
#include <stdio.h>

#include "BufferFill.h"
#include "Utilities.h"

//============================================================================
//
//============================================================================


BufferFill::BufferFill()
{
   // initialize empty
   m_i_NumberOfContinuePoints = 0;
   m_i_MaxStack               = 0;
   m_pc_SourceBuffer          = NULL;
   m_pc_DestinationBuffer     = NULL;

   m_i_Distance               = 0;
   m_p_DistanceColor          = 0x808080;
   m_i_DistanceMode           = BufferFillDistanceOrigin;

} // BufferFill()

//============================================================================
//
//============================================================================

void BufferFill::Push(int i_X, int i_Y)
{
   if(m_i_NumberOfContinuePoints+1 >= NUMBER_OF_CONTINUE_POINTS_MAX)
   {
      // first attemt to reduce the list by already filled points
      int i_From;
      int i_To;
      int i_Removed;

      i_Removed =0;
      i_From    =0;
      i_To      =0;

      while(i_From < m_i_NumberOfContinuePoints)
      {
         int i_CurrentX;
         int i_CurrentY;
         i_CurrentX = m_ContinuePoints[i_From].m_i_X;
         i_CurrentY = m_ContinuePoints[i_From].m_i_X;
         if( ! HasToFill(i_CurrentX, i_CurrentY))
         {
            // point already filled, drop it
            i_From ++;
            i_Removed ++;
         }
         else // has to fill
         {
            if(i_Removed >0)
            {
               m_ContinuePoints[i_To] = m_ContinuePoints[i_From];
            }
            i_From++;
            i_To  ++;
         }

      } // while trough all points
   } // if list is full


   if(m_i_NumberOfContinuePoints+1 < NUMBER_OF_CONTINUE_POINTS_MAX)
   {
      m_ContinuePoints[m_i_NumberOfContinuePoints].m_i_X = i_X;
      m_ContinuePoints[m_i_NumberOfContinuePoints].m_i_Y = i_Y;
      m_i_NumberOfContinuePoints ++;
      if(m_i_MaxStack<m_i_NumberOfContinuePoints)
      {
         m_i_MaxStack=m_i_NumberOfContinuePoints;
      }
   }

} // Push()

//============================================================================
//
//============================================================================

bool BufferFill::Pop(int &i_X, int &i_Y)
{
   if(m_i_NumberOfContinuePoints <=0)
   {
      return false;
   }
   m_i_NumberOfContinuePoints --;
   i_X = m_ContinuePoints[m_i_NumberOfContinuePoints].m_i_X ;
   i_Y = m_ContinuePoints[m_i_NumberOfContinuePoints].m_i_Y ;

   return true;
} // Pop()

//============================================================================
//
//============================================================================

void BufferFill::StartFill(int i_StartX, int i_StartY)
{
   bool b_CheckUp;
   bool b_CheckDown;
   bool b_CheckUpSave;
   bool b_CheckDownSave;
   bool b_NextPoint;
   int  i_RunX;

   m_i_SourceWidth  = m_pc_SourceBuffer->GetWidth();
   m_i_SourceHeight = m_pc_SourceBuffer->GetHeight();

   // set region to start point
   m_i_MinX = i_StartX;
   m_i_MinY = i_StartY;
   m_i_MaxX = i_StartX;
   m_i_MaxY = i_StartY;

   // set number of filled points
   m_i_FilledPoint =0;



   if(m_pc_DestinationBuffer  == NULL)
   {
      // no seperate Destination buffer
      if(m_NewColor == m_ChangeColor)
      {
         // new and change color are equal. stop filling
         return ;
      }
   }

   if( m_i_DistanceMode == BufferFillDistanceOrigin)
   {
      m_p_RealUsedDistanceColor = m_pc_SourceBuffer->Get(i_StartX,i_StartY);
   }
   else
   {
      m_p_RealUsedDistanceColor = m_p_DistanceColor;
   }


   do
   {

      if(HasToFill(i_StartX, i_StartY))
      {
         // mark this point as filled
         FillPoint(i_StartX,i_StartY);

         //
         if(HasToFill(i_StartX,i_StartY-1) )
         {
            Push(i_StartX, i_StartY-1);
            b_CheckUp = false;
         }
         else
         {
            b_CheckUp = true;
         }

         if(HasToFill(i_StartX,i_StartY+1) )
         {
            Push(i_StartX, i_StartY+1);
            b_CheckDown = false;
         }
         else
         {
            b_CheckDown = true;
         }

         // prepare tun to left
         i_RunX          = i_StartX-1;
         b_CheckUpSave   = b_CheckUp;
         b_CheckDownSave = b_CheckDown;

         // go to left
         while(   i_RunX >=0
               && HasToFill(i_RunX,i_StartY)
              )
         {
            bool b_Up;
            bool b_Down;

            // mark this point as filled
            FillPoint(i_RunX,i_StartY);

            b_Up   = HasToFill(i_RunX,i_StartY-1);
            b_Down = HasToFill(i_RunX,i_StartY+1);

            if(b_Up && b_CheckUp)
            {
               Push(i_RunX, i_StartY-1);
               b_CheckUp = false;

            }
            else if(!b_Up)
            {
               b_CheckUp =true;
            }

            if(b_Down && b_CheckDown)
            {
               Push(i_RunX, i_StartY+1);
               b_CheckDown = false;

            }
            else if(!b_Down)
            {
               b_CheckDown =true;
            }

            i_RunX --;
         } // while go to left

         // prepare run to right

         b_CheckUp   = b_CheckUpSave;
         b_CheckDown = b_CheckDownSave;
         i_RunX      = i_StartX+1;

         // go to right
         while(   i_RunX < m_i_SourceWidth
               && HasToFill(i_RunX,i_StartY)
              )
         {
            bool b_Up;
            bool b_Down;

            // mark this point as filled
            FillPoint(i_RunX,i_StartY);

            b_Up   = HasToFill(i_RunX,i_StartY-1);
            b_Down = HasToFill(i_RunX,i_StartY+1);

            if(b_Up && b_CheckUp)
            {
               Push(i_RunX, i_StartY-1);
               b_CheckUp = false;

            }
            else if(!b_Up)
            {
               b_CheckUp =true;
            }

            if(b_Down && b_CheckDown)
            {
               Push(i_RunX, i_StartY+1);
               b_CheckDown = false;

            }
            else if(!b_Down)
            {
               b_CheckDown =true;
            }

            i_RunX ++;
         } // while go to right
      } // if Has to Fill

      // check for a next point
      b_NextPoint = Pop(i_StartX,i_StartY);

   }  while(b_NextPoint );

} // StartFill()

//============================================================================
//
//============================================================================

bool BufferFill::HasToFill(int i_X, int i_Y)
{
   // border reached
   if(    i_X < 0
       || i_Y < 0
       || i_X >= m_i_SourceWidth
       || i_Y >= m_i_SourceHeight
      )
   {
      return false;
   }

   if(m_pc_DestinationBuffer != NULL)
   {
      // point already filled ??
      if(m_pc_DestinationBuffer->Get(i_X,i_Y) == m_NewColor)
      {
         return false;
      }

      if(m_i_Distance<=0)
      {
         // point in source has to be filled
         if(m_pc_SourceBuffer->Get(i_X,i_Y) == m_ChangeColor)
         {
            return true;
         }
      }
      else // distance greater 0
      {
         Pixel p_Actual;
         p_Actual = m_pc_SourceBuffer->Get(i_X,i_Y);
         if( Utilities::PixelDistance(p_Actual,m_p_RealUsedDistanceColor ) < m_i_Distance)
         {
            return true;
         }

      }

      // point in source has not to be filled !
      return false;
   }
   else // no seperate destination buffer
   {
      if(m_pc_SourceBuffer->Get(i_X,i_Y) ==  m_ChangeColor)
      {
         return true;
      }
      return false;
   } // else no seperate destination buffer

} // HasToFill

//============================================================================
//
//============================================================================

void BufferFill::FillPoint(int i_X, int i_Y)
{
   if(m_pc_DestinationBuffer == NULL)
   {
      m_pc_SourceBuffer->Set(i_X,i_Y,m_NewColor);
   }
   else
   {
      m_pc_DestinationBuffer->Set(i_X,i_Y,m_NewColor);
   }
   
   // increase the number of filled points
   m_i_FilledPoint ++;
   
   // update the changed region
   if(i_X < m_i_MinX)
   {
      m_i_MinX = i_X;
   }
   else if (i_X > m_i_MaxX)
   {
      m_i_MaxX = i_X;
   }

   // also in y-dimension
   if(i_Y < m_i_MinY)
   {
      m_i_MinY = i_Y;
   }
   else if (i_Y > m_i_MaxY)
   {
      m_i_MaxY = i_Y;
   }

} // FillPoint()

//============================================================================
//
//============================================================================

RectRegion BufferFill::GetChangedRegion()
{
   RectRegion c_ReturnRegion;
   c_ReturnRegion.SetXYXY(m_i_MinX,m_i_MinY,m_i_MaxX,m_i_MaxY);

   return c_ReturnRegion;

} // GetChangedRegion()

//============================================================================
//
//============================================================================

void BufferFill::SetColorDistance(int i_Distance)
{
   m_i_Distance = i_Distance;
} // SetColorDistance()

//============================================================================
//
//============================================================================

void BufferFill::SetColorDistanceColor(Pixel p)
{
   m_p_DistanceColor = p;
} // SetColorDistanceColor()

//============================================================================
//
//============================================================================

void BufferFill::SetColorDistanceFrom(int i_DistanceMode)
{
   m_i_DistanceMode = i_DistanceMode;
} // SetColorDistanceFrom()

//============================================================================
//
//============================================================================



