
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "BufferPainter.h"

#include "Utilities.h"
#include "PixelBuffer.h"
#include "ColorCalculation.h"


//===========================================================================
//
//===========================================================================


BufferPainter::BufferPainter()
{
   m_BorderBrush.m_Color =(0xffaa88);
   m_InnerBrush.m_Color  =(0x55ff55);

   m_BlackBrush.m_Color  =(0x000000);
   m_WhiteBrush.m_Color  =(0xFFFFFF);

   m_i_LineWidth          = 7;
   m_b_DrawInnerPart      = true;
   m_b_DrawTransparent    = false;
   m_i_TransparencyFactor = 0;

   m_i_FilterWidth              = 1;
   m_b_FilterNearDistanceActive = false;
   m_i_FilterNearDistance       = 30;
   m_i_FilterAlgorithm          = FilterAlgorithmMiddle;
   // disable the onetimedraw buffer
   m_b_UseOneTimeDrawBuffer = false;
//   m_pc_OneTimeDrawBuffer   = NULL;

   SetMaskMode(false,false);

} // BufferPainter

//===========================================================================
//
//===========================================================================

void BufferPainter::FilterPixelMiddle(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y)
{
   PixelBuffer *pc_Original;
   Pixel p_Value;
   Pixel p_Original;
   int i_Width;
   int i_Height;

   int i_RangeFromY;
   int i_RangeFromX;
   int i_RangeToX;
   int i_RangeToY;
   int i_RangeTotalPoints;

   int i_InnerX;
   int i_InnerY;

   int i_RedSum;
   int i_GreenSum;
   int i_BlueSum;

   // get the original picture, which must not get modified
   pc_Original = pc_Buffer->GetUsedPixelMask();

   // get the image dimension
   i_Width  = pc_Original->GetWidth();
   i_Height = pc_Original->GetHeight();

   // calculate the filter region
   i_RangeFromY = i_Y - m_i_FilterWidth;
   if(i_RangeFromY <0)
   {
      i_RangeFromY =0;
   }
   i_RangeToY = i_Y + m_i_FilterWidth;
   if(i_RangeToY >=i_Height)
   {
      i_RangeToY = i_Height-1;
   }

   i_RangeFromX = i_X - m_i_FilterWidth;
   if(i_RangeFromX <0)
   {
      i_RangeFromX =0;
   }
   i_RangeToX = i_X + m_i_FilterWidth;
   if(i_RangeToX >= i_Width)
   {
      i_RangeToX = i_Width-1;
   }

   // calculate the total points in the range
   i_RangeTotalPoints=   (i_RangeToX - i_RangeFromX +1)
                        *(i_RangeToY - i_RangeFromY +1);

   // set sums to zero
   i_RedSum  = 0;
   i_GreenSum= 0;
   i_BlueSum = 0;

   // in some modes the original is needed
   if(   m_b_FilterNearDistanceActive)
   {
      p_Original = pc_Original->Get(i_X,i_Y);
   }
         
   // inner loop - calculate the pixelsumms
   for(i_InnerY =  i_RangeFromY;
       i_InnerY <= i_RangeToY;
       i_InnerY ++)
   {
      for(i_InnerX =  i_RangeFromX;
          i_InnerX <= i_RangeToX;
          i_InnerX ++)
      {
         p_Value = pc_Original->Get(i_InnerX,i_InnerY);

         if(m_b_FilterNearDistanceActive)
         {
            int i_ActualDistance;
            i_ActualDistance = Utilities::PixelDistance(p_Value,p_Original);
            if( i_ActualDistance > m_i_FilterNearDistance)
            {
               // forget this point
               i_RangeTotalPoints -= 1;
               continue;
            }
         }
         
         i_RedSum   += PIXEL_RED  (p_Value);
         i_GreenSum += PIXEL_GREEN(p_Value);
         i_BlueSum  += PIXEL_BLUE (p_Value);

      } // for i_InnerX
   } // for i_InnerY

   if(i_RangeTotalPoints >= 1)
   {
      // divide summ by number of pixels
      i_RedSum   /= i_RangeTotalPoints;
      i_GreenSum /= i_RangeTotalPoints;
      i_BlueSum  /= i_RangeTotalPoints;
      // clipp maximum to 255
      if(i_RedSum>255)
      {
         i_RedSum = 255;
      }
      if(i_GreenSum>255)
      {
         i_GreenSum = 255;
      }
      if(i_BlueSum>255)
      {
         i_BlueSum = 255;
      }

      // compute the new pixel
      p_Value = PIXEL_MAKE(i_RedSum,i_GreenSum,i_BlueSum);

      // set the new pixel into the image
      pc_Buffer->Set(i_X,i_Y,p_Value);
   }


} // FilterPixelMiddle()

//===========================================================================
//
//===========================================================================

void BufferPainter::FilterPixelMedian(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y)
{
   PixelBuffer *pc_Original;
   Pixel p_Value;
   Pixel p_Original;
   int i_Width;
   int i_Height;

   int i_RangeFromY;
   int i_RangeFromX;
   int i_RangeToX;
   int i_RangeToY;
   int i_RangeTotalPoints;

   int i_InnerX;
   int i_InnerY;
   int i_ActualPoint;

   unsigned char *p_ArrayRed;
   unsigned char *p_ArrayGreen;
   unsigned char *p_ArrayBlue;

   // get the original picture, which must not get modified
   pc_Original = pc_Buffer->GetUsedPixelMask();

   // get the image dimension
   i_Width  = pc_Original->GetWidth();
   i_Height = pc_Original->GetHeight();

   // calculate the filter region
   i_RangeFromY = i_Y - m_i_FilterWidth;
   if(i_RangeFromY <0)
   {
      i_RangeFromY =0;
   }
   i_RangeToY = i_Y + m_i_FilterWidth;
   if(i_RangeToY >=i_Height)
   {
      i_RangeToY = i_Height-1;
   }

   i_RangeFromX = i_X - m_i_FilterWidth;
   if(i_RangeFromX <0)
   {
      i_RangeFromX =0;
   }
   i_RangeToX = i_X + m_i_FilterWidth;
   if(i_RangeToX >= i_Width)
   {
      i_RangeToX = i_Width-1;
   }

   // calculate the total points in the range
   i_RangeTotalPoints=   (i_RangeToX - i_RangeFromX +1)
                        *(i_RangeToY - i_RangeFromY +1);

   // allocate 3 arrays for the sorting vector
   p_ArrayRed    = new unsigned char[i_RangeTotalPoints];
   if(p_ArrayRed == NULL)
   {
      return ;
   }
   p_ArrayGreen  = new unsigned char[i_RangeTotalPoints];
   if(p_ArrayGreen == NULL)
   {
      delete p_ArrayRed;
      return ;
   }
   p_ArrayBlue   = new unsigned char[i_RangeTotalPoints];
   if(p_ArrayBlue == NULL)
   {
      delete p_ArrayRed;
      delete p_ArrayGreen;
      return ;
   }


   // in some modes the original is needed
   if(   m_b_FilterNearDistanceActive)
   {
      p_Original = pc_Original->Get(i_X,i_Y);
   }

   i_ActualPoint =0;

   // inner loop - calculate the pixelsumms
   for(i_InnerY =  i_RangeFromY;
       i_InnerY <= i_RangeToY;
       i_InnerY ++)
   {
      for(i_InnerX =  i_RangeFromX;
          i_InnerX <= i_RangeToX;
          i_InnerX ++)
      {
         p_Value = pc_Original->Get(i_InnerX,i_InnerY);

         if(m_b_FilterNearDistanceActive)
         {
            int i_ActualDistance;
            i_ActualDistance = Utilities::PixelDistance(p_Value,p_Original);
            if( i_ActualDistance > m_i_FilterNearDistance)
            {
               // forget this point
               continue;
            }
         }

         p_ArrayRed  [i_ActualPoint] = (unsigned char) PIXEL_RED  (p_Value);
         p_ArrayGreen[i_ActualPoint] = (unsigned char) PIXEL_GREEN(p_Value);
         p_ArrayBlue [i_ActualPoint] = (unsigned char) PIXEL_BLUE (p_Value);
         i_ActualPoint ++;

      } // for i_InnerX
   } // for i_InnerY

   // middle pixel makes only sense, if we have >=3 pixel
   if(i_ActualPoint > 2)
   {
      int i_Red;
      int i_Green;
      int i_Blue;

      // now sort the array
      Utilities::QSortBytes(p_ArrayRed  ,0,i_ActualPoint -1);
      Utilities::QSortBytes(p_ArrayGreen,0,i_ActualPoint -1);
      Utilities::QSortBytes(p_ArrayBlue ,0,i_ActualPoint -1);

      // take the point in the middle
      i_ActualPoint /=2;
      i_Red   = p_ArrayRed  [i_ActualPoint];
      i_Green = p_ArrayGreen[i_ActualPoint];
      i_Blue  = p_ArrayBlue [i_ActualPoint];

      // compute the new pixel
      p_Value = PIXEL_MAKE(i_Red,i_Green,i_Blue);

      // set the new pixel into the image
      pc_Buffer->Set(i_X,i_Y,p_Value);
   }

   // free the allocated arrays
   delete p_ArrayRed;
   delete p_ArrayGreen;
   delete p_ArrayBlue;

} // FilterPixelMedian()

//===========================================================================
//
//===========================================================================


void BufferPainter::DrawPixel(PixelBuffer *pc_Buffer,
                              PainterBrush *pc_Brush,
                              int i_X,
                              int i_Y)
{
   Pixel p_NewPixel;

   // is the only overwrite once buffer used?
   if(    m_b_UseOneTimeDrawBuffer
      &&  pc_Buffer->UsedPixelMaskAvailable())
   {
      if(pc_Buffer->UsedPixelMaskGet(i_X,i_Y))
      {
         // already set
         return;
      }
      else
      {
         // set it for this pixel, to prevent the next time
         pc_Buffer->UsedPixelMaskSet(i_X,i_Y,1);
      }
   }

   if(pc_Brush->m_b_Lightness)
   {
      Pixel p_Old;
      Pixel p_New;
      int   i_Red;
      int   i_Green;
      int   i_Blue;
      int   i_Hue;
      int   i_Lightness;
      int   i_Saturation;

      p_Old = pc_Buffer->Get(i_X, i_Y);
      i_Red   = PIXEL_RED  (p_Old);
      i_Green = PIXEL_GREEN(p_Old);
      i_Blue  = PIXEL_BLUE (p_Old);

      // convert into HLS
      ColorCalculation::RGBToHLS(i_Red,i_Green,i_Blue,i_Hue,i_Lightness,i_Saturation);

      // modify the lightness
      i_Lightness += pc_Brush->m_i_LightnessFactor;
      if(i_Lightness <0)
      {
         i_Lightness =0;
      }
      else if(i_Lightness >100)
      {
         i_Lightness = 100;
      }

      // convert back into rgb
      ColorCalculation::HLSToRGB(i_Hue,i_Lightness,i_Saturation,i_Red,i_Green,i_Blue);
      p_New = PIXEL_MAKE(i_Red,i_Green,i_Blue);

      pc_Buffer->Set(i_X,i_Y,p_New);

   }
   else if(pc_Brush->m_b_Saturation)
   {
      Pixel p_Old;
      Pixel p_New;
      int   i_Red;
      int   i_Green;
      int   i_Blue;
      int   i_Hue;
      int   i_Lightness;
      int   i_Saturation;

      p_Old = pc_Buffer->Get(i_X, i_Y);
      i_Red   = PIXEL_RED  (p_Old);
      i_Green = PIXEL_GREEN(p_Old);
      i_Blue  = PIXEL_BLUE (p_Old);

      //convert into HLS
      ColorCalculation::RGBToHLS(i_Red,i_Green,i_Blue,i_Hue,i_Lightness,i_Saturation);

      // modify the saturation
      i_Saturation += pc_Brush->m_i_LightnessFactor;
      if(i_Saturation <0)
      {
         i_Saturation =0;
      }
      else if(i_Saturation >100)
      {
         i_Saturation = 100;
      }

      // convert back into rgb
      ColorCalculation::HLSToRGB(i_Hue,i_Lightness,i_Saturation,i_Red,i_Green,i_Blue);
      p_New = PIXEL_MAKE(i_Red,i_Green,i_Blue);

      pc_Buffer->Set(i_X,i_Y,p_New);

   }
   else if(pc_Brush->m_b_Filter)
   {
      if(m_i_FilterAlgorithm == FilterAlgorithmMiddle)
      {
         FilterPixelMiddle(pc_Buffer,i_X,i_Y);
      }
      else if(m_i_FilterAlgorithm == FilterAlgorithmMedian)
      {
         FilterPixelMedian(pc_Buffer,i_X,i_Y);
      }
   }
   else
   {
      bool b_Valid;
      // regular pixel paint
      p_NewPixel = pc_Brush->GetBrushPixel(i_X,i_Y,b_Valid);

      // if pixel is not valid
      if(!b_Valid)
      {
         // leave it untouched
         return;
      }
      
      if(m_b_DrawTransparent
         && m_i_TransparencyFactor>0)
      {
         // make a transparent color
         Pixel p_Old;

         p_Old = pc_Buffer->Get(i_X, i_Y);

         p_NewPixel = ColorCalculation::TransparentColor(p_Old,
                                                         p_NewPixel,
                                                         m_i_TransparencyFactor);
         pc_Buffer->Set(i_X,i_Y,p_NewPixel);

      }
      else // not transparent
      {
         pc_Buffer->Set(i_X,i_Y,p_NewPixel);
      }
   }


} // DrawPixel()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawHorizontalLine(PixelBuffer *pc_Buffer,
                                       PainterBrush *pc_Brush,
                                       int i_StartX,
                                       int i_EndX,
                                       int i_Y)
{
   int i;
   // fprintf(stderr,"Horizontal %d %d\n",i_StartX,i_EndX);
   if(i_StartX > i_EndX)
   {
      Utilities::swap(i_StartX,i_EndX);
   }

   for(i=i_StartX; i<=i_EndX;i++)
   {
      DrawPixel(pc_Buffer,pc_Brush,i,i_Y);
   }

} // DrawHorizontalLine()


//===========================================================================
//
//===========================================================================

void BufferPainter::DrawPoint(PixelBuffer *pc_Buffer,int i_X, int i_Y)
{
   DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_X,i_Y);

   m_ChangedRegion.SetXYWH(i_X,i_Y,1,1);

} // DrawPoint()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawLine(PixelBuffer *pc_Buffer,
                             int i_X1, int i_Y1,
                             int i_X2, int i_Y2)
{
   BaseDrawLine(  pc_Buffer,
                 m_pc_CurrentBorderBrush,
                i_X1,i_Y1,
                i_X2,i_Y2,
                m_i_LineWidth,
                 m_ChangedRegion);


} // DrawLine()


//===========================================================================
//
//===========================================================================

void BufferPainter::DrawLineWithWidth1(PixelBuffer *pc_Buffer,
				PainterBrush *pc_Brush,
                             int i_X1, int i_Y1,
                             int i_X2, int i_Y2)
{

   int i_DiffX;
   int i_DiffY;
   
   // calculate differences
   i_DiffX = i_X2 - i_X1;
   if(i_DiffX<0)
   {
      i_DiffX = - i_DiffX;
   }

   i_DiffY = i_Y2 - i_Y1;
   if(i_DiffY<0)
   {
      i_DiffY = - i_DiffY;
   }

   if(i_DiffX >= i_DiffY)
   {
      int i_RunX;
      int i_RunY;
      int i_YOffset;
      int i_Divider;

      // make loop for X
      if(i_X1> i_X2)
      {
         Utilities::swap(i_X1,i_X2);   
         Utilities::swap(i_Y1,i_Y2);   
      }
      if(i_Y1>i_Y2) 
      {
         i_YOffset = -1;
      }
      else if(i_Y1<i_Y2)
      {
         i_YOffset = 1;
      }
      else
      {
         i_YOffset = 0;
      }

      i_Divider = i_DiffX /2;
      i_RunY = i_Y1;
      for(i_RunX= i_X1; i_RunX <= i_X2;i_RunX++)
      {
         DrawPixel(pc_Buffer,pc_Brush,i_RunX,i_RunY);
         i_Divider -= i_DiffY;
         if(i_Divider <0)
         {
            i_Divider += i_DiffX;
            i_RunY    += i_YOffset;
         }
      }

   }
   else  // DiffY ist larger
   {
      // make loop for Y
      int i_RunX;
      int i_RunY;
      int i_XOffset;
      int i_Divider;

      if(i_Y1> i_Y2)
      {
         Utilities::swap(i_X1,i_X2);   
         Utilities::swap(i_Y1,i_Y2);   
      }
      if(i_X1>i_X2) 
      {
         i_XOffset = -1;
      }
      else if(i_X1<i_X2)
      {
         i_XOffset = 1;
      }
      else
      {
         i_XOffset = 0;
      }

      i_Divider = i_DiffY /2;
      i_RunX = i_X1;
      for(i_RunY= i_Y1; i_RunY <= i_Y2;i_RunY++)
      {
         DrawPixel(pc_Buffer,pc_Brush,i_RunX,i_RunY);
         i_Divider -= i_DiffX;
         if(i_Divider <0)
         {
            i_Divider += i_DiffY;
            i_RunX    += i_XOffset;
         }
      }

   }


} // DrawLineWithWidth1()

//===========================================================================
//
//===========================================================================

RectRegion BufferPainter::GetChangedRegion()
{
   return m_ChangedRegion;
} // GetChangedRegion()

//===========================================================================
//
//===========================================================================


void BufferPainter::DrawRectangle(PixelBuffer *pc_Buffer,
                             int i_X1, int i_Y1,
                             int i_X2, int i_Y2)
{
   int i_RunX;
   int i_RunY;

   int i_Height;
   int i_Width;

   if(i_X1 > i_X2)
   {
      Utilities::swap(i_X1,i_X2);
   }

   if(i_Y1 > i_Y2)
   {
      Utilities::swap(i_Y1,i_Y2);
   }

   i_Width  = (i_X2 - i_X1)+1;
   i_Height = (i_Y2 - i_Y1)+1;


   // now x1 <= x2 and y1 <= y2

   if(    ( m_i_LineWidth * 2) >= i_Width
       || ( m_i_LineWidth * 2) >= i_Height )
   {
      // Draw full filled rectangle
      for(i_RunY = i_Y1; i_RunY <= i_Y2; i_RunY++)
      {
         for(i_RunX = i_X1; i_RunX <= i_X2; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
      }
   }
   else
   {
      // draw inner part in another color

      // start and end of the inner part - including
      int i_LeftInner;
      int i_RightInner;


      i_LeftInner  = i_X1 + m_i_LineWidth;
      i_RightInner = i_X2 - m_i_LineWidth;


      // draw the upper line part
      for(i_RunY = i_Y1; i_RunY < i_Y1+m_i_LineWidth; i_RunY++)
      {
         for(i_RunX = i_X1; i_RunX <= i_X2; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
      }
      // draw the middle Part
      for(i_RunY = i_Y1+m_i_LineWidth; 
          i_RunY <= i_Y2-(m_i_LineWidth);
          i_RunY++)
      {
         // draw left line
         for(i_RunX = i_X1; i_RunX < i_LeftInner; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
         // draw middle part
         if(m_b_DrawInnerPart)
         {
            for(i_RunX = i_LeftInner; i_RunX <= i_RightInner; i_RunX++)
            {
               DrawPixel(pc_Buffer,m_pc_CurrentInnerBrush,i_RunX,i_RunY);
            }
         }
         // draw right line
         for(i_RunX = i_RightInner+1; i_RunX <= i_X2; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
      }

      // draw the lower line part
      for(i_RunY = i_Y2-(m_i_LineWidth-1); i_RunY <= i_Y2; i_RunY++)
      {
         for(i_RunX = i_X1; i_RunX <= i_X2; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
      }
   }




   m_ChangedRegion.SetXYXY(i_X1,i_Y1,i_X2,i_Y2);


} // DrawRectangle

//===========================================================================
//
//===========================================================================

void BufferPainter::BaseDrawLine (PixelBuffer *pc_Buffer,
   				PainterBrush *pc_Brush,
   	      			int i_StartX,
				int i_StartY,
    	     			int i_EndX,
				int i_EndY,
     	    			int i_LineWidth,
				RectRegion &c_ChangedRegion)
{

   float  f_DifferenceX;
   float  f_DifferenceY;
   float  f_Length;
   float  f_Width;
   int    i_DeltaX;
   int    i_DeltaY;
   int    i_PointX[4];
   int    i_PointY[4];
   int    i_RunningY;
   int    i_MaximumDifference;
   int    i_DeltaRight;
   int    i_DeltaLeft;

   // calculate the delta value
   i_DeltaX = i_EndX - i_StartX;
   i_DeltaY = i_EndY - i_StartY;


   // is it a vertical line ?
   if (i_DeltaX == 0)
   {

      int i_RunningX;
      int i_RunningY;

      // let startY smaller than end
      if (i_EndY < i_StartY)
      {
         Utilities::swap (i_StartY, i_EndY);
      }


      if (i_LineWidth == 1)
      {
         // set the modified region
         c_ChangedRegion.SetXYXY(i_StartX, i_StartY, i_EndX, i_EndY);

         // draw the vertical line
         for (i_RunningY = i_StartY; i_RunningY <= i_EndY; i_RunningY++)
         {
            DrawPixel(pc_Buffer,pc_Brush, i_StartX, i_RunningY);
         }
      }
      else // linewidth larger one
      {
         int i_RunningEndX;
         i_RunningX    = i_StartX   - i_LineWidth / 2;
         i_RunningEndX = i_RunningX + i_LineWidth - 1;
         c_ChangedRegion.SetXYXY( i_RunningX, i_StartY,
                                  i_RunningEndX, i_EndY);
         for (i_RunningY = i_StartY; i_RunningY <= i_EndY; i_RunningY++)
         {
            DrawHorizontalLine(pc_Buffer,
                               pc_Brush,
                               i_RunningX,
                               i_RunningEndX,
                               i_RunningY);
         }

      }

   }
   // is it a horizonal line
   else if (i_DeltaY == 0)
   {

      if (i_EndX < i_StartX)
      {
          Utilities::swap  ( i_StartX, i_EndX);
      }
 
      if (i_LineWidth == 1)
      {
         c_ChangedRegion.SetXYXY(i_StartX, i_StartY,
                                  i_EndX, i_EndY);
         DrawHorizontalLine(pc_Buffer,
                               pc_Brush,
                               i_StartX, i_EndX, i_StartY);
      }
      else
      {
         int i_Y;
         i_RunningY = i_StartY - i_LineWidth / 2;

         c_ChangedRegion.SetXYXY( i_StartX, i_RunningY,
                                  i_EndX, i_RunningY + i_LineWidth - 1);

         for (i_Y = 0; 
              i_Y < i_LineWidth;
              i_Y++, i_RunningY++)
         {
            DrawHorizontalLine(pc_Buffer,
                               pc_Brush,
                               i_StartX, 
                               i_EndX, 
                               i_RunningY);
         }
      }

   }

   // simple line with width 1
   else if (i_LineWidth == 1)

   {
      int i;
      c_ChangedRegion.SetXYXY(i_StartX, i_StartY,
                              i_EndX, i_EndY);
      i_MaximumDifference = abs (i_DeltaX);

      if (abs (i_DeltaY) > i_MaximumDifference)
      {
         i_MaximumDifference = abs (i_DeltaY);
      }

      for (i = 0; i <= i_MaximumDifference; i++)
      {
         int  i_XPosition;
         int  i_YPosition;
    
         i_XPosition = i_StartX + i_DeltaX * i / i_MaximumDifference;
         i_YPosition = i_StartY + i_DeltaY * i / i_MaximumDifference;
         DrawPixel (pc_Buffer,pc_Brush, i_XPosition, i_YPosition);
      }
   }

   // default case : not vertical, horizontal and width > 1
   else
   {

      int i_ClimbRight;
      int i_ClimbLeft;

      if (i_StartX > i_EndX)
      {
         Utilities::swap (i_StartX, i_EndX);
         Utilities::swap (i_StartY, i_EndY);
         i_DeltaX = -i_DeltaX;
         i_DeltaY = -i_DeltaY;
      }

      // calculate 4 points
      // point 0 upper most
      // point 1 left most
      // point 2 right most
      // point 3 lowest

      if (i_EndY > i_StartY)
      {
         f_Length = Utilities::Hypothenusis (i_DeltaX, i_DeltaY);
         f_Width = i_LineWidth / 2.0;
         f_DifferenceX = (float) f_Width *(float) i_DeltaY / (float) f_Length;
         f_DifferenceY = (float) f_Width *(float) i_DeltaX / (float) f_Length;

         i_PointX[0] = i_StartX + (int) (f_DifferenceX+0.5);
         i_PointY[0] = i_StartY - (int) (f_DifferenceY+0.5);
         i_PointX[1] = i_StartX - (int) (f_DifferenceX+0.5);
         i_PointY[1] = i_StartY + (int) (f_DifferenceY+0.5);
         i_PointX[2] = i_PointX[0] + i_DeltaX;
         i_PointY[2] = i_PointY[0] + i_DeltaY;
         i_PointX[3] = i_PointX[1] + i_DeltaX;
         i_PointY[3] = i_PointY[1] + i_DeltaY;
         i_DeltaRight = i_DeltaX;
         i_DeltaLeft = i_DeltaY;
         i_ClimbRight = i_DeltaLeft / 2;
         i_ClimbLeft = i_DeltaRight / 2;
      }
      else
      {
         f_Length = Utilities::Hypothenusis (i_DeltaX, i_DeltaY);
         f_Width = i_LineWidth / 2.0;
         f_DifferenceX = (float) f_Width * (-(float) i_DeltaY) / (float) f_Length;
         f_DifferenceY = (float) f_Width *   (float) i_DeltaX  / (float) f_Length;

         i_PointX[1] = i_StartX - (int) (f_DifferenceX+0.5);
         i_PointY[1] = i_StartY - (int) (f_DifferenceY+0.5);
         i_PointX[3] = i_StartX + (int) (f_DifferenceX+0.5);
         i_PointY[3] = i_StartY + (int) (f_DifferenceY+0.5);
         i_PointX[0] = i_PointX[1] + i_DeltaX;
         i_PointY[0] = i_PointY[1] + i_DeltaY;
         i_PointX[2] = i_PointX[3] + i_DeltaX;
         i_PointY[2] = i_PointY[3] + i_DeltaY;
         i_DeltaRight = -i_DeltaY;
         i_DeltaLeft  = i_DeltaX;
         i_ClimbRight = i_DeltaLeft / 2;
         i_ClimbLeft  = i_DeltaRight / 2;
      }

      {
         int i_ActualRightPosition;
         int i_ActualLeftPosition;
         int i_RunningY;

         i_ActualLeftPosition  = i_PointX[0];
         i_ActualRightPosition = i_PointX[0];

         // now calculate the extension for the uppermost row
         i_ClimbLeft -= i_DeltaLeft/2;
         while (i_ClimbLeft < 0)
         {
            i_ClimbLeft += i_DeltaRight;
            i_ActualLeftPosition--;
         }
         i_ClimbRight -= i_DeltaRight/2;
         while (i_ClimbRight < 0)
         {
            i_ClimbRight += i_DeltaLeft;
            i_ActualRightPosition++;
         }


         for (i_RunningY = i_PointY[0]; i_RunningY <= i_PointY[3]; i_RunningY++)
         {


            DrawHorizontalLine(pc_Buffer,
                               pc_Brush, 
                               i_ActualLeftPosition,
                               i_ActualRightPosition,
                               i_RunningY);

            // calculate the left startpoint of the drawing line
            if (i_RunningY < i_PointY[1])

            {
               i_ClimbLeft -= i_DeltaLeft;
               while (i_ClimbLeft < 0)
               {
                  i_ClimbLeft += i_DeltaRight;
                  i_ActualLeftPosition--;
               }
               if (i_ActualLeftPosition < i_PointX[1])
               {
                  i_ActualLeftPosition = i_PointX[1];
               }
            }
            else if (i_RunningY > i_PointY[1])

            {
               i_ClimbLeft -= i_DeltaRight;
               while (i_ClimbLeft <= 0)
               {
                  i_ClimbLeft += i_DeltaLeft;
                  i_ActualLeftPosition++;
               }
               if (i_ActualLeftPosition > i_PointX[3])
               {
                  i_ActualLeftPosition = i_PointX[3];
               }
            }
            else // point 1 reached
            {
               i_ActualLeftPosition = i_PointX[1];
               i_ClimbLeft = i_DeltaLeft / 2;
            }


            // calculate the right end of the line
            if (i_RunningY < i_PointY[2])
            {
               i_ClimbRight -= i_DeltaRight;
               while (i_ClimbRight < 0)
               {
                  i_ClimbRight += i_DeltaLeft;
                  i_ActualRightPosition++;
               }
               if (i_ActualRightPosition > i_PointX[2])
               {
                  i_ActualRightPosition = i_PointX[2];
               }
            }
            else if (i_RunningY > i_PointY[2])

            {
               i_ClimbRight -= i_DeltaLeft;
               while (i_ClimbRight < 0)
               {
                  i_ClimbRight += i_DeltaRight;
                  i_ActualRightPosition--;
               }
               if (i_ActualRightPosition < i_PointX[3])
               {
                  i_ActualRightPosition = i_PointX[3];
               }
            }
            else // point 2 reached
            {
               i_ActualRightPosition = i_PointX[2];
               i_ClimbRight = i_DeltaRight / 2;
            }
         }
      }

      c_ChangedRegion.SetXYXY (i_PointX[1], i_PointY[0],
                               i_PointX[2], i_PointY[3]);


   } // else ordinary line


} // BaseDrawLine

//===========================================================================
//
//===========================================================================

void BufferPainter::SetLineWidth(int i_LineWidth)
{
   m_i_LineWidth = i_LineWidth;
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetDrawInnerEnable(bool b_On)
{
   m_b_DrawInnerPart = b_On;

}  // SetDrawInnerEnable()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetTransparencyEnable(bool b_on)
{
   m_b_DrawTransparent = b_on;
} // SetTransparencyEnable()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetTransparencyFactor(int i_Factor)
{
   m_i_TransparencyFactor = i_Factor;
} // SetTransparencyFactor()


//===========================================================================
//
//===========================================================================

void BufferPainter::SetInnerColor(Pixel c_Color)
{
   m_InnerBrush.m_Color = c_Color;
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetBorderColor(Pixel c_Color)
{
   m_BorderBrush.m_Color = c_Color;
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetBrushShape(PixelBuffer *pc_BrushShape)
{
   if(m_BorderBrush.m_pc_BrushShape != NULL)
   {
      delete m_BorderBrush.m_pc_BrushShape;
      m_BorderBrush.m_pc_BrushShape = NULL;
   }
   m_BorderBrush.m_pc_BrushShape = pc_BrushShape;

   // create a brush shape for the white brush

   if(m_WhiteBrush.m_pc_BrushShape != NULL )
   {
      delete m_WhiteBrush.m_pc_BrushShape;
      m_WhiteBrush.m_pc_BrushShape = NULL;
   }

   if(pc_BrushShape != NULL)
   {
      m_WhiteBrush.m_pc_BrushShape = new PixelBuffer(pc_BrushShape);
   }

   // create a brush shape for the black brush
   if(m_BlackBrush.m_pc_BrushShape != NULL)
   {
      delete m_BlackBrush.m_pc_BrushShape;
      m_BlackBrush.m_pc_BrushShape = NULL;
   }
   if(pc_BrushShape != NULL)
   {
      m_BlackBrush.m_pc_BrushShape = new PixelBuffer(pc_BrushShape);
   }
} // SetBrushShape()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawBrush(PixelBuffer *pc_Buffer,int i_X, int i_Y)
{
   int i_RunX;
   int i_RunY;
   int i_Width;
   int i_Height;

   if(m_pc_CurrentBorderBrush->m_pc_BrushShape ==NULL)
   {
      return;
   }

   i_Width  = m_BorderBrush.m_pc_BrushShape->GetWidth();
   i_Height = m_BorderBrush.m_pc_BrushShape->GetHeight();

   for(i_RunY=0; i_RunY < i_Height; i_RunY ++)
   {
      for(i_RunX=0; i_RunX < i_Width; i_RunX ++)
      {
         // is the brush set for this pixel
         if( m_BorderBrush.m_pc_BrushShape->Get(i_RunX,i_RunY) > 0 )
         {
            DrawPixel(pc_Buffer,
                      m_pc_CurrentBorderBrush,
                      i_X+i_RunX,
                      i_Y+i_RunY);

         }

      } // for RunX
   } // for RunY


   m_ChangedRegion.SetXYWH(i_X,i_Y,i_Width,i_Height);

} // DrawBrush()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawMarkedPixel(PixelBuffer *pc_Buffer,
                        PixelBuffer *pc_MarkedBuffer,
                        RectRegion  *pc_ReducedRegion)
{
   int i_RunX;
   int i_RunY;

   int i_StartX;
   int i_StartY;
   int i_EndX;
   int i_EndY;

   int i_MarkStartX;
   int i_MarkStartY;
   int i_MarkEndX;
   int i_MarkEndY;

   if(pc_ReducedRegion == NULL)
   {
      i_StartX = 0;
      i_StartY = 0;
      i_EndX   = pc_Buffer->GetWidth()-1;
      i_EndY   = pc_Buffer->GetHeight()-1;
   }
   else
   {
      i_StartX = pc_ReducedRegion->GetStartX();
      i_StartY = pc_ReducedRegion->GetStartY();
      i_EndX   = pc_ReducedRegion->GetEndX();
      i_EndY   = pc_ReducedRegion->GetEndY();
   }

   // set mark to unused
   i_MarkStartX = 1000000;
   i_MarkStartY = 1000000;
   i_MarkEndX   = -1;
   i_MarkEndY   = -1;


   for(i_RunY= i_StartY; i_RunY<= i_EndY; i_RunY ++)
   {
      for(i_RunX= i_StartX; i_RunX<= i_EndX; i_RunX ++)
      {
         if(pc_MarkedBuffer->Get(i_RunX,i_RunY))
         {
            // remember the changed region
            if(i_RunX <i_MarkStartX)
            {
               i_MarkStartX = i_RunX;
            }
            if(i_RunY <i_MarkStartY)
            {
               i_MarkStartY = i_RunY;
            }

            if(i_RunX >i_MarkEndX)
            {
               i_MarkEndX = i_RunX;
            }
            if(i_RunY >i_MarkEndY)
            {
               i_MarkEndY = i_RunY;
            }
            DrawPixel(pc_Buffer,
                      m_pc_CurrentBorderBrush,
                      i_RunX,
                      i_RunY);

         } // if marked

      } // for X
   } // for y
   if(i_MarkStartX > i_MarkEndX)
   {
      // no pixel
      m_ChangedRegion.SetEmpty();
   }
   else
   {
      m_ChangedRegion.SetXYXY(i_MarkStartX,i_MarkStartY,i_MarkEndX,i_MarkEndX);
   }

} // DrawMarkedPixel



//===========================================================================
//
//===========================================================================

void BufferPainter::SetOneTimeDraw(bool b_On)
{
   m_b_UseOneTimeDrawBuffer = b_On;

} // SetOneTimeDraw()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawEllipse(PixelBuffer *pc_Buffer,
                             int i_CenterX, int i_CenterY,
                             int i_RadiusX, int i_RadiusY)
{
   float f_Factor;
   float f_InnerFactor;
   float f_WidthX;
   float f_InnerWidthX;
   int i_RunY;

   int i_InnerRadiusX;
   int i_InnerRadiusY;
   int i_Width;
   int i_InnerWidth;


   if(    m_i_LineWidth ==0
      && !m_b_DrawInnerPart)
   {
      // draw nothing
      m_ChangedRegion.SetEmpty();
      return;
   }


   i_InnerRadiusX = (i_RadiusX-m_i_LineWidth);
   i_InnerRadiusY = (i_RadiusY-m_i_LineWidth);

   if(    i_InnerRadiusX <=0
       || i_InnerRadiusY <=0)
   {
      // only draw width the border color
      DrawEllipseSimple(pc_Buffer,
                             m_pc_CurrentBorderBrush,
                             i_CenterX, i_CenterY,
                             i_RadiusX, i_RadiusY);
      return;

   }
   else if( m_i_LineWidth ==0)
   {
      // only draw with inner Color
      DrawEllipseSimple(pc_Buffer,
                             m_pc_CurrentInnerBrush,
                             i_CenterX, i_CenterY,
                             i_RadiusX, i_RadiusY);
      return;
   }


   // line width >0 but inner is to see
   f_Factor =   (float)  (i_RadiusX * i_RadiusX)
              / ((float) (i_RadiusY * i_RadiusY));


   f_InnerFactor =     (float) (i_InnerRadiusX * i_InnerRadiusX)
                    / ((float) (i_InnerRadiusY * i_InnerRadiusY) );

   for(i_RunY= 1;i_RunY<= i_RadiusY; i_RunY ++)
   {

      f_WidthX = sqrt (( float) (f_Factor * (  i_RadiusY * i_RadiusY
                                             - i_RunY * i_RunY)));

      if(i_RunY < i_InnerRadiusY)
      {
         f_InnerWidthX = sqrt (( float) (f_InnerFactor * (  i_InnerRadiusY * i_InnerRadiusY
                                             - i_RunY * i_RunY)));

         i_Width      = (int) (f_WidthX +0.5);
         i_InnerWidth = (int) (f_InnerWidthX +0.5);
         if(f_WidthX >0)
         {
            // draw upper lines
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX-i_InnerWidth-1,
                            i_CenterY+i_RunY);

            if(m_b_DrawInnerPart)
            {
               DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentInnerBrush,
                            i_CenterX-i_InnerWidth,i_CenterX+i_InnerWidth,
                            i_CenterY+i_RunY);
            }
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX+i_InnerWidth+1,i_CenterX+i_Width,
                            i_CenterY+i_RunY);





            // draw lower line
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX-i_InnerWidth-1,
                            i_CenterY-i_RunY);

            if(m_b_DrawInnerPart)
            {
               DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentInnerBrush,
                            i_CenterX-i_InnerWidth,i_CenterX+i_InnerWidth,
                            i_CenterY-i_RunY);
            }
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX+i_InnerWidth+1,i_CenterX+i_Width,
                            i_CenterY-i_RunY);
         }

      }
      else // inner not part is visible at this line
      {
         i_Width = (int) (f_WidthX +0.5);
         if(i_Width >0)
         {
            // draw upper line
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY+i_RunY);

            // draw lower line
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY-i_RunY);
         }
      }
   } // for all lines

   // now draw the middle line
   f_WidthX = sqrt (( float) (f_Factor * (  i_RadiusY * i_RadiusY)  ));

   f_InnerWidthX = sqrt (( float) (f_InnerFactor * (  i_InnerRadiusY * i_InnerRadiusY)));

   i_Width = (int) (f_WidthX +0.5);
   i_InnerWidth = (int) (f_InnerWidthX +0.5);
   DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX-i_InnerWidth-1,
                            i_CenterY);
   if(m_b_DrawInnerPart)
   {
      DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentInnerBrush,
                            i_CenterX-i_InnerWidth,i_CenterX+i_InnerWidth,
                            i_CenterY);
   }
   DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX+i_InnerWidth+1,i_CenterX+i_Width,
                            i_CenterY);




   m_ChangedRegion.SetXYXY(i_CenterX-i_RadiusX,i_CenterY-i_RadiusY,
                           i_CenterX+i_RadiusX,i_CenterY+i_RadiusY);


} // DrawEllipse

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawEllipseSimple(PixelBuffer *pc_Buffer,
                                      PainterBrush *pc_Brush,
                                      int i_CenterX, int i_CenterY,
                                      int i_RadiusX, int i_RadiusY)
{
   float f_Factor;
   float f_WidthX;
   int   i_RunY;
   int   i_Width;

   f_Factor =   (float)  (i_RadiusX * i_RadiusX)
              / ((float) (i_RadiusY * i_RadiusY));

   for(i_RunY= 1;i_RunY<= i_RadiusY; i_RunY ++)
   {

      f_WidthX = sqrt (( float) (f_Factor * (  i_RadiusY * i_RadiusY
                                             - i_RunY * i_RunY)));
      if(f_WidthX >0)
      {
         i_Width = (int) (f_WidthX +0.5);
         // draw upper line
         DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY+i_RunY);

         // draw lower line
         DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY-i_RunY);

      }
   } // for all lines

   // now draw the line in the middle
   f_WidthX = sqrt (( float) (f_Factor * (  i_RadiusY * i_RadiusY)));
   if(f_WidthX >0)
   {
      i_Width = (int) (f_WidthX +0.5);
      DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY);
   }


   m_ChangedRegion.SetXYXY(i_CenterX-i_RadiusX,i_CenterY-i_RadiusY,
                           i_CenterX+i_RadiusX,i_CenterY+i_RadiusY);


} // DrawEllipseSimple

//===========================================================================
// version with floating point
//===========================================================================

void BufferPainter::DrawEllipseSimple(PixelBuffer *pc_Buffer,
                                      PainterBrush *pc_Brush,
                                      float f_CenterX, float f_CenterY,
                                      float f_RadiusX, float f_RadiusY)
{
   float f_Factor;
   float f_WidthX;
   int   i_RunY;
   int   i_Width;

   f_Factor =   (float)  (f_RadiusX * f_RadiusX)
              / ((float) (f_RadiusY * f_RadiusY));

   for(i_RunY= 1;i_RunY<= f_RadiusY; i_RunY ++)
   {

      f_WidthX = sqrt (( float) (f_Factor * (  f_RadiusY * f_RadiusY
                                             - i_RunY * i_RunY)));
      if(f_WidthX >0)
      {
         i_Width = (int) (f_WidthX );
         // draw upper line
         DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            (int)(f_CenterX-i_Width),
                            (int)(f_CenterX+i_Width),
                            (int) (f_CenterY+i_RunY));

         // draw lower line
         DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            (int)(f_CenterX-i_Width),
                            (int)(f_CenterX+i_Width),
                            (int)(f_CenterY-i_RunY));

      }
   } // for all lines

   // now draw the line in the middle
   f_WidthX = sqrt (( float) (f_Factor * (  f_RadiusY * f_RadiusY)));
   if(f_WidthX >0)
   {
      i_Width = (int) (f_WidthX );
      DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            (int)(f_CenterX-i_Width),
                            (int)(f_CenterX+i_Width),
                            (int)(f_CenterY));
   }


   m_ChangedRegion.SetXYXY((int)f_CenterX-f_RadiusX,(int)f_CenterY-f_RadiusY,
                           (int)f_CenterX+f_RadiusX,(int)f_CenterY+f_RadiusY);


} // DrawEllipseSimple


//===========================================================================
//
//===========================================================================

void BufferPainter::DrawFreeFigure(PixelBuffer *pc_Buffer,
                       QValueVector<PixelCoordinate> *pc_List)
{
   PixelCoordinate c_Last;
   PixelCoordinate c_Actual;
   RectRegion      c_SingleRegion;
   RectRegion      c_TotalRegion;
   int             i_Size;
   int             i;

   bool b_OneTimeDrawSave;

   // save an eventual active buffer
   b_OneTimeDrawSave =  m_b_UseOneTimeDrawBuffer;

   m_b_UseOneTimeDrawBuffer = true;
   pc_Buffer->CreateUsedPixelMask();
   pc_Buffer->UsedPixelMaskClean();


   i_Size = pc_List->size();

   c_Last = pc_List->at(i_Size-1);

   c_TotalRegion.SetEmpty();

   // draw the first starting point
   DrawEllipseSimple(pc_Buffer,
             m_pc_CurrentBorderBrush,
             (float) c_Last.m_i_X  ,(float) c_Last.m_i_Y,
             (float)(m_i_LineWidth-0.5)/2.0 ,(float) (m_i_LineWidth-0.5)/2.0);
   c_SingleRegion = GetChangedRegion();

   c_TotalRegion.AddRegion(c_SingleRegion);
   for (i=0;i<i_Size;i++)
   {
      int i_DiffX;
      int i_DiffY;
      // get two coordinates
      c_Actual = pc_List->at(i);

	//fprintf(stderr,"%d draw %d %d %d %d\n",i,c_Last.m_i_X  ,c_Last.m_i_Y, c_Actual.m_i_X,c_Actual.m_i_Y);

      i_DiffX= c_Last.m_i_X - c_Actual.m_i_X;
      if(i_DiffX <0)
      {
         i_DiffX = - i_DiffX;
      }
      i_DiffY= c_Last.m_i_Y - c_Actual.m_i_Y;
      if(i_DiffY <0)
      {
         i_DiffY = - i_DiffY;
      }

      // only draw a line, if the differences are large enough
      if(i_DiffX>1 || i_DiffY>1)
      {
         DrawLine(pc_Buffer,
                c_Last.m_i_X  ,c_Last.m_i_Y,
                c_Actual.m_i_X,c_Actual.m_i_Y);
      }

      DrawEllipseSimple(
             pc_Buffer,
             m_pc_CurrentBorderBrush,
             (float) c_Last.m_i_X  ,(float) c_Last.m_i_Y,
             (float)(m_i_LineWidth-0.5)/2.0 ,(float) (m_i_LineWidth-0.5)/2.0);


      c_SingleRegion = GetChangedRegion();

      c_TotalRegion.AddRegion(c_SingleRegion);

      c_Last = c_Actual;
   } // for all points

   m_ChangedRegion = c_TotalRegion;

   // now fill the inner part
   if(m_b_DrawInnerPart)
   {
      int i_StartLine;
      int i_EndLine;
      int i_LastDirection;

      BufferPainterEdges * pc_Edges;

      // calculate the maxiumum extension
      CalculateStartEndLine(pc_List,
                            i_StartLine,
                            i_EndLine);
      // create a structure for the edges
      pc_Edges = new BufferPainterEdges(i_StartLine,i_EndLine);

      // now draw all the line
      c_Last = pc_List->at(i_Size-1);

      i_LastDirection = CalculateInitialLastDirection(pc_List);

      for(i=0;i<i_Size;i++)
      {
         c_Actual = pc_List->at(i);
         // draw line from last to actual
         CalculateEdgesOfLine (pc_Edges,
                    c_Last,
                    c_Actual,
                    i_LastDirection);

         c_Last = c_Actual;
      }

      // now draw the inner figure
      DrawEdgesField(pc_Buffer,
                     m_pc_CurrentInnerBrush,
                     pc_Edges);

      // after drawing delete the edges field
      delete pc_Edges;
   } // if draw inner part

   // restore the original buffer
   //m_pc_OneTimeDrawBuffer   = pc_OneTimeDrawBufferSave;
   m_b_UseOneTimeDrawBuffer = b_OneTimeDrawSave ;


} // DrawFreeFigure()

//===========================================================================
//
//===========================================================================

void BufferPainter::CalculateStartEndLine(QValueVector<PixelCoordinate> *pc_List,
                 int &i_StartLine,
                 int &i_EndLine)
{
   PixelCoordinate c_Actual;
   int i;
   int i_Size;

   i_Size = pc_List->size();

   if(i_Size <=0)
   {
      i_StartLine = -1;
      i_EndLine   = -1;
      return;
   }

   c_Actual = pc_List->at(0);

   i_StartLine = c_Actual.m_i_Y;
   i_EndLine   = c_Actual.m_i_Y;

   for(i=1;i<i_Size ; i++)
   {
      c_Actual = pc_List->at(i);

      if(i_StartLine > c_Actual.m_i_Y)
      {
         i_StartLine = c_Actual.m_i_Y;
      }

      if(i_EndLine < c_Actual.m_i_Y)
      {
         i_EndLine = c_Actual.m_i_Y;
      }

   } // for i
}// CalculateStartEndLine()

//===========================================================================
//
//===========================================================================


void BufferPainter::CalculateEdgesOfLine (BufferPainterEdges * pc_Edges,
                    PixelCoordinate c_From,
                    PixelCoordinate c_To,
                    int &i_LastDirection)
{
   int i_DiffX;
   int i_DiffY;
   int i_AbsolutDiffY;
   int i_BufferYIncrement;
   int i_YPosition;
   int i_XPosition;
   int i_StartIndex;
   int i_MaximumDifference;
   int i_LastYPosition;
   int i;

   /*
fprintf(stderr,"Caluclate Edges of (%d/%d) - (%d/%d)\n",
	c_To.m_i_X,
        c_To.m_i_Y,
        c_From.m_i_X,
        c_From.m_i_Y);
*/
   i_DiffX = c_To.m_i_X  - c_From.m_i_X;
   i_DiffY = c_To.m_i_Y  - c_From.m_i_Y;
   if(i_DiffY == 0)
   {
      // horizontal line, do nothing
   }
   else
   {
      i_AbsolutDiffY = abs (i_DiffY);
      if (i_DiffY > 0)
      {
         i_BufferYIncrement = 1;
      }
      else
      {
         i_BufferYIncrement = -1;
      }
      i_YPosition = c_From.m_i_Y;
//fprintf(stderr,"iLAst = %d , current =%d\n",i_LastDirection,i_DiffY);
      if ((i_LastDirection < 0 && i_DiffY >= 0) ||
          (i_LastDirection > 0 && i_DiffY <= 0))
      {
         i_StartIndex = 0;
      }
      else
      {
         i_StartIndex = 1;
         i_YPosition += i_BufferYIncrement;


      }

      i_MaximumDifference = i_AbsolutDiffY;
      if (abs (i_DiffX) > i_MaximumDifference)
      {
         i_MaximumDifference = abs (i_DiffX);
      }

      i_LastYPosition = i_YPosition - i_BufferYIncrement;

      for (i=i_StartIndex ; i <= i_MaximumDifference; i++)
      {
         i_XPosition  = c_From.m_i_X + i_DiffX * i / i_MaximumDifference;
         i_YPosition  = c_From.m_i_Y + i_DiffY * i / i_MaximumDifference;
         if (   (i_BufferYIncrement > 0 && i_YPosition > i_LastYPosition)
             || (i_BufferYIncrement < 0 && i_YPosition < i_LastYPosition)
             )
         {
//fprintf(stderr,"AddPoint %d %d \n",i_XPosition, i_YPosition);

            pc_Edges->AddLinePoint (i_XPosition,i_YPosition);
            i_LastYPosition = i_YPosition;
         }

      }                         /*for BufI */

      i_LastDirection = i_DiffY;

   } // else not horizontal

} // CalculateEdgesOfLine();

//===========================================================================
//
//===========================================================================

int BufferPainter::CalculateInitialLastDirection(QValueVector<PixelCoordinate> *pc_List)
{
   int i_LastDirection;
   int i_LastIndex;

   PixelCoordinate c_First;
   PixelCoordinate c_Last;

   i_LastIndex = pc_List->size()-1;

   c_First = pc_List->at(0);
   c_Last  = pc_List->at(i_LastIndex);

   i_LastDirection = c_First.m_i_Y - c_Last.m_i_Y;

   // do we have a horizonalt line at the end??
   // then go back, until a direction is found
   while (   i_LastDirection == 0
          && i_LastIndex > 0)
   {
      i_LastIndex --;
      c_Last  = pc_List->at(i_LastIndex);


      i_LastDirection = c_Last.m_i_Y - c_First.m_i_Y;

   }

   return i_LastDirection;

} // CalculateInitialLastDirection()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawEdgesField(PixelBuffer *pc_Buffer,
                 PainterBrush *pc_Brush,
                 BufferPainterEdges * pc_Edges)
{
   int i_ActualLine;
   int i_StartLine;
   int i_EndLine;
   
   i_StartLine = pc_Edges->GetStartLine();
   i_EndLine   = pc_Edges->GetEndLine();

// fprintf(stderr,"DrawEdgesField %d %d\n",i_StartLine,i_EndLine);
   for(i_ActualLine = i_StartLine;
       i_ActualLine <= i_EndLine;
       i_ActualLine++)
   {
      BufferPainterEdgesLine * pc_Line;

      pc_Line = pc_Edges->GetLine(i_ActualLine);
      if(pc_Line != NULL)
      {
         int i;
         int i_Number;
         i_Number = pc_Line->GetNumber();
//fprintf(stderr,"%d edges in line %d\n",i_Number,i_ActualLine);

         for(i=0;i+1<i_Number; i+= 2)
         {
            int i_From;
            int i_To;
            i_From = pc_Line->GetPoint(i);
            i_To   = pc_Line->GetPoint(i+1);

            DrawHorizontalLine(pc_Buffer,
                                pc_Brush,
                                i_From,
                                i_To,
                                i_ActualLine);

         } // for i

      } // if Line != NULL

   } // for i_ActualLine

} // DrawEdgesField

//===========================================================================
//
//===========================================================================

void BufferPainter::SetMaskMode(bool b_EnableMode,bool b_White)
{
   if(b_EnableMode)
   {
      if(b_White)
      {
         m_pc_CurrentBorderBrush = &m_WhiteBrush;
         m_pc_CurrentInnerBrush  = &m_WhiteBrush;
      }
      else
      {
         m_pc_CurrentBorderBrush = &m_BlackBrush;
         m_pc_CurrentInnerBrush  = &m_BlackBrush;
      }
      m_b_MaskModeEnabled = true;
   }
   else
   {
      m_pc_CurrentBorderBrush = &m_BorderBrush;
      m_pc_CurrentInnerBrush  = &m_InnerBrush;
      m_b_MaskModeEnabled = false;
   }
} // SetMaskMode()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetLightnessMode(bool b_EnableMode)
{
   m_BorderBrush.m_b_Lightness = b_EnableMode;
} // SetLightnessMode()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetLightnessFactor(int i_Factor)
{
   m_BorderBrush.m_i_LightnessFactor = i_Factor;
} //SetLightnessFactor()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetSaturationMode(bool b_EnableMode)
{
   m_BorderBrush.m_b_Saturation = b_EnableMode;
} // SetLightnessMode()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterMode(bool b_EnableMode)
{
   m_BorderBrush.m_b_Filter = b_EnableMode;
} // SetLightnessMode()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterWidth(int i_Width)
{
   m_i_FilterWidth = i_Width;
} // SetFilterWidth()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterNearDistanceActive(bool b_Active)
{
   m_b_FilterNearDistanceActive = b_Active;
} // SetFilterNearDistanceActive()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterNearDistance(int i_Distance)
{
   m_i_FilterNearDistance = i_Distance;
} // SetFilterNearDistance()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterAlgorithm(int i_Algorithm)
{
   m_i_FilterAlgorithm = i_Algorithm;
} // SetFilterAlgorithm()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetInnerPattern(PixelBuffer * pc_Buffer)
{
   m_InnerBrush.SetPattern(pc_Buffer);
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetBorderPattern(PixelBuffer * pc_Buffer)
{
   m_BorderBrush.SetPattern(pc_Buffer);
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetInnerDrawMode(bool b_UsePattern)
{
   m_InnerBrush.m_b_UsePattern = b_UsePattern;
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetBorderDrawMode(bool b_UsePattern)
{
   m_BorderBrush.m_b_UsePattern = b_UsePattern;
}

//===========================================================================
//
//===========================================================================

