#ifndef _BUFFERPAINTER_H
#define _BUFFERPAINTER_H

#include <qvaluevector.h>
#include <qvaluelist.h>

#include "PixelBuffer.h"
#include "RectRegion.h"
#include "BufferPainterEdges.h"
#include "PainterBrush.h"

//==================================================================

class BufferPainter
{
public:
   BufferPainter();

   void DrawPoint(PixelBuffer *pc_Buffer, int i_X, int i_Y);
   void DrawLine(PixelBuffer *pc_Buffer,int i_X1, int i_Y1,int i_X2, int i_Y2);
   void DrawRectangle(PixelBuffer *pc_Buffer,int i_X1, int i_Y1,int i_X2, int i_Y2);

   void DrawEllipse(PixelBuffer *pc_Buffer,int i_CenterX, int i_CenterY,
                                           int i_RadiusX, int i_Radiusy);

   void DrawBrush(PixelBuffer *pc_Buffer, int i_X, int i_Y);
   
   void DrawMarkedPixel(PixelBuffer *pc_Buffer,
                        PixelBuffer *pc_MarkedBuffer,
                        RectRegion  *pc_ReducedRegion);

   RectRegion GetChangedRegion();

   //--------- set functions for properties

   void SetLineWidth(int i_LineWidth);
   void SetDrawInnerEnable(bool b_On);
   void SetTransparencyEnable(bool b_on);
   void SetTransparencyFactor(int i_Factor);

   void SetInnerColor(Pixel c_Color);
   void SetBorderColor(Pixel c_Color);
   void SetInnerPattern(PixelBuffer * pc_Buffer);
   void SetBorderPattern(PixelBuffer * pc_Buffer);
   void SetInnerDrawMode(bool b_UsePattern);
   void SetBorderDrawMode(bool b_UsePattern);

   void SetBrushShape(PixelBuffer *pc_BrushShape);
   void SetOneTimeDraw(bool b_On);

   void DrawEllipseSimple(PixelBuffer *pc_Buffer,
                 PainterBrush *pc_Brush,
                 float i_CenterX, float i_CenterY,
                 float i_RadiusX, float i_Radiusy);

   void DrawFreeFigure(PixelBuffer *pc_Buffer,
                       QValueVector<PixelCoordinate> *pc_List);

   void SetMaskMode(bool b_EnableMode,bool b_White);
   void SetLightnessMode(bool b_EnableMode);
   void SetLightnessFactor(int i_Factor);
   void SetSaturationMode(bool b_EnableMode);
   
   
   enum 
   {
      FilterAlgorithmMiddle =0,
      FilterAlgorithmMedian
   };
   void SetFilterMode(bool b_EnableMode);
   void SetFilterWidth(int i_Width);
   void SetFilterNearDistanceActive(bool b_Active);
   void SetFilterNearDistance(int i_Distance);
   void SetFilterAlgorithm(int i_Algorithm);


private:
   void FilterPixelMiddle(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y);
   void FilterPixelMedian(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y);
   void DrawPixel(PixelBuffer *pc_Buffer,
                                PainterBrush *pc_Brush,
                                int i_X,
                                int i_Y);
   void DrawHorizontalLine(PixelBuffer *pc_Buffer,
                                PainterBrush *pc_Brush,
                                int i_StartX,
                                int i_EndX,
                                int i_Y);
   void DrawLineWithWidth1(PixelBuffer *pc_Buffer,
                        PainterBrush *pc_Brush,
                        int i_X1, int i_Y1,int i_X2, int i_Y2);


   void BaseDrawLine(PixelBuffer *pc_Buffer,
                PainterBrush *pc_Brush,
                int i_StartX, int i_StartY,
                int i_EndX, int i_EndY,
                int i_LineWidth,
                RectRegion &c_ChangedRegion);

   void DrawEllipseSimple(PixelBuffer *pc_Buffer,
                 PainterBrush *pc_Brush,
                 int i_CenterX, int i_CenterY,
                 int i_RadiusX, int i_Radiusy);

  void CalculateStartEndLine(QValueVector<PixelCoordinate> *pc_List,
                 int &i_StartLine,
                 int &i_EndLine);
   void CalculateEdgesOfLine (BufferPainterEdges * pc_Edges,
                    PixelCoordinate c_From,
                    PixelCoordinate c_To,
                    int &LastDirection);
   int CalculateInitialLastDirection(QValueVector<PixelCoordinate> *pc_List);

   void DrawEdgesField(PixelBuffer *pc_Buffer,
                 PainterBrush *pc_Brush,
                 BufferPainterEdges * pc_Edges);


private:
    PainterBrush m_BorderBrush;
    PainterBrush m_InnerBrush;

    PainterBrush m_BlackBrush;
    PainterBrush m_WhiteBrush;

    PainterBrush *m_pc_CurrentBorderBrush;
    PainterBrush *m_pc_CurrentInnerBrush;

    bool         m_b_DrawInnerPart;

    bool         m_b_DrawTransparent;
    int          m_i_TransparencyFactor;

    int          m_i_LineWidth;

    int          m_i_FilterWidth;
    bool         m_b_FilterNearDistanceActive;
    int          m_i_FilterNearDistance;
    int          m_i_FilterAlgorithm;

   RectRegion m_ChangedRegion;

//   PixelBuffer *m_pc_OneTimeDrawBuffer;
   bool        m_b_UseOneTimeDrawBuffer;
   
   bool        m_b_MaskModeEnabled;

}; // class BufferPainter


#endif // _BUFFERPAINTER_H
