// module to calculate color representations into others

#include <math.h>
#include <stdio.h>
#include "ColorCalculation.h"


//===========================================================================
//
//===========================================================================

void ColorCalculation::RGBToHLS(int i_R  , int i_G  , int i_B,
                                int &i_H , int &i_L , int &i_S)
{

   double d_Maximum;
   double d_Minimum;
   double d_Red;
   double d_Green;
   double d_Blue;
   double d_Hue;
   double d_Lightness;
   double d_Saturation;


   // adjust to 0 .. 1
   d_Red   = (double)i_R / 255.0;
   d_Green = (double)i_G / 255.0;
   d_Blue  = (double)i_B / 255.0;

   // calculate the maximum
   d_Maximum = d_Red;
   if(d_Green>d_Maximum)
   {
      d_Maximum = d_Green;
   }
   if(d_Blue>d_Maximum)
   {
      d_Maximum = d_Blue;
   }

   d_Minimum = d_Red;
   if(d_Green<d_Minimum)
   {
      d_Minimum = d_Green;
   }
   if(d_Blue<d_Minimum)
   {
      d_Minimum = d_Blue;
   }

   d_Lightness = 50.0 * (d_Maximum+d_Minimum);

   if ( fabs(d_Minimum-d_Maximum) < 0.0001 )
   {
      d_Hue         = 0.0;
      d_Saturation  = 0.0;
   }
   else
   {
      double d_Summ;
      double d_Difference;
      double d_HueHelp;
      
      d_Summ       = d_Maximum + d_Minimum;
      d_Difference = d_Maximum - d_Minimum;

      if ( d_Summ <= 1.0 )
      {
         d_Saturation  = 100.0 * (d_Difference) / (d_Summ) ;
      }
      else
      {
         d_Saturation  = 100.0 * (d_Difference) / (2.0 - d_Maximum - d_Minimum) ;
      }

      d_Red   = (d_Red   - d_Minimum) / (d_Difference);
      d_Green = (d_Green - d_Minimum) / (d_Difference);
      d_Blue  = (d_Blue  - d_Minimum) / (d_Difference);


      if ( d_Red >= 0.999 ) // near to 1
      {
         d_HueHelp = d_Green - d_Blue;
      }
      else if ( d_Green >= 0.999 ) // near to 1
      {
         d_HueHelp = 2.0 + d_Blue - d_Red;
      }
      else
      {
         d_HueHelp = 4.0 + d_Red - d_Green;
      }


      if ( d_HueHelp < 0.0 )
      {
         d_HueHelp += 6.0;
      }

      d_Hue = 60.0 * d_HueHelp ;
      if ( d_Hue >= 360.0 )
      {
         d_Hue -= 360.0;
      }
   } // else difference is greater

   i_H = (int) d_Hue;
   i_L = (int) d_Lightness;
   i_S = (int) d_Saturation;


} // RGBToHLS


//===========================================================================
//
//===========================================================================

static double HueValue (double d_Metric1,
                        double d_Metric2,
                        double d_Hue
			)
{
   if ( d_Hue < 0.0 )
   {
      d_Hue += 360.0;
   }
   if ( d_Hue > 360.0 )
   {
      d_Hue -= 360.0;
   }


   if ( d_Hue < 60.0 )
   {
      return d_Metric1 + (d_Metric2-d_Metric1) * d_Hue / 60.0;
   }

   if ( d_Hue < 180.0 )
   {
      return d_Metric2;
   }

   if ( d_Hue < 240.0 )
   {
      return d_Metric1 + (d_Metric2-d_Metric1) * (240.0 - d_Hue) / 60.0;
   }

   return d_Metric1;
} // HueValue

//===========================================================================
//
//===========================================================================

void ColorCalculation::HLSToRGB(int i_H  , int i_L  , int i_S,
                                int &i_R , int &i_G , int &i_B)
{

   double d_Hue;
   double d_Lightness;
   double d_Saturation;

   double d_Metric1;
   double d_Metric2;

   double d_Red;
   double d_Green;
   double d_Blue;

   // convert into double
   d_Hue        = (double) i_H;
   d_Lightness  = (double) i_L / 100.0;
   d_Saturation = (double) i_S / 100.0;

   if ( d_Lightness <= 0.5 )
   {
      d_Metric2 = d_Lightness * ( 1.0 + d_Saturation);
   }
   else
   {
      d_Metric2 = d_Lightness + d_Saturation - d_Lightness * d_Saturation;
   }

   d_Metric1 = 2.0 * d_Lightness - d_Metric2;

   d_Red   = 255.0 * HueValue (d_Metric1, d_Metric2, d_Hue + 120.0) ;
   d_Green = 255.0 * HueValue (d_Metric1, d_Metric2, d_Hue        ) ;
   d_Blue  = 255.0 * HueValue (d_Metric1, d_Metric2, d_Hue + 240.0) ;


   // back to int

   i_R = (int) d_Red;
   i_G = (int) d_Green;
   i_B = (int) d_Blue;


} // HLStoRGB


//===========================================================================
//
//===========================================================================

Pixel ColorCalculation::TransparentColor(Pixel p_Old,
                                         Pixel p_New,
                                         int i_TransparencyFactor)
{
   Pixel p_Result;
   int   i_InversFactor;
   int   i_Red;
   int   i_Green;
   int   i_Blue;

   i_InversFactor = 100 - i_TransparencyFactor;

   i_Red   = PIXEL_RED(p_Old )     * i_TransparencyFactor +
             PIXEL_RED(p_New) * i_InversFactor;
   i_Green = PIXEL_GREEN(p_Old )     * i_TransparencyFactor +
             PIXEL_GREEN(p_New) * i_InversFactor;
   i_Blue  = PIXEL_BLUE(p_Old )     * i_TransparencyFactor +
             PIXEL_BLUE(p_New) * i_InversFactor;

   i_Red   /= 100;
   i_Green /= 100;
   i_Blue  /= 100;

   p_Result =  PIXEL_MAKE(i_Red,i_Green,i_Blue);

   return p_Result;
} // TransparentColor()

//===========================================================================
//
//===========================================================================



