// box to select a color

#include <math.h>

#include "ColorSelectBox.h"

#include "ColorCalculation.h"
#include "QtUtilities.h"
#include "Geometry.h"
#include "MouseEvent.h"
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include "QUColorDragWidget.h"

//===========================================================================
//
//===========================================================================


ColorSelectBox::ColorSelectBox(QWidget *parent,
                  GlobalStorage *pc_ParentGlobalStorage)
		: QDialog(parent)
{
   QBoxLayout   *l_DialogButtons;

   m_pc_ParentGlobalStorage = pc_ParentGlobalStorage;

   m_i_CircleRadius                 = 100;
   m_i_LightnessWidth               = 40;
   m_i_LightnessHeight              = 202;
   m_i_CurrentLightnessBoxHue       = -1; // undefined
   m_i_CurrentLightnessBoxLightness = -1; // undefined


   setCaption("Color Selection");

   // create a vertical box
   m_l_TopLayout = new QVBoxLayout( this,1);


   m_w_GraphicWidget = new QWidget (this);
   m_l_TopLayout->addWidget(m_w_GraphicWidget);
   {
      QHBoxLayout *l_GraphicLayout;
      l_GraphicLayout = new QHBoxLayout( m_w_GraphicWidget,1);

      m_ZoomSlider = new QSlider(1,4,1,1,
                                 Qt::Vertical,
                                 m_w_GraphicWidget
                                 );
      l_GraphicLayout->addWidget(m_ZoomSlider);
      m_ZoomSlider->setMaximumHeight(100 );

      // create the color circle image
      m_pc_ImageFrame = new ImageFrame(m_w_GraphicWidget,NULL,NULL);

      l_GraphicLayout->addWidget(m_pc_ImageFrame);

      m_pc_ImageFrame->setFixedSize(250,250 );

      connect( m_ZoomSlider, SIGNAL(valueChanged(int)),
                SLOT( ZoomSliderChanged(int )) );

      // create the frame for the lightness image
      {
         m_pc_LightnessFrame = new ImageFrame(m_w_GraphicWidget,
                               NULL,NULL,
                               ImageFrame::FixedSize | ImageFrame::NoSelection,
                               m_i_LightnessWidth,
                               m_i_LightnessHeight);

         l_GraphicLayout->addWidget(m_pc_LightnessFrame);
      }

      // sub-box for the History
      {
         QVGroupBox *w_HistoryBox;
         int        i;
         w_HistoryBox = new QVGroupBox("History",m_w_GraphicWidget);
         l_GraphicLayout->addWidget(w_HistoryBox);
         w_HistoryBox->setColumns(2);
         for(i=0;i<COLORSELECTBOX_HISTORYWIDGETS ;i++)
         {
            m_w_ColorHistory[i]= new QUColorDragWidget(w_HistoryBox,
                                         QUColorDragWidget::ModePipette,
                                         m_pc_ParentGlobalStorage);
            m_w_ColorHistory[i]->setFixedSize(20,20);
            m_w_ColorHistory[i]->SetColor(0);

         } // for i;
      }
      

      // right box with the colorselect widget
      {
         m_ColorWidget = new QUColorDragWidget(m_w_GraphicWidget,
                                               QUColorDragWidget::ModePipette,
                                               m_pc_ParentGlobalStorage);
         l_GraphicLayout->addWidget(m_ColorWidget);

         connect(m_ColorWidget,
                 SIGNAL(ColorChanged(Pixel)),
                 SLOT(ColorWidgetColorChanged(Pixel))
                );
         connect(m_ColorWidget,
                 SIGNAL(DragStarted(Pixel)),
                 SLOT(StartedDrag(Pixel))
                );

      }

   } // block for the graphics box

   // create the lower region with the slider
   m_w_GridWidget = new QWidget (this);
   m_l_TopLayout->addWidget(m_w_GridWidget);
   {
      QGridLayout  *grid;
      grid = new QGridLayout( m_w_GridWidget, 3, 2, 1 );

      m_RedSlider= new QULabelSlider(m_w_GridWidget,
                                     true,
                                     0,
                                     255,
                                     255,
                                     "Red",
                                     3
      );
      m_GreenSlider= new QULabelSlider(m_w_GridWidget,
                                     true,
                                     0,
                                     255,
                                     255,
                                     "Green",
                                     3
      );
      m_BlueSlider= new QULabelSlider(m_w_GridWidget,
                                     true,
                                     0,
                                     255,
                                     255,
                                     "Blue",
                                     3
      );
      m_HueSlider= new QULabelSlider(m_w_GridWidget,
                                     true,
                                     0,
                                     360,
                                     0,
                                     "Hue",
                                     5
      );
      m_LightnessSlider= new QULabelSlider(m_w_GridWidget,
                                     true,
                                     0,
                                     100,
                                     0,
                                     "Lightness",
                                     5
      );
      m_SaturationSlider= new QULabelSlider(m_w_GridWidget,
                                     true,
                                     0,
                                     100,
                                     0,
                                     "Saturation",
                                     5
      );

      grid->addWidget(m_RedSlider,0,0);
      grid->addWidget(m_GreenSlider,1,0);
      grid->addWidget(m_BlueSlider,2,0);

      grid->addWidget(m_HueSlider,0,2);
      grid->addWidget(m_LightnessSlider,1,2);
      grid->addWidget(m_SaturationSlider,2,2);

      grid->addColSpacing ( 1, 8 ) ;

      connect( m_RedSlider, 
               SIGNAL(valueChanged(int)),
               SLOT( RGBSliderChanged(int )) );
      connect( m_GreenSlider,
               SIGNAL(valueChanged(int)),
               SLOT( RGBSliderChanged(int )) );
      connect( m_BlueSlider, 
               SIGNAL(valueChanged(int)),
               SLOT( RGBSliderChanged(int )) );

      connect( m_HueSlider, 
               SIGNAL(valueChanged(int)),
               SLOT( HLSSliderChanged(int )) );
      connect( m_LightnessSlider, 
               SIGNAL(valueChanged(int)),
               SLOT( HLSSliderChanged(int )) );
      connect( m_SaturationSlider, 
               SIGNAL(valueChanged(int)),
               SLOT( HLSSliderChanged(int )) );
   }

   // --- the close button
   l_DialogButtons = new QHBoxLayout( m_l_TopLayout,5);

   m_w_CloseButton = new QPushButton( "Close", this );
   connect( m_w_CloseButton, 
            SIGNAL(clicked()), 
            SLOT( ClosePressed()) );

   l_DialogButtons->addWidget(new QtUtilSpacer(true,false,this) );
   l_DialogButtons->addWidget(m_w_CloseButton);

   CreateColorImage();

   m_ColorRed   = 0xff;
   m_ColorGreen = 0xcc;
   m_ColorBlue  = 0xaa;
   ColorCalculation::RGBToHLS(m_ColorRed,m_ColorGreen,m_ColorBlue,
                              m_ColorHue,m_ColorLightness,m_ColorSaturation
                              );
   SetRGBSlider();
   SetHLSSlider();
   SetObject();
   SetLightness();
   SetColorWidget();


} // ColorSelectBox - constructor

//===========================================================================
//
//===========================================================================

void ColorSelectBox::CreateColorImage()
{
   GeoCoordinate c_Center;
   int           i_X;
   int           i_Y;
   int           i_Dimension;

   i_Dimension = m_i_CircleRadius *2 +1;

   // allocate a new buffer
   m_pc_ColorCircle = new PixelBuffer(i_Dimension,
                                      i_Dimension,
                                      32,
                                      false,false);

   if(m_pc_ColorCircle == NULL)
   {
      return;
   }

   c_Center.SetXY(m_i_CircleRadius,m_i_CircleRadius);
   for(i_Y=0; i_Y< i_Dimension; i_Y++)
   {
      for(i_X =0; i_X < i_Dimension; i_X++)
      {
         Pixel p_Pixel;
         float f_Angle;
         int   i_Distance;
         int   i_Angle;
         int   i_R;
         int   i_G;
         int   i_B;

         i_Distance = (int) c_Center.DifferenceTo(i_X,i_Y);

         // norm to 0..100
         i_Distance = i_Distance*100 / m_i_CircleRadius;
         if(i_Distance >100)
         {
            i_Distance = 100;
         }
         f_Angle = c_Center.AngleTo(i_X,i_Y);
         f_Angle = Geometry::AngleRadianToDegree(f_Angle);

         // calculate angle 0..359
         i_Angle = (int) f_Angle;
         if(i_Angle <0)
         {
            i_Angle += 360;
         }
         if(i_Angle >=360)
         {
            i_Angle -= 360;
         }
         ColorCalculation::HLSToRGB(i_Angle , 50  ,  i_Distance,
                                    i_R , i_G , i_B);

         p_Pixel = PIXEL_MAKE(i_R,i_G,i_B);

         m_pc_ColorCircle->SetRaw(i_X,i_Y,p_Pixel);


      } //for i_X
   } // for i_Y

   m_pc_ImageFrame->ReplacePixelbuffer(m_pc_ColorCircle);

   // create the object in the layer
   {

      m_pc_GlobalStorage = m_pc_ImageFrame->GetGlobalStorage();
      m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(true);
      // add some default points
      GraphicObjectLayer     *pc_ActiveLayer;
      m_pc_GlobalStorage->m_pc_ObjectMain->CreateLayer("default");
      pc_ActiveLayer = m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer();
      if(pc_ActiveLayer != NULL)
      {
         m_f_CoordinateX = 100.0;
         m_f_CoordinateY = 100.0;
         m_pc_Point = new GraphicObjectPoint (m_f_CoordinateX,m_f_CoordinateY);
         m_pc_Point->SetColor(0x000000);
         pc_ActiveLayer->AddObject(m_pc_Point);

      }

      m_pc_GlobalStorage->m_pc_MouseEvent->AddEventCallback(Callback(MouseEventArrivedCB,this));

   }

    m_pc_LightnessBuffer = new PixelBuffer(m_i_LightnessWidth,
                                           m_i_LightnessHeight,
                                           32,
                                           false,false);
    m_pc_LightnessFrame->ReplacePixelbuffer(m_pc_LightnessBuffer);
    // create the object in the layer
    {
      GraphicObjectLayer     *pc_ActiveLayer;

      m_pc_LightnessGlobalStorage = m_pc_LightnessFrame->GetGlobalStorage();
      m_pc_LightnessGlobalStorage->m_pc_ObjectMain->SetVisible(true);
      m_pc_LightnessGlobalStorage->m_pc_ObjectMain->CreateLayer("default");
      
      pc_ActiveLayer = m_pc_LightnessGlobalStorage->m_pc_ObjectMain->GetActiveInputLayer();
      if(pc_ActiveLayer != NULL)
      {
         m_f_CoordinateX = m_i_LightnessWidth /2;
         m_f_CoordinateY = 100.0;
         m_pc_LightnessPoint = new GraphicObjectPoint (m_f_CoordinateX,m_f_CoordinateY);
         m_pc_LightnessPoint->SetColor(0x003300);
         pc_ActiveLayer->AddObject(m_pc_LightnessPoint);

      }

      m_pc_LightnessGlobalStorage->m_pc_MouseEvent->AddEventCallback(Callback(LightnessMouseEventArrivedCB,this));

   }

} // CreateColorImage()

//===========================================================================
//
//===========================================================================

void ColorSelectBox::SetRGBSlider()
{
   m_RedSlider  ->SetValue(m_ColorRed);
   m_GreenSlider->SetValue(m_ColorGreen);
   m_BlueSlider ->SetValue(m_ColorBlue);
}

//===========================================================================
//
//===========================================================================

void ColorSelectBox::SetHLSSlider()
{
   m_HueSlider       ->SetValue(m_ColorHue);
   m_LightnessSlider ->SetValue(m_ColorLightness);
   m_SaturationSlider->SetValue(m_ColorSaturation);

} // SetHLSSlider()

//===========================================================================
//
//===========================================================================

void ColorSelectBox::SetObject()
{
   // calculate the new XY position of the object
   RectRegion c_ChangedRegion;
   RectRegion c_TotalImage;
   float      f_X;
   float      f_Y;
   float      f_Angle;


   f_Angle = Geometry::AngleDegreeToRadian((float) m_ColorHue);

   // multiply the length
   f_X = -sin(f_Angle) * (float) m_ColorSaturation;
   f_Y = -cos(f_Angle) * (float) m_ColorSaturation;

   // add center coordinate
   f_X += m_i_CircleRadius;
   f_Y += m_i_CircleRadius;

   // calculate the region old an new point
   c_ChangedRegion.SetXYXY(m_f_CoordinateX,m_f_CoordinateY,f_X,f_Y);

   // now move the point to the new coordinate


   m_f_CoordinateX = f_X;
   m_f_CoordinateY = f_Y;
   m_pc_Point->MoveTo(m_f_CoordinateX,m_f_CoordinateY);

   // now redraw the object region
   // TODO make function for the constants
   c_ChangedRegion.Extend(5,5,5,5);
   c_TotalImage.SetXYXY(0,0,2*m_i_CircleRadius,2*m_i_CircleRadius);

   c_ChangedRegion.ClipAt(c_TotalImage);

   m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

} // SetObject

//===========================================================================
//
//===========================================================================

void ColorSelectBox::SetLightness()
{
   RectRegion c_ChangedRegion;
   bool       b_RedrawAll;

   if( m_i_CurrentLightnessBoxHue != m_ColorHue)
   {
      // build up a new buffer
      int i;

      b_RedrawAll                 = true;
      m_i_CurrentLightnessBoxHue  = m_ColorHue;

      for(i=0;i<=100;i++)
      {
         Pixel p_Color;
         int i_R;
         int i_G;
         int i_B;
         int i_RunX;
         ColorCalculation::HLSToRGB(m_ColorHue,i,m_ColorSaturation,
                                    i_R, i_G, i_B);

         p_Color = PIXEL_MAKE(i_R, i_G,i_B);
         for(i_RunX=0;i_RunX<m_i_LightnessWidth; i_RunX++)
         {
            m_pc_LightnessBuffer->SetRaw(i_RunX,i*2,p_Color);
            m_pc_LightnessBuffer->SetRaw(i_RunX,i*2+1,p_Color);
         } // for i_RunX

      } // for i;
   }

   // calculate the region old an new point
   c_ChangedRegion.SetXYXY(m_i_LightnessWidth/2,m_i_CurrentLightnessBoxLightness*2,
                           m_i_LightnessWidth/2+1,m_ColorLightness);

   m_pc_LightnessPoint->MoveTo(m_i_LightnessWidth/2,m_ColorLightness*2);
   m_i_CurrentLightnessBoxLightness = m_ColorLightness;

   // change object color from black to white depending on the
   // background
   if(m_ColorLightness < 34)
   {
      m_pc_LightnessPoint->SetColor(Pixel(0xffffff));
   }
   else
   {
      m_pc_LightnessPoint->SetColor(Pixel(0x000000));
   }

   if(!b_RedrawAll)
   {
      RectRegion c_TotalImage;

      c_ChangedRegion.Extend(5,5,5,5);
      c_TotalImage.SetXYXY(0,0,m_i_LightnessWidth,m_i_LightnessHeight);

      c_ChangedRegion.ClipAt(c_TotalImage);

      m_pc_LightnessFrame->RedrawBufferRegion(c_ChangedRegion);
   }
   else // redraw all
   {
      m_pc_LightnessFrame->RedrawAll();
   }


} // SetLightness

//===========================================================================
//
//===========================================================================

void ColorSelectBox::SetColorWidget()
{
   Pixel c_Color;
   
   c_Color = PIXEL_MAKE(m_ColorRed,m_ColorGreen,m_ColorBlue);
   
   m_ColorWidget->SetColor(c_Color);

} // SetColorWidget()

//===========================================================================
//
//===========================================================================

void ColorSelectBox::RGBSliderChanged(int i_Value)
{
   // get the current RGB values
   m_ColorRed   = m_RedSlider->GetValue();
   m_ColorGreen = m_GreenSlider->GetValue();
   m_ColorBlue  = m_BlueSlider->GetValue();

   ColorCalculation::RGBToHLS(m_ColorRed,m_ColorGreen,m_ColorBlue,
                              m_ColorHue,m_ColorLightness,m_ColorSaturation);
   SetHLSSlider();
   SetObject();
   SetLightness();
   SetColorWidget();
} // RGBSliderChanged()

//===========================================================================
//
//===========================================================================

void ColorSelectBox::HLSSliderChanged(int i_Value)
{
   // get the current RGB values
   m_ColorHue        = m_HueSlider->GetValue();
   m_ColorLightness  = m_LightnessSlider->GetValue();
   m_ColorSaturation = m_SaturationSlider->GetValue();

   ColorCalculation::HLSToRGB(m_ColorHue,m_ColorLightness,m_ColorSaturation,
                              m_ColorRed,m_ColorGreen,m_ColorBlue
                              );
   SetRGBSlider();
   SetObject();
   SetLightness();
   SetColorWidget();

} // HLSSliderChanged()

//===========================================================================
//
//===========================================================================

void ColorSelectBox::ZoomSliderChanged(int i_Value)
{
   m_i_ZoomFactor = i_Value;
   m_pc_ImageFrame->SetZoomFactor(m_i_ZoomFactor);

} // ZoomSliderChanged()


//===========================================================================
//
//===========================================================================



void *ColorSelectBox::MouseEventArrivedCB(void *Thisdata,void *pc_Data)
{
   ColorSelectBox *pc_This = (ColorSelectBox *) Thisdata;
   return pc_This->MouseEventArrived(pc_Data);

}

//===========================================================================
//
//===========================================================================


void *ColorSelectBox::MouseEventArrived(void *pc_Data)
{
   int i_MouseEventType ;

   i_MouseEventType = m_pc_GlobalStorage->m_pc_MouseEvent->GetEventType( );

   if(   i_MouseEventType == MouseEvent::EventStart
      ||  i_MouseEventType == MouseEvent::EventDo
     )
   {
      // get the coordinates and calculate a valid value
      GeoCoordinate c_Center;
      float         f_Angle;
      int           i_X;
      int           i_Y;
      int           i_Distance;
      int           i_Angle;

      m_pc_GlobalStorage->m_pc_MouseEvent->GetLastBufferCoordinates(i_X,i_Y );

      c_Center.SetXY(m_i_CircleRadius,m_i_CircleRadius);
      i_Distance = (int) c_Center.DifferenceTo(i_X,i_Y);

      // norm to 0..100
      i_Distance = i_Distance*100 / m_i_CircleRadius;
      if(i_Distance >100)
      {
         i_Distance = 100;
      }
      f_Angle = c_Center.AngleTo(i_X,i_Y);
      f_Angle = Geometry::AngleRadianToDegree(f_Angle);

      // calculate angle 0..359
      i_Angle = (int) f_Angle;
      if(i_Angle <0)
      {
         i_Angle += 360;
      }
      if(i_Angle >=360)
      {
         i_Angle -= 360;
      }

      m_ColorHue        = i_Angle;
      m_ColorSaturation = i_Distance;
      ColorCalculation::HLSToRGB(m_ColorHue,m_ColorLightness,m_ColorSaturation,
                              m_ColorRed,m_ColorGreen,m_ColorBlue
                              );

      SetRGBSlider();
      SetHLSSlider();
      SetObject();
      SetLightness();
      SetColorWidget();
   } // if mouse start or do

   return NULL;
} // MouseEventArrived()

//===========================================================================
//
//===========================================================================

void *ColorSelectBox::LightnessMouseEventArrivedCB(void *Thisdata,void *pc_Data)
{
   ColorSelectBox *pc_This = (ColorSelectBox *) Thisdata;
   return pc_This->LightnessMouseEventArrived(pc_Data);

}

//===========================================================================
//
//===========================================================================


void *ColorSelectBox::LightnessMouseEventArrived(void *pc_Data)
{
   int i_MouseEventType ;
   
   i_MouseEventType = m_pc_LightnessGlobalStorage->m_pc_MouseEvent->GetEventType( );

   if(   i_MouseEventType == MouseEvent::EventStart
      || i_MouseEventType == MouseEvent::EventDo
     )
   {
      // get the coordinates and calculate a valid value
      int i_X;
      int i_Y;

      m_pc_LightnessGlobalStorage->m_pc_MouseEvent->GetLastBufferCoordinates(i_X,i_Y );

      m_ColorLightness = i_Y /2;

      ColorCalculation::HLSToRGB(m_ColorHue,m_ColorLightness,m_ColorSaturation,
                              m_ColorRed,m_ColorGreen,m_ColorBlue
                              );
      SetRGBSlider();
      SetHLSSlider();
      // no need to change the object on the color circle
      // SetObject();
      SetLightness();
      SetColorWidget();
   }

   return NULL;
} // LightnessMouseEventArrived()

//===========================================================================
//
//===========================================================================

void ColorSelectBox::ColorWidgetColorChanged(Pixel c_Color)
{
   m_ColorRed   = PIXEL_RED  (c_Color);
   m_ColorGreen = PIXEL_GREEN(c_Color);
   m_ColorBlue  = PIXEL_BLUE (c_Color);

   ColorCalculation::RGBToHLS(m_ColorRed,m_ColorGreen,m_ColorBlue,
                              m_ColorHue,m_ColorLightness,m_ColorSaturation);
   SetRGBSlider();
   SetHLSSlider();
   SetObject();
   SetLightness();
   SetColorWidget();

} // ColorWidgetColorChanged()

//===========================================================================
//
//===========================================================================

void ColorSelectBox::ClosePressed()
{

   this->hide();
} // ClosePressed()

//===========================================================================
//
//===========================================================================

void ColorSelectBox::StartedDrag(Pixel c_Color)
{
   Pixel p_Color;
   int   i;

   // save the color in the history
   for(i=COLORSELECTBOX_HISTORYWIDGETS-1; i>0;i--)
   {
      p_Color = m_w_ColorHistory[i-1]->GetColor();
      m_w_ColorHistory[i]->SetColor(p_Color);

   } // for i;

   p_Color = m_ColorWidget->GetColor();
   m_w_ColorHistory[0]->SetColor(p_Color);

} // StartedDrag()

//===========================================================================
//
//===========================================================================


