#ifndef _COLORSELECTBOX_H
#define _COLORSELECTBOX_H

#include <qdialog.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include "PixelBuffer.h"
#include "ImageFrame.h"
#include "QULabelSlider.h"
#include "GlobalStorage.h"
#include "QUColorDragWidget.h"
#include "GraphicObjectLayer.h"
#include "GraphicObjectMain.h"

class ColorSelectBox : public QDialog
{
Q_OBJECT

public:
   ColorSelectBox(QWidget *parent,
                  GlobalStorage *pc_ParentGlobalStorage);




public slots:
   void RGBSliderChanged(int i_Value);
   void HLSSliderChanged(int i_Value);
   void ZoomSliderChanged(int i_ZoomValue);

   void ColorWidgetColorChanged(Pixel c_Color);
   void StartedDrag(Pixel c_Color);
   void ClosePressed();
private:


   void CreateColorImage();

   void SetRGBSlider();
   void SetHLSSlider();
   void SetObject();
   void SetLightness();
   void SetColorWidget();

   void *MouseEventArrived(void *pc_Data);

   static void *MouseEventArrivedCB(void *Thisdata,void *pc_Data);

   void *LightnessMouseEventArrived(void *pc_Data);

   static void *LightnessMouseEventArrivedCB(void *Thisdata,void *pc_Data);



   ImageFrame      *m_pc_ImageFrame;
   ImageFrame      *m_pc_LightnessFrame;


   int m_ColorRed;
   int m_ColorGreen;
   int m_ColorBlue;
   int m_ColorHue;
   int m_ColorLightness;
   int m_ColorSaturation;

   float m_f_CoordinateX;
   float m_f_CoordinateY;

   PixelBuffer *m_pc_ColorCircle;
   PixelBuffer *m_pc_LightnessBuffer;
   GlobalStorage *m_pc_GlobalStorage;
   GlobalStorage *m_pc_LightnessGlobalStorage;
   GraphicObject *m_pc_Point;
   GraphicObject *m_pc_LightnessPoint;

   GlobalStorage *m_pc_ParentGlobalStorage;


   QBoxLayout    *m_l_TopLayout;
   QWidget       *m_w_GraphicWidget;
   QWidget       *m_w_GridWidget;

   QPushButton   *m_w_CloseButton;

   QULabelSlider  *m_RedSlider;
   QULabelSlider  *m_GreenSlider;
   QULabelSlider  *m_BlueSlider;
   QULabelSlider  *m_HueSlider;
   QULabelSlider  *m_LightnessSlider;
   QULabelSlider  *m_SaturationSlider;

   QUColorDragWidget *m_ColorWidget;
#define COLORSELECTBOX_HISTORYWIDGETS 18
   QUColorDragWidget *m_w_ColorHistory[COLORSELECTBOX_HISTORYWIDGETS];

   QSlider        *m_ZoomSlider;
   int             m_i_ZoomFactor;


   int  m_i_CircleRadius ;
   int  m_i_LightnessWidth;
   int  m_i_LightnessHeight;

   int m_i_CurrentLightnessBoxHue;
   int m_i_CurrentLightnessBoxLightness;
} ; // ColorSelectBox


#endif // _COLORSELECTBOX_H
