
#include <qsizepolicy.h>
#include <qpainter.h>

#include "DrawingArea.h"

//===========================================================================
//
//===========================================================================

DrawingArea::DrawingArea(QWidget *parent,GlobalStorage *pc_GlobalStorage,
               int i_PixelSizeX, int i_PixelSizeY)
   : QWidget(parent),
     Tracer("DrawingArea")
{
   QSizePolicy  DrawingPolicy;

   m_pc_GlobalStorage = pc_GlobalStorage;
   m_pc_GlobalStorage->m_pc_DrawingArea = this;

   DrawingPolicy.setVerData(QSizePolicy::Ignored);
   DrawingPolicy.setHorData(QSizePolicy::Ignored);
   setSizePolicy(DrawingPolicy);
   setMouseTracking(true);
   m_b_InMouseEvent = false;

   // not valid
   m_i_Width     = 1;
   m_i_Height    = 1;
   m_i_OldWidth  = 1;
   m_i_OldHeight = 1;

   m_ResizeCallback.Clear();

   if(   i_PixelSizeX>0
      && i_PixelSizeY>0)
   {
      // must be fixed size
      // allocate only enough for the drawing area
      m_CurrentImage = new QImage(i_PixelSizeX,i_PixelSizeY,32);
   }
   else
   {
      // default, whole screen
      m_CurrentImage = new QImage(1600,1200,32);
   }

} // DrawingArea() constructor

//===========================================================================
//
//===========================================================================

DrawingArea::~DrawingArea()
{
   delete m_CurrentImage;
}

//===========================================================================
//
//===========================================================================


void DrawingArea::resizeEvent(QResizeEvent *theEvent)
{
   int i_NewHeight;
   int i_NewWidth;

   // fprintf(stderr,"resize \n");

   i_NewWidth  = width();
   i_NewHeight = height();

   // fprintf(stderr,"w= %d , h=%d \n",i_NewWidth,i_NewHeight);

   // remember the old values
   m_i_OldWidth  = m_i_Width;
   m_i_OldHeight = m_i_Height;

   // store the new values
   m_i_Width  = i_NewWidth;
   m_i_Height = i_NewHeight;

   // call resize event if registered
   m_ResizeCallback.Execute(theEvent);

} // resizeEvent()

//===========================================================================
//
//===========================================================================


void DrawingArea::paintEvent(QPaintEvent *theEvent)
{
   QRect s_ModifiedRect;
   int i_X,i_Y, i_ChangedWidth,i_ChangedHeight;


   s_ModifiedRect = theEvent->rect();

   i_X = s_ModifiedRect.x();
   i_Y = s_ModifiedRect.y();
   i_ChangedWidth = s_ModifiedRect.width();
   i_ChangedHeight= s_ModifiedRect.height();
/*
   fprintf(stderr,"Paint in %d %d w=%d h=%d\n",
          i_X, i_Y, i_ChangedWidth, i_ChangedHeight);
*/


   if(    m_i_Width  >= 0
       && m_i_Height >= 0)
   {
      QPainter painter(this);
//      painter->drawLine(0,0,m_i_Width,m_i_Height);
//      painter->drawLine(0,m_i_Height,m_i_Width,0);

//      painter.drawImage(i_X,i_Y,    // destination
//                        *m_CurrentImage,
//                        i_X,i_Y,     // image start
//                        i_ChangedWidth,i_ChangedHeight,
//                        0);

      // call expose event if registered
      m_ExposeCallback.Execute(theEvent);


   }
} // paintEvent()

//===========================================================================
//
//===========================================================================

void DrawingArea::InstallResizeCallback(Callback c_ResizeCallback)
{
   m_ResizeCallback = c_ResizeCallback;

} // InstallResizeCallback()

//===========================================================================
//
//===========================================================================

void DrawingArea::InstallExposeCallback(Callback c_ExposeCallback)
{
   m_ExposeCallback = c_ExposeCallback;

} // InstallResizeCallback()

//===========================================================================
//
//===========================================================================

void DrawingArea::InstallMouseStartCallback(Callback c_Callback)
{
   m_MouseStartCallback = c_Callback;

} // InstallMouseStartCallback()

//===========================================================================
//
//===========================================================================

void DrawingArea::InstallMouseDoCallback(Callback c_Callback)
{
   m_MouseDoCallback = c_Callback;

} // InstallMouseDoCallback()

//===========================================================================
//
//===========================================================================

void DrawingArea::InstallMouseEndCallback(Callback c_Callback)
{
   m_MouseEndCallback = c_Callback;

} // InstallMouseEndCallback()

//===========================================================================
//
//===========================================================================

void DrawingArea::InstallMouseOverCallback(Callback c_Callback)
{
   m_MouseOverCallback = c_Callback;

} // InstallMouseEndCallback()

//===========================================================================
//
//===========================================================================

int DrawingArea::GetWidth()
{
   return m_i_Width;
}

//===========================================================================
//
//===========================================================================

int DrawingArea::GetHeight()
{
   return m_i_Height;
}

//===========================================================================
//
//===========================================================================


int DrawingArea::GetOldWidth()
{
   return m_i_OldWidth;
}

//===========================================================================
//
//===========================================================================

int DrawingArea::GetOldHeight()
{
   return m_i_OldHeight;
}

//===========================================================================
//
//===========================================================================


QImage *DrawingArea::GetImage()
{
   return m_CurrentImage;
}

//===========================================================================
//
//===========================================================================




void DrawingArea::mousePressEvent(QMouseEvent *theEvent)
{
/*
fprintf(stderr,"MousePress %d  (%d/%d)\n",
   theEvent->button(),
   theEvent->x(),
   theEvent->y());
*/
   m_MouseStartCallback.Execute(theEvent);
   m_b_InMouseEvent = true;

} // mousePressEvent()

//===========================================================================
//
//===========================================================================


void DrawingArea::mouseMoveEvent(QMouseEvent *theEvent)
{
/*
fprintf(stderr,"MouseMove %d  (%d/%d)  in MouseEvent %d\n",
   theEvent->state(),
   theEvent->x(),
   theEvent->y(),
   m_b_InMouseEvent);
*/

   if(m_b_InMouseEvent)
   {
      m_MouseDoCallback.Execute(theEvent);
      return;
   }


   m_MouseOverCallback.Execute(theEvent);

} // mouseMoveEvent()

//===========================================================================
//
//===========================================================================

void DrawingArea::mouseReleaseEvent(QMouseEvent *theEvent)
{
/*
fprintf(stderr,"MouseRelease %d  (%d/%d)\n",
   theEvent->button(),
   theEvent->x(),
   theEvent->y());
*/
   m_MouseEndCallback.Execute(theEvent);

   m_b_InMouseEvent = false;

} // mouseReleaseEvent()

//===========================================================================
//
//===========================================================================


