// file load and write functions for the Perico program

#include <qfiledialog.h>

#include "FileLoader.h"
#include "PixelBuffer.h"
#include "ApplicationStorage.h"

extern "C"
{
#include <jpeglib.h>
#include <gif_lib.h>
#include <tiffio.h>
#include <png.h>
}

#define JPEG_FILTER "Jpeg (*.jpg )"
#define TIFF_FILTER "Tiff (*.tif )"
#define PNG_FILTER  "Png (*.png )"
#define BMP_FILTER  "True Color Bmp (*.bmp )"
#define ALL_FILTER  "Images (*.jpg *.gif *.tif *.png *.bmp)"

//===========================================================================
//
//===========================================================================


FileLoader::FileLoader(QWidget *MainWidget)
{
   m_MainWidget = MainWidget;
}

//===========================================================================
//
//===========================================================================

FileLoader::~FileLoader()
{
}

//===========================================================================
//
//===========================================================================

QString FileLoader::AskForFilename(bool b_OpenMode,QString qs_DefaultName)
{
   QString qs_ReturnValue;
   qs_ReturnValue ="";

   if(b_OpenMode)
   {
      QFileDialog *w_FileDialog;
      int         i_ExecReturn;

      w_FileDialog = new QFileDialog(m_MainWidget);
      
      w_FileDialog->setCaption("Perico: Load a File");

      w_FileDialog->addFilter(JPEG_FILTER );
      w_FileDialog->addFilter(PNG_FILTER );
      w_FileDialog->addFilter(TIFF_FILTER );
      w_FileDialog->addFilter(BMP_FILTER );
      w_FileDialog->addFilter(ALL_FILTER );

      if( ! g_Application->m_qs_LoadDirectory.isEmpty() )
      {
         w_FileDialog->setDir(g_Application->m_qs_LoadDirectory);
      }

      // ask for a name
      i_ExecReturn = w_FileDialog->exec();

      // if abort, then return an empty filename
      if(i_ExecReturn == 0)
      {
         // delete the allocated dialog
         delete w_FileDialog;
         return qs_ReturnValue;
      }

      // get the filename
      qs_ReturnValue = w_FileDialog->selectedFile();

      // save the current path
      g_Application->m_qs_LoadDirectory = w_FileDialog->dirPath();

      // delete the allocated dialog
      delete w_FileDialog;

   }
   else // save mode
   {

      QFileDialog *w_FileDialog;
      QString     qs_SelectedFile;
      QString     qs_SelectedFilter;
      QString     qs_SelectedEnding;
      QString     qs_ActualEnding;
      int         i_ExecReturn;

      w_FileDialog = new QFileDialog(m_MainWidget);
      
      w_FileDialog->setCaption("Perico: Save a File");

      w_FileDialog->addFilter(JPEG_FILTER );
      w_FileDialog->addFilter(PNG_FILTER );
      w_FileDialog->addFilter(TIFF_FILTER );
      w_FileDialog->setMode(QFileDialog::AnyFile);

      // if we have a default name, then try to set into
      // that directory
      if( !qs_DefaultName.isEmpty() )
      {
         QString qs_Extension;
         qs_Extension = GetExtension(qs_DefaultName);
         
         // set the selection to the filename
         w_FileDialog->setSelection(qs_DefaultName);

         // try to set the extension
         if(qs_Extension.compare("jpg") == 0)
         {
            w_FileDialog->setSelectedFilter( JPEG_FILTER );
         }
         else if(qs_Extension.compare("tiff") == 0)
         {
            w_FileDialog->setSelectedFilter( TIFF_FILTER );
         }
         else if(qs_Extension.compare("png") == 0)
         {
            w_FileDialog->setSelectedFilter( PNG_FILTER );
         }

      }
      else
      {
         if( ! g_Application->m_qs_LoadDirectory.isEmpty() )
         {
            w_FileDialog->setDir(g_Application->m_qs_LoadDirectory);
         }
      }

      // ask for a file
      i_ExecReturn = w_FileDialog->exec();

      // if abort, then return an empty filename
      if(i_ExecReturn == 0)
      {
         // delete the allocated dialog
         delete w_FileDialog;
         return qs_ReturnValue;
      }
      // now get back the current filter an the filename

      qs_SelectedFile = w_FileDialog->selectedFile();
      qs_SelectedFilter = w_FileDialog->selectedFilter();

      // get the current selected ending
      if(qs_SelectedFilter.compare( JPEG_FILTER ) ==0)
      {
         qs_SelectedEnding = "jpg";
      }
      else if(qs_SelectedFilter.compare( TIFF_FILTER ) ==0)
      {
         qs_SelectedEnding = "tif";
      }
      else if(qs_SelectedFilter.compare( PNG_FILTER ) ==0)
      {
         qs_SelectedEnding = "png";
      }
      else if(qs_SelectedFilter.compare( BMP_FILTER ) ==0)
      {
         qs_SelectedEnding = "bmp";
      }
      else
      {
         // default is tiff
         qs_SelectedEnding = "tif";
      }

      // look whether the file has an ending
      qs_ActualEnding = GetExtension(qs_SelectedFile);


      // are file and selected ending the same?
      if(qs_ActualEnding.compare(qs_SelectedEnding) != 0)
      {
         // no, then
         // put selected ending behing name
         qs_SelectedFile = qs_SelectedFile+"."+qs_SelectedEnding;
      }

      // delete the allocated dialog
      delete w_FileDialog;

      // return the filename
      qs_ReturnValue = qs_SelectedFile;

   } // save mode

   return qs_ReturnValue;

} // AskForFilename()

//===========================================================================
//
//===========================================================================


PixelBuffer *FileLoader::LoadFile(QString qs_Filename)
{
   QString qs_Extension;
   PixelBuffer *pc_NewBuffer;
   pc_NewBuffer = NULL;

   qs_Extension = GetExtension(qs_Filename);

   if(qs_Extension.compare("jpg") == 0)
   {
      pc_NewBuffer = LoadFileJpeg(qs_Filename);
   }
   else if(qs_Extension.compare("gif") == 0)
   {
      pc_NewBuffer = LoadFileGif(qs_Filename);
   }
   else if(qs_Extension.compare("tif") == 0)
   {
      pc_NewBuffer = LoadFileTif(qs_Filename);
   }
   else if(qs_Extension.compare("png") == 0)
   {
      pc_NewBuffer = LoadFilePng(qs_Filename);
   }
   else if(qs_Extension.compare("bmp") == 0)
   {
      pc_NewBuffer = LoadFileBmp(qs_Filename);
   }

   return pc_NewBuffer;

} // LoadFile()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::LoadFilePng(QString qs_Filename)
{
   unsigned char *pc_ReadBuffer;
   PixelBuffer   *pc_NewImage ;
   png_structp    pc_Png;
   png_infop      pc_PngInfo;

   FILE          *fp_Infile;
   int            i_Width;
   int            i_Height;
   int            i_Bitdepth;
   int            i_ColorType;
   int            i_Y;
   int            i_X;

   // open the infile
   fp_Infile = fopen(qs_Filename, "rb");
   if(fp_Infile == NULL)
   {
      return NULL;
   }

   pc_NewImage = NULL;

   // create the png struct
   pc_Png = png_create_read_struct(PNG_LIBPNG_VER_STRING,  0, 0, 0);
   if(pc_Png == NULL)
   {
      fclose(fp_Infile);
      return NULL;
   }
   
   // create the info struct
   pc_PngInfo = png_create_info_struct(pc_Png);
   if(pc_PngInfo == NULL)
   {
      fclose(fp_Infile);
      return NULL;
   }

   // attach infile to the io
   png_init_io(pc_Png, fp_Infile);
   
   // read header of the file
   png_read_info(pc_Png, pc_PngInfo);

   i_Width = pc_PngInfo->width;
   i_Height = pc_PngInfo->height;
   i_ColorType = pc_PngInfo->color_type;
   i_Bitdepth = pc_PngInfo->bit_depth;


   // need a valid image with RGB
   if(   i_Width<=0
      || i_Height <=0
      || i_ColorType != PNG_COLOR_TYPE_RGB)
   {
      png_read_end(pc_Png, 0);
      png_destroy_read_struct(&pc_Png, &pc_PngInfo, 0);
      fclose(fp_Infile);
      return NULL;
   }

   // allocate a new buffer for the pictrue
   pc_NewImage =  new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);
   if(pc_NewImage == NULL)
   {
      png_read_end(pc_Png, 0);
      png_destroy_read_struct(&pc_Png, &pc_PngInfo, 0);
      fclose(fp_Infile);
      return NULL;
   }

   // allocate a buffer for one line
   pc_ReadBuffer = new unsigned char [i_Width *4];

   // loop for reading the image
   for(i_Y = 0; i_Y < i_Height; i_Y ++)
   {
      unsigned char *pc_In;

      png_read_row(pc_Png, pc_ReadBuffer, 0);
      pc_In = pc_ReadBuffer;
      for(i_X=0;i_X < i_Width;i_X ++)
      {
          unsigned int i_Red;
          unsigned int i_Green;
          unsigned int i_Blue;
          Pixel p;

          i_Red   = (unsigned int) *pc_In++;
          i_Green = (unsigned int) *pc_In++;
          i_Blue  = (unsigned int) *pc_In++;

          p = PIXEL_MAKE(i_Red,i_Green,i_Blue);

          pc_NewImage->SetRaw(i_X,i_Y,p);

      }
   }
   delete pc_ReadBuffer;

   // destroy the png structures
   png_read_end(pc_Png, 0);
   png_destroy_read_struct(&pc_Png, &pc_PngInfo, 0);

   // close the file
   fclose(fp_Infile);

   // return the new image
   return pc_NewImage;
} // LoadFilePng()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::LoadFileTif(QString qs_Filename)
{
   TIFFRGBAImage  img;
   PixelBuffer   *pc_NewImage ;
   TIFF          *tif;
   char           emsg[1024];

   pc_NewImage = NULL;

   tif = TIFFOpen(qs_Filename, "r");

   if (tif == NULL)
   {
      return NULL;
   }
   
   if (TIFFRGBAImageBegin(&img, tif, 0, emsg))
   {
      uint32  * raster;
      size_t    npixels;

      npixels = img.width * img.height;
      raster = (uint32*) _TIFFmalloc(npixels * sizeof (uint32));

      if (raster != NULL)
      {
         if (TIFFRGBAImageGet(&img, raster, img.width, img.height))
         {
            unsigned char* pc_Source;
            int            i_X;
            int            i_Y;

            pc_NewImage =  new PixelBuffer(img.width,img.height,BITNUMBER_LONG);
            pc_Source   = (unsigned char *) raster;

            // copy all pixel into the new buffer
            for(i_Y = img.height-1;i_Y >=0; i_Y --)
            {
               for(i_X = 0; i_X< (int) img.width; i_X ++)
               {
                  unsigned int i_Red;
                  unsigned int i_Green;
                  unsigned int i_Blue;
                  Pixel        p;

                  i_Red   = (unsigned int) *pc_Source ++;
                  i_Green = (unsigned int) *pc_Source ++;
                  i_Blue  = (unsigned int) *pc_Source ++;
                  pc_Source ++;
                  p= PIXEL_MAKE(i_Red,i_Green,i_Blue);
                  pc_NewImage->SetRaw(i_X,i_Y,p);
               } // for i_X
            } // for i_Y
         }
         _TIFFfree(raster);
      }
      TIFFRGBAImageEnd(&img);
   }
   
   TIFFClose(tif);

   return pc_NewImage;
} // LoadFileTif()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::LoadFileGif(QString qs_Filename)
{
   ColorMapObject *pc_ColorMap;
   GifPixelType   *pc_PixelSource;
   GifColorType   *Colors;
   GifFileType    * pc_GifFile;
   PixelBuffer    * pc_NewImage ;
   SavedImage     * pc_Image;
   int            i_Width;
   int            i_Height;
   int            i_Y;

   // open the gif file
   pc_GifFile = DGifOpenFileName( qs_Filename);
   if(pc_GifFile == NULL)
   {
      return NULL;
   }

   // get dimensions
   i_Width   = pc_GifFile->SWidth;
   i_Height  = pc_GifFile->SHeight;

   // allocate a buffer for the new image
   pc_NewImage = new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);

   // read in the complete file
   DGifSlurp(pc_GifFile);

   // get the first image (in case of a multiimage file
   pc_Image = pc_GifFile->SavedImages;

   // get the colormap
   pc_ColorMap = pc_GifFile->SColorMap;
   if( pc_ColorMap == NULL)
   {
      // get map of the single image
      pc_ColorMap = pc_Image->ImageDesc.ColorMap;
   }

   // get pointer to the single colors
   Colors = pc_ColorMap->Colors;

   // get pointer to the rasterbits
   pc_PixelSource = (GifPixelType *) pc_Image->RasterBits;

   // now copy the file into the new pixelbuffer
   if(  pc_Image-> ImageDesc.Interlace != 0
     )
   {
      // interlaced lines
      for(i_Y =0; i_Y < i_Height; i_Y += 8)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);
      }
      for(i_Y =4; i_Y < i_Height; i_Y += 8)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);
      }
      for(i_Y =2; i_Y < i_Height; i_Y += 4)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);
      }
      for(i_Y =1; i_Y < i_Height; i_Y += 2)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);
      }

   }
   else // not interlaced
   {
      for(i_Y =0; i_Y < i_Height; i_Y++)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);

      } // for all lines
   }

   // close the input file
   DGifCloseFile(pc_GifFile);

   // return the result
   return pc_NewImage;

} // LoadFileGif()

//===========================================================================
//
//===========================================================================

void FileLoader::SetGifLine(int i_Line,
                    unsigned char * &pc_Source,
                    PixelBuffer* pc_Destination,
                    GifColorType *Colors)
{
   int i_X;
   int i_Width = pc_Destination->GetWidth();

   for(i_X=0 ; i_X < i_Width; i_X ++)
   {
      int i_Red;
      int i_Green;
      int i_Blue;
      unsigned int i_Index;

      Pixel p_Color;
      i_Index = (unsigned int) *pc_Source++;

      i_Red   = Colors[i_Index].Red;
      i_Green = Colors[i_Index].Green;
      i_Blue  = Colors[i_Index].Blue;

      p_Color =PIXEL_MAKE(i_Red,i_Green,i_Blue);

      //fprintf(stderr,"%d %x\n",i_X,ReadBuffer[i_X]);
      pc_Destination->Set(i_X,i_Line,p_Color);
   }

} // SetGifLine()

//===========================================================================
//
//===========================================================================


PixelBuffer *FileLoader::LoadFileJpeg(QString qs_Filename)
{
   int i_OpenResult;
   int i_Width;
   int i_Height;
   int y;

   PixelBuffer * pc_NewImage = NULL;

   i_OpenResult = JpegSetFilenameForLoad(qs_Filename,
                                        i_Width,
                                        i_Height);

   if(    i_OpenResult <0
       || i_Width  <=0
       || i_Height <=0
      )
   {
      return NULL;
   }
   
   pc_NewImage = new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);

   for(y=0; y< i_Height;y++)
   {
      unsigned char *puc_Line;

      puc_Line = JpegReadLine();
      if(puc_Line != NULL)
      {
         int x;

         // put line into the pixelbuffer
         for(x=0; x < i_Width; x++)
         {
            unsigned char r,g,b;
            Pixel p_Pixel;
            r = (unsigned char) *puc_Line++;
            g = (unsigned char) *puc_Line++;
            b = (unsigned char) *puc_Line++;
            p_Pixel = PIXEL_MAKE(r,g,b) ;
            pc_NewImage->SetRaw(x,y,p_Pixel);
         }

      }
   } // for all lines

   JpegClose();

   return pc_NewImage;

} // LoadFile()

//===========================================================================
//
//===========================================================================


int FileLoader::JpegSetFilenameForLoad(QString qs_Filename,
                                       int &i_Width,
                                       int &i_Height)
{
   int  i_RowStride;

   jpegInfo.err =jpeg_std_error(&jpegError);

   jpeg_create_decompress(&jpegInfo);
     
   m_JpegFile = fopen(qs_Filename,"r");

   // file open success?
   if(m_JpegFile == NULL)
   {
      i_Width  = -1;
      i_Height = -1;
      return -1;
   }

   jpeg_stdio_src(&jpegInfo,m_JpegFile);
   
   /*===================================*/
   
   jpeg_read_header(&jpegInfo,TRUE);


   jpeg_start_decompress(&jpegInfo);

   i_RowStride = jpegInfo.output_width * jpegInfo.output_components;

   m_JpegOutputLine = (*jpegInfo.mem->alloc_sarray)
                       ((j_common_ptr) &jpegInfo, 
                       JPOOL_IMAGE,
                       i_RowStride,
                       1);


   i_Width  = jpegInfo.image_width;
   i_Height = jpegInfo.image_height;

   return 0;

} // JpegSetFilenameForLoad()

//===========================================================================
//
//===========================================================================

void FileLoader::JpegClose()
{

   jpeg_finish_decompress(&jpegInfo);

   jpeg_destroy_decompress(&jpegInfo);

   if(m_JpegFile != NULL)
   {
      fclose(m_JpegFile);
      m_JpegFile=NULL;
   }

} // JpegClose()

//===========================================================================
//
//===========================================================================

unsigned char * FileLoader::JpegReadLine()
{
   unsigned char *outbuffer;

   jpeg_read_scanlines(&jpegInfo,m_JpegOutputLine,1);
   outbuffer = (unsigned char *) *(m_JpegOutputLine);

   return outbuffer;

} // JpegReadLine()


//===========================================================================
//
//===========================================================================

int FileLoader::SaveFile(QString qs_Filename,PixelBuffer *pc_Image)
{
   QString qs_Extension;
   int     i_Result;

   qs_Extension = GetExtension(qs_Filename);
   // fprintf(stderr,"extension = >%s<\n",(const char *) qs_Extension);


   if(qs_Extension.compare("jpg") == 0)
   {
      i_Result = SaveFileJpeg(qs_Filename,pc_Image);
   }
   else if(qs_Extension.compare("tif") == 0)
   {
      i_Result = SaveFileTiff(qs_Filename,pc_Image);
   }
   else if(qs_Extension.compare("png") == 0)
   {
      i_Result = SaveFilePng(qs_Filename,pc_Image);
   }

   return 0;
} // SaveFile

//===========================================================================
//
//===========================================================================

int FileLoader::SaveFileJpeg(QString qs_Filename,PixelBuffer *pc_Image)
{
   struct jpeg_compress_struct jpegWriteInfo;
   unsigned char              *pc_Line;
   FILE                       *fp_Outfile;
   int                        i_Y;
   int                        i_X;
   int                        i_Width;
   // create the output file
   fp_Outfile = fopen(qs_Filename,"w");
   if(fp_Outfile == NULL)
   {
      return -1;
   }

   // clean up the write parameter buffer
   memset(&jpegWriteInfo,0,sizeof(jpegWriteInfo));

   jpegWriteInfo.err = jpeg_std_error(&jpegError);

   jpeg_create_compress(&jpegWriteInfo);

   jpeg_stdio_dest(&jpegWriteInfo, fp_Outfile);


   i_Width = pc_Image->GetWidth();
   jpegWriteInfo.image_width      = i_Width;
   jpegWriteInfo.image_height     = pc_Image->GetHeight();
   jpegWriteInfo.input_components = 3;
   jpegWriteInfo.in_color_space   = JCS_RGB;

   jpeg_set_defaults(&jpegWriteInfo);

   jpeg_set_quality(&jpegWriteInfo,97,TRUE);

   jpeg_start_compress(&jpegWriteInfo,TRUE);


   // now write all lines
   pc_Line = new unsigned char[ (i_Width+1)*3];

   for(i_Y=0; i_Y < pc_Image->GetHeight(); i_Y ++)
   {
      unsigned char * pc_InLine;
      JSAMPROW        row_pointer[1];

      pc_InLine = pc_Line;
      for(i_X=0; i_X < i_Width; i_X ++)
      {
         Pixel p_Pixel;
         p_Pixel = pc_Image->Get(i_X,i_Y);
         *pc_InLine ++ = PIXEL_RED(p_Pixel);
         *pc_InLine ++ = PIXEL_GREEN(p_Pixel);
         *pc_InLine ++ = PIXEL_BLUE(p_Pixel);

      }
      row_pointer[0] = (JSAMPROW) pc_Line;
      jpeg_write_scanlines(&jpegWriteInfo,row_pointer,1);
   }

   delete pc_Line;


   // finish compression
   jpeg_finish_compress(&jpegWriteInfo);

   fclose(fp_Outfile);

   jpeg_destroy_compress(&jpegWriteInfo);

   return 0;
   
} // SaveFileJpeg

//===========================================================================
//
//===========================================================================

int FileLoader::SaveFileTiff(QString qs_Filename,PixelBuffer *pc_Image)
{
   unsigned char * puc_Outbuffer;
   TIFF          * p_TiffOut;
   int             i_Outbytes;
   int             i_Y;
   int             i_X;
   int             i_Width;
   int             i_Height;


   p_TiffOut=TIFFOpen(qs_Filename,"w");

   if(p_TiffOut ==NULL)
   {
      return -1;
   }


   i_Width  = pc_Image->GetWidth();
   i_Height = pc_Image->GetHeight();

   TIFFSetField( p_TiffOut, TIFFTAG_IMAGELENGTH, i_Height);
   TIFFSetField( p_TiffOut, TIFFTAG_IMAGEWIDTH, i_Width);
   TIFFSetField( p_TiffOut, TIFFTAG_SAMPLESPERPIXEL, 3);
   TIFFSetField( p_TiffOut, TIFFTAG_BITSPERSAMPLE, 8);
   TIFFSetField( p_TiffOut, TIFFTAG_ORIENTATION, 1);

   TIFFSetField ( p_TiffOut, TIFFTAG_SUBFILETYPE, 0);

   // does not work because of patents
   // TIFFSetField ( out, TIFFTAG_COMPRESSION, COMPRESSION_LZW);
   TIFFSetField ( p_TiffOut, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
   TIFFSetField ( p_TiffOut, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
   TIFFSetField ( p_TiffOut, TIFFTAG_ROWSPERSTRIP, 1);


   i_Outbytes = TIFFScanlineSize(p_TiffOut);

   puc_Outbuffer = (unsigned char *) _TIFFmalloc( i_Outbytes );

   for(i_Y =0; i_Y < i_Height; i_Y ++)
   {
      unsigned char * pc_Inout;

      pc_Inout = puc_Outbuffer;

      for(i_X =0; i_X < i_Width; i_X ++)
      {
         Pixel p;

         p = pc_Image->Get(i_X,i_Y);
         *pc_Inout++ = PIXEL_RED(p);
         *pc_Inout++ = PIXEL_GREEN(p);
         *pc_Inout++ = PIXEL_BLUE(p);

      }

      TIFFWriteScanline(p_TiffOut, puc_Outbuffer, i_Y, 0 );

   } // for i_Y


   TIFFClose(p_TiffOut);

   _TIFFfree(puc_Outbuffer);

   return 0;

} // SaveFileTiff

//===========================================================================
//
//===========================================================================

int FileLoader::SaveFilePng(QString qs_Filename,PixelBuffer *pc_Image)
{
   unsigned char * puc_Outbuffer;
   png_structp     pc_Png;
   png_infop       pc_PngInfo;
   FILE          * fp_Outfile;
   int             i_Y;
   int             i_X;
   int             i_Width;
   int             i_Height;

   // open the outfile;
   fp_Outfile = fopen(qs_Filename, "wb");
   if(fp_Outfile == NULL)
   {
      return -1;
   }

   // get image dimensions
   i_Width  = pc_Image->GetWidth();
   i_Height = pc_Image->GetHeight();

   // create the write structure
   pc_Png = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   if(pc_Png ==NULL)
   {
      fclose(fp_Outfile);
      return -1;
   }

   // create the info structure
   pc_PngInfo = png_create_info_struct(pc_Png);
   if(pc_PngInfo ==NULL)
   {
      png_destroy_write_struct(&pc_Png, NULL);
      fclose(fp_Outfile);
      return -1;
   }

   // attach the outfile
   png_init_io(pc_Png, fp_Outfile);

   // write the header information
   png_set_IHDR(pc_Png,
                pc_PngInfo,
                i_Width,
                i_Height,
                8,
                PNG_COLOR_TYPE_RGB,
                PNG_INTERLACE_NONE,
                PNG_COMPRESSION_TYPE_BASE,
                PNG_FILTER_TYPE_BASE);

   png_write_info(pc_Png, pc_PngInfo);


   // create buffer for a single line
   puc_Outbuffer = new unsigned char[i_Width *4];

   // write all lines
   for(i_Y =0; i_Y < i_Height; i_Y ++)
   {
      unsigned char * pc_Inout;

      pc_Inout = puc_Outbuffer;

      for(i_X =0; i_X < i_Width; i_X ++)
      {
         Pixel p;

         p = pc_Image->Get(i_X,i_Y);
         *pc_Inout++ = PIXEL_RED(p);
         *pc_Inout++ = PIXEL_GREEN(p);
         *pc_Inout++ = PIXEL_BLUE(p);

      }
      // write line
      png_write_row(pc_Png, (png_byte *) puc_Outbuffer);

   } // for i_Y

   delete puc_Outbuffer;

   // clean up the png structure
   png_write_end(pc_Png, pc_PngInfo);
   png_destroy_write_struct(&pc_Png, &pc_PngInfo);

   // close the outfile
   fclose(fp_Outfile);
   
   return 0;

} // SaveFilePng();

//===========================================================================
// get extension without the point
//===========================================================================

QString FileLoader::GetExtension(QString qs_Name)
{
   QString qs_Result;
   int     i_PointPosition;

   // set empty
   qs_Result= "";

   i_PointPosition = qs_Name.findRev('.');
   if(i_PointPosition <=0)
   {
      return qs_Result;
   }

   // get only extension
   qs_Result = qs_Name.mid(i_PointPosition+1);

   // make it lowercase
   qs_Result = qs_Result.lower();

   return qs_Result;
} // GetExtension()

//===========================================================================
//
//===========================================================================

// get value a pointer as an int (LSB first / intel)

static int GetInt(void *pv_P)
{
   unsigned char *puc_P;
   unsigned int  i_Return;

   puc_P = (unsigned char *) pv_P;

   i_Return  = ((unsigned int) (*puc_P++));
   i_Return |= ((unsigned int) (*puc_P++))<<8;
   i_Return |= ((unsigned int) (*puc_P++))<<16;
   i_Return |= ((unsigned int) (*puc_P++))<<24;
   
   return (int) i_Return;

} // GetInt()
   
//===========================================================================
//
//===========================================================================

// structures for the bmpfile
typedef struct{
   unsigned char m_uc_SignatureB;
   unsigned char m_uc_SignatureM;
   unsigned char m_uc_FLen[4];
   unsigned char m_uc_Dummy[4];
   unsigned char m_uc_Offset[4];
} BMPHEADER;

typedef struct{
   unsigned char m_uc_ISize[4];
   unsigned char m_uc_XMax[4];
   unsigned char m_uc_YMax[4];
   unsigned char m_uc_Planes[2];
   unsigned char m_uc_BitsPerPixel[2];
   unsigned char m_uc_Compress[4];
   unsigned char m_uc_XSize[4];
   unsigned char m_uc_Hdpi[4];
   unsigned char m_uc_Vdpi[4];
   unsigned char m_uc_Cols[4];
   unsigned char m_uc_Colsi[4];
} BMPINFO;


PixelBuffer *FileLoader::LoadFileBmp(QString qs_Filename)
{
   PixelBuffer *pc_NewImage;
   BMPHEADER    c_BmpHeader;
   BMPINFO      c_BmpInfoHeader;
   FILE        *fp_Infile;
   int          i_Read;
   int          i_Width ;
   int          i_Height;
   int          i_BitsPerPixel;

   pc_NewImage = NULL;

   // open the file for reading
   fp_Infile = fopen(qs_Filename,"r");
   if(fp_Infile == NULL)
   {
      return NULL;
   }

   // read the header
   i_Read = fread(&c_BmpHeader,1,sizeof(c_BmpHeader), fp_Infile );
   if(i_Read != sizeof(c_BmpHeader))
   {
      fclose(fp_Infile);
      return NULL;
   }
   
   // read the info header
   i_Read = fread(&c_BmpInfoHeader,1,sizeof(c_BmpInfoHeader), fp_Infile );
   if(i_Read != sizeof(c_BmpInfoHeader))
   {
      fclose(fp_Infile);
      return NULL;
   }

   // check for the signature
   if ( !(     (c_BmpHeader.m_uc_SignatureB == 'B')
           &&  (c_BmpHeader.m_uc_SignatureM == 'M') )
      )
   {
      fclose(fp_Infile);
      return NULL;
   }
      
   i_Width  = GetInt(&c_BmpInfoHeader.m_uc_XMax);
   i_Height = GetInt(&c_BmpInfoHeader.m_uc_YMax);


   i_BitsPerPixel =   (c_BmpInfoHeader.m_uc_BitsPerPixel[0])
                    + (c_BmpInfoHeader.m_uc_BitsPerPixel[1]<<8);

   // only load true color pictures
   if( i_BitsPerPixel == 24)
   {
      unsigned char *puc_LineBuffer;
      int            i_LoadOffset;
      int            i_RunY;
      int            i_LineWidth;

      
      i_LoadOffset = GetInt(& c_BmpHeader.m_uc_Offset);
      fseek( fp_Infile,i_LoadOffset,SEEK_SET);

      // allocate a new buffer for the pictrue
      pc_NewImage =  new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);

      if(pc_NewImage == NULL)
      {
         fclose(fp_Infile);
         return NULL;
      }

      // allocate a buffer to read a line
      i_LineWidth = 3*i_Width;

      // let it be a multiple of 4
      i_LineWidth = (i_LineWidth+3) & 0xfffffffc;
      
      puc_LineBuffer = new unsigned char[i_LineWidth];
      if( puc_LineBuffer == NULL)
      {
         delete pc_NewImage;
         fclose(fp_Infile);
         return NULL;
      }

      // read all lines
      for(i_RunY=0;i_RunY<i_Height;i_RunY++)
      {
         unsigned char * puc_InBuffer;
         int            i_RunX;

         // read single line
         i_Read = fread(puc_LineBuffer,1,i_LineWidth, fp_Infile );
         if(i_Read != i_LineWidth)
         {
            fclose(fp_Infile);
            delete puc_LineBuffer;
            delete pc_NewImage;
            return NULL;
         }

         
         // copy line into the image
         puc_InBuffer = puc_LineBuffer;
         for(i_RunX=0;i_RunX < i_Width; i_RunX ++)
         {
            unsigned char uc_Red;
            unsigned char uc_Green;
            unsigned char uc_Blue;
            Pixel p_Pixel;

            // get rgb values
            uc_Blue   = *puc_InBuffer++;
            uc_Green  = *puc_InBuffer++;
            uc_Red    = *puc_InBuffer++;

            // make a pixel
            p_Pixel  = PIXEL_MAKE(uc_Red,uc_Green,uc_Blue);

            // put into image
            pc_NewImage->Set(i_RunX,i_Height-(i_RunY+1),p_Pixel);
         } // for RunX
         
      } // for runY

      // delete the linebuffer
      delete puc_LineBuffer;

   } // if ==24 bits per pixel


   // close file
   fclose(fp_Infile);

   // return the new image
   return pc_NewImage;

} // LoadFileBmp()

//===========================================================================
//
//===========================================================================

