
#include <stdlib.h>
#include <stdio.h>

#include "Filter.h"
#include "Undo.h"
#include "Utilities.h"


//============================================================================
//
//============================================================================

Filter::Filter(GlobalStorage *pc_Global)
{
   m_pc_GlobalStorage = pc_Global;

   m_pc_Original   = NULL;
   m_pc_Copy       = NULL;

   m_i_FilterType  = FilterNothing;

   m_i_FilterWidth          = 1;
   m_i_NearDistance         = 30;
   m_b_NearEnabled          = false;
   m_b_OnlyOneColorActive   = false;
   m_i_OnlyOneColorDistance = 30;
   m_p_OnlyOneColor         = 0x000000;


} // Filter()


//============================================================================
//
//============================================================================

Filter::~Filter()
{
   // nothing special now
}

//============================================================================
//
//============================================================================

void Filter::SetFilterType(int i_Type)
{

   m_i_FilterType = i_Type;

} // SetFilterType

//============================================================================
//
//============================================================================

void Filter::SetFilterWidth(int i_Width)
{

   m_i_FilterWidth = i_Width;

} // SetFilterWidth


//============================================================================
//
//============================================================================

void Filter::DoFilter()
{
   // do a filter depending on the filter type
   switch (m_i_FilterType)
   {
      case FilterMiddle:
         DoFilterMiddle();
         break;

      case FilterMedian:
         DoFilterMedian();
         break;

      default:
         break;
   }

} // DoFilter()

//============================================================================
//
//============================================================================

void Filter::SetOriginalBuffer(PixelBuffer * pc_Image)
{
   m_pc_Original = pc_Image;

} // SetOriginalBuffer

//============================================================================
//
//============================================================================

void Filter::DoFilterMiddle()
{
   PixelBuffer *pc_SelectionMask;
   RectRegion   c_MaskRegion;
   Pixel        p_Value;
   Pixel        p_Original;
   int i_RunX;
   int i_RunY;
   int i_Width;
   int i_Height;

   int i_FromY;
   int i_FromX;
   int i_ToX;
   int i_ToY;

   int i_RangeFromY;
   int i_RangeFromX;
   int i_RangeToX;
   int i_RangeToY;
   int i_RangeTotalPoints;

   int i_InnerX;
   int i_InnerY;

   int i_RedSum;
   int i_GreenSum;
   int i_BlueSum;

   // create a copy of the buffer
   CreateCopy();

   i_Width  = m_pc_Original->GetWidth();
   i_Height = m_pc_Original->GetHeight();

   i_FromX = 0;
   i_FromY = 0;
   i_ToX   = i_Width  -1;
   i_ToY   = i_Height -1;

   pc_SelectionMask = NULL;

   if(m_pc_Original->GetSelectionMaskActive() )
   {

      pc_SelectionMask = m_pc_Copy->GetSelectionMask();
      c_MaskRegion = pc_SelectionMask->GetUsedRegion(1);

      if(c_MaskRegion.IsEmpty())
      {
         // empty mask - do nothing
         // clean up the processing
         CleanUp();
         return;
      }

      i_FromX = c_MaskRegion.GetStartX();
      i_FromY = c_MaskRegion.GetStartY();
      i_ToX   = c_MaskRegion.GetEndX();
      i_ToY   = c_MaskRegion.GetEndY();
   }


   for(i_RunY= i_FromY; i_RunY <= i_ToY ;i_RunY ++)
   {
      i_RangeFromY = i_RunY - m_i_FilterWidth;
      if(i_RangeFromY <0)
      {
         i_RangeFromY =0;
      }
      i_RangeToY = i_RunY + m_i_FilterWidth;
      if(i_RangeToY >=i_Height)
      {
         i_RangeToY = i_Height-1;
      }

      for(i_RunX= i_FromX; i_RunX <= i_ToX; i_RunX ++)
      {
         // if selection mask aktive
         if(pc_SelectionMask != NULL)
         {
            if(! pc_SelectionMask->Get(i_RunX,i_RunY) )
            {
               // and point not selected, then do no calculation
               continue;
            }
         }

         i_RangeFromX = i_RunX - m_i_FilterWidth;
         if(i_RangeFromX <0)
         {
            i_RangeFromX =0;
         }
         i_RangeToX = i_RunX + m_i_FilterWidth;
         if(i_RangeToX >= i_Width)
         {
            i_RangeToX = i_Width-1;
         }

         // calculate the total points in the range
         i_RangeTotalPoints=   (i_RangeToX - i_RangeFromX +1)
                              *(i_RangeToY - i_RangeFromY +1);

         // set sums to zero
         i_RedSum  = 0;
         i_GreenSum= 0;
         i_BlueSum = 0;

         // in some modes the original is needed
         if(   m_b_NearEnabled
            || m_b_OnlyOneColorActive)
         {
            p_Original = m_pc_Copy->Get(i_RunX,i_RunY);
         }

         // is filter for only one color active?
         if(m_b_OnlyOneColorActive)
         {
            int i_ActualDistance;
            // is the distance to this color in the range
            i_ActualDistance = Utilities::PixelDistance(m_p_OnlyOneColor,
                                                        p_Original);
            if(i_ActualDistance > m_i_OnlyOneColorDistance)
            {
               // do nothing more for this point
               continue;
            }
         }

         // inner loop - calculate the pixelsumms
         for(i_InnerY =  i_RangeFromY;
             i_InnerY <= i_RangeToY;
             i_InnerY ++)
         {
            for(i_InnerX =  i_RangeFromX;
                i_InnerX <= i_RangeToX;
                i_InnerX ++)
            {
               p_Value = m_pc_Copy->Get(i_InnerX,i_InnerY);

               if(m_b_NearEnabled)
               {
                  int i_ActualDistance;
                  i_ActualDistance = Utilities::PixelDistance(p_Value,p_Original);
                  if( i_ActualDistance > m_i_NearDistance)
                  {
                     // forget this point
                     i_RangeTotalPoints -= 1;
                     continue;
                  }

               }

               i_RedSum   += PIXEL_RED  (p_Value);
               i_GreenSum += PIXEL_GREEN(p_Value);
               i_BlueSum  += PIXEL_BLUE (p_Value);

            } // for i_InnerX
         } // for i_InnerY

         if(i_RangeTotalPoints >= 1)
         {
            // divide summ by number of pixels
            i_RedSum   /= i_RangeTotalPoints;
            i_GreenSum /= i_RangeTotalPoints;
            i_BlueSum  /= i_RangeTotalPoints;
            // clipp maximum to 255
            if(i_RedSum>255)
            {
               i_RedSum = 255;
            }
            if(i_GreenSum>255)
            {
               i_GreenSum = 255;
            }
            if(i_BlueSum>255)
            {
               i_BlueSum = 255;
            }

            // compute the new pixel
            p_Value = PIXEL_MAKE(i_RedSum,i_GreenSum,i_BlueSum);

            // set the new pixel into the image
            m_pc_Original->Set(i_RunX,i_RunY,p_Value);
         } // if totalpoints >= 1
      } // for i_Run_X
   } // for i_Run_Y

   // clean up the processing
   CleanUp();

} // DoFilterMiddle()

//============================================================================
//
//============================================================================

void Filter::DoFilterMedian()
{
   PixelBuffer *pc_SelectionMask;
   RectRegion   c_MaskRegion;
   Pixel        p_Value;
   Pixel        p_Original;
   int i_RunX;
   int i_RunY;
   int i_Width;
   int i_Height;

   int i_FromY;
   int i_FromX;
   int i_ToX;
   int i_ToY;

   int i_RangeFromY;
   int i_RangeFromX;
   int i_RangeToX;
   int i_RangeToY;
   int i_RangeTotalPoints;

   int i_InnerX;
   int i_InnerY;

   int i_MaximumPoints;

   unsigned char *p_ArrayRed;
   unsigned char *p_ArrayGreen;
   unsigned char *p_ArrayBlue;


   // create a copy of the buffer
   CreateCopy();

   // calculate maximum points for array allocation
   i_MaximumPoints = m_i_FilterWidth*2+1;
   i_MaximumPoints *= i_MaximumPoints;



   i_Width  = m_pc_Original->GetWidth();
   i_Height = m_pc_Original->GetHeight();

   i_FromX = 0;
   i_FromY = 0;
   i_ToX   = i_Width   -1;
   i_ToY   = i_Height  -1;

   pc_SelectionMask = NULL;

   if(m_pc_Original->GetSelectionMaskActive() )
   {

      pc_SelectionMask = m_pc_Copy->GetSelectionMask();
      c_MaskRegion = pc_SelectionMask->GetUsedRegion(1);

      if(c_MaskRegion.IsEmpty())
      {
         // empty mask - do nothing
         // clean up the processing
         CleanUp();
         return;
      }

      i_FromX = c_MaskRegion.GetStartX();
      i_FromY = c_MaskRegion.GetStartY();
      i_ToX   = c_MaskRegion.GetEndX();
      i_ToY   = c_MaskRegion.GetEndY();
   }

   // allocate 3 arrays for the sorting vector
   p_ArrayRed    = new unsigned char[i_MaximumPoints];
   p_ArrayGreen  = new unsigned char[i_MaximumPoints];
   p_ArrayBlue   = new unsigned char[i_MaximumPoints];

   // test, whether the array are valid
   if(   p_ArrayRed   == NULL
      || p_ArrayGreen == NULL
      || p_ArrayBlue  == NULL)
   {
      // one allocation has failed
      if(p_ArrayRed != NULL)
      {
         delete p_ArrayRed;
      }
      if(p_ArrayGreen != NULL)
      {
         delete p_ArrayGreen;
      }
      if(p_ArrayBlue != NULL)
      {
         delete p_ArrayBlue;
      }

      CleanUp();
      return ;
   } // if array allocation failed

   for(i_RunY=i_FromY; i_RunY <= i_ToY;i_RunY ++)
   {
      i_RangeFromY = i_RunY - m_i_FilterWidth;
      if(i_RangeFromY <0)
      {
         i_RangeFromY =0;
      }
      i_RangeToY = i_RunY + m_i_FilterWidth;
      if(i_RangeToY >=i_Height)
      {
         i_RangeToY = i_Height-1;
      }

      for(i_RunX=i_FromX; i_RunX <= i_ToX;i_RunX ++)
      {
         int i_ActualPoint;
         int i_Red;
         int i_Green;
         int i_Blue;

         // if selection mask aktive
         if(pc_SelectionMask != NULL)
         {
            if(! pc_SelectionMask->Get(i_RunX,i_RunY) )
            {
               // and point not selected, then do no calculation
               continue;
            }
         }

         i_RangeFromX = i_RunX - m_i_FilterWidth;
         if(i_RangeFromX <0)
         {
            i_RangeFromX =0;
         }
         i_RangeToX = i_RunX + m_i_FilterWidth;
         if(i_RangeToX >= i_Width)
         {
            i_RangeToX = i_Width-1;
         }

         // in some modes the original is needed
         if(   m_b_NearEnabled
            || m_b_OnlyOneColorActive)
         {
            p_Original = m_pc_Copy->Get(i_RunX,i_RunY);
         }

         // is filter for only one color active?
         if(m_b_OnlyOneColorActive)
         {
            int i_ActualDistance;
            // is the distance to this color in the range
            i_ActualDistance = Utilities::PixelDistance(m_p_OnlyOneColor,
                                                        p_Original);
            if(i_ActualDistance > m_i_OnlyOneColorDistance)
            {
               // do nothing more for this point
               continue;
            }
         }

         // calculate the total points in the range
         i_RangeTotalPoints=   (i_RangeToX - i_RangeFromX +1)
                              *(i_RangeToY - i_RangeFromY +1);

         // start in the begin of the array
         i_ActualPoint =0;

         // inner loop - calculate the pixelsumms
         for(i_InnerY =  i_RangeFromY;
             i_InnerY <= i_RangeToY;
             i_InnerY ++)
         {
            for(i_InnerX =  i_RangeFromX;
                i_InnerX <= i_RangeToX;
                i_InnerX ++)
            {
               p_Value = m_pc_Copy->Get(i_InnerX,i_InnerY);

               if(m_b_NearEnabled)
               {
                  int i_ActualDistance;
                  i_ActualDistance = Utilities::PixelDistance(p_Value,p_Original);
                  if( i_ActualDistance > m_i_NearDistance)
                  {
                     // forget this point
                     continue;
                  }
               }

               p_ArrayRed  [i_ActualPoint] = (unsigned char) PIXEL_RED  (p_Value);
               p_ArrayGreen[i_ActualPoint] = (unsigned char) PIXEL_GREEN(p_Value);
               p_ArrayBlue [i_ActualPoint] = (unsigned char) PIXEL_BLUE (p_Value);

               // go to next array element
               i_ActualPoint += 1;

            } // for i_InnerX
         } // for i_InnerY

         // middle pixel makes only sense, if we have >=3 pixel
         if(i_ActualPoint > 2)
         {
            // now sort the array
            Utilities::QSortBytes(p_ArrayRed  ,0,i_ActualPoint -1);
            Utilities::QSortBytes(p_ArrayGreen,0,i_ActualPoint -1);
            Utilities::QSortBytes(p_ArrayBlue ,0,i_ActualPoint -1);

            // take the point in the middle
            i_ActualPoint /=2;
            i_Red   = p_ArrayRed[i_ActualPoint];
            i_Green = p_ArrayGreen[i_ActualPoint];
            i_Blue  = p_ArrayBlue[i_ActualPoint];

            // compute the new pixel
            p_Value = PIXEL_MAKE(i_Red,i_Green,i_Blue);

            // set the new pixel into the image
            m_pc_Original->Set(i_RunX,i_RunY,p_Value);
         } // if we have at least 3 pixel
      } // for i_Run_X
   } // for i_Run_Y

   // delete the allocated arrays
   delete p_ArrayRed;
   delete p_ArrayGreen;
   delete p_ArrayBlue;

   // clean up the processing
   CleanUp();

} // DoFilterMedian()

//============================================================================
//
//============================================================================

void Filter::CreateCopy()
{
   // need at least the original image
   if(m_pc_Original == NULL)
   {
      return;
   }

   // is the undo - picture available
   m_pc_Copy=NULL;
   if(m_pc_GlobalStorage->m_pc_Undo != NULL)
   {
      m_pc_Copy = m_pc_GlobalStorage->m_pc_Undo->GetActualPicture();
   }

   // if the undo copy is there
   if(m_pc_Copy != NULL)
   {
      // compareOnIndentity
      bool b_Identical;

      b_Identical = m_pc_Copy->CompareBuffer(m_pc_Original,false);

      if(! b_Identical)
      {
         m_pc_Copy = NULL;
      }
      else
      {
         m_b_CopySelfAllocated = false;
      }
   }

   // nothing copied until now?
   if(m_pc_Copy == NULL)
   {
      // then duplicate the buffer
      m_pc_Copy = new PixelBuffer(m_pc_Original);
      m_b_CopySelfAllocated = true;
   }

} // CreateCopy()

//============================================================================
//
//============================================================================

void Filter::CleanUp()
{
   // after the filter the copy is not more needed
   if(m_pc_Copy != NULL)
   {
      // if self allocated
      if(m_b_CopySelfAllocated)
      {
         // then delete it
         delete m_pc_Copy;

      }
      m_pc_Copy = NULL;
   }
   m_b_CopySelfAllocated = false;

} // CleanUp()

//============================================================================
//
//============================================================================

void Filter::SetNearDistance(int i_Distance)
{
   m_i_NearDistance = i_Distance;
} // SetNearDistance()

//============================================================================
//
//============================================================================

void Filter::SetNearMode(bool b_On)
{
   m_b_NearEnabled = b_On;
} // SetNearMode()

//============================================================================
//
//============================================================================

void Filter::SetOnlyOneMode(bool b_On)
{
   m_b_OnlyOneColorActive = b_On;
} // SetOnlyOneMode()

//============================================================================
//
//============================================================================

void Filter::SetOnlyOneColor(Pixel p_Color)
{
   m_p_OnlyOneColor = p_Color;
} // SetOnlyOneColor()

//============================================================================
//
//============================================================================

void Filter::SetOnlyOneColorDistance(int i_Distance)
{
   m_i_OnlyOneColorDistance = i_Distance;
} //SetOnlyOneColorDistance()

//============================================================================
//
//============================================================================



