
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>

#include "FilterBox.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Utilities.h"

#include "QULabelSlider.h"


FilterBox::FilterBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global),
                Tracer("FilterBox")
{
   QBoxLayout    *l_Layout;

   QVGroupBox   *FilterStyleBox;
   QHGroupBox   *ActivationBox;

   setCaption("Filter");
   TRACE << "Filter Started\n" ;

   m_i_FilterWidth          = 1;
   m_i_NearDistance         = 30;
   m_b_NearEnabled          = false;

   m_b_OnlyOneColorActive   = false;
   m_i_OnlyOneColorDistance = 30;
   m_p_OnlyOneColor         = 0x564855;


   m_pc_GlobalStorage = pc_Global;

   m_pc_Filter = new Filter(pc_Global);

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);


   // box for Filter style
   FilterStyleBox = new QVGroupBox("Filter Style",m_w_TopWidget);
   l_Layout->addWidget(FilterStyleBox);

   {
      m_FilterWidthSlider = new QULabelSlider(FilterStyleBox,true,
                                       0,15,
                                       m_i_FilterWidth,
                                       "Width",6);

      connect( m_FilterWidthSlider,
               SIGNAL(valueChanged(int)),
               SLOT( FilterWidthChanged(int )) );


   } // box for Filter Settings

   m_OnlyOneColorBox = new QVGroupBox("Filter only one Color",m_w_TopWidget);
   l_Layout->addWidget(m_OnlyOneColorBox);
   {

      m_OnlyOneColorCheck = new QCheckBox("Active",
                                              m_OnlyOneColorBox);

      m_OnlyOneColorButton = new QUColorDragLabeledWidget(m_OnlyOneColorBox,
                                               "Color to filter",
                                               QUColorDragWidget::ModePipette,
                                               m_pc_GlobalStorage);
      m_OnlyOneColorButton->SetColor(m_p_OnlyOneColor);

      m_OnlyOneColorDistanceSlider = new QULabelSlider(m_OnlyOneColorBox,
                                       true,
                                       0,100,
                                       m_i_OnlyOneColorDistance,
                                       "Distance",6);
      connect( m_OnlyOneColorCheck ,
               SIGNAL(toggled(bool)),
               SLOT( FilterOnlyOneClicked(bool) ) );
      connect( m_OnlyOneColorButton,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( OnlyOneColorChanged(Pixel ) )
             );
      connect( m_OnlyOneColorDistanceSlider,
               SIGNAL(valueChanged(int)),
               SLOT( OnlyOneDistanceChanged(int )) );


   } // box for nearColor selection

   
   m_NearBox = new QVGroupBox("Extended Option",m_w_TopWidget);
   l_Layout->addWidget(m_NearBox);
   {

      m_OnlyFilterNearColorsCheck = new QCheckBox("Include only near colors",
                                              m_NearBox);

      m_NearDistanceSlider = new QULabelSlider(m_NearBox,true,
                                       0,100,
                                       m_i_NearDistance,
                                       "Distance",6);
      connect( m_OnlyFilterNearColorsCheck ,
               SIGNAL(toggled(bool)),
               SLOT( FilterOnlyNearClicked(bool) ) );
      connect( m_NearDistanceSlider,
               SIGNAL(valueChanged(int)),
               SLOT( NearDistanceChanged(int )) );


   } // box for nearColor selection

   // --- box for activation
   ActivationBox = new QHGroupBox("Perform Filter",m_w_TopWidget);
   l_Layout->addWidget(ActivationBox);
   {
      // buttons to establish painting mode

      m_MiddleButton  = new QPushButton("Middle",ActivationBox);
      m_MedianButton  = new QPushButton("Median",ActivationBox);

      connect( m_MiddleButton ,
               SIGNAL(clicked()),
               SLOT( ActivateMiddle() ) );

      connect( m_MedianButton ,
               SIGNAL(clicked()),
               SLOT( ActivateMedian() ) );
   }

} // FilterBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void FilterBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()


//===========================================================================
//
//===========================================================================

void FilterBox::FilterWidthChanged(int i_Width)
{

   m_i_FilterWidth = i_Width;

} // FilterWidthChanged()

//===========================================================================
//
//===========================================================================

void FilterBox::ActivateMiddle()
{
   // initialize structures
   PrepareFiltering(Filter::FilterMiddle);

   // do filtering
   m_pc_Filter->DoFilter();

   // cleanup, redraw and undo
   EndFiltering();
} // ActivateMiddle()

//===========================================================================
//
//===========================================================================

void FilterBox::ActivateMedian()
{
   // initialize structures
   PrepareFiltering(Filter::FilterMedian);

   // do filtering
   m_pc_Filter->DoFilter();

   // cleanup, redraw and undo
   EndFiltering();
} // ActivateMedian()

//===========================================================================
//
//===========================================================================

void FilterBox::PrepareFiltering(int i_FilterType)
{
   // set cursor in waiting mode
   QtUtility::StartWaitingCursor();

   // transfer all stati into filter object
   m_pc_Filter->SetFilterType(i_FilterType);
   m_pc_Filter->SetFilterWidth(m_i_FilterWidth);
   m_pc_Filter->SetOriginalBuffer(m_pc_GlobalStorage->m_pc_MainImage);
   m_pc_Filter->SetNearDistance(m_i_NearDistance);
   m_pc_Filter->SetNearMode(m_b_NearEnabled);
   m_pc_Filter->SetOnlyOneMode(m_b_OnlyOneColorActive);
   m_pc_Filter->SetOnlyOneColor(m_p_OnlyOneColor);
   m_pc_Filter->SetOnlyOneColorDistance(m_i_OnlyOneColorDistance);

} // PrepareFiltering

//===========================================================================
//
//===========================================================================

void FilterBox::EndFiltering()
{
   // force a global redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   // send image changed message for undo processing
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   // cursor back to normal
   QtUtility::EndWaitingCursor();
} // EndFiltering()

//===========================================================================
//
//===========================================================================

void FilterBox::NearDistanceChanged(int i_Distance)
{
   m_i_NearDistance = i_Distance;
} // NearDistanceChanged()

//===========================================================================
//
//===========================================================================

void FilterBox::FilterOnlyNearClicked(bool b_On)
{
   m_b_NearEnabled = b_On;
} // FilterOnlyNearClicked()

//===========================================================================
//
//===========================================================================

void FilterBox::OnlyOneDistanceChanged(int i_Distance)
{
   m_i_OnlyOneColorDistance = i_Distance;
} // OnlyOneDistanceChanged()

//===========================================================================
//
//===========================================================================

void FilterBox::FilterOnlyOneClicked(bool b_On)
{
   m_b_OnlyOneColorActive = b_On;
} // FilterOnlyOneClicked()

//===========================================================================
//
//===========================================================================

void FilterBox::OnlyOneColorChanged(Pixel p_NewColor)
{
   m_p_OnlyOneColor = p_NewColor;
} // OnlyOneColorChanged()

//===========================================================================
//
//===========================================================================

