#ifndef _GEOMETRY_H
#define _GEOMETRY_H

#include <qmemarray.h>

// utility class to work with geometrical coordinates and list
// of coordinates

#include "RectRegion.h"

class Geometry
{
public:
   static double AngleRadianToDegree(double d_RadianAngle);
   static double AngleDegreeToRadian(double d_DegreeAngle);
};


class GeoCoordinate
{
   // allow RectRegion to access my private members
   friend class RectRegion;

public:
   GeoCoordinate();
   GeoCoordinate(float f_X, float f_Y);
   ~GeoCoordinate();

   void SetXY(float f_X, float f_Y);
   void SetX( float f_X);
   void SetY( float f_Y);
   
   float GetX();
   float GetY();

   void Transform(float f_DifferenceX, float f_DifferenceY);

   float DifferenceTo(GeoCoordinate c_Other);
   float DifferenceTo(float f_X, float f_Y);
   float AngleTo(float f_X, float f_Y);
   float AngleTo(GeoCoordinate c_Other);

private:
   float m_f_X;
   float m_f_Y;

}; // GeoCoordinate



class GeoCoordinateList
{

public:
   GeoCoordinateList();

   int GetCount();
   int AddElement(GeoCoordinate c_Element);
   int InsertElement(GeoCoordinate c_Element,int i_Position);

   GeoCoordinate Get(int i_Index);

   void Transform(float f_DifferenceX, float f_DifferenceY);





   QMemArray<GeoCoordinate> m_CoordinateList;
private:
   int m_i_NumberOfElements;

}; // GeoCoordinateList


class GeoLine
{
public:
   GeoLine();
   GeoLine(float f_StartX,float f_StartY,float f_EndX,float f_EndY);
   GeoLine(GeoCoordinate c_Start,GeoCoordinate c_End);

   void Set(float f_StartX,float f_StartY,float f_EndX,float f_EndY);
   void Set(GeoCoordinate c_Start,GeoCoordinate c_End);


   GeoCoordinate GetStartPoint();
   GeoCoordinate GetEndPoint();
   void Get( float &f_StartX, float &f_StartY, float &f_EndX,float &f_EndY);

   void Transform(float f_DifferenceX, float f_DifferenceY);

   bool CutsLine(GeoLine c_SecondLine, GeoCoordinate &c_CutPoint);
   
   bool ClipInRectRegion(RectRegion c_Region,GeoLine &c_ResultLine);

private:
   GeoCoordinate m_Start;
   GeoCoordinate m_End;

   float CalcDeterminante(float x1,float y1,float x2,float y2);
   
   bool ClipInRectRegionHorizontal(RectRegion c_Region,GeoLine &c_ResultLine);

   bool ClipInRectRegionVertical(RectRegion c_Region,GeoLine &c_ResultLine);

} ; // GeoLine


#endif  // _GEOMETRY_H

