
#include <stdio.h>

#include "GraphicObject.h"

//===========================================================================
//
//===========================================================================

GraphicObject::GraphicObject()
{

   // no group
   m_b_IsGroup = false;
   // and no childs
   m_ChildObjects= NULL;

   // default : not selected
   m_b_IsSelected = false;

   // no type so far
   m_b_ObjectType = TypeInvalid;
   
   // initital no chached size
   m_b_CachedSizeValid = false;
   
   // set some default object attributes
   m_b_IsResizeable  = false;
   m_b_IsSelectable = true;
   m_b_IsVisible     = true;


} // GraphicObject()

//===========================================================================
//
//===========================================================================

GraphicObject::~GraphicObject()
{
   if(m_ChildObjects != NULL)
   {
      delete m_ChildObjects;
   }
} // ~GraphicObject()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObject::GetAttribute(int e_AttributeType)
{
   return m_Attributes.GetAttribute( e_AttributeType);

} // GetAttribute()

//===========================================================================
//
//===========================================================================

int GraphicObject::GetObjectType()
{
   return m_b_ObjectType;

} // GetObjectType()

//===========================================================================
//
//===========================================================================

void GraphicObject::MoveTo( float f_OriginX, float f_OriginY)
{
} // MoveTo

//===========================================================================
//
//===========================================================================

void GraphicObject::MoveOffset( float f_OffsetX, float f_OffsetY)
{
} // MoveOffset()

//===========================================================================
//
//===========================================================================

void GraphicObject::SetColor(Pixel p_Color)
{
   GraphicObjectAttribute *pc_ColorAttribute;
   // look whether available
   pc_ColorAttribute = GetAttribute(GraphicObjectAttribute::Color);
   if(pc_ColorAttribute == NULL)
   {
      // create new
      pc_ColorAttribute = new GraphicObjectAttribute;
      pc_ColorAttribute->m_e_ObjectAttributeType = GraphicObjectAttribute::Color;
      pc_ColorAttribute->m_p_Color = p_Color;
      // add to list
      m_Attributes.AddAttribute(pc_ColorAttribute);
   }
   else
   {
      // modify existing
      pc_ColorAttribute->m_p_Color = p_Color;
   }
} // SetColor()

//===========================================================================
//
//===========================================================================

Pixel GraphicObject::GetColor()
{
   GraphicObjectAttribute *pc_ColorAttribute;
   // look whether available
   pc_ColorAttribute = GetAttribute(GraphicObjectAttribute::Color);

   if(pc_ColorAttribute == NULL)
   {
      return 0;
   }

   return pc_ColorAttribute->m_p_Color;

} // GetColor()

//===========================================================================
//
//===========================================================================

RectRegion GraphicObject::GetSize()
{
   if( m_b_CachedSizeValid)
   {
      return m_CachedSize;
   }

   m_CachedSize.SetEmpty();
   m_b_CachedSizeValid = true;
   return m_CachedSize;

} // GetSize()

//===========================================================================
//
//===========================================================================

void GraphicObject::SetSize(RectRegion c_NewSize)
{
   // do nothing on a empty object.
   // has to be overloaded
} // SetSize()

//===========================================================================
//
//===========================================================================

bool GraphicObject::IsSelected()
{
   return m_b_IsSelected;
} // IsSelected()

//===========================================================================
//
//===========================================================================

void GraphicObject::SelectObject(bool b_State)
{
   m_b_IsSelected = b_State;
} // SelectObject()

//===========================================================================
//
//===========================================================================

bool GraphicObject::IsResizeable()
{
   return m_b_IsResizeable;
} // IsResizeable()

//===========================================================================
//
//===========================================================================

bool GraphicObject::IsSelectable()
{
   return m_b_IsSelectable;
} // IsSelectable()

//===========================================================================
//
//===========================================================================


void GraphicObject::SetSelectable(bool b_Selectable)
{
   m_b_IsSelectable = b_Selectable;
} // SetSelectable()

//===========================================================================
//
//===========================================================================

bool GraphicObject::IsVisible()
{
   return m_b_IsVisible;
} // IsVisible()

//===========================================================================
//
//===========================================================================

void GraphicObject::SetVisible(bool b_Visible)
{
   m_b_IsVisible = b_Visible;
} // SetVisible()

//===========================================================================
//###########################################################################
//===========================================================================

GraphicObjectList::GraphicObjectList(bool b_SetAutoDelete)
{
   m_ObjectList.setAutoDelete( b_SetAutoDelete );

} // GraphicObjectList()

//===========================================================================
//
//===========================================================================

void GraphicObjectList::AddObject(GraphicObject *pc_Object)
{
   m_ObjectList.append(pc_Object);

} // AddObject()

//===========================================================================
//
//===========================================================================

int  GraphicObjectList::GetCount()
{
   return m_ObjectList.count();

} // GetGount()

//===========================================================================
//
//===========================================================================

GraphicObject *GraphicObjectList::GetObject(int i_Index)
{
   return m_ObjectList.at(i_Index);

} // GetObject()

//===========================================================================
//
//===========================================================================

void GraphicObjectList::DeleteObject(GraphicObject *pc_ObjectToRemove)
{
   m_ObjectList.remove(pc_ObjectToRemove);
} // DeleteObject()

//===========================================================================
//
//===========================================================================

void GraphicObjectList::DeleteAllObjects()
{
   m_ObjectList.clear();
} // DeleteAllObjects()

//===========================================================================
//
//===========================================================================

