#ifndef _GRAPHIC_OBJECT_H
#define _GRAPHIC_OBJECT_H

#include <qptrlist.h>
#include <qstring.h>

#include "PTypes.h"
#include "Geometry.h"

#include "GlobalStorage.h"
#include "RectRegion.h"

#include "GraphicObjectAttribute.h"

//#########################################################################

class GraphicObjectList;

class GraphicObject
{
public:
   GraphicObject();
   virtual ~GraphicObject();
   GraphicObjectAttribute *GetAttribute(int e_AttributeType);

   // different types of objects
   enum
   {
       TypeInvalid =0,
       TypePoint,
       TypeLine,
       TypeRectangle,
       TypeEllipse,
       TypeImage,
       TypeGroup
   };


   int GetObjectType();
   virtual void SetColor(Pixel p_Color);
   virtual Pixel GetColor();

   virtual bool IsResizeable();
   virtual bool IsSelected();
   virtual void SelectObject(bool b_State);
   
   virtual bool IsSelectable();
   virtual bool IsVisible();
   virtual void SetSelectable(bool b_Selectable);
   virtual void SetVisible(bool b_Visible);

   virtual void MoveTo( float f_OriginX, float f_OriginY);
   virtual void MoveOffset( float f_OffsetX, float f_OffsetY);
   virtual RectRegion GetSize();
   virtual void SetSize(RectRegion c_NewSize);

protected:

   int m_b_ObjectType;
   GraphicObjectAttributeList m_Attributes;
   bool m_b_IsGroup;
   bool m_b_IsSelected;
   bool m_b_IsResizeable;
   bool m_b_IsSelectable;
   bool m_b_IsVisible;


   GraphicObjectList *m_ChildObjects;

   RectRegion m_CachedSize;
   bool       m_b_CachedSizeValid;
};

//#########################################################################
//#########################################################################


class GraphicObjectList
{
public :
   GraphicObjectList(bool SetAutoDelete=true);
   void AddObject(GraphicObject *pc_Object);
   int  GetCount();
   GraphicObject *GetObject(int i_Index);

   void DeleteObject(GraphicObject *pc_ObjectToRemove);
   void DeleteAllObjects();

private:
	QPtrList<GraphicObject> m_ObjectList;

};

#endif //  _GRAPHIC_OBJECT_H

