
#include "GraphicObjectAttribute.h"

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute::GraphicObjectAttribute()
{
   m_e_ObjectAttributeType = Invalid;
   m_pc_ImageData          = NULL;
} // GraphicObjectAttribute()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute::~GraphicObjectAttribute()
{
   if( m_pc_ImageData != NULL)
   {
      delete m_pc_ImageData;
      m_pc_ImageData = NULL;
   }
} // ~GraphicObjectAttribute()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeStartpoint(
                                 GeoCoordinate c_Coordinate)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = StartPoint;
   p_Return->m_c_Coordinate        = c_Coordinate;

   return p_Return;

} // GraphicAttributeStartpoint()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeEndpoint(
                                 GeoCoordinate c_Coordinate)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = EndPoint;
   p_Return->m_c_Coordinate        = c_Coordinate;

   return p_Return;

} // GraphicAttributeEndpoint()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeColor(
                                 Pixel p_TheColor)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = Color;
   p_Return->m_p_Color               = p_TheColor;

   return p_Return;

} // GraphicAttributeColor()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeRotationAngel(
                                 double d_Angle)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = RotationAngle;
   p_Return->m_d_Value = d_Angle;
   
   return p_Return;

} //  GraphicAttributeRorationAngel()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeImage( PixelBuffer *pc_Image)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = Image;
   p_Return->m_pc_ImageData = pc_Image;

   return p_Return;

} // GraphicAttributeImage();

//===========================================================================
//
//===========================================================================

Pixel GraphicObjectAttribute::GetColor()
{
   if( m_e_ObjectAttributeType != Color)
   {
      return 0;
   }
   return m_p_Color;

} // GetColor()

//===========================================================================
//
//===========================================================================

void  GraphicObjectAttribute::SetColor(Pixel p_Color )
{
   if( m_e_ObjectAttributeType != Color)
   {
      return ;
   }
   m_p_Color = p_Color;
} //  SetColor()

//===========================================================================
//
//===========================================================================

double GraphicObjectAttribute::GetRotationAngle()
{
   if( m_e_ObjectAttributeType != RotationAngle)
   {
      return 0.0;
   }
   return m_d_Value ;

} // GetRotationAngle()

//===========================================================================
//
//===========================================================================

void   GraphicObjectAttribute::SetRotationAngle(double d_Angle)
{
   if( m_e_ObjectAttributeType != RotationAngle)
   {
      return;
   }
   m_d_Value = d_Angle;
   
} // SetRotationAngle()

//===========================================================================
//
//===========================================================================

GeoCoordinate GraphicObjectAttribute::GetStartPoint()
{
   if(m_e_ObjectAttributeType != StartPoint)
   {
      return GeoCoordinate(0,0);
   }
   return m_c_Coordinate;

} // GetStartPoint()

//===========================================================================
//
//===========================================================================

GeoCoordinate GraphicObjectAttribute::GetEndPoint()
{
   if(m_e_ObjectAttributeType != EndPoint)
   {
      return GeoCoordinate(0,0);
   }
   return m_c_Coordinate;

} // GetEndPoint()

//===========================================================================
//###########################################################################
//###########################################################################
//###########################################################################
//===========================================================================

GraphicObjectAttributeList::GraphicObjectAttributeList()
{
   m_AttributeList.setAutoDelete( true );

} // GraphicObjectAttributeList()

//===========================================================================
//
//===========================================================================

GraphicObjectAttributeList::~GraphicObjectAttributeList()
{
} // ~GraphicObjectAttributeList()

//===========================================================================
//
//===========================================================================

void GraphicObjectAttributeList::AddAttribute(GraphicObjectAttribute *pc_Attribute)
{
   m_AttributeList.append(pc_Attribute);

} // AddAttribute()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttributeList::GetAttribute(int e_AttributeType)
{
   int i_Number;
   int i;

   i_Number = m_AttributeList.count();
   for(i=0;i< i_Number; i++)
   {
      GraphicObjectAttribute *pc_ActualAttribute;

      pc_ActualAttribute = m_AttributeList.at(i);
      if(e_AttributeType == pc_ActualAttribute->m_e_ObjectAttributeType)
      {
         return pc_ActualAttribute;
      }
   } // for i

   // nothing found
   return NULL;

} // GetAttribute()

//===========================================================================
//
//===========================================================================

