
#include <stdio.h>

#include "GraphicObjectEllipse.h"

//===========================================================================
//
//===========================================================================

GraphicObjectEllipse :: GraphicObjectEllipse(float f_StartX, float f_StartY,
                                             float f_EndX, float f_EndY)
   				:GraphicObject()
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;

   // store the object type
   m_b_ObjectType = TypeEllipse;

   m_StartPoint = new GraphicObjectAttribute();
   m_EndPoint   = new GraphicObjectAttribute();

   if(    m_StartPoint == NULL
       || m_EndPoint   == NULL
     )
   {
      return ;
   }

   // create a startpoint
   m_StartPoint->m_e_ObjectAttributeType = GraphicObjectAttribute::StartPoint;

   m_StartPoint->m_c_Coordinate.SetXY(f_StartX, f_StartY);

   m_Attributes.AddAttribute(m_StartPoint);

   // create a endpoint
   m_EndPoint->m_e_ObjectAttributeType = GraphicObjectAttribute::EndPoint;

   m_EndPoint->m_c_Coordinate.SetXY(f_EndX, f_EndY);

   m_Attributes.AddAttribute(m_EndPoint);

   m_b_IsResizeable = true;

   // currently no other attributes

} // GraphicObjectEllipse()

//===========================================================================
//
//===========================================================================

RectRegion GraphicObjectEllipse::GetSize()
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;
   GeoCoordinate            c_StartPosition;
   GeoCoordinate            c_EndPosition;

   if( m_b_CachedSizeValid)
   {
      return m_CachedSize;
   }

   m_CachedSize.SetEmpty();

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return m_CachedSize;
   }
   c_StartPosition= m_StartPoint->m_c_Coordinate;

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return m_CachedSize;
   }
   c_EndPosition= m_EndPoint->m_c_Coordinate;

   m_CachedSize.SetXYXY(c_StartPosition.GetX(),
                    c_StartPosition.GetY(),
                    c_EndPosition.GetX(),
                    c_EndPosition.GetY()
                    );

   m_b_CachedSizeValid = true;

   return m_CachedSize;
} // GetSize()

//===========================================================================
//
//===========================================================================

void GraphicObjectEllipse::SetSize(RectRegion c_NewSize)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;

   // disable the cache size
   m_b_CachedSizeValid = false;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return ;
   }

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return ;
   }

   m_StartPoint->m_c_Coordinate.SetXY(c_NewSize.GetStartXFloat(),
                                      c_NewSize.GetStartYFloat());
   m_EndPoint  ->m_c_Coordinate.SetXY(c_NewSize.GetEndXFloat(),
                                      c_NewSize.GetEndYFloat());

} // SetSize()

//===========================================================================
//
//===========================================================================

void GraphicObjectEllipse::MoveTo(float f_OriginX, float f_OriginY)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;
   GeoCoordinate            c_StartPosition;
   GeoCoordinate            c_EndPosition;

   double                   d_OffsetX;
   double                   d_OffsetY;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return;
   }
   c_StartPosition= m_StartPoint->m_c_Coordinate;

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return;
   }
   c_EndPosition= m_EndPoint->m_c_Coordinate;

   d_OffsetX = c_EndPosition.GetX() - c_StartPosition.GetX();
   d_OffsetY = c_EndPosition.GetY() - c_StartPosition.GetY();

   m_StartPoint->m_c_Coordinate.SetXY(f_OriginX, f_OriginY);
   m_EndPoint  ->m_c_Coordinate.SetXY(f_OriginX+d_OffsetX,
                                    f_OriginY+d_OffsetY);

   m_b_CachedSizeValid = false;
} // MoveTo()

//===========================================================================
//
//===========================================================================

void GraphicObjectEllipse::MoveOffset( float f_OffsetX, float f_OffsetY)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;


   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return;
   }

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return;
   }


   m_StartPoint->m_c_Coordinate.Transform(f_OffsetX, f_OffsetY);
   m_EndPoint  ->m_c_Coordinate.Transform(f_OffsetX, f_OffsetY);

   m_b_CachedSizeValid = false;
} // MoveOffset()

//===========================================================================
//
//===========================================================================

