
#include <stdio.h>

#include "GraphicObjectImage.h"

//===========================================================================
//
//===========================================================================

GraphicObjectImage :: GraphicObjectImage(float f_StartX, float f_StartY,
                                         PixelBuffer *pc_Image)
                       :GraphicObject()
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;
   GraphicObjectAttribute * m_RotationAngle;
   GraphicObjectAttribute * m_Image;
   float                    f_EndX;
   float                    f_EndY;

   if(pc_Image ==NULL)
   {
      return;
   }
   
   f_EndX = f_StartX + (pc_Image->GetWidth ()-1);
   f_EndY = f_StartY + (pc_Image->GetHeight()-1);
   
   
   // store the object type
   m_b_ObjectType = TypeImage;

   m_StartPoint    = GraphicObjectAttribute::GraphicAttributeStartpoint(
                  GeoCoordinate(f_StartX, f_StartY));
   m_EndPoint    = GraphicObjectAttribute::GraphicAttributeEndpoint(
                  GeoCoordinate(f_EndX, f_EndY));
   
   m_RotationAngle = GraphicObjectAttribute::GraphicAttributeRotationAngel(
                  0.0);

   m_Image = GraphicObjectAttribute::GraphicAttributeImage(pc_Image);

   if(   m_StartPoint == NULL
      || m_EndPoint   == NULL
      || m_RotationAngle == NULL
      || m_Image         == NULL
      )
   {
      m_b_ObjectType = TypeInvalid;
      return;
   }
   m_Attributes.AddAttribute(m_StartPoint);
   m_Attributes.AddAttribute(m_EndPoint);
   m_Attributes.AddAttribute(m_RotationAngle);
   m_Attributes.AddAttribute(m_Image);
   
   m_b_IsResizeable = true;


   // currently no other attributes

} // GraphicObjectImage()

//===========================================================================
//
//===========================================================================

RectRegion GraphicObjectImage::GetSize()
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;
   GeoCoordinate            c_StartPosition;
   GeoCoordinate            c_EndPosition;

   if( m_b_CachedSizeValid)
   {
      return m_CachedSize;
   }

   m_CachedSize.SetEmpty();

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return m_CachedSize;
   }
   c_StartPosition= m_StartPoint->m_c_Coordinate;

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return m_CachedSize;
   }
   c_EndPosition= m_EndPoint->m_c_Coordinate;

   m_CachedSize.SetXYXY(c_StartPosition.GetX(),
                    c_StartPosition.GetY(),
                    c_EndPosition.GetX(),
                    c_EndPosition.GetY()
                    );

   m_b_CachedSizeValid = true;

   return m_CachedSize;
} // GetSize()

//===========================================================================
//
//===========================================================================

void GraphicObjectImage::SetSize(RectRegion c_NewSize)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;



   // disable the cache size
   m_b_CachedSizeValid = false;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return ;
   }

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return ;
   }

   if(m_StartPoint->m_c_Coordinate.GetX() < m_EndPoint->m_c_Coordinate.GetX())
   {
      if(m_StartPoint->m_c_Coordinate.GetY() < m_EndPoint->m_c_Coordinate.GetY())
      {
         m_StartPoint->m_c_Coordinate.SetXY(c_NewSize.GetStartXFloat(),
                                      c_NewSize.GetStartYFloat());
         m_EndPoint  ->m_c_Coordinate.SetXY(c_NewSize.GetEndXFloat(),
                                      c_NewSize.GetEndYFloat());
      }
      else //startY > endY
      {
         m_StartPoint->m_c_Coordinate.SetXY(c_NewSize.GetStartXFloat(),
                                      c_NewSize.GetEndYFloat());
         m_EndPoint  ->m_c_Coordinate.SetXY(c_NewSize.GetEndXFloat(),
                                      c_NewSize.GetStartYFloat());
      }
   }
   else // startX > endX
   {
      if(m_StartPoint->m_c_Coordinate.GetY() < m_EndPoint->m_c_Coordinate.GetY())
      {
         m_StartPoint->m_c_Coordinate.SetXY(c_NewSize.GetEndXFloat(),
                                      c_NewSize.GetStartYFloat());
         m_EndPoint  ->m_c_Coordinate.SetXY(c_NewSize.GetStartXFloat(),
                                      c_NewSize.GetEndYFloat());
      }
      else //startY > endY
      {
         m_StartPoint->m_c_Coordinate.SetXY(c_NewSize.GetEndXFloat(),
                                      c_NewSize.GetEndYFloat());
         m_EndPoint  ->m_c_Coordinate.SetXY(c_NewSize.GetStartXFloat(),
                                      c_NewSize.GetStartYFloat());
      }
   }

} // SetSize()

//===========================================================================
//
//===========================================================================

void GraphicObjectImage::MoveTo(float f_OriginX, float f_OriginY)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;
   GeoCoordinate            c_StartPosition;
   GeoCoordinate            c_EndPosition;

   double                   d_OffsetX;
   double                   d_OffsetY;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return;
   }
   c_StartPosition= m_StartPoint->m_c_Coordinate;

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return;
   }
   c_EndPosition= m_EndPoint->m_c_Coordinate;

   d_OffsetX = c_EndPosition.GetX() - c_StartPosition.GetX();
   d_OffsetY = c_EndPosition.GetY() - c_StartPosition.GetY();

   m_StartPoint->m_c_Coordinate.SetXY(f_OriginX, f_OriginY);
   m_EndPoint  ->m_c_Coordinate.SetXY(f_OriginX+d_OffsetX,
                                    f_OriginY+d_OffsetY);

   m_b_CachedSizeValid = false;
} // MoveTo()

//===========================================================================
//
//===========================================================================

void GraphicObjectImage::MoveOffset( float f_OffsetX, float f_OffsetY)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return;
   }

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return;
   }

   m_StartPoint->m_c_Coordinate.Transform(f_OffsetX,f_OffsetY);
   m_EndPoint  ->m_c_Coordinate.Transform(f_OffsetX,f_OffsetY);


   m_b_CachedSizeValid = false;

} // MoveOffset()

//===========================================================================
//
//===========================================================================

void GraphicObjectImage::SetPosition(float f_StartX, float f_StartY,
                                    float f_EndX,   float f_EndY   )
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;
   GeoCoordinate            c_StartPosition;
   GeoCoordinate            c_EndPosition;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return;
   }
   c_StartPosition= m_StartPoint->m_c_Coordinate;

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return;
   }
   c_EndPosition= m_EndPoint->m_c_Coordinate;

   m_StartPoint->m_c_Coordinate.SetXY(f_StartX, f_StartY);
   m_EndPoint  ->m_c_Coordinate.SetXY(f_EndX,f_EndY);

   m_b_CachedSizeValid = false;

} // SetPosition()

//===========================================================================
//
//===========================================================================


PixelBuffer *GraphicObjectImage::GetImageData()
{
   GraphicObjectAttribute * pc_Attribute;

   pc_Attribute = GetAttribute(GraphicObjectAttribute::Image);
   if(pc_Attribute ==NULL)
   {
      return NULL;
   }
   return pc_Attribute->m_pc_ImageData;


}  // GetImageData()

//===========================================================================
//
//===========================================================================



