
#include "GraphicObject.h"
#include "GraphicObjectLayer.h"

//===========================================================================
//
//===========================================================================

GraphicObjectLayer::GraphicObjectLayer(QString qs_LayerName)
{
   m_qs_LayerName = qs_LayerName;

   m_b_IsActive = true;

   // allocate the list with all objects
   m_Objects         = new GraphicObjectList(true);

   // allocate the list for the selected objects
   // objects are same pointers into the all objects list!
   // so no autodelete
   m_SelectedObjects = new GraphicObjectList(false);

   // layer is visible by default
   m_b_IsVisible = true;

} // GraphicObjectLayer()

//===========================================================================
//
//===========================================================================

GraphicObjectLayer::~GraphicObjectLayer()
{
   // first delete all objects
   delete m_Objects;
   
} // ~GraphicObjectLayer()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::AddObject(GraphicObject *pc_NewObject)
{
   m_Objects->AddObject(pc_NewObject);

} // AddObject()

//===========================================================================
//
//===========================================================================

int  GraphicObjectLayer::GetObjectCount()
{
   return m_Objects->GetCount();

} // GetObjectGount()

//===========================================================================
//
//===========================================================================

GraphicObject *GraphicObjectLayer::GetObject(int i_Index)
{
   return m_Objects->GetObject(i_Index);

} // GetObject()

//===========================================================================
//
//===========================================================================


bool GraphicObjectLayer::IsVisible()
{
   return m_b_IsVisible;

} // IsVisible()

//===========================================================================
//
//===========================================================================


void GraphicObjectLayer::SetVisible(bool b_Visible)
{
   m_b_IsVisible = b_Visible;

} // SetVisible()

//===========================================================================
//
//===========================================================================

GraphicObject *GraphicObjectLayer::SearchObjectAt(float f_PositionX,
                                                  float f_PositionY,
                                 GraphicObject *pc_StartObject)
{
   GraphicObject * pc_Actual;
   RectRegion      c_ObjectSize;
   int i_NumberOfObjects;
   int i;

   i_NumberOfObjects = m_Objects->GetCount();

   if(pc_StartObject == NULL)
   {
      for(i=0;i<i_NumberOfObjects;i++)
      {
         pc_Actual = m_Objects->GetObject(i);
         if(    pc_Actual->IsSelectable()
             && pc_Actual->IsVisible()    )
         {
            c_ObjectSize = pc_Actual->GetSize();
            if( c_ObjectSize.PointInRegion(f_PositionX,f_PositionY))
            {
               // found an object
               return pc_Actual;
            }
         } // if visible and selectable

      } // for i;

   }
   else // startobject available
   {
      // search from the object to the last, and then from the start
      GraphicObject *pc_Before;
      int            i_ObjectIndex;

      // unused
      i_ObjectIndex = -1;
      pc_Before     = NULL;

      for(i=0;i <i_NumberOfObjects; i++)
      {
         pc_Actual = m_Objects->GetObject(i);
         if(pc_Actual == pc_StartObject)
         {
            i_ObjectIndex =i;
            continue;
         }
         
         // object must be selectable and visible
         if(    !pc_Actual->IsSelectable()
             || !pc_Actual->IsVisible()    )
         {
            continue;
         }

         c_ObjectSize = pc_Actual->GetSize();
         if( c_ObjectSize.PointInRegion(f_PositionX,f_PositionY))
         {
            if(i_ObjectIndex <0)
            {
               // before object
               if(pc_Before == NULL)
               {
                  pc_Before = pc_Actual;
               }
            }
            else
            {
               // found an object behind
               return pc_Actual;
            }
         } // if point in region

      } // for i;

      // found nothing behinde
      if(pc_Before != NULL)
      {
         // but something before.
         return pc_Before;
      }

   } // else startobject available

   // nothing found
   return NULL;
} // SearchObjectAt()

//===========================================================================
//
//===========================================================================

QString GraphicObjectLayer::GetName()
{
   return m_qs_LayerName;
} // GetName()

//===========================================================================
//
//===========================================================================

GraphicObject *GraphicObjectLayer::GetLastSelectedObject()
{
   GraphicObject *pc_LastObject;
   int            i_NumberOfObjects;
   int            i;

   pc_LastObject = NULL;

   i_NumberOfObjects = GetObjectCount();
   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);
      if(pc_Actual -> IsSelected())
      {
         pc_LastObject = pc_Actual;
      } // if is selected
   } // for i

   return pc_LastObject;
} // GetLastSelectedObject()

//===========================================================================
//
//===========================================================================

int GraphicObjectLayer::GetNumberOfSelectedObjects(GraphicObject * & pc_FirstObject)
{
   int  i_NumberOfObjects;
   int  i_SelectedObjects;
   int  i;

   i_SelectedObjects = 0;

   i_NumberOfObjects = GetObjectCount();

   pc_FirstObject    = NULL;

   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);
      if(pc_Actual -> IsSelected())
      {
         i_SelectedObjects += 1;

         if(pc_FirstObject == NULL)
         {
            pc_FirstObject = pc_Actual;
         }
      } // if is selected
   } // for i

   return i_SelectedObjects;

} // GetNumberOfSelectedObjects()

//===========================================================================
//
//===========================================================================

GraphicObject *GraphicObjectLayer::GetNextSelectedObject(
                               GraphicObject *pc_PreviousObject)
{
   bool b_TakeNext;
   int  i_NumberOfObjects;
   int  i;

   i_NumberOfObjects = GetObjectCount();
   b_TakeNext = false;

   // go through all objects
   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);

      // is object selected?
      if(pc_Actual -> IsSelected())
      {
         // signal to take it set??
         if(b_TakeNext)
         {
            // the return it
            return pc_Actual;
         }

         // is it the previous, then signal to take the next
         if(pc_Actual == pc_PreviousObject)
         {
            b_TakeNext = true;
         }
      } // if is selected
   } // for i

   // nothing more found
   return NULL;

} // GetNextSelectedObject()

//===========================================================================
//
//===========================================================================
bool GraphicObjectLayer::CursorOverSelectedObjects(int i_CursorX, int i_CursorY)
{
   RectRegion  c_ObjectSize;
   int         i_NumberOfObjects;
   int         i;


   i_NumberOfObjects = GetObjectCount();

   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);
      if(pc_Actual -> IsSelected())
      {
         c_ObjectSize = pc_Actual->GetSize();
         if(c_ObjectSize.PointInRegion(i_CursorX,i_CursorY))
         {
            return true;
         }

      } // if is selected

   } // for i

   return false;

} // CursorOverSelectedObjects()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::MoveSelectedObjects(int i_DifferenceX,
                         int i_DifferenceY,
                         RectRegion &c_ChangedRegion)
{
   bool b_FirstObject;
   
   b_FirstObject = true;

   RectRegion  c_ObjectSize;
   int         i_NumberOfObjects;
   int         i;

   // get number of objects
   i_NumberOfObjects = GetObjectCount();

   // go through all objects
   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);

      // but move only selected objects
      if(pc_Actual -> IsSelected())
      {
         // add region of unmoved object
         c_ObjectSize = pc_Actual->GetSize();
         if(b_FirstObject)
         {
            b_FirstObject = false;
            c_ChangedRegion = c_ObjectSize;
         }
         else
         {
            c_ChangedRegion.AddRegion(c_ObjectSize);
         }

         pc_Actual->MoveOffset((float) i_DifferenceX,(float) i_DifferenceY);

         // add region of moved object
         c_ObjectSize = pc_Actual->GetSize();
         c_ChangedRegion.AddRegion(c_ObjectSize);

      } // if is selected

   } // for i



} // MoveSelectedObjects()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::DeleteObject(GraphicObject *pc_ObjectToDelete)
{
   // remove object from the list
   m_Objects->DeleteObject(pc_ObjectToDelete);

} // DeleteObject()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::DeleteAllObjects()
{
   // remove object from the list
   m_Objects->DeleteAllObjects();

} // DeleteAllObjects()

//===========================================================================
//###########################################################################
//===========================================================================

GraphicObjectLayerList::GraphicObjectLayerList()
{

} // GraphicObjectLayerList()

//===========================================================================
//
//===========================================================================

GraphicObjectLayerList::~GraphicObjectLayerList()
{
   // should delete the entire layer, because auto-remove
   // is on
   m_ObjectLayerList.setAutoDelete(true);
   m_ObjectLayerList.remove();
} // GraphicObjectLayerList()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayerList::AddLayer(GraphicObjectLayer *pc_Layer)
{
   m_ObjectLayerList.append(pc_Layer);

} // Addlayer()

//===========================================================================
//
//===========================================================================

int GraphicObjectLayerList::GetCount()
{
   return m_ObjectLayerList.count();
} // Addlayer()

//===========================================================================
//
//===========================================================================

GraphicObjectLayer *GraphicObjectLayerList::GetLayer(int i_Index)
{
   return m_ObjectLayerList.at(i_Index);
} // GetLayer()

//===========================================================================
//
//===========================================================================

