#ifndef _GRAPHIC_OBJECT_LAYER_H
#define _GRAPHIC_OBJECT_LAYER_H


#include "GraphicObjectAttribute.h"
#include "GraphicObject.h"
#include "GraphicObjectEllipse.h"
#include "GraphicObjectImage.h"
#include "GraphicObjectLine.h"
#include "GraphicObjectPoint.h"
#include "GraphicObjectRectangle.h"

//#########################################################################

class GraphicObjectLayer
{
public:
   GraphicObjectLayer(QString qs_LayerName);
   ~GraphicObjectLayer();

   void AddObject(GraphicObject *pc_NewObject);
   int  GetObjectCount();
   GraphicObject *GetObject(int i_Index);
   bool IsVisible();
   void SetVisible(bool b_Visible);
   
   GraphicObject *SearchObjectAt(float f_PositionX,float f_PositionY,
                                 GraphicObject *pc_StartObject=NULL);


   GraphicObject *GetLastSelectedObject();                                                                  

   QString GetName();
   
   int GetNumberOfSelectedObjects(GraphicObject * & pc_FirstObject);
   GraphicObject *GetNextSelectedObject(GraphicObject *pc_PreviousObject);
   bool CursorOverSelectedObjects(int i_CursorX, int i_CursorY);
   
   void MoveSelectedObjects(int i_DifferenceX, int i_DifferenceY,
                            RectRegion &c_ChangedRegion);

   void DeleteObject( GraphicObject *pc_ObjectToDelete);
   void DeleteAllObjects( );

private: // variables

   bool m_b_IsActive;
   bool m_b_IsVisible;

   GraphicObjectList *m_Objects;
   GraphicObjectList *m_SelectedObjects;

   // every layer gets a name
   QString m_qs_LayerName;


}; // GraphicObjectLayer


//#########################################################################
//#########################################################################

class GraphicObjectLayerList
{
public :
	GraphicObjectLayerList();
	~GraphicObjectLayerList();
	void AddLayer(GraphicObjectLayer *pc_Layer);
	int GetCount();
	GraphicObjectLayer *GetLayer(int i_Index);

private:
	QPtrList<GraphicObjectLayer> m_ObjectLayerList;

};


#endif //  _GRAPHIC_OBJECT_LAYER_H
