
#include <qcursor.h>

#include "GraphicObjectMain.h"
#include "MouseEvent.h"
#include "Selection.h"
#include "ImageFrame.h"

//===========================================================================
//
//===========================================================================

GraphicObjectMain::GraphicObjectMain(GlobalStorage *pc_Global)
{
   m_pc_GlobalStorage = pc_Global;

   // initital no callback
   m_ObjectEventCallback.Clear();

   // default is invisible
   m_b_Visible = false;

   m_p_WorkColor = 0xeeeeee;

} // GraphicObjectMain()

//===========================================================================
//
//===========================================================================

GraphicObjectMain::~GraphicObjectMain()
{
}  // ~GraphicObjectMain()

//===========================================================================
//
//===========================================================================

void GraphicObjectMain::SetVisible(bool b_Visible)
{
    m_b_Visible = b_Visible;
} // SetVisible()

//===========================================================================
//
//===========================================================================

bool GraphicObjectMain::IsVisible()
{
    return m_b_Visible;

} // IsVisible()

//===========================================================================
//
//===========================================================================

GraphicObjectLayer * GraphicObjectMain::CreateLayer(QString qs_LayerName)
{
   GraphicObjectLayer *pc_NewLayer;

   // create the layer
   pc_NewLayer = new GraphicObjectLayer(qs_LayerName);

   // add it to the list
   m_AllLayer.AddLayer(pc_NewLayer);

   // if the only layer, then set it active
   if(m_AllLayer.GetCount() <=1 )
   {
      m_pc_ActiveInputLayer = pc_NewLayer;
   }

   return pc_NewLayer;
} // CreateLayer()

//===========================================================================
//
//===========================================================================

GraphicObjectLayer *GraphicObjectMain::GetActiveInputLayer()
{
   return m_pc_ActiveInputLayer;

} // GetActiveInputLayer()

//===========================================================================
//
//===========================================================================

GraphicObjectLayer *GraphicObjectMain::GetLayer(int i_Index)
{
   return m_AllLayer.GetLayer(i_Index);

} // GetActiveInputLayer()

//===========================================================================
//
//===========================================================================

int  GraphicObjectMain::GetLayerCount()
{
   return m_AllLayer.GetCount();

} // GetLayerCount()

//===========================================================================
//
//===========================================================================

void GraphicObjectMain::CallEventForSelectedObjects(int i_EventType)
{
   GraphicObjectMainEvent Event;
   int                    i_NumberOfObjects;
   int                    i;

   if(  m_ObjectEventCallback.IsEmpty() )
   {
      return;
   }

   i_NumberOfObjects = m_pc_ActiveInputLayer->GetObjectCount();

   // go through all objects
   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      // get actual
      pc_Actual =  m_pc_ActiveInputLayer->GetObject(i);

      // look, whether it is selected
      if(pc_Actual -> IsSelected())
      {

         Event.i_CallbackType = i_EventType;
         Event.pc_Object      = pc_Actual;
         m_ObjectEventCallback.Execute(&Event);
      } // if is selected
   } // for i

} // CallEventForSelectedObjects()

//===========================================================================
//
//===========================================================================

RectRegion GraphicObjectMain::DeselectAllObjects()
{
   RectRegion c_ReturnRegion;
   int        i_NumberOfObjects;
   int        i;

   c_ReturnRegion.SetEmpty();

   i_NumberOfObjects = m_pc_ActiveInputLayer->GetObjectCount();

   // go through all objects
   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      // get actual
      pc_Actual =  m_pc_ActiveInputLayer->GetObject(i);

      // look, whether it is selected
      if(pc_Actual -> IsSelected())
      {
         RectRegion c_ObjectRegion;

         // remember the region for a possible redraw
         c_ObjectRegion = pc_Actual->GetSize();
         c_ReturnRegion.AddRegion(c_ObjectRegion);

         // deselect the object
         pc_Actual ->SelectObject(false);

         // call the deselect callback
         if( ! m_ObjectEventCallback.IsEmpty() )
         {
            GraphicObjectMainEvent Event;

            Event.i_CallbackType =GraphicObjectMainEvent::Deselected;
            Event.pc_Object = pc_Actual;
            m_ObjectEventCallback.Execute(&Event);
         }

      } // if is selected
   } // for i

   return c_ReturnRegion;
} // DeselectAllObjects()

//===========================================================================
//
//===========================================================================

void *GraphicObjectMain::SelectionEventArrived(void *pc_Data)
{
   GraphicObject *pc_Object;
   RectRegion    c_DeselectRegion;
   RectRegion    c_SelectRegion;
   int           i_ButtonState;
   int           i_Dummy;

//fprintf(stderr,"Selection arrived\n");

   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      GraphicObjectMainEvent Event;
      // clean up unused structures
      
      // set the cursor to normal (if not so)
      m_i_StartOperation = CursorNormal;
      SetSpecialCursor(CursorNormal);


      // tell parent, that the selection has been aborted
      Event.i_CallbackType = GraphicObjectMainEvent::SelectionDisabled;
      Event.pc_Object      = NULL;
      m_ObjectEventCallback.Execute(&Event);

      return NULL;
   }
   // from now only real selections
//fprintf(stderr,"Selection is valid\n");

   m_i_MouseEventType = m_pc_GlobalStorage->m_pc_Selection->GetMouseEventType( );
   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(
                                           m_i_MouseEventPositionX,
                                           m_i_MouseEventPositionY);

   // get state of shift and ctrl
   i_ButtonState =m_pc_GlobalStorage->m_pc_Selection->GetButtonState();


   // need an active input layer
   if(m_pc_ActiveInputLayer == NULL)
   {
      return NULL;
   }
/*
fprintf(stderr,"Type %d Position state %d\n",m_i_MouseEventType,
                  m_i_MouseEventPositionX,
                  m_i_MouseEventPositionY,
                  i_ButtonState);
*/
   if( m_i_MouseEventType == MouseEvent::EventOver)
   {
      // at event over on the cursor is recalculated

      CalculateCursor(m_i_MouseEventPositionX,m_i_MouseEventPositionY,i_Dummy,i_Dummy);
      return NULL;
   }

   else if( m_i_MouseEventType == MouseEvent::EventStart)
   {
      GraphicObject *pc_StartSearchObject;

      m_b_ObjectsMoved=false;

      // search from the beginning
      pc_StartSearchObject = NULL;

      // on a control button objects below the current selected or seareched
      if(i_ButtonState & Qt::ControlButton)
      {
         // Control button is active
//fprintf(stderr,"ControlButton\n") ;
         pc_StartSearchObject = m_pc_ActiveInputLayer->GetLastSelectedObject();
      }

      // deselect all objects
      c_DeselectRegion = DeselectAllObjects();


      // search for an object below the current one
      pc_Object = m_pc_ActiveInputLayer->SearchObjectAt(m_i_MouseEventPositionX,
                                                     m_i_MouseEventPositionY,
                                                     pc_StartSearchObject);


//fprintf(stderr,"Search found Object %x \n",(int) pc_Object);
      c_SelectRegion.SetEmpty();
      if( pc_Object != NULL)
      {
         // found one
         // select it
         pc_Object->SelectObject(true);
//fprintf(stderr,"Set object to selected\n");
         c_SelectRegion = pc_Object->GetSize();

         // call the select callback
         if( ! m_ObjectEventCallback.IsEmpty() )
         {
            GraphicObjectMainEvent Event;

            Event.i_CallbackType =GraphicObjectMainEvent::Selected;
            Event.pc_Object = pc_Object;
            m_ObjectEventCallback.Execute(&Event);
         }
      }

      // get the changed region and redraw the screen
      c_SelectRegion.AddRegion(c_DeselectRegion);

      // redraw the changed region
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_SelectRegion);

      // switch the cursor to the current state
      CalculateCursor(m_i_MouseEventPositionX,
                      m_i_MouseEventPositionY,
                      m_i_StartOperation,
                      m_i_ResizeStartBorder);

      // remeber the positions for further do operations
      m_i_StartMouseX = m_i_MouseEventPositionX;
      m_i_StartMouseY = m_i_MouseEventPositionY;
      m_i_LastMouseX  = m_i_MouseEventPositionX;
      m_i_LastMouseY  = m_i_MouseEventPositionY;


// fprintf(stderr,"StartOperation =%d\n",m_i_StartOperation);
   } // if event start

   else if( m_i_MouseEventType == MouseEvent::EventDo)
   {
      int i_DiffX;
      int i_DiffY;

      i_DiffX = m_i_MouseEventPositionX - m_i_LastMouseX;
      i_DiffY = m_i_MouseEventPositionY - m_i_LastMouseY;

      // in a cursor move operation
      if(m_i_StartOperation == CursorMove)
      {
         RectRegion c_ChangedRegion;

         // move the selected objects
         m_pc_ActiveInputLayer->MoveSelectedObjects(i_DiffX, i_DiffY, c_ChangedRegion);

         // redraw the changed Region
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

         CallEventForSelectedObjects(GraphicObjectMainEvent::Moved);
         m_b_ObjectsMoved=true;
      }
      else if(   m_i_StartOperation == CursorSizeUpDown
              || m_i_StartOperation == CursorSizeLeftRight
              || m_i_StartOperation == CursorSizeDiagonalUp
              || m_i_StartOperation == CursorSizeDiagonalDown
              )
      {
         ResizeAfterMouse(m_i_MouseEventPositionX,m_i_MouseEventPositionY);
      }

      // remember this position
      m_i_LastMouseX  = m_i_MouseEventPositionX;
      m_i_LastMouseY  = m_i_MouseEventPositionY;
   }
   else if( m_i_MouseEventType == MouseEvent::EventEnd)
   {
      // at the end recalcualte the cursor
      m_i_StartOperation = CursorNormal;
      CalculateCursor(m_i_MouseEventPositionX,m_i_MouseEventPositionY,i_Dummy,i_Dummy);
      
      if(m_b_ObjectsMoved)
      {
         GraphicObjectMainEvent Event;

         Event.i_CallbackType = GraphicObjectMainEvent::DragEnd;
         Event.pc_Object      = NULL;
         m_ObjectEventCallback.Execute(&Event);
      }

   }
   return NULL;

} // MouseEventArrived()

//===========================================================================

void *GraphicObjectMain::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   GraphicObjectMain *pc_Main;

   pc_Main = (GraphicObjectMain *) Thisdata;

   return pc_Main->SelectionEventArrived(pc_Data);
} // MouseEventArrivedCB()

//===========================================================================
//
//===========================================================================

void  GraphicObjectMain::ActivateInput(bool b_Activate)
{
   // need the mouse event pointer
   if(   m_pc_GlobalStorage == NULL
      || m_pc_GlobalStorage->m_pc_MouseEvent == NULL)
   {
      return;
   }
   
   if(b_Activate)
   {

      m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  "Object Work",
                                                  Callback(SelectionEventArrivedCB,this));
   }
   else
   {
      m_pc_GlobalStorage->m_pc_Selection->DeactivateSelection(Selection::SelectionStream,
                                                  "Object Work",
                                                  Callback(SelectionEventArrivedCB,this));
      
   }
} // ActivateInput()

//===========================================================================
//
//===========================================================================

void GraphicObjectMain::SetWorkColor(Pixel p_Color)
{
   m_p_WorkColor = p_Color;
} // SetWorkColor

//===========================================================================
//
//===========================================================================

Pixel GraphicObjectMain::GetWorkColor()
{
   return m_p_WorkColor;
} // GetWorkColor

//===========================================================================
//
//===========================================================================

void GraphicObjectMain::SetActiveLayer(GraphicObjectLayer * pc_Layer)
{
   m_pc_ActiveInputLayer = pc_Layer;
} // SetActiveLayer()

//===========================================================================
//
//===========================================================================

GraphicObjectLayer *GraphicObjectMain::SetActiveLayer(QString qs_LayerName)
{
   GraphicObjectLayer * pc_ActualLayer;
   QString              qs_ActualName;
   int                  i_LayerNumber;
   int                  i;

   if( m_pc_ActiveInputLayer != pc_ActualLayer &&
       m_pc_ActiveInputLayer != NULL)
   {
      // disable the old layer first
      // tell parent, that the selection has been aborted
      GraphicObjectMainEvent Event;
      Event.i_CallbackType = GraphicObjectMainEvent::SelectionDisabled;
      Event.pc_Object      = NULL;
      m_ObjectEventCallback.Execute(&Event);
   }
   
   i_LayerNumber = m_AllLayer.GetCount();

   for(i=0; i < i_LayerNumber; i++)
   {
      pc_ActualLayer =  m_AllLayer.GetLayer(i);
      qs_ActualName = pc_ActualLayer->GetName();
      if( qs_ActualName.compare(qs_LayerName)  ==0)
      {
         m_pc_ActiveInputLayer = pc_ActualLayer;
         return pc_ActualLayer;
      }
   } // for all layer

   return NULL;
} // SetActiveLayer()

//===========================================================================
//
//===========================================================================

void GraphicObjectMain::SetSpecialCursor(int i_Cursor)
{
   QWidget *pc_DrawingArea;

   pc_DrawingArea = m_pc_GlobalStorage->m_pc_DrawingArea;

   // selected the best looking cursor for an operation
   switch(i_Cursor)
   {
   case CursorSizeUpDown:
      pc_DrawingArea->setCursor( QCursor(Qt::SizeVerCursor) );
      break;
   case CursorSizeLeftRight:
      pc_DrawingArea->setCursor( QCursor(Qt::SizeHorCursor) );
      break;
   case CursorSizeDiagonalUp:
      pc_DrawingArea->setCursor( QCursor(Qt::SizeBDiagCursor) );
      break;
   case CursorSizeDiagonalDown:
      pc_DrawingArea->setCursor( QCursor(Qt::SizeFDiagCursor) );
      break;
   case CursorMove:
      pc_DrawingArea->setCursor( QCursor(Qt::SizeAllCursor) );
      break;
   case CursorNormal:
   default:
      pc_DrawingArea->setCursor(QCursor(Qt::ArrowCursor));
      break;
   };

} // SetSpecialCursor()

//===========================================================================
//
//===========================================================================

void GraphicObjectMain::CalculateCursor(int i_X, int i_Y,
                                        int &i_StartOperation,
                                        int &i_ResizeBorder)
{
   // look whether we are over an object
   GraphicObject *pc_FirstObject = NULL;
   int            i_NumberOfSelectedObjects;
   int            i_Operation;

   // default : no special cursor
   i_Operation = CursorNormal;

   // initial no border selected
   i_ResizeBorder = ResizeBorderNone;

   // get number of current selected objects
   i_NumberOfSelectedObjects = m_pc_ActiveInputLayer->GetNumberOfSelectedObjects(pc_FirstObject);
   
   // only one, than resizing might be possible
   if( i_NumberOfSelectedObjects == 1)
   {
      // enable sizing operators
      RectRegion      c_ObjectSize;
      
      // get currents object size
      c_ObjectSize = pc_FirstObject->GetSize();

      // is resize possible
      if(  pc_FirstObject->IsResizeable())
      {
         RectRegion r_CheckRegion;
         bool b_Right;
         bool b_Left;
         bool b_Upper;
         bool b_Lower;

         m_c_ResizeObjectInitialSize = c_ObjectSize;

         // check the different resize regions

         b_Right = b_Left = b_Upper = b_Lower = false;

         // region above the rectangle
         r_CheckRegion.SetXYXY (c_ObjectSize.GetStartX(),
                                c_ObjectSize.GetStartY() -5,
                                c_ObjectSize.GetEndX(),
                                c_ObjectSize.GetStartY() +5);
         if(r_CheckRegion.PointInRegion(i_X,
                                        i_Y))
         {
            b_Upper = true;
         }

         // region below the rectangle
         r_CheckRegion.SetXYXY (c_ObjectSize.GetStartX(),
                                   c_ObjectSize.GetEndY() -5,
                                   c_ObjectSize.GetEndX(),
                                   c_ObjectSize.GetEndY() +5);
         if(r_CheckRegion.PointInRegion(i_X,
                                        i_Y))
         {
            b_Lower = true;
         }

         // region left of the rectangle
         r_CheckRegion.SetXYXY (c_ObjectSize.GetStartX()-5,
                                c_ObjectSize.GetStartY() ,
                                c_ObjectSize.GetStartX()+5,
                                c_ObjectSize.GetEndY() );
         if(r_CheckRegion.PointInRegion(i_X,
                                        i_Y))
         {
            b_Left = true;
         }

         // region right of the reavtangle
         r_CheckRegion.SetXYXY (c_ObjectSize.GetEndX()-5,
                                   c_ObjectSize.GetStartY() ,
                                   c_ObjectSize.GetEndX()+5,
                                   c_ObjectSize.GetEndY() );
         if(r_CheckRegion.PointInRegion(i_X,
                                        i_Y))
         {
            b_Right = true;
         }

         
         // is in one of the resize regions?
         if(   b_Upper
            || b_Lower
            || b_Left
            || b_Right)
         {
            // check for the corners
            if(b_Upper)
            {
               if(b_Right)
               {
                  i_Operation = CursorSizeDiagonalUp;
                  i_ResizeBorder = ResizeBorderRightUp;
               }
               else if(b_Left)
               {
                  i_Operation = CursorSizeDiagonalDown;
                  i_ResizeBorder = ResizeBorderLeftUp;
               }
               else
               {
                  i_Operation = CursorSizeUpDown;
                  i_ResizeBorder = ResizeBorderUp;
               }
            }
            else if(b_Lower)
            {
               if(b_Right)
               {
                  i_Operation = CursorSizeDiagonalDown;
                  i_ResizeBorder = ResizeBorderRightDown;
               }
               else if(b_Left)
               {
                  i_Operation = CursorSizeDiagonalUp;
                  i_ResizeBorder = ResizeBorderLeftDown;
               }
               else
               {
                  i_Operation = CursorSizeUpDown;
                  i_ResizeBorder = ResizeBorderDown;
               }
            }
            else if(b_Left)
            {
               i_Operation = CursorSizeLeftRight;
               i_ResizeBorder = ResizeBorderLeft;
            }
            else if(b_Right)
            {
               i_Operation = CursorSizeLeftRight;
               i_ResizeBorder = ResizeBorderRight;
            }
         }
         else // not in resize region.
         {
            // check for the middle region
            if( c_ObjectSize.PointInRegion(i_X,
                                           i_Y))
            {
               i_Operation = CursorMove;
            }
            else
            {
               i_Operation = CursorNormal;
            }
         }
      }
      else // not resizeable
      {
         // check, whether in the middle of an object
         if( c_ObjectSize.PointInRegion(i_X,
                                           i_Y))
         {
            i_Operation = CursorMove;
         }
         else
         {
            i_Operation = CursorNormal;
         }
      } // else not resizeable

   }
   else if( i_NumberOfSelectedObjects > 1)
   {
      // TODO
      // check for cursor in one of all selected rectangles
   }
   else // no active object
   {
      i_Operation = CursorNormal;
   }

   // set the cursor in the drawing area
   SetSpecialCursor(i_Operation);

   // remember this cursor
   i_StartOperation = i_Operation;

} // CalculateCursor()

//===========================================================================
//
//===========================================================================

void GraphicObjectMain::SetEventCallback(Callback EventCallback)
{
   m_ObjectEventCallback = EventCallback;

} // SetEventCallback()

//===========================================================================
//
//===========================================================================

void GraphicObjectMain::ResizeAfterMouse(int i_MouseX,int i_MouseY)
{
   int i_NumberSelectedObjects;
   GraphicObject *pc_FirstObject = NULL;

   RectRegion c_NewSize;
   RectRegion c_SizeBefore;
   RectRegion c_SizeAfter;
   
   i_NumberSelectedObjects = m_pc_ActiveInputLayer->
                                 GetNumberOfSelectedObjects(pc_FirstObject);
                                 
   if(i_NumberSelectedObjects != 1)
   {
      return;
   }

   c_SizeBefore = pc_FirstObject->GetSize();
   // calculate the new dimension

   // first set it to original size
   c_NewSize = m_c_ResizeObjectInitialSize;
   switch(m_i_ResizeStartBorder)
   {
   case ResizeBorderUp:
      if( i_MouseY <  m_c_ResizeObjectInitialSize.GetEndY())
      {
         c_NewSize.SetStartY(i_MouseY);  
      }
      else
      {
         c_NewSize.SetStartY(c_SizeBefore.GetStartYFloat());
      }
      break;

   case ResizeBorderRightUp:
      if( i_MouseX >  m_c_ResizeObjectInitialSize.GetStartX())
      {
         c_NewSize.SetEndX(i_MouseX);
      }
      else
      {
         c_NewSize.SetEndX(c_SizeBefore.GetEndXFloat());
      }

      if( i_MouseY <  m_c_ResizeObjectInitialSize.GetEndY())
      {
         c_NewSize.SetStartY(i_MouseY);
      }
      else
      {
         c_NewSize.SetStartY(c_SizeBefore.GetStartYFloat());
      }
      break;
   case ResizeBorderRight:
      if( i_MouseX >  m_c_ResizeObjectInitialSize.GetStartX())
      {
         c_NewSize.SetEndX(i_MouseX);
      }
      else
      {
         c_NewSize.SetEndX(c_SizeBefore.GetEndXFloat());
      }

      break;
   case ResizeBorderRightDown:
      if( i_MouseX >  m_c_ResizeObjectInitialSize.GetStartX())
      {
         c_NewSize.SetEndX(i_MouseX);
      }
      else
      {
         c_NewSize.SetEndX(c_SizeBefore.GetEndXFloat());
      }

      if( i_MouseY >  m_c_ResizeObjectInitialSize.GetStartY())
      {
         c_NewSize.SetEndY(i_MouseY);
      }
      else
      {
         c_NewSize.SetEndY(c_SizeBefore.GetEndYFloat());
      }

      break;
   case ResizeBorderDown:
      if( i_MouseY >  m_c_ResizeObjectInitialSize.GetStartY())
      {
         c_NewSize.SetEndY(i_MouseY);
      }
      else
      {
         c_NewSize.SetEndY(c_SizeBefore.GetEndYFloat());
      }

      break;
   case ResizeBorderLeftDown:
      if( i_MouseX <  m_c_ResizeObjectInitialSize.GetEndX())
      {
         c_NewSize.SetStartX(i_MouseX);
      }
      else
      {
         c_NewSize.SetStartX(c_SizeBefore.GetStartXFloat());
      }

      if( i_MouseY >  m_c_ResizeObjectInitialSize.GetStartY())
      {
         c_NewSize.SetEndY(i_MouseY);
      }
      else
      {
         c_NewSize.SetEndY(c_SizeBefore.GetEndYFloat());
      }
      break;
   case ResizeBorderLeft:
      if( i_MouseX <  m_c_ResizeObjectInitialSize.GetEndX())
      {
         c_NewSize.SetStartX(i_MouseX);
      }
      else
      {
         c_NewSize.SetStartX(c_SizeBefore.GetStartXFloat());
      }

      break;
   case ResizeBorderLeftUp:
      if( i_MouseX <  m_c_ResizeObjectInitialSize.GetEndX())
      {
         c_NewSize.SetStartX(i_MouseX);
      }
      else
      {
         c_NewSize.SetStartX(c_SizeBefore.GetStartXFloat());
      }

      if( i_MouseY <  m_c_ResizeObjectInitialSize.GetEndY())
      {
         c_NewSize.SetStartY(i_MouseY);
      }
      else
      {
         c_NewSize.SetStartY(c_SizeBefore.GetStartYFloat());
      }
      
      break;
   default:
      break;
   } // swtich

   pc_FirstObject->SetSize(c_NewSize);
   
   c_SizeAfter = pc_FirstObject->GetSize();

   c_SizeAfter.AddRegion(c_SizeBefore);
   // redraw the changed Region
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_SizeAfter);

   if(  !m_ObjectEventCallback.IsEmpty() )
   {
      GraphicObjectMainEvent Event;

      Event.i_CallbackType = GraphicObjectMainEvent::Resized;
      Event.pc_Object      = pc_FirstObject;
      m_ObjectEventCallback.Execute(&Event);
   } // if callback enabled
      
} // ResizeAfterMouse()

//===========================================================================
//
//===========================================================================


   


