#ifndef _GRAPHIC_OBJECT_MAIN_H
#define _GRAPHIC_OBJECT_MAIN_H


#include "GraphicObjectAttribute.h"
#include "GraphicObject.h"
#include "GraphicObjectLayer.h"
#include "Callback.h"


//#########################################################################
//#########################################################################

class GraphicObjectMainEvent
{
public:
   enum
   {

      Moved,         // object is moved
      DragEnd,       // end of moving operation
      Selected,      // object switch to selected
      Deselected,    // object switched to deselected
      Resized,       // object size has changed
      SelectionDisabled
   };

   int i_CallbackType;
   GraphicObject *pc_Object;
};


class GraphicObjectMain
{
public:
   GraphicObjectMain(GlobalStorage *pc_Global);
   ~GraphicObjectMain();
   void SetVisible(bool b_Visible);
   bool IsVisible();

   int  GetLayerCount();
   GraphicObjectLayer *GetLayer(int i_Index);

   void ActivateInput(bool b_Activate = true);
   
   void SetWorkColor(Pixel p_Color);
   Pixel GetWorkColor();

   void SetEventCallback(Callback EventCallback);


   GraphicObjectLayer *CreateLayer(QString qs_LayerName);
   GraphicObjectLayer *GetActiveInputLayer();
   void SetActiveLayer(GraphicObjectLayer * pc_Layer);
   GraphicObjectLayer *SetActiveLayer(QString qs_LayerName);

private: // functions

   RectRegion DeselectAllObjects();
   void CallEventForSelectedObjects(int i_EventType);

   void *SelectionEventArrived(void *pc_Data);

   static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);
   void ResizeAfterMouse(int i_MouseX,int i_MouseY);
   void SetSpecialCursor(int i_Cursor=0);
   void CalculateCursor(int i_X, int i_Y,
                        int &i_StartOperation,
                        int &i_ResizeBorder);


private: // variables
   // whether objects are drawn at all
   bool m_b_Visible;
   GraphicObjectLayerList m_AllLayer;
   GraphicObjectLayer     *m_pc_ActiveInputLayer;

   GlobalStorage *m_pc_GlobalStorage;
   Pixel          m_p_WorkColor;

   Callback      m_ObjectEventCallback;

   bool m_b_ObjectsMoved;

   int m_i_MouseEventType;
   int m_i_MouseEventPositionX;
   int m_i_MouseEventPositionY;

   int m_i_StartOperation;
   int m_i_StartMouseX;
   int m_i_StartMouseY;
   int m_i_LastMouseX;
   int m_i_LastMouseY;
   int m_i_ResizeStartBorder;
   RectRegion m_c_ResizeObjectInitialSize;

   enum
   {
      ResizeBorderNone=0,
      ResizeBorderUp,
      ResizeBorderRightUp,
      ResizeBorderRight,
      ResizeBorderRightDown,
      ResizeBorderDown,
      ResizeBorderLeftDown,
      ResizeBorderLeft,
      ResizeBorderLeftUp,
   };
   
   enum
   {
      CursorNormal =0,
      CursorSizeUpDown,
      CursorSizeLeftRight,
      CursorSizeDiagonalUp,
      CursorSizeDiagonalDown,
      CursorMove
   };

};

//#########################################################################
//#########################################################################

#endif //  _GRAPHIC_OBJECT_MAIN_H
