#ifndef _GRAPHIKBOX_H
#define _GRAPHIKBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qvgroupbox.h>


#include "StandardDialog.h"
#include "BufferPainter.h"
#include "QULabelSlider.h"
#include "QUColorDragWidget.h"
#include "QUPixelButton.h"
#include "Tracer.h"
#include "QUColorOrPattern.h"


class GraphikBox :  public StandardDialog,
		    public Tracer
{
Q_OBJECT

public:
   GraphikBox(QWidget *parent=0,
			GlobalStorage *pc_Global=0);

public slots:

   void ActivatePoint();
   void ActivateLine();
   void ActivateRectangle();
   void ActivateQuadrat();
   void ActivateEllipse();
   void ActivateCircle();
   void ActivateFreeDraw();

   void LineThicknessChanged(int i_Width);
   void DrawInnerChanged(bool b_on);
   void TransparencyEnableChanged(bool b_on);
   void TransparencyFactorChanged(int i_Factor);

   void LineColorChanged(Pixel c_Color);
   void LinePatternChanged(PixelBuffer *pc_Buffer);
   void LineModeChanged(bool b_PatternMode);
   void InnerColorChanged(Pixel c_Color);
   void InnerPatternChanged(PixelBuffer *pc_Buffer);
   void InnerModeChanged(bool b_PatternMode);


private:


   enum
   {
      GraphikPoint,
      GraphikLine,
      GraphikRectangle,
      GraphikQuadrat,
      GraphikEllipse,
      GraphikCircle,
      GraphikFreeDraw


   };

    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   void *SelectionEventArrived(void *pc_Data);

   static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);

   void DrawPoint();
   void DrawLine();
   void DrawRectangle();
   void DrawEllipse();
   void DrawCircle();
   void DrawQuadrat();
   void DrawFreeDraw();

   PixelBuffer *GetDestinationBuffer();

   QUPixelButton *m_PointButton;
   QUPixelButton *m_LineButton;
   QUPixelButton *m_RectangleButton;
   QUPixelButton *m_QuadratButton;
   QUPixelButton *m_EllipseButton;
   QUPixelButton *m_CircleButton;
   QUPixelButton *m_FreeDrawButton;

   QUColorOrPattern *m_LineColorWidget;
   QUColorOrPattern *m_InnerColorWidget;

   QSlider *m_LineWidthSlider;

   QCheckBox *m_DrawInnerEnable;

   QULabelSlider *m_TransparencySlider;
   QCheckBox *m_TransparencyEnable;


   int m_i_CurrentGraphikMode;
   int m_i_LineWidth;

   BufferPainter *m_pc_BufferPainter;
   
   QVGroupBox   *InnerBox;

};


#endif // _GRAPHIKBOX_H
