
#include <stdio.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>


#include "PixelBuffer.h"
#include "GridBox.h"
#include "QtUtilities.h"
#include "Zoom.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "ApplicationFrame.h"
#include "Selection.h"


//===========================================================================
//
//===========================================================================

GridBox::GridBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global)
{
   QBoxLayout   *l_Layout;
   QVGroupBox   *SizeBox;
   QVGroupBox   *VisibleBox;


   setCaption("Image Grid");

   // initialize settings
   m_i_Width     = 20;
   m_i_Height    = 30;
   m_i_OffsetX   = 10;
   m_i_OffsetY   = 15;
   m_b_Visible   = false;
   m_p_GridColor = 0xccffcc;

   // transfer these settings to the zoom
   m_pc_GlobalStorage->m_pc_Zoom->SetGridVisible(m_b_Visible);
   m_pc_GlobalStorage->m_pc_Zoom->SetGridParameters(m_i_OffsetX,
                                             m_i_OffsetY,
                                             m_i_Width,
                                             m_i_Height);
   m_pc_GlobalStorage->m_pc_Zoom->SetGridColor(m_p_GridColor);

   // create the box
   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);

   SizeBox = new QVGroupBox("Size",m_w_TopWidget);
   l_Layout->addWidget(SizeBox);
   {
      QHGroupBox   *w_ButtonBox;
      QGridLayout  *grid;
      QWidget      *GridBox;
      int           i_CharWidth;

      GridBox = new QWidget(SizeBox);

      grid = new QGridLayout( GridBox, 2, 4, 5 );

      m_WidthEdit      = new QLineEdit(GridBox);
      m_OffsetXEdit    = new QLineEdit(GridBox);
      m_HeightEdit     = new QLineEdit(GridBox);
      m_OffsetYEdit    = new QLineEdit(GridBox);

      i_CharWidth = g_Application->m_pc_QtUtility->GetMaximalFontWidth();
      m_WidthEdit    ->setMaximumWidth(i_CharWidth*4);
      m_OffsetXEdit  ->setMaximumWidth(i_CharWidth*4);
      m_HeightEdit   ->setMaximumWidth(i_CharWidth*4);
      m_OffsetYEdit  ->setMaximumWidth(i_CharWidth*4);

      grid->addWidget(new QLabel("Width",GridBox),0,0);
      grid->addWidget(m_WidthEdit,0,1);
      grid->addWidget(new QLabel("Horizontal Offset",GridBox),0,2);
      grid->addWidget(m_OffsetXEdit,0,3);
      grid->addWidget(new QLabel("Height",GridBox),1,0);
      grid->addWidget(m_HeightEdit,1,1);
      grid->addWidget(new QLabel("Vertical Offset",GridBox),1,2);
      grid->addWidget(m_OffsetYEdit,1,3);

      w_ButtonBox = new QHGroupBox("",SizeBox);

      m_ApplyButton = new QPushButton("Apply Size",w_ButtonBox);
      m_SelectImageButton = new QPushButton("Select in Image",w_ButtonBox);
      m_ResetButton = new QPushButton("Reset",w_ButtonBox);

      connect( m_ApplyButton ,
               SIGNAL(clicked()),
               SLOT( ApplySettings() ) );
      connect( m_SelectImageButton ,
               SIGNAL(clicked()),
               SLOT( SelectClicked() ) );
      connect( m_ResetButton ,
               SIGNAL(clicked()),
               SLOT( DoReset() ) );

      // set input focus to the first item
      m_WidthEdit->setFocus();
   } // SizeBox

   VisibleBox = new QVGroupBox("Visibility Options",m_w_TopWidget);
   l_Layout->addWidget(VisibleBox);
   {
      // create the visible checkbox
      m_VisibleCheck = new QCheckBox("Visible",VisibleBox);
      // create the grid color
      m_GridcolorWidget = new QUColorDragLabeledWidget(VisibleBox,
                                             "Grid Display Color",
                                             QUColorDragWidget::ModePipette,
                                             m_pc_GlobalStorage);
      m_GridcolorWidget->SetColor(m_p_GridColor);
   }

   // connect widgets and events
   connect( m_GridcolorWidget,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( GridColorChanged(Pixel ) )
             );

   connect( m_VisibleCheck ,
               SIGNAL(toggled(bool)),
               SLOT( VisibleClicked(bool) ) );

   connect( m_WidthEdit,
            SIGNAL(textChanged ( const QString & )),
            SLOT( TextChanged(const QString &) ) );
   connect( m_HeightEdit,
            SIGNAL(textChanged ( const QString & )),
            SLOT( TextChanged(const QString &) ) );
   connect( m_OffsetXEdit,
            SIGNAL(textChanged ( const QString & )),
            SLOT( TextChanged(const QString &) ) );
   connect( m_OffsetYEdit,
            SIGNAL(textChanged ( const QString & )),
            SLOT( TextChanged(const QString &) ) );

   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction) OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageNewBuffer");



   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction)OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageResize");

   // copy number into the edit fields
   SetValuesIntoFields();

} // ManipulationBox()

//===========================================================================
//
//===========================================================================

void GridBox::SelectClicked()
{
   if(  m_pc_GlobalStorage != NULL)
   {

      m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionRectangle,
                                 "Select a Grid",
                                 Callback(SelectionEventArrivedCB,this) );

   }
} // SelectClicked()

//===========================================================================
//
//===========================================================================

void *GridBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures

   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      RectRegion c_Region;
      int i_X1;
      int i_X2;
      int i_Y1;
      int i_Y2;


      if(m_pc_GlobalStorage == NULL)
      {
         return NULL;
      }
      m_pc_GlobalStorage->m_pc_Selection->GetSelectionRectangle(i_X1,i_Y1,i_X2,i_Y2);
      c_Region.SetXYXY(i_X1,i_Y1,i_X2,i_Y2);

      m_i_Width   = c_Region.GetWidth();
      m_i_Height  = c_Region.GetHeight();
      m_i_OffsetX = c_Region.GetStartX();
      m_i_OffsetY = c_Region.GetStartY();
      
      // set into zoom
      if(m_i_OffsetX > m_i_Width)
      {
         m_i_OffsetX = m_i_OffsetX % m_i_Width;
      }
      if(m_i_OffsetY > m_i_Height)
      {
         m_i_OffsetY = m_i_OffsetY % m_i_Height;
      }

      // copy number into the edit fields
      SetValuesIntoFields();

      m_pc_GlobalStorage->m_pc_Zoom->SetGridParameters(m_i_OffsetX,
                                             m_i_OffsetY,
                                             m_i_Width,
                                             m_i_Height);
      if(m_b_Visible)
      {
         // force a redraw
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
      }
   } // If ReasonSelection
   
   return NULL;
} // SelectionEventArrived()

void *GridBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   GridBox *pc_This = (GridBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()


//===========================================================================
//
//===========================================================================

void GridBox::DoReset()
{
   // set back the stored values into the edit fields
   SetValuesIntoFields();
} // DoReset()

//===========================================================================
//
//===========================================================================

void GridBox::ApplySettings()
{
   QString qs_Text;

   // get values out of the edit fields
   qs_Text  = m_WidthEdit->text();
   sscanf((const char *) qs_Text,"%d",&m_i_Width);
   qs_Text  = m_HeightEdit->text();
   sscanf((const char *) qs_Text,"%d",&m_i_Height);
   qs_Text  = m_OffsetXEdit->text();
   sscanf((const char *) qs_Text,"%d",&m_i_OffsetX);
   qs_Text  = m_OffsetYEdit->text();
   sscanf((const char *) qs_Text,"%d",&m_i_OffsetY);

   // set into zoom
   if(m_i_OffsetX > m_i_Width)
   {
      m_i_OffsetX = m_i_OffsetX % m_i_Width;
   }
   if(m_i_OffsetY > m_i_Height)
   {
      m_i_OffsetY = m_i_OffsetY % m_i_Height;
   }

   m_pc_GlobalStorage->m_pc_Zoom->SetGridParameters(m_i_OffsetX,
                                             m_i_OffsetY,
                                             m_i_Width,
                                             m_i_Height);
   if(m_b_Visible)
   {
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // ApplySettings()

//===========================================================================
//
//===========================================================================

void GridBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   if(qs_MessageType.compare("ImageResize") ==0)
   {
      CheckValues();
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================

void GridBox::VisibleClicked(bool b_On)
{
   m_b_Visible = b_On;
   m_pc_GlobalStorage->m_pc_Zoom->SetGridVisible(b_On);

   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

} // VisibleClicked()

//===========================================================================
//
//===========================================================================

void GridBox::SetValuesIntoFields()
{
   QString qs_Out;

   // put values into the single edit fields
   qs_Out.sprintf("%d",m_i_Width);
   m_WidthEdit->setText(qs_Out);
   qs_Out.sprintf("%d",m_i_Height);
   m_HeightEdit->setText(qs_Out);
   qs_Out.sprintf("%d",m_i_OffsetX);
   m_OffsetXEdit->setText(qs_Out);
   qs_Out.sprintf("%d",m_i_OffsetY);
   m_OffsetYEdit->setText(qs_Out);

} // SetValuesIntoFields()

//===========================================================================
//
//===========================================================================

void GridBox::CheckValues()
{
   QString qs_Text;
   char    c_Dummy;
   bool    b_Valid;
   int     i_Value;
   int     i_Result;
   int     i_OriginalWidth;
   int     i_OriginalHeight;

   // get original dimensions
   i_OriginalWidth  = m_pc_GlobalStorage->m_pc_MainImage->GetWidth();
   i_OriginalHeight = m_pc_GlobalStorage->m_pc_MainImage->GetHeight();

   // assume : its ok
   b_Valid =true;

   qs_Text  = m_WidthEdit->text();
   i_Result = sscanf((const char *) qs_Text,"%d%c",&i_Value,&c_Dummy);
   if(!(   i_Result == 1
        && i_Value >1
        && i_Value < i_OriginalWidth))
   {
      b_Valid = false;
   }

   qs_Text  = m_HeightEdit->text();
   i_Result = sscanf((const char *) qs_Text,"%d%c",&i_Value,&c_Dummy);
   if(!(   i_Result == 1
        && i_Value >1
        && i_Value < i_OriginalHeight))
   {
      b_Valid = false;
   }

   qs_Text  = m_OffsetXEdit->text();
   i_Result = sscanf((const char *) qs_Text,"%d%c",&i_Value,&c_Dummy);
   if(!(   i_Result == 1
        && i_Value >=0
        && i_Value < i_OriginalWidth))
   {
      b_Valid = false;
   }

   qs_Text  = m_OffsetYEdit->text();
   i_Result = sscanf((const char *) qs_Text,"%d%c",&i_Value,&c_Dummy);
   if(!(   i_Result == 1
        && i_Value >=0
        && i_Value < i_OriginalHeight))
   {
      b_Valid = false;
   }
   
   // if valid then enalbe the apply button
   if(b_Valid)
   {
      m_ApplyButton->setEnabled(true);
   }
   else
   {
      m_ApplyButton->setEnabled(false);
   }

} // CheckValues()

//===========================================================================
//
//===========================================================================

void GridBox::TextChanged(const QString &qs_NewText)
{
   // on a text change check again the input values
   CheckValues();
} // TextChanged()

//===========================================================================
//
//===========================================================================

void GridBox::GridColorChanged(Pixel c_Color)
{
   m_p_GridColor = c_Color;

   // copy new color to the zomm
   m_pc_GlobalStorage->m_pc_Zoom->SetGridColor(m_p_GridColor);

   if(m_b_Visible)
   {
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }
} // GridColorChanged()

//===========================================================================
//
//===========================================================================


