
#include <qimage.h>

#include "IconLibrary.h"
#include "FileLoader.h"

//============================================================================
//
//============================================================================

IconEntry::IconEntry()
{
   // inititalize empty
   m_Name[0]='\0';
   m_Buffer = NULL;
} // IconEntry()

//============================================================================
//
//============================================================================

IconLibrary::IconLibrary(void *pc_IconStructure)
{
   //save the structture
   m_pc_IconStructure = pc_IconStructure;
   
   if(m_pc_IconStructure != NULL)
   {
      // if structure is not empty, enable the memory read mode
      m_b_ReadFromMemory =true;
      m_i_NumberOfIcons = * (int *)(m_pc_IconStructure);
      
      // set pointer to icon directory into the strucure
      m_IconDirectory = (IconEntry *) ((unsigned char *)m_pc_IconStructure +4);

   }
   else
   {
      // no predefined icons in the memory
      m_b_ReadFromMemory =false;

      // create an array in memory to store pixelbuffer

      m_i_MaximumNumberInArray=64;
      m_IconDirectory   = new IconEntry[m_i_MaximumNumberInArray];
      m_i_NumberOfIcons = 0;
   }

} // IconLibrary()

//============================================================================
//
//============================================================================

IconLibrary::~IconLibrary()
{
   if(m_b_ReadFromMemory)
   {
      // do nothing
   }
   else // else allocated elements
   {
      int i;
      for(i=0;i< m_i_NumberOfIcons; i++)
      {
         delete m_IconDirectory[i].m_Buffer;
         m_IconDirectory[i].m_Buffer = NULL;
      } // for i
      m_i_NumberOfIcons =0;
      delete m_IconDirectory;
   }
} // ~IconLibrary()

//============================================================================
//
//============================================================================

QPixmap *IconLibrary::GetPixmap(QString qs_IconName)
{
   PixelBuffer *pc_Buffer;
   QPixmap     *pc_NewPixmap;
   QImage      *pc_Image;
   int         i_Width;
   int         i_Height;
   int         i_RunX;
   int         i_RunY;

   pc_Buffer = GetIcon(qs_IconName);

   if(pc_Buffer == NULL)
   {
      return NULL;
   }

   i_Width  =pc_Buffer->GetWidth();
   i_Height =pc_Buffer->GetHeight();
   pc_NewPixmap = new QPixmap(i_Width,
                              i_Height,
                              -1);
   if(pc_NewPixmap == NULL)
   {
      delete pc_Buffer;
      return NULL;
   }

   // now copy from buffer into pixmap
   pc_Image = new QImage(i_Width,
                         i_Height,
                         32);
   for(i_RunY=0; i_RunY < i_Height; i_RunY++)
   {
      for(i_RunX=0; i_RunX < i_Width; i_RunX++)
      {
         pc_Image->setPixel(i_RunX,i_RunY,
                            pc_Buffer->Get(i_RunX,i_RunY));
      }
   }

   pc_NewPixmap->convertFromImage(*pc_Image,QPixmap::Color);

   delete pc_Buffer;
   delete pc_Image;

   return pc_NewPixmap;

} // GetPixmap()

//============================================================================
//
//============================================================================

QImage *IconLibrary::GetImage(QString qs_IconName)
{
   PixelBuffer *pc_Buffer;
   QImage      *pc_Image;
   int         i_Width;
   int         i_Height;
   int         i_RunX;
   int         i_RunY;

   // get the icon into an pixelbuffer
   pc_Buffer = GetIcon(qs_IconName);

   if(pc_Buffer == NULL)
   {
      return NULL;
   }

   i_Width  =pc_Buffer->GetWidth();
   i_Height =pc_Buffer->GetHeight();

   // now copy from buffer into pixmap
   pc_Image = new QImage(i_Width,
                         i_Height,
                         32);
   for(i_RunY=0; i_RunY < i_Height; i_RunY++)
   {
      for(i_RunX=0; i_RunX < i_Width; i_RunX++)
      {
         pc_Image->setPixel(i_RunX,i_RunY,
                            pc_Buffer->Get(i_RunX,i_RunY));
      }
   }

   // delete the pixelbuffer
   delete pc_Buffer;

   // return the image
   return  pc_Image;
} // GetImage()

//============================================================================
//
//============================================================================

void IconLibrary::SaveToCFile(QString cs_FileName)
{
   int  i_WritePosition;
   int  i_SizeOfPreviousImages;
   int  i_SizeOfHeader;
   int  i;

   if(m_b_ReadFromMemory)
   {
      return;
   }

   i_SizeOfPreviousImages = 0;
   i_WritePosition        = 0;


   fp_Outfile = fopen(cs_FileName,"wb");
   if(fp_Outfile == NULL)
   {
      return ;
   }
   WriteHeader();


   WriteInt(m_i_NumberOfIcons);
   i_WritePosition += 4;

   // first write all headers
   i_SizeOfHeader = 4 + m_i_NumberOfIcons *(64+4);
   for(i=0;i<m_i_NumberOfIcons;i++)
   {
      PixelBuffer *pc_CurrentImage;
      int          i_SizeOfCurrentImage;

      pc_CurrentImage = m_IconDirectory[i]. m_Buffer;
      // write the icon name
      WriteChar((unsigned char *) m_IconDirectory[i].m_Name,
                64);
      WriteInt(i_SizeOfPreviousImages + i_SizeOfHeader);

      // get size of current image;
      i_SizeOfCurrentImage=4+4
                            +(pc_CurrentImage->GetWidth() * pc_CurrentImage->GetHeight()*4);
      i_SizeOfPreviousImages += i_SizeOfCurrentImage;

   }

   // write all icon pixel data
   for(i=0;i<m_i_NumberOfIcons;i++)
   {
      PixelBuffer *pc_CurrentImage;
      int          i_Width;
      int          i_Height;
      int          i_RunX;
      int          i_RunY;

      pc_CurrentImage = m_IconDirectory[i]. m_Buffer;

      // get size of current image;
      i_Width = pc_CurrentImage->GetWidth();
      i_Height= pc_CurrentImage->GetHeight();
      WriteInt(i_Width);
      WriteInt(i_Height);

      // write all pixel
      for(i_RunY=0; i_RunY <i_Height; i_RunY ++)
      {
         for(i_RunX=0; i_RunX <i_Width; i_RunX ++)
         {
            Pixel p;
            p = pc_CurrentImage->Get(i_RunX,i_RunY);
            WriteInt(p);
         }
      } // for i_RunY

   } // for all images

   WriteFooter();

   fclose(fp_Outfile);
} // SaveToCFile

//============================================================================
//
//============================================================================

void IconLibrary::AddIconFromFile(QString qs_Name,QString cs_FileName)
{
   PixelBuffer *pc_NewIcon;
   FileLoader  *pc_Loader;

   if(m_b_ReadFromMemory )
   {
      return;
   }

   // create a file-loader to load the icon
   pc_Loader = new FileLoader(NULL);
   if(pc_Loader == NULL)
   {
      return;
   }
   pc_NewIcon = pc_Loader->LoadFile(cs_FileName);
   delete pc_Loader;

   if(pc_NewIcon == NULL)
   {
      return;
   }

   // at this point the icon is loaded. add it to the directory
   if(m_i_NumberOfIcons+1 >= m_i_MaximumNumberInArray)
   {
      // increase the directory-memory
      IconEntry *pc_NewDirectory;
      int        i_NewSize;
      int        i;


      i_NewSize = 2 * m_i_MaximumNumberInArray;

      pc_NewDirectory = new IconEntry[i_NewSize];
      if(pc_NewDirectory ==NULL)
      {
         delete pc_NewIcon;
         return;
      }
      for(i=0; i< m_i_NumberOfIcons; i++)
      {
         pc_NewDirectory[i] = m_IconDirectory[i];
      } // for i
      m_i_MaximumNumberInArray = i_NewSize;
      delete m_IconDirectory;

      m_IconDirectory = pc_NewDirectory;

   } // if array to small

   // copy the pointer into the field
   m_IconDirectory[m_i_NumberOfIcons].m_Buffer = pc_NewIcon;

   // copy the name
   strncpy(m_IconDirectory[m_i_NumberOfIcons].m_Name,(const char *) qs_Name,62);
   m_IconDirectory[m_i_NumberOfIcons].m_Name[62] = '\0';

   m_i_NumberOfIcons += 1;


} // AddIconFromFile()

//============================================================================
//
//============================================================================

PixelBuffer *IconLibrary::GetIcon(QString cs_IconName)
{
   int i;
   
   if(m_b_ReadFromMemory )
   {
      for(i=0; i< m_i_NumberOfIcons; i++)
      {
         // is it the icon which is looked for
         if(strcmp((const char *)cs_IconName,m_IconDirectory[i].m_Name) ==0)
         {
            unsigned char *pc_Memory;
            unsigned int  *pc_Image;
            PixelBuffer   *pc_Buffer;
            int            i_Width;
            int            i_Height;
            int            i_RunX;
            int            i_RunY;

            pc_Memory = (unsigned char *) m_pc_IconStructure;

            pc_Memory = &pc_Memory[m_IconDirectory[i].m_OffsetPixelData];
            pc_Image = ( unsigned int *) pc_Memory;

            i_Width  = (int) *pc_Image++;
            i_Height = (int) *pc_Image++;
            
            // safety check against wrong parameters
            if(i_Width>1000 || i_Height>1000)
            {
               return NULL;
            }

            pc_Buffer = new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG,false,false);
            if(pc_Buffer == NULL)
            {
               return NULL;
            }
            for(i_RunY=0; i_RunY <i_Height; i_RunY ++)
            {
               for(i_RunX=0; i_RunX <i_Width; i_RunX ++)
               {
                  Pixel p;
                  p= (Pixel) * pc_Image++;
                  pc_Buffer->SetRaw(i_RunX,i_RunY,p);
               }
            } // for i_RunY
            return pc_Buffer;
         } // if strcmp
      } // for i;

      // nothing found
      return NULL;
   }
   else // not read from memory
   {
      for(i=0;i< m_i_NumberOfIcons; i++)
      {
         if(strcmp((const char *)cs_IconName,m_IconDirectory[i].m_Name) ==0)
         {
            // found it
            // return a copy
            return new PixelBuffer(m_IconDirectory[i].m_Buffer);
         }
      } // for i
      return NULL;
   } // not read from memory

   return NULL;
}

//============================================================================
//
//============================================================================

int IconLibrary::WriteHeader()
{
   m_i_CurrentCharPosition= 0;
   m_b_FirstNumber = true;

   fprintf(fp_Outfile,"// this is an automatic generated file\n");
   fprintf(fp_Outfile,"// Do not edit by hand\n");
   fprintf(fp_Outfile,"\n");
   fprintf(fp_Outfile,"unsigned char IconLibraryMemoryData[] = { \n");

   return 0;
} // WriteHeader

//============================================================================
//
//============================================================================

int IconLibrary::WriteFooter()
{

   fprintf(fp_Outfile,"\n");
   fprintf(fp_Outfile,"};\n");

   return 0;
} // WriteFooter

//============================================================================
//
//============================================================================

int IconLibrary::WriteInt(unsigned int i_Value)
{
   unsigned char uc_Value;


   uc_Value = (unsigned char) (i_Value & 0xff);
   WriteOne(uc_Value);
   uc_Value = (unsigned char) ((i_Value >>8) & 0xff);
   WriteOne(uc_Value);
   uc_Value = (unsigned char) ((i_Value >>16) & 0xff);
   WriteOne(uc_Value);
   uc_Value = (unsigned char) ((i_Value >>24) & 0xff);
   WriteOne(uc_Value);

   return 0;
} // WriteInt

//============================================================================
//
//============================================================================

int IconLibrary::WriteChar(unsigned char *uc_Memory,int i_Length)
{
   int i;
   
   for(i=0; i<i_Length; i++)
   {
      WriteOne(uc_Memory[i]);
   }

   return 0;
} // WriteChar()

//============================================================================
//
//============================================================================

int IconLibrary::WriteOne(unsigned char uc_Value)
{

   if(!m_b_FirstNumber)
   {
      if(m_i_CurrentCharPosition >= 32)
      {
         fprintf(fp_Outfile,",\n");
         m_i_CurrentCharPosition =0;
      }
     else
     {
         fprintf(fp_Outfile,",");
     }
   }
   else
   {
      m_b_FirstNumber =false;
   }
   fprintf(fp_Outfile ,"0x%02x",uc_Value);

   m_i_CurrentCharPosition +=1;

   return 0;
   
} // WriteOne()

//============================================================================
//
//============================================================================



