

#include <qwidget.h>
#include <qlayout.h>
#include <qgrid.h>
#include <qvbox.h>
#include <qstatusbar.h>
#include <qlabel.h>
#include <qscrollview.h>
#include <qscrollbar.h>
#include <qsizepolicy.h>
#include <qimage.h>
#include <qcursor.h>
#include <qbitmap.h>

#include <qpainter.h>

#include "ImageFrame.h"
#include "DrawingArea.h"
#include "MouseEvent.h"
#include "MessageServer.h"
#include "Selection.h"
#include "GraphicObjectLayer.h"
#include "GraphicObjectMain.h"
#include "MaskSettings.h"

//===========================================================================
//
//===========================================================================

ImageFrame::ImageFrame(QWidget *parent,
                  QApplication *mainApplication,
                  void *data,
                  int i_Flags,
                  int i_SizeX,
                  int i_SizeY)
   : QWidget(parent,
                 "ImageFrame" )

{
   QBoxLayout  *TopLayout;
   QWidget     *TopWidget;

   m_pc_GlobalStorage = new GlobalStorage();
   m_pc_GlobalStorage->m_pc_ImageFrame = this;

   // create a MessageServer
   m_pc_GlobalStorage->m_pc_MessageServer = new MessageServer();

   // create the object Layer
   m_pc_GlobalStorage->m_pc_ObjectMain    = new GraphicObjectMain(m_pc_GlobalStorage);

   m_pc_GlobalStorage->m_pc_MaskSettings = new MaskSettings();

   if(i_Flags & FixedSize)
   {
      m_b_FixedSize = true;
   }
   else
   {
      m_b_FixedSize = false;
   }

   m_b_XHalfPixel    = false;
   m_b_YHalfPixel    = false;
   m_i_ZoomFactor    = 1;
   m_pc_Picture      = NULL;
   m_b_PreventRedraw = false;

   m_i_LeftStartPixel         = 0;
   m_i_TopStartPixel          = 0;
   m_i_OutputAreaZoomedWidth  = 1;
   m_i_OutputAreaZoomedHeight = 1;


  // topwidgets erzeugen
   TopWidget = this;

   TopLayout     = new QVBoxLayout( TopWidget,1);

   // block for the drawing area
   if(! m_b_FixedSize)
   {
      QGridLayout  *grid;
      QWidget      *DrawingWidget;
      DrawingWidget = new QWidget(TopWidget);
      TopLayout->addWidget(DrawingWidget);
      grid = new QGridLayout( DrawingWidget, 2, 2, 3 );

      m_BottomScrollbar=new QScrollBar(Qt::Horizontal,DrawingWidget);
      m_RightScrollbar =new QScrollBar(Qt::Vertical,DrawingWidget);

      m_DrawingAreaWidget =  new DrawingArea(DrawingWidget,m_pc_GlobalStorage);

      grid->addWidget(m_DrawingAreaWidget,0,0);
      grid->addWidget(m_RightScrollbar ,0,1);
      grid->addWidget(m_BottomScrollbar,1,0);

      if(i_Flags & UndoButton)
      {
         m_UndoButton = new QPushButton("Undo",DrawingWidget);
         {
            QFont f( "Helvetica", 7, QFont::Normal );
            m_UndoButton->setFont( f );
         }
      
         grid->addWidget(m_UndoButton,1,1);

         connect(m_UndoButton,
                 SIGNAL(clicked()),
                 SLOT(UndoButtonClicked())
                );
         m_UndoAccelerator = new QAccel(this);
         m_UndoAccelerator->connectItem(m_UndoAccelerator->insertItem(Key_U),
                                 this,
                                 SLOT(UndoButtonClicked())
                                 );
         
      }
      else // no undo button
      {
         m_UndoButton      = NULL;
         m_UndoAccelerator = NULL;
         grid->addWidget(new QLabel(" ",DrawingWidget),1,1);
      }

   }
   else // FixedSize;
   {
      m_DrawingAreaWidget =  new DrawingArea(TopWidget,m_pc_GlobalStorage,i_SizeX,i_SizeY);
      m_DrawingAreaWidget->setFixedSize(i_SizeX,i_SizeY );
      this->setFixedSize(i_SizeX,i_SizeY );

      m_BottomScrollbar=NULL;
      m_RightScrollbar =NULL;
   }

   m_DrawingAreaWidget->InstallResizeCallback(Callback(ResizeCB,this));
   m_DrawingAreaWidget->InstallExposeCallback(Callback(ExposeCB,this));

   m_pc_Zoomer = new Zoom(m_pc_GlobalStorage);
   m_pc_GlobalStorage->m_pc_MouseEvent = new MouseEvent(m_pc_GlobalStorage);

   if( ! ( i_Flags & NoSelection) )
   {
      m_pc_GlobalStorage->m_pc_Selection  = new Selection(m_pc_GlobalStorage);
   }

   // add signal to the scrollbar
   if(m_BottomScrollbar != NULL)
   {
     connect(m_BottomScrollbar,
           SIGNAL(valueChanged(int)),
           this,
           SLOT(ScrollbarBottomMoved(int)) );
   }

   // add signal to the scrollbar
   if(m_RightScrollbar != NULL)
   {
      connect(m_RightScrollbar,
           SIGNAL(valueChanged(int)),
           this,
           SLOT(ScrollbarRightMoved(int)) );
   }

   CreateDummyImage();

} // ImageFrame() constructor

//===========================================================================
//
//===========================================================================

void ImageFrame::UndoButtonClicked()
{
   UndoClicked();   
} // UndoButtonClicked()


//===========================================================================
//
//===========================================================================

ImageFrame::~ImageFrame()
{
   if(m_DrawingAreaWidget != NULL)
   {
      m_DrawingAreaWidget->InstallResizeCallback(Callback(NULL,NULL));
      m_DrawingAreaWidget->InstallExposeCallback(Callback(NULL,NULL));
   }
   
   if(m_pc_GlobalStorage->m_pc_Selection != NULL)
   {
      delete m_pc_GlobalStorage->m_pc_Selection;
      m_pc_GlobalStorage->m_pc_Selection = NULL;
   }
   if(m_pc_GlobalStorage->m_pc_MouseEvent != NULL)
   {
      delete m_pc_GlobalStorage->m_pc_MouseEvent;
      m_pc_GlobalStorage->m_pc_MouseEvent = NULL;
   }
   if(m_pc_GlobalStorage->m_pc_Zoom != NULL)
   {
      delete m_pc_GlobalStorage->m_pc_Zoom;
      m_pc_GlobalStorage->m_pc_Zoom =NULL;
   }
   if(m_pc_GlobalStorage->m_pc_DrawingArea != NULL)
   {
      delete m_pc_GlobalStorage->m_pc_DrawingArea;
      m_pc_GlobalStorage->m_pc_DrawingArea =NULL;
   }
   if(m_pc_GlobalStorage->m_pc_MaskSettings != NULL)
   {
      delete m_pc_GlobalStorage->m_pc_MaskSettings;
      m_pc_GlobalStorage->m_pc_MaskSettings =NULL;
   }
   if(m_pc_GlobalStorage->m_pc_ObjectMain != NULL)
   {
      delete m_pc_GlobalStorage->m_pc_ObjectMain;
      m_pc_GlobalStorage->m_pc_ObjectMain =NULL;
   }
   if(m_pc_GlobalStorage->m_pc_MessageServer != NULL)
   {
      delete m_pc_GlobalStorage->m_pc_MessageServer;
      m_pc_GlobalStorage->m_pc_MessageServer =NULL;
   }
   // delete my pointer
   m_pc_GlobalStorage->m_pc_ImageFrame =NULL;
   
   if(m_pc_GlobalStorage != NULL)
   {
      delete m_pc_GlobalStorage;
      m_pc_GlobalStorage =NULL;
   }

} // ~ImageFrame()

//===========================================================================
//
//===========================================================================


void * ImageFrame::ResizeCB(void *ThisData, void * CallData)
{
   ImageFrame *pc_This;
   pc_This = (ImageFrame *) ThisData;
   pc_This->Resize(CallData);
   return NULL;
}

void ImageFrame::Resize(void *CallData)
{

   int i_OldWidth;
   int i_NewWidth;
   int i_OldHeight;
   int i_NewHeight;

   int i_ZoomedPictureWidth;
   int i_ZoomedPictureHeight;


//   fprintf(stderr,"ImageFrame Resize Callback\n");

   // no picture?
   if(m_pc_Picture ==NULL)
   {
      return;
   }


   i_NewWidth  = m_DrawingAreaWidget->GetWidth();
   i_OldWidth  = m_DrawingAreaWidget->GetOldWidth();
   i_NewHeight = m_DrawingAreaWidget->GetHeight();
   i_OldHeight = m_DrawingAreaWidget->GetOldHeight();

   // Calculate maximum pixel size of current picture if zoomed
   // with the current zoom factor
   if(m_i_ZoomFactor >=1)
   {
      i_ZoomedPictureWidth  = m_pc_Picture->GetWidth()  *m_i_ZoomFactor;
      i_ZoomedPictureHeight = m_pc_Picture->GetHeight() *m_i_ZoomFactor;
   }
   else
   {
      int i_Smaller;
      i_Smaller = - m_i_ZoomFactor;

      i_ZoomedPictureWidth  = m_pc_Picture->GetWidth()  / i_Smaller;
      i_ZoomedPictureHeight = m_pc_Picture->GetHeight() / i_Smaller;
   }


   m_b_PreventRedraw = true;

   // calculate Bottom scrollbar position
   CalculatePosition(i_NewWidth,
                     i_OldWidth,
                     m_BottomScrollbar,
                     m_b_XHalfPixel,
                     i_ZoomedPictureWidth,
                     m_i_LeftStartPixel ,
                     m_i_OutputAreaZoomedWidth);

   m_b_PreventRedraw = false;

   // calculate right scrollbar position
   CalculatePosition(i_NewHeight,
                     i_OldHeight,
                     m_RightScrollbar,
                     m_b_YHalfPixel,
                     i_ZoomedPictureHeight,
                     m_i_TopStartPixel ,
                     m_i_OutputAreaZoomedHeight);

}

//===========================================================================
//
//===========================================================================


void ImageFrame::CalculatePosition(int i_NewDimension,
                          int i_OldDimension,
                          QScrollBar *pq_Scrollbar,
                          bool &b_HalfPixelDisplay,
                          int i_ImageDimension,
                          int &i_TopDisplayPixel,
                          int &i_FullScreenPixel
                         )
{
   int i_HalfScreenPixel;
   int i_Smaller;
   
   if(pq_Scrollbar == NULL)
   {
      return;
   }
   

   i_Smaller = - m_i_ZoomFactor;

   if(i_NewDimension < i_OldDimension)
   {
      // smaller output dimension
      // keep the same leftstartpixel


      // initialize with false;
      b_HalfPixelDisplay = false;

      if(m_i_ZoomFactor >= 1)
      {
         i_FullScreenPixel = i_NewDimension / m_i_ZoomFactor;
         if(m_i_ZoomFactor >1)
         {
            i_HalfScreenPixel = i_NewDimension % m_i_ZoomFactor;

            if(i_HalfScreenPixel != 0)
            {
               b_HalfPixelDisplay = false;
            }
         } // if zoomfactor >1
      }
      else // zoomfactor < 0
      {
         i_FullScreenPixel = i_NewDimension * (i_Smaller);
         i_HalfScreenPixel = 0;
         b_HalfPixelDisplay = false;

      }

      if(i_TopDisplayPixel == 0)
      {
         // scroll inactive
         //is new area to small ?
         if(i_FullScreenPixel < i_ImageDimension)
         {
            // yes enable scroll
            SetScrollbar(pq_Scrollbar,
                         i_ImageDimension,
                         GetZoomedSize(i_TopDisplayPixel,m_i_ZoomFactor),
                         GetZoomedSize(i_FullScreenPixel,m_i_ZoomFactor)
                        );
         }
         else
         {
            // remain with unscrolled view
            SetScrollbar(pq_Scrollbar,1,0,1);
         }
      }
      else // scroll bas yet enabled
      {
         // set the new scroll Range
            SetScrollbar(pq_Scrollbar,
                         i_ImageDimension,
                         GetZoomedSize(i_TopDisplayPixel,m_i_ZoomFactor),
                         GetZoomedSize(i_FullScreenPixel,m_i_ZoomFactor)
                        );
      }



   } // if smaller
   else
   {
      // larger
      // initialize with false;
      b_HalfPixelDisplay = false;

      if(m_i_ZoomFactor >=1)
      {
         i_FullScreenPixel = i_NewDimension / m_i_ZoomFactor;
         if(m_i_ZoomFactor >1)
         {
            i_HalfScreenPixel = i_NewDimension % m_i_ZoomFactor;

            if(i_HalfScreenPixel != 0)
            {
               b_HalfPixelDisplay = false;
            }
         } // if zoomfactor >1
      }
      else // smaller
      {
         i_FullScreenPixel = i_NewDimension * i_Smaller;
         i_HalfScreenPixel = 0;
         b_HalfPixelDisplay = false;
      
      }

      if( (i_FullScreenPixel + i_TopDisplayPixel) >i_ImageDimension)
      {
          // behind image there is space
          if(i_TopDisplayPixel > 0)
          {
              // try to move the top position down
              i_TopDisplayPixel = i_ImageDimension - i_FullScreenPixel;
              if(i_TopDisplayPixel <0)
              {
                  i_TopDisplayPixel =0;
              }
          }
      }  // room behind image

      if(i_TopDisplayPixel == 0)
      {
         // scroll inactive
         //is new area to small ?
         if(i_FullScreenPixel < i_ImageDimension)
         {
            // yes enable scroll
            SetScrollbar(pq_Scrollbar,
                         i_ImageDimension,
                         GetZoomedSize(i_TopDisplayPixel ,m_i_ZoomFactor),
                         GetZoomedSize(i_FullScreenPixel ,m_i_ZoomFactor)
                        );
         }
         else
         {
            // remain with unscrolled view
            SetScrollbar(pq_Scrollbar,1,0,1);
         }
      }
      else // scroll bas yet enabled
      {
         // set the new scroll Range
            SetScrollbar(pq_Scrollbar,
                         i_ImageDimension,
                         GetZoomedSize(i_TopDisplayPixel ,m_i_ZoomFactor),
                         GetZoomedSize(i_FullScreenPixel ,m_i_ZoomFactor)
                        );
      }
   } // else larger

} // CalculatePosition()

//===========================================================================
//
//===========================================================================


void ImageFrame::SetScrollbar(QScrollBar *pq_Scrollbar,
                  int i_ImageSize,
                  int i_TopPixel,
                  int i_VisibleSize)
{
//fprintf(stderr,"SetScrollbar Image =%d, top =%d, Visible = %d\n", i_ImageSize, i_TopPixel,  i_VisibleSize);
   if(pq_Scrollbar == NULL)
   {
      return;
   }

   if(i_ImageSize <= i_VisibleSize)
   {
      // make a disabled scrollbar

      pq_Scrollbar->setValue(0);
      pq_Scrollbar->setRange(0,0);
      pq_Scrollbar->setSteps(1,1);
   }
   else
   {
      // make a real scrollbar

      pq_Scrollbar->setValue(0);
      pq_Scrollbar->setRange(0,i_ImageSize-i_VisibleSize);
      pq_Scrollbar->setSteps(1,i_VisibleSize);
      pq_Scrollbar->setValue(i_TopPixel);

   }

} // SetScrollbar()


//===========================================================================
//
//===========================================================================

void ImageFrame::ScrollbarBottomMoved(int i_NewValue)
{

//   fprintf(stderr,"Scrollbar bottom to %d / Zoom =%d\n",i_NewValue,m_i_ZoomFactor);

   if(m_i_ZoomFactor >=1)
   {
      m_i_LeftStartPixel = i_NewValue / m_i_ZoomFactor;
   }
   else
   {
      m_i_LeftStartPixel = i_NewValue * (-m_i_ZoomFactor);
   }

//   fprintf(stderr,"Bottom Range %d - %d\n",m_i_LeftStartPixel, m_i_LeftStartPixel + m_i_OutputAreaZoomedWidth);


   RedrawAll();
}


//===========================================================================
//
//===========================================================================

void ImageFrame::ScrollbarRightMoved(int i_NewValue)
{
//   fprintf(stderr,"Scrollbar right to %d\n",i_NewValue);

   if(m_i_ZoomFactor >=1)
   {
      m_i_TopStartPixel = i_NewValue / m_i_ZoomFactor;
   }
   else
   {
      m_i_TopStartPixel = i_NewValue * (- m_i_ZoomFactor) ;
   }


/*
   fprintf(stderr,"Range %d - %d \n",
                  m_i_TopStartPixel,
                  m_i_TopStartPixel + m_i_OutputAreaZoomedHeight);
*/
   RedrawAll();
}


//===========================================================================
//
//===========================================================================


void ImageFrame::CreateDummyImage()
{
   PixelBuffer * pc_Safety;
   PixelBuffer * pc_Selection;
   int i_X;
   int i_Y;
   int i;

   int i_MaxX;
   int i_MaxY;


   i_MaxX = 333;
   i_MaxY = 432;

   // DUMMY
   // install a test picture
   m_pc_Picture = new PixelBuffer(i_MaxX,i_MaxY,32,true,true);

   for(i_Y =0; i_Y < i_MaxY; i_Y ++)
   {
      for(i_X =0; i_X < i_MaxX; i_X ++)
      {
          m_pc_Picture->SetRaw(i_X,i_Y, PIXEL_MAKE(255, i_Y/2, i_X/2));
      }
   }

   for(i_Y =0; i_Y < i_MaxY; i_Y +=25)
   {
      for(i_X =0; i_X < i_MaxX; i_X ++)
      {
          m_pc_Picture->SetRaw(i_X,i_Y, PIXEL_MAKE(255, 255, 0));
      }
   }

   for(i_Y =0; i_Y < i_MaxY; i_Y ++)
   {
      for(i_X =0; i_X < i_MaxX; i_X += 25)
      {
          m_pc_Picture->SetRaw(i_X,i_Y,PIXEL_MAKE(255, 0, 255));
      }
   }

   for(i =100; i< 120; i ++)
   {
      m_pc_Picture->SetRaw(i,0,PIXEL_MAKE(0, 255, 255));
      m_pc_Picture->SetRaw(0,i,PIXEL_MAKE(0, 255, 255));
      m_pc_Picture->SetRaw(i,i_MaxY-1,PIXEL_MAKE(0, 255, 255));
      m_pc_Picture->SetRaw(i_MaxX-1,i,PIXEL_MAKE(0, 255, 255));
   }
   m_pc_GlobalStorage->m_pc_MainImage = m_pc_Picture;

   pc_Safety    = m_pc_Picture->GetSafetyMask();
   pc_Selection = m_pc_Picture->GetSelectionMask();

   for(i_X=22;i_X< 200; i_X ++)
   {
       for(i=12;i<34;i++)
       {
          pc_Safety->SetRaw(i_X,i_X+i,1);
          pc_Selection->SetRaw(212-i_X,i_X+i,1);
       }
   }
   //m_pc_Zoomer->SetSafetyMaskVisible(true);
   m_pc_Zoomer->SetSafetyMaskColor(0xFFBB88);

   //m_pc_Zoomer->SetSelectionMaskVisible(true);
   //m_pc_Zoomer->SetSelectionMaskColor(0x88FFBB);


   pc_Safety    = m_pc_Picture->GetSafetyMask();
   pc_Selection = m_pc_Picture->GetSelectionMask();


} // CreateDummyImage

//===========================================================================
//
//===========================================================================



void * ImageFrame::ExposeCB(void *ThisData, void * CallData)
{
   ImageFrame *pc_This;
   pc_This = (ImageFrame *) ThisData;
   pc_This->Expose(CallData);
   return NULL;
}

void ImageFrame::Expose(void *CallData)
{
   QPaintEvent *theEvent ;
   RectRegion   c_DestinationRegion;
   RectRegion   c_RealZoomedRegion;
   RectRegion   c_MaxmimumRegion;
   QRect        s_ModifiedRect;


   if(CallData == NULL)
   {
      return;
   }

   theEvent =  (QPaintEvent *) CallData;
   s_ModifiedRect = theEvent->rect();

   c_DestinationRegion = RectRegion( s_ModifiedRect.x(),
                                     s_ModifiedRect.y(),
                                     s_ModifiedRect.width(),
                                     s_ModifiedRect.height()
                                   );
   c_MaxmimumRegion = CalculateMaximumPossibleZoomArea();

   // now prepare the zoomer to redraw the image;

   m_pc_Zoomer-> SetSourceBuffer(m_pc_Picture);
   m_pc_Zoomer-> SetSourceLeftTop( m_i_LeftStartPixel,m_i_TopStartPixel);
   m_pc_Zoomer-> SetZoomFactor(m_i_ZoomFactor);
   m_pc_Zoomer-> SetDestinationImage(m_DrawingAreaWidget->GetImage());
   m_pc_Zoomer-> SetDestinationArea(c_DestinationRegion);
   m_pc_Zoomer-> SetMaximumPossibleDestinationArea(c_MaxmimumRegion);

   if(m_b_PreventRedraw)
   {
      return;
   }

   // zoom from PixelBuffer into the image
   m_pc_Zoomer->DoZoom();

   c_RealZoomedRegion = m_pc_Zoomer->GetZoomedOutputArea();
/*
   fprintf(stderr,"Zoomed area is");
   c_RealZoomedRegion.printf();
   fprintf(stderr,"\n");
*/
   // create a painter object
   QPainter painter(m_DrawingAreaWidget);

   // draw the zoomed buffer
   if(c_RealZoomedRegion.IsEmpty())
   {
      // fill white
   }
   else
   {
      painter.drawImage(s_ModifiedRect.x(),s_ModifiedRect.y(),    // destination
                     * (m_DrawingAreaWidget->GetImage()),
                        c_RealZoomedRegion.GetStartX(),c_RealZoomedRegion.GetStartY(),     // image start
                        c_RealZoomedRegion.GetWidth(),c_RealZoomedRegion.GetHeight(),
                        0);
      
   }
}   // Expose()

//===========================================================================
//
//===========================================================================

void ImageFrame::RedrawAll(bool b_EraseBackground )
{
   RectRegion  c_DestinationRegion;

   c_DestinationRegion = RectRegion( 0,
                                     0,
                                     m_DrawingAreaWidget->GetWidth(),
                                     m_DrawingAreaWidget->GetHeight()
                                   );

   // erase the entire background
   if(b_EraseBackground)
   {
      EraseBackground();
   }
      
   // redraw only the image region                                  
   RedrawWindowRegion(c_DestinationRegion);
} // RedrawAll()

//===========================================================================
//
//===========================================================================

void ImageFrame::RedrawWindowRegion(RectRegion c_DestinationRegion)
{
   RectRegion  c_RealZoomedRegion;
   RectRegion  c_MaximumRegion;

   // now prepare the zoomer to redraw the image;

   c_MaximumRegion = CalculateMaximumPossibleZoomArea();

   m_pc_Zoomer-> SetSourceBuffer(m_pc_Picture);
   m_pc_Zoomer-> SetSourceLeftTop( m_i_LeftStartPixel,m_i_TopStartPixel);
   m_pc_Zoomer-> SetZoomFactor(m_i_ZoomFactor);
   m_pc_Zoomer-> SetDestinationImage(m_DrawingAreaWidget->GetImage());
   m_pc_Zoomer-> SetDestinationArea(c_DestinationRegion);
   m_pc_Zoomer-> SetMaximumPossibleDestinationArea(c_MaximumRegion);

   if(m_b_PreventRedraw)
   {
      return;
   }


   // zoom from PixelBuffer into the image
   m_pc_Zoomer->DoZoom();
   c_RealZoomedRegion = m_pc_Zoomer->GetZoomedOutputArea();

   QPainter painter(m_DrawingAreaWidget);


   // draw the zoomed buffer
   if(c_RealZoomedRegion.IsEmpty())
   {
   }
   else
   {
      painter.drawImage(c_RealZoomedRegion.GetStartX(),c_RealZoomedRegion.GetStartY(),    // destination
                     * (m_DrawingAreaWidget->GetImage()),
                        c_RealZoomedRegion.GetStartX(),c_RealZoomedRegion.GetStartY(),     // image start
                        c_RealZoomedRegion.GetWidth(),c_RealZoomedRegion.GetHeight(),
                        0);
   }


} // RedrawWindowRegion()

//===========================================================================
//
//===========================================================================

void ImageFrame::ReplacePixelbuffer(PixelBuffer *pc_NewBuffer)
{
   PixelBuffer *pc_Old;


   pc_Old = m_pc_Picture;

   // put in the new buffer
   m_pc_Picture = pc_NewBuffer;

   // tell it also to the zoom objekt
   m_pc_GlobalStorage->m_pc_Zoom->SetSourceBuffer(pc_NewBuffer);
   m_pc_GlobalStorage->m_pc_MainImage = pc_NewBuffer;

   // put scrolling to upper left
   m_i_LeftStartPixel =0;
   m_i_TopStartPixel  =0;
   Resize(NULL);
   Expose(NULL);

   delete pc_Old;

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageNewBuffer");

} // ReplacePixelbuffer


//===========================================================================
//
//===========================================================================

void ImageFrame::ChangeImageDimensions()
{
   // tell it also to the zoom objekt
   m_pc_GlobalStorage->m_pc_Zoom->SetSourceBuffer(m_pc_Picture);

   // put scrolling to upper left
   m_i_LeftStartPixel =0;
   m_i_TopStartPixel  =0;
   Resize(NULL);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageResize");

} // ChangeImageDimensions

//===========================================================================
//
//===========================================================================

PixelBuffer *ImageFrame::GetActualPixelBuffer()
{
   return m_pc_Picture;

} // GetActualPixelBuffer()


//===========================================================================
//
//===========================================================================

void ImageFrame::CalculateZoomRegionChange(int i_NewZoomFactor,
                               int i_OldZoomFactor,
                               int i_DrawingAreaSize,
                               int i_ImageSize,
                               int &i_StartPixel,
                               int &i_OutputAreaZoomedSize,
                               QScrollBar *Scrollbar)
{
   int i_OldFullSize;
   int i_NewFullSize;
   int i_NewPixelInArea;
   int i_Rest;

   // calculate new position of bottom scrollbar
   if(i_NewZoomFactor > i_OldZoomFactor)
   {
      // zoom in


      i_OldFullSize = GetZoomedSize(i_ImageSize,i_OldZoomFactor);
      i_NewFullSize = GetZoomedSize(i_ImageSize,i_NewZoomFactor);
          
      i_NewPixelInArea = GetZoomedPixelForArea(i_DrawingAreaSize,
                                               i_NewZoomFactor,
                                               i_Rest);

      if(i_NewFullSize < i_DrawingAreaSize)
      {
         // new size fits fully in the drawing area
         i_StartPixel =0;

      }
      else // new full is larger and must get scrolled
      {
         // center to the middle
         int i_OldMiddlePoint;
         if(i_OldFullSize < i_DrawingAreaSize)
         {
            i_OldMiddlePoint = i_ImageSize /2;
         }
         else
         {
// fprintf(stderr,"old m_i_StartPixel =%d\n",i_StartPixel);
// fprintf(stderr,"i_OutputAreaZoomedWidth =%d\n",i_OutputAreaZoomedSize);
            i_OldMiddlePoint = i_StartPixel+ i_OutputAreaZoomedSize/2;

         }
// fprintf(stderr,"i_OldMiddlePoint =%d\n",i_OldMiddlePoint);
         i_StartPixel = i_OldMiddlePoint -(i_NewPixelInArea/2);
// fprintf(stderr,"m_i_LeftStartPixel =%d\n",i_StartPixel);
      } // else must be scrolled

      SetScrollbar(Scrollbar,
                   i_NewFullSize,
                   GetZoomedSize(i_StartPixel,i_NewZoomFactor),
                   GetZoomedSize(i_NewPixelInArea,i_NewZoomFactor)
                  );
      i_OutputAreaZoomedSize = i_NewPixelInArea;

   }
   else if (i_NewZoomFactor < i_OldZoomFactor)
   {
      // zoom out

      i_OldFullSize = GetZoomedSize(i_ImageSize,i_OldZoomFactor);
      i_NewFullSize = GetZoomedSize(i_ImageSize,i_NewZoomFactor);

      i_NewPixelInArea = GetZoomedPixelForArea(i_DrawingAreaSize,
                                               i_NewZoomFactor,
                                               i_Rest);

      if(i_NewFullSize < i_DrawingAreaSize)
      {
         // new size fits fully in the drawing area
         i_StartPixel =0;

      }
      else // new full is larger and must get scrolled
      {
         // center to the middle
         int i_OldMiddlePoint;
         if(i_OldFullSize < i_DrawingAreaSize)
         {
            i_OldMiddlePoint = i_ImageSize /2;
         }
         else
         {
// fprintf(stderr,"old i_StartPixel =%d\n",i_StartPixel);
// fprintf(stderr,"m_i_OutputAreaZoomedSize =%d\n",i_OutputAreaZoomedSize);
            i_OldMiddlePoint = i_StartPixel+ i_OutputAreaZoomedSize/2;

         }
// fprintf(stderr,"i_OldMiddlePoint =%d\n",i_OldMiddlePoint);
         i_StartPixel = i_OldMiddlePoint -(i_NewPixelInArea/2);
         if(i_StartPixel<0)
         {
            i_StartPixel =0;
         }
// fprintf(stderr,"i_StartPixel =%d\n",i_StartPixel);
      } // else must be scrolled

      SetScrollbar(Scrollbar,
                   i_NewFullSize,
                   GetZoomedSize(i_StartPixel,i_NewZoomFactor),
                   GetZoomedSize(i_NewPixelInArea,i_NewZoomFactor)
                  );
      i_OutputAreaZoomedSize = i_NewPixelInArea;

   }


} // CalculateZoomRegionChange()

//===========================================================================
//
//===========================================================================

void ImageFrame::SetZoomRegionChange(int i_NewZoomFactor,
                               int i_OldZoomFactor,
                               int i_DrawingAreaSize,
                               int i_ImageSize,
                               int &i_StartPixel,
                               int &i_OutputAreaZoomedSize,
                               QScrollBar *Scrollbar)
{
   int i_OldFullSize;
   int i_NewFullSize;
   int i_NewPixelInArea;
   int i_Rest;


   i_OldFullSize = GetZoomedSize(i_ImageSize,i_OldZoomFactor);
   i_NewFullSize = GetZoomedSize(i_ImageSize,i_NewZoomFactor);

   i_NewPixelInArea = GetZoomedPixelForArea(i_DrawingAreaSize,
                                            i_NewZoomFactor,
                                            i_Rest);

   if(i_NewFullSize < i_DrawingAreaSize)
   {
      // new size fits fully in the drawing area
      i_StartPixel =0;

   }
   else // new full is larger and must get scrolled
   {
      if( (i_StartPixel + i_NewPixelInArea ) > i_ImageSize)
      {
         i_StartPixel = i_ImageSize - i_NewPixelInArea;
         if(i_StartPixel<0)
         {
            i_StartPixel =0;
         }
      }
                  
   } // else must be scrolled


   SetScrollbar(Scrollbar,
                   i_NewFullSize,
                   GetZoomedSize(i_StartPixel,i_NewZoomFactor),
                   GetZoomedSize(i_NewPixelInArea,i_NewZoomFactor)
                  );
   i_OutputAreaZoomedSize = i_NewPixelInArea;



} // SetZoomRegionChange()

//===========================================================================
//
//===========================================================================


void ImageFrame::SetZoomFactor(int i_NewZoomFactor)
{
   int i_OldZoomFactor ;

   i_OldZoomFactor = m_i_ZoomFactor;
   m_i_ZoomFactor = i_NewZoomFactor;

   m_b_PreventRedraw = true;
   // calculate for the bottom scrollbar
   CalculateZoomRegionChange(i_NewZoomFactor,
                             i_OldZoomFactor,
                             m_DrawingAreaWidget->GetWidth(),
			     m_pc_Picture->GetWidth(),
                             m_i_LeftStartPixel,
                             m_i_OutputAreaZoomedWidth,
                             m_BottomScrollbar);

   // calculate for the right scrollbar
   CalculateZoomRegionChange(i_NewZoomFactor,
                             i_OldZoomFactor,
                             m_DrawingAreaWidget->GetHeight(),
			     m_pc_Picture->GetHeight(),
                             m_i_TopStartPixel,
                             m_i_OutputAreaZoomedHeight,
                             m_RightScrollbar);


   m_b_PreventRedraw = false;
     

   EraseBackground();
   RedrawAll();
} // SetZoomFactor()


//===========================================================================
//
//===========================================================================

int ImageFrame::GetZoomedSize(int i_Size,int i_ZoomFactor)
{
   if(i_ZoomFactor >0)
   {
      return i_Size * i_ZoomFactor;
   }
   else
   {
      return i_Size / (-i_ZoomFactor);
   }
} // GetZoomedSize()

//===========================================================================
//
//===========================================================================

int ImageFrame::DivideByZoom(int i_Size,int i_ZoomFactor)
{
   if(i_ZoomFactor >0)
   {
      return i_Size / i_ZoomFactor;
   }
   else
   {
      return i_Size * (-i_ZoomFactor);
   }
} // DivideByZoom()

//===========================================================================
//
//===========================================================================


int ImageFrame::GetZoomedPixelForArea(int i_Area,int i_ZoomFactor,int &i_Rest)
{
   if(i_ZoomFactor >0)
   {
      i_Rest = i_Area % i_ZoomFactor;
      return i_Area / i_ZoomFactor;
   }
   else
   {
      i_Rest=0;
      return i_Area * (-i_ZoomFactor);
   }
} // GetZoomedPixelForArea()

//===========================================================================
//
//===========================================================================

int ImageFrame::GetZoomFactor()
{
   return m_i_ZoomFactor;
} // GetZoomFactor()

//===========================================================================
//
//===========================================================================

void ImageFrame::RedrawBufferRegion(RectRegion c_BufferRegion)
{
   RectRegion c_WindowRegion;
   int        i_StartX;
   int        i_StartY;
   int        i_EndX;
   int        i_EndY;
   int        i_WindowStartX;
   int        i_WindowStartY;
   int        i_WindowEndX;
   int        i_WindowEndY;

// fprintf(stderr,"RedrawBufferRegion ");
// c_BufferRegion.printf();
// fprintf(stderr,"\n");

   // get the coordinates
   i_StartX = c_BufferRegion.GetStartX();
   i_StartY = c_BufferRegion.GetStartY();
   i_EndX = c_BufferRegion.GetEndX();
   i_EndY = c_BufferRegion.GetEndY();

   // increase end coordiante by 1
   i_EndX ++;
   i_EndY ++;

   // calculate from buffer into window coordiantes
   m_pc_Zoomer->GetWindowCoordinate(i_StartX,i_StartY,
                                    i_WindowStartX,i_WindowStartY,
                                    true);

   m_pc_Zoomer->GetWindowCoordinate(i_EndX,i_EndY,
                                    i_WindowEndX,i_WindowEndY,
                                    true);

   // set coordinates in structure
   c_WindowRegion.SetXYXY(i_WindowStartX,i_WindowStartY,
                          i_WindowEndX  ,i_WindowEndY   );

// fprintf(stderr,"Window Region is ");
// c_WindowRegion.printf();
// fprintf(stderr,"\n");

   RedrawWindowRegion(c_WindowRegion);

} // RedrawBufferRegion()


//===========================================================================
//
//===========================================================================

RectRegion ImageFrame::CalculateMaximumPossibleZoomArea()
{
   RectRegion c_ReturnRegion;
   int        i_Width;
   int        i_Height;
   int        i_EndX;
   int        i_EndY;

   i_Width   = m_DrawingAreaWidget->GetWidth();
   i_Height  = m_DrawingAreaWidget->GetHeight();
   i_Width   = DivideByZoom(i_Width,m_i_ZoomFactor);
   i_Height  = DivideByZoom(i_Height,m_i_ZoomFactor);
   i_EndX    = m_i_LeftStartPixel + i_Width;
   i_EndY    = m_i_TopStartPixel  + i_Height;

   if(i_EndX >= m_pc_Picture->GetWidth())
   {
      i_EndX = m_pc_Picture->GetWidth() -1;
   }
   if(i_EndY >= m_pc_Picture->GetHeight())
   {
      i_EndY = m_pc_Picture->GetHeight() -1;
   }
   c_ReturnRegion.SetXYXY(m_i_LeftStartPixel,m_i_TopStartPixel,
                               i_EndX,i_EndY);



// fprintf(stderr,"Maximum possible =");
// c_ReturnRegion.printf();
// fprintf(stderr,"\n");

   return c_ReturnRegion;


} // CalculatedMaximumPossibleZoomArea()

//===========================================================================
//
//===========================================================================

GlobalStorage *ImageFrame::GetGlobalStorage()
{
   return m_pc_GlobalStorage;

} // GetGlobalStorage()

//===========================================================================
//
//===========================================================================
static unsigned int Pipette[32] = {
0x000000,0x0000f0,0x0001f8,0x0001fc,
0x0001fc,0x0000fe,0x0003fe,0x000ffe,
0x0007fc,0x000bfc,0x001338,0x002400,
0x004800,0x009000,0x012000,0x024000,
0x048000,0x090000,0x120000,0x140000,
0x280000,0x100000,
0,0,0,0,0,0,0,0,0,0,};
static unsigned int PipetteMask[32] = {
0x0000f0,0x0001f8,0x0003fc,0x0003fe,
0x0003fe,0x0003ff,0x001fff,0x001fff,
0x000ffe,0x001ffe,0x003ffc,0x007f38,
0x00fc00,0x01f800,0x03f000,0x07e000,
0x0fc000,0x1f8000,0x3f0000,0x3e0000,
0x6c0000,0x380000,0x100000,
0,0,0,0,0,0,0,0,0};


void ImageFrame::SetCursor(int i_CursorStyle)
{
   if(i_CursorStyle == CursorNormal)
   {
      m_DrawingAreaWidget->setCursor(QCursor(Qt::ArrowCursor));
   }
   else if(i_CursorStyle == CursorPipette)
   {
      QBitmap Cursor(32,32,(const uchar *)Pipette,true);
      QBitmap CursorMask(32,32,(const uchar *)PipetteMask,true);
      QCursor cur(Cursor,CursorMask,21,21);

      m_DrawingAreaWidget->setCursor(cur);
   }
} // SetCursor

//===========================================================================
//
//===========================================================================


void ImageFrame::SetZoomAndRegion(int i_NewZoomFactor,RectRegion c_VisibleRegion)
{
   int i_OldZoomFactor ;

   i_OldZoomFactor = m_i_ZoomFactor;
   m_i_ZoomFactor = i_NewZoomFactor;


   m_i_LeftStartPixel = c_VisibleRegion.GetStartX();
   m_i_TopStartPixel  = c_VisibleRegion.GetStartY();
   
   
   m_b_PreventRedraw = true;
   // calculate for the bottom scrollbar
   SetZoomRegionChange(i_NewZoomFactor,
                             i_OldZoomFactor,
                             m_DrawingAreaWidget->GetWidth(),
                             m_pc_Picture->GetWidth(),
                             m_i_LeftStartPixel,
                             m_i_OutputAreaZoomedWidth,
                             m_BottomScrollbar);

   // calculate for the right scrollbar
   SetZoomRegionChange(i_NewZoomFactor,
                             i_OldZoomFactor,
                             m_DrawingAreaWidget->GetHeight(),
			                    m_pc_Picture->GetHeight(),
                             m_i_TopStartPixel,
                             m_i_OutputAreaZoomedHeight,
                             m_RightScrollbar);


   m_b_PreventRedraw = false;

   EraseBackground();
   RedrawAll();
} //  SetZoomAndRegion()

//===========================================================================
//
//===========================================================================

void ImageFrame::EraseBackground()
{
   m_DrawingAreaWidget->erase();
} // EraseBackground()
   
//===========================================================================
//
//===========================================================================

void ImageFrame::SetUndoText(QString qs_Text)
{
   if(m_UndoButton != NULL)
   {
      m_UndoButton->setText(qs_Text);
   }
}  // SetUndoText()

//===========================================================================
//
//===========================================================================


