// ImageFrame.h
// Dialog, which contains an image

#ifndef _IMAGE_FRAME_H
#define _IMAGE_FRAME_H


#include <qwidget.h>

#include <qapplication.h>
#include <qmainwindow.h>
#include <qscrollbar.h>
#include <qpushbutton.h>
#include <qaccel.h>

#include "DrawingArea.h"

#include "PixelBuffer.h"

#include "Zoom.h"
#include "GlobalStorage.h"



class ImageFrame : public QWidget
{
  Q_OBJECT
public:
   ImageFrame(QWidget *parent,
                  QApplication *mainApplication,
                  void *data,
                  int i_Flags=0,
                  int i_SizeX=0,
                  int i_SizeY=0
                  );

   ~ImageFrame();

   enum
   {
      FixedSize    = 1,
      NoSelection  = 2,
      NoPixelImage = 4,
      UndoButton   = 8
   };


public slots:
    void ScrollbarBottomMoved(int i_NewPosition);
    void ScrollbarRightMoved(int i_NewPosition);
    void UndoButtonClicked();

signals:
   void	UndoClicked();

public:

   static void * ResizeCB(void *ThisData, void * CallData);
   static void * ExposeCB(void *ThisData, void * CallData);

   void RedrawAll(bool b_EraseBackground = false);
   void RedrawWindowRegion(RectRegion c_WindowRegion);
   void RedrawBufferRegion(RectRegion c_BufferRegion);

   void ReplacePixelbuffer(PixelBuffer *pc_NewBuffer);
   PixelBuffer *GetActualPixelBuffer();
   void ChangeImageDimensions();
   void SetZoomFactor(int i_ZoomFactor);
   void SetZoomAndRegion(int i_ZoomFactor,RectRegion c_VisibleRegion);
   int  GetZoomFactor();
   GlobalStorage *GetGlobalStorage();
   
   enum
   {
      CursorNormal,
      CursorPipette
   };
   void SetCursor(int CursorStyle);
   void SetUndoText(QString qs_Text);

private:
   void Resize(void *CallData);
   void Expose(void *CallData);

   void CalculatePosition(int i_NewDimension,
                          int i_OldDimension,
                          QScrollBar *pq_Scrollbar,
                          bool &b_HalfPixelDispay,
                          int i_ImageDimension,
                          int &i_TopDisplayPixel,
                          int &i_FullScreenPixel
                         );

   void SetScrollbar(QScrollBar *pq_Scrollbar,
                  int i_ImageSize,
                  int i_TopPixel,
                  int i_VisibleSize);



   void CreateMenu();
   void CreateDummyImage();
   
   int GetZoomedSize(int i_Size,int i_ZoomFactor);
   int DivideByZoom(int i_Size,int i_ZoomFactor);
   int GetZoomedPixelForArea(int i_Area,int i_ZoomFactor,int &i_Rest);
   void CalculateZoomRegionChange(int i_NewZoomFactor,
                               int i_OldZoomFactor,
                               int i_DrawingAreaSize,
                               int i_ImageSize,
                               int &i_StartPixel,
                               int &i_OutputAreaZoomedSize,
                               QScrollBar *Scrollbar);
   void SetZoomRegionChange(int i_NewZoomFactor,
                               int i_OldZoomFactor,
                               int i_DrawingAreaSize,
                               int i_ImageSize,
                               int &i_StartPixel,
                               int &i_OutputAreaZoomedSize,
                               QScrollBar *Scrollbar);
   RectRegion CalculateMaximumPossibleZoomArea();
   void EraseBackground();

protected:
   DrawingArea *m_DrawingAreaWidget;
   QScrollBar  *m_BottomScrollbar;
   QScrollBar  *m_RightScrollbar;
   QPushButton *m_UndoButton;
   QAccel      *m_UndoAccelerator;


   int m_i_ZoomFactor;
   int m_i_LeftStartPixel;
   int m_i_TopStartPixel;

   int m_i_OutputAreaZoomedWidth;
   int m_i_OutputAreaZoomedHeight;

   bool m_b_XHalfPixel;
   bool m_b_YHalfPixel;


   PixelBuffer * m_pc_Picture;
   Zoom        *m_pc_Zoomer;

   bool m_b_PreventRedraw;
   bool m_b_FixedSize;
   
   

   GlobalStorage *m_pc_GlobalStorage;


};


#endif // _IMAGE_FRAME_H
