
#include <stdlib.h>

#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qsizepolicy.h>

#include "MainFrame.h"
#include "FileLoader.h"
#include "GlobalStorage.h"
#include "ColorSelectBox.h"
#include "QtUtilities.h"
#include "StatusBar.h"
#include "Undo.h"
#include "ApplicationFrame.h"
#include "NewImageBox.h"
#include "Selection.h"
#include "QtUtilities.h"
#include "DragAndDropConstants.h"

//===========================================================================
//
//===========================================================================

MainFrame::MainFrame(QWidget *parent,
                  QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame)
   : QMainWindow( parent,
                 "MainFrameFrame",
                 WType_TopLevel )

{
   CreateSimple(mainApplication,pc_ApplicationFrame);

   // remember the changes for undo
   DoChangedProcessing();

} // MainFrame constructor

//===========================================================================
//
//===========================================================================

MainFrame::MainFrame(QWidget *parent,
                  QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame,
                  int i_Width, int i_Height)
   : QMainWindow( parent,
                 "MainFrameFrame",
                 WType_TopLevel )

{
   PixelBuffer *pc_EmptyImage;

   CreateSimple(mainApplication,pc_ApplicationFrame);

   pc_EmptyImage = new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);

   // replace the old by the new image
   m_pc_ImageFrame->ReplacePixelbuffer(pc_EmptyImage);

   // remember the changes for undo
   DoChangedProcessing();

} // MainFrame constructor

//===========================================================================
//
//===========================================================================

MainFrame::~MainFrame()
{
   // TODO clean everything up

   if(m_pc_GlobalStorage->m_pc_Undo != NULL)
   {
      delete m_pc_GlobalStorage->m_pc_Undo;
      m_pc_GlobalStorage->m_pc_Undo = NULL;
   }

   if(m_pc_ImageFrame != NULL)
   {
      delete m_pc_ImageFrame;
      m_pc_ImageFrame = NULL;
   }

} // ~MainFrame()

//===========================================================================
//
//===========================================================================

void MainFrame::CreateSimple(QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame)
{
   // set Titel
   setCaption("Perico");

   // i dont know why, but events must be created here to have
   // a correct drawing area
   //mainApplication->processEvents();

   // clear the filename
   m_qs_Filename ="";

   // save the application frame
   m_pc_ApplicationFrame = pc_ApplicationFrame;
   // create substructures
   m_pc_ImageFrame = new ImageFrame(this,mainApplication,NULL,ImageFrame::UndoButton);
   m_pc_GlobalStorage = m_pc_ImageFrame->GetGlobalStorage();
   connect(m_pc_ImageFrame,
           SIGNAL(UndoClicked()),
           SLOT(UndoButtonPressed())
           );
   
   setAcceptDrops(true);
   
   m_pc_GlobalStorage->m_pc_Undo = new Undo();

   // create the boxes
   m_pc_ManipulationBox = new ManipulationBox(this,m_pc_GlobalStorage);

   m_pc_ZoomBox         = new ZoomBox(this,m_pc_GlobalStorage);

   m_pc_GraphikBox      = new GraphikBox(this,m_pc_GlobalStorage);

   m_pc_PaintBox        = new PaintBox(this,m_pc_GlobalStorage);

   m_pc_ColorSelectBox  = new ColorSelectBox(this,m_pc_GlobalStorage);

   m_pc_TracerBox       = new TracerBox(this);

   m_pc_FilterBox       = new FilterBox(this,m_pc_GlobalStorage);

   m_pc_MaskBox         = new MaskBox(this,m_pc_GlobalStorage);

   m_pc_AboutBox        = new AboutBox(this);

   m_pc_GridBox         = new GridBox(this,m_pc_GlobalStorage);

   m_pc_TestObjectBox   = new TestObjectBox(this,m_pc_GlobalStorage);
   
   m_pc_CorrectionBox   = new CorrectionBox(this,m_pc_GlobalStorage);

   m_pc_RedEyeBox       = new RedEyeBox(this,m_pc_GlobalStorage);

   m_pc_MotiveBox       = new MotiveBox(this,m_pc_GlobalStorage);

   setCentralWidget(m_pc_ImageFrame);

   // create the menus
   CreateMenu();

   m_pc_GlobalStorage->m_pc_StatusBar = new StatusBar(this);
   m_pc_GlobalStorage->mainApplication = mainApplication;

    // set state of Undo menu
   SetUndoMenu();

   // add a message receiver
   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction)OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageChanged");

   // this is the default size
   this->resize(400,400 );

   DoChangedProcessing();

   // create the accelerators
   m_Accelerator = new QAccel(this);
   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_Less),
                                 this,
                                 SLOT(ZoomOutAccelPressed())
                                 );
   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_Greater),
                                 this,
                                 SLOT(ZoomInAccelPressed())
                                 );

} // CreateSimple()

//===========================================================================
//
//===========================================================================

void MainFrame::CreateMenu()
{
   QPopupMenu  *m_FileMenu;
   QPopupMenu  *m_Test1Menu;
   QPopupMenu  *m_Test2Menu;
   QPopupMenu  *m_Test3Menu;
   QPopupMenu  *m_NewMenu;

   m_NewMenu = new QPopupMenu(this);
   m_NewMenu ->insertItem("Load File",   this,SLOT(NewFileLoad() )  );
   m_NewMenu ->insertItem("Create Empty",this,SLOT(NewFileBlank() )  );

   // items for the file menu
   m_FileMenu = new QPopupMenu( this );
   m_FileMenu->insertItem("About Perico", this, SLOT( About() )  );
   m_FileMenu->insertItem("Load Image", this, SLOT( LoadFile() ) );
   m_FileMenu->insertItem("Save Image", this, SLOT( SaveFile() ) );
   m_FileMenu->insertItem("Close Image", this, SLOT( CloseFile() ) );
   m_FileMenu->insertItem("New Image", m_NewMenu );
   m_FileMenu->insertSeparator();
   m_FileMenu->insertItem("Exit All",this,SLOT(ExitFile() ));

   // Items for the work menu
   m_Test1Menu = new QPopupMenu( this );
   m_Test1Menu->insertItem("Manipulate", this,SLOT( ManipulationBoxMenu() ) );
   m_Test1Menu->insertItem("Graphik", this,SLOT( GraphikBoxMenu() ) );
   m_Test1Menu->insertItem("Paint", this,SLOT( PaintBoxMenu() ) );
   m_Test1Menu->insertItem("Filter", this,SLOT( FilterBoxMenu() ) );
   m_Test1Menu->insertItem("Correction", this,SLOT( CorrectionBoxMenu() ) );
   m_Test1Menu->insertItem("Red Eye Reduction", this,SLOT( RedEyeBoxMenu() ) );
   m_Test1Menu->insertItem("Motive", this,SLOT( MotiveBoxMenu() ) );

   // items for the view menu
   m_Test2Menu = new QPopupMenu( this );
   m_Test2Menu->insertItem("Zoom", this,SLOT( ZoomBoxMenu() ) );
   m_Test2Menu->insertItem("Mask", this,SLOT( MaskBoxMenu() ) );
   m_Test2Menu->insertItem("Color Selection", this,SLOT( ColorSelectMenu() ) );
   m_Test2Menu->insertItem("Grid", this,SLOT( GridBoxMenu() ) );

   // items for the undo menu
   m_UndoMenu = new QPopupMenu( this );
   m_i_UndoId = m_UndoMenu->insertItem("Undo", this,SLOT( UndoMenu() ) );

   m_Test3Menu = new QPopupMenu( this );
   m_Test3Menu->insertItem("Trace-Messages", this,SLOT( TracerBoxMenu() ) );
   m_Test3Menu->insertItem("Test Object", this,SLOT( TestObjectBoxMenu() ) );

   // add menus to menubars
   menuBar()->insertItem("File" , m_FileMenu );
   menuBar()->insertItem("Work", m_Test1Menu );
   menuBar()->insertItem("View", m_Test2Menu );
   menuBar()->insertItem("Undo", m_UndoMenu );
   menuBar()->insertItem("Debug", m_Test3Menu );

} // Createmenu

//===========================================================================
//
//===========================================================================

void MainFrame::About()
{
   // open the about box
   m_pc_AboutBox->show();

} // About()

//===========================================================================
//
//===========================================================================


void MainFrame::LoadFile()
{
   PixelBuffer *pc_NewImage;
   FileLoader  *mp_FileLoader;
   QString      qs_Filename;

   // create a file loader
   mp_FileLoader = new FileLoader(this);

   // open a dialog and ask for the filename
   qs_Filename = mp_FileLoader->AskForFilename(true);

   if(qs_Filename.isEmpty())
   {
      // empty name, then return
      delete mp_FileLoader;
      return;
   }

   // deactivate all selections first, to clean up selection
   // private storage and modes
   m_pc_GlobalStorage->m_pc_Selection->DeactivateSelection();
   
   // load the file
   pc_NewImage = mp_FileLoader->LoadFile(qs_Filename);
   delete mp_FileLoader;

   // success?
   if(pc_NewImage == NULL)
   {
      return ;
   }

   ReplaceNewImage(pc_NewImage,qs_Filename);

} // LoadFile

//===========================================================================
//
//===========================================================================

void MainFrame::SaveFile()
{
   FileLoader *mp_FileLoader;
   QString     qs_Filename;

   // create the file loader
   mp_FileLoader = new FileLoader(this);

   // ask for a filename
   qs_Filename = mp_FileLoader->AskForFilename(false,m_qs_Filename);

   // empty file, then finsh
   if(qs_Filename.isEmpty())
   {
      delete mp_FileLoader;
      return;
   }

   // save image as new file
   mp_FileLoader->SaveFile(qs_Filename,
                            m_pc_ImageFrame->GetActualPixelBuffer());

   delete mp_FileLoader;

} // SaveFile

//===========================================================================
//
//===========================================================================

void MainFrame::CloseFile()
{
   // tell the MainFram about it
   m_pc_ApplicationFrame->RemoveMainFrame(this);

   // delete myself
   delete this;
} // CloseFile()

//===========================================================================
//
//===========================================================================

void MainFrame::ExitFile()
{
   // tell the MainFram about it
   m_pc_ApplicationFrame->ExitCall(this);

} // ExitFile()

//===========================================================================
//
//===========================================================================

void MainFrame::GridBoxMenu()
{
   m_pc_GridBox->show();

} // GridBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::ManipulationBoxMenu()
{
   m_pc_ManipulationBox->show();

} // ManipulationBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::ZoomBoxMenu()
{
   m_pc_ZoomBox->show();

} // ZoomBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::GraphikBoxMenu()
{
   m_pc_GraphikBox->show();

} // GraphikBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::PaintBoxMenu()
{
   m_pc_PaintBox->show();

} // PaintBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::FilterBoxMenu()
{
   m_pc_FilterBox->show();

} // FilterBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::MaskBoxMenu()
{
   m_pc_MaskBox->show();

} // FilterBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::TracerBoxMenu()
{
   m_pc_TracerBox->Reload();
   m_pc_TracerBox->show();

} // TracerBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::TestObjectBoxMenu()
{
   m_pc_TestObjectBox->show();

} // TestObjectBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::ColorSelectMenu()
{
   m_pc_ColorSelectBox->show();

} // ColorSelectMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::CorrectionBoxMenu()
{
   m_pc_CorrectionBox->show();

} // CorrectionBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::RedEyeBoxMenu()
{
   m_pc_RedEyeBox->show();
   m_pc_RedEyeBox->DoActivate();

} // RedEyeBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::MotiveBoxMenu()
{
   m_pc_MotiveBox->show();

} // MotiveBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::closeEvent(QCloseEvent *)
{
   CloseFile();
} // closeEvent()

//===========================================================================
//
//===========================================================================

void MainFrame::UndoMenu()
{
   bool b_SizeChanged;

   if(m_pc_GlobalStorage->m_pc_Undo->IsUndoPossible())
   {
      m_pc_GlobalStorage->m_pc_Undo->DoUndo(
                              m_pc_GlobalStorage->m_pc_MainImage,
                              b_SizeChanged);

      // repaint area
      m_pc_ImageFrame->RedrawAll();

   }
   else if(m_pc_GlobalStorage->m_pc_Undo->IsRedoPossible())
   {
      m_pc_GlobalStorage->m_pc_Undo->DoRedo(
                              m_pc_GlobalStorage->m_pc_MainImage,
                              b_SizeChanged);

      // repaint area
      m_pc_ImageFrame->RedrawAll();

   }

   // if size changed
   if(b_SizeChanged)
   {
      // tell it to the image frame
      m_pc_ImageFrame->ChangeImageDimensions();

      // and send a message to all the other modules
      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageResize");
   }

   // set the new state of the menu
   SetUndoMenu();

} // UndoMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::UndoButtonPressed()
{
   // do the same as in the undo menu
   UndoMenu();
      
}  // UndoButtonPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::OnMessageReceiveCB(void *p_ThisData,
                                   QString qs_MessageType,
                                   void *p_Data)
{
   MainFrame *p_This;

   p_This =(MainFrame *) p_ThisData;
   p_This->OnMessageReceive(qs_MessageType,p_Data);

} // OnMessageReceiveCB()

//----------------------------------------------------------------------

void MainFrame::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   if(qs_MessageType.compare("ImageChanged") == 0)
   {
      DoChangedProcessing();
   }
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================

void MainFrame::SetUndoMenu()
{
   QString qs_Text;
   // set the menu depending on the state of the
   // undo state machine
   if(m_pc_GlobalStorage->m_pc_Undo->IsUndoPossible())
   {
      qs_Text = "Undo";
      m_pc_ImageFrame->SetUndoText(qs_Text);
      m_UndoMenu->changeItem(m_i_UndoId,qs_Text);
      m_UndoMenu->setItemEnabled(m_i_UndoId,true);
   }
   else if(m_pc_GlobalStorage->m_pc_Undo->IsRedoPossible())
   {
      qs_Text = "Redo";
      m_pc_ImageFrame->SetUndoText(qs_Text);
      m_UndoMenu->changeItem(m_i_UndoId,qs_Text);
      m_UndoMenu->setItemEnabled(m_i_UndoId,true);
   }
   else
   {
      // nothing is possible
      qs_Text ="----";
      m_pc_ImageFrame->SetUndoText(qs_Text);
      m_UndoMenu->changeItem(m_i_UndoId,qs_Text);
      m_UndoMenu->setItemEnabled(m_i_UndoId,false);
   }

} // SetUndoMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::DoChangedProcessing()
{
   // save the image for Undo
   m_pc_GlobalStorage->m_pc_Undo->SaveBufferToUndo(
                              m_pc_GlobalStorage->m_pc_MainImage);


   SetUndoMenu();
} // ChangedProcessing()

//===========================================================================
//
//===========================================================================

int MainFrame::LoadImage(QString qs_Filename)
{
   PixelBuffer *pc_NewImage;
   FileLoader  *mp_FileLoader;

   // filename must not be ampty
   if(qs_Filename.isEmpty())
   {
      return -1;
   }

   // load the files
   mp_FileLoader = new FileLoader(this);
   if(mp_FileLoader == NULL)
   {
      return -1;
   }

   pc_NewImage = mp_FileLoader->LoadFile(qs_Filename);

   // if loaded, replace the old
   if(pc_NewImage != NULL)
   {
      m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);
   }
   
   // set zoom to default values
   m_pc_ImageFrame->SetZoomFactor(1);
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ZoomChanged");

   // remember the changes for undo
   DoChangedProcessing();

   delete mp_FileLoader;

   // set caption the the new image name
   setCaption(qs_Filename);

   return 0;
} // LoadImage()

//===========================================================================
//
//===========================================================================

void MainFrame::NewFileLoad()
{
   PixelBuffer *pc_NewImage;
   FileLoader  *mp_FileLoader;
   MainFrame   *pc_Image;
   QString      qs_Filename;

   // create a file loader
   mp_FileLoader = new FileLoader(this);
   if(mp_FileLoader == NULL)
   {
      return;
   }

   // open a dialog and ask for the filename
   qs_Filename = mp_FileLoader->AskForFilename(true);

   if(qs_Filename.isEmpty())
   {
      // empty name, then return
      delete mp_FileLoader;
      return;
   }

   // load the file
   pc_NewImage = mp_FileLoader->LoadFile(qs_Filename);
   delete mp_FileLoader;

   // success?
   if(pc_NewImage == NULL)
   {
      return ;
   }

   // create a new blank image
   pc_Image = new MainFrame(0,
                               m_pc_GlobalStorage->mainApplication,
                               m_pc_ApplicationFrame,100,100);

    m_pc_ApplicationFrame ->AddMainFrame(pc_Image);

   // replace the old by the new image
   pc_Image->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);


   // remember the changes for undo
   pc_Image->DoChangedProcessing();

   // set caption the the new image name
   pc_Image->setCaption(qs_Filename);
   pc_Image->m_qs_Filename = qs_Filename;

   // show the window
   pc_Image->show();

} // NewFileLoad()

//===========================================================================
//
//===========================================================================

void MainFrame::NewFileBlank()
{
   NewImageBox *pc_NewBox;
   int          i_Result;

   // create a box to ask for dimensions
   pc_NewBox = new NewImageBox(this,m_pc_GlobalStorage);
   if(pc_NewBox == NULL)
   {
      return;
   }

   // show the box and wait until finished
   pc_NewBox->show();
   i_Result = pc_NewBox->exec();

   // if dialog was finished with OK button
   if(i_Result == QDialog::Accepted)
   {
      PixelBuffer *pc_Buffer;
      MainFrame   *pc_Image;
      int          i_NewWidth;
      int          i_NewHeight;
      int          i_RunX;
      int          i_RunY;

      // ask for new values
      i_NewWidth  = pc_NewBox->GetWidth();
      i_NewHeight = pc_NewBox->GetHeight();
      
      // create a new blank image
      pc_Image = new MainFrame(0,
                               m_pc_GlobalStorage->mainApplication,
                               m_pc_ApplicationFrame,
                               i_NewWidth,
                               i_NewHeight);
      m_pc_ApplicationFrame ->AddMainFrame(pc_Image);
      
      // set a white default image into it
      pc_Buffer = pc_Image->m_pc_GlobalStorage->m_pc_MainImage;
      for(i_RunY=0;i_RunY<i_NewHeight;i_RunY++)
      {
         for(i_RunX=0;i_RunX<i_NewWidth;i_RunX++)
         {
            pc_Buffer->Set(i_RunX,i_RunY,0xffffff);
         }
      }

      // show the window
      pc_Image->show();

   }
   delete pc_NewBox;

} // NewFileBlank()

//===========================================================================
//
//===========================================================================

ImageFrame  *MainFrame::GetImageFrame()
{
   return m_pc_ImageFrame;

} // GetImageFrame()

//===========================================================================
//
//===========================================================================

void MainFrame::ZoomInAccelPressed()
{
   m_pc_ZoomBox->ZoomInOneStep();

} // ZoomInAccelPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::ZoomOutAccelPressed()
{
   m_pc_ZoomBox->ZoomOutOneStep();

} // ZoomOutAccelPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::dragEnterEvent(QDragEnterEvent* event)
{
   QByteArray ba_Data;
   QString    qs_Source;
   bool       b_DoAccept;

   // block for checking external types
   {
      char *pc_Result;
      int  i_FormatNumber ;
      
      for(i_FormatNumber =0;;i_FormatNumber ++)
      {
         pc_Result = (char *) event->format(i_FormatNumber);
         if(pc_Result ==NULL)
         {
            break;
         }

         // accept plain text
         if(strcmp(pc_Result,"text/plain")==0)
         {
            event->accept( true );
            return;
            
         }
      }
   } // block for external types
   
   b_DoAccept = false;
   ba_Data    = event->encodedData(DRAGANDDROP_APPLICATION);


   qs_Source =  QtUtility::GetDragSourceName(&ba_Data);

   // currently only PixelBuffers are accepted
   if(qs_Source.compare( DRAGANDDROP_TYPE_PIXELBUFFER) ==0)
   {
      b_DoAccept = true;
   }

   event->accept( b_DoAccept );

} // dragEnterEvent()

//===========================================================================
//
//===========================================================================

void MainFrame::dragMoveEvent(QDragMoveEvent* event)
{
}

//===========================================================================
//
//===========================================================================

void MainFrame::dropEvent(QDropEvent* event)
{
   QByteArray ba_Data;
   QString    qs_Source;
   QString    qs_Data;

   // block for checking external types
   {
      QString qs_PlainText;
      char   *pc_Result;
      int     i_FormatNumber ;

      for(i_FormatNumber =0;;i_FormatNumber ++)
      {
         pc_Result = (char *) event->format(i_FormatNumber);
         if(pc_Result ==NULL)
         {
            break;
         }
         // accept plain text
         if(strcmp(pc_Result,"text/plain")==0)
         {
            QString qs_Start;
            ba_Data      = event->encodedData(pc_Result);
            qs_PlainText = QtUtility::GetDragSourceText(&ba_Data);

            qs_Start = qs_PlainText.left(5);

            if(qs_Start.compare("file:") ==0)
            {
               // try to load the file
               PixelBuffer *pc_Buffer;
               FileLoader  *pc_Loader;
               QString      qs_Filename;

               // remove the file: from the filename
               qs_Filename = qs_PlainText.mid(5);

               // create a file loader
               pc_Loader = new FileLoader(this);
               if(pc_Loader == NULL)
               {
                  return;
               }

               // load the file
               pc_Buffer = pc_Loader->LoadFile(qs_Filename);

               // success?
               if(pc_Buffer != NULL)
               {
                  // then replace the current
                  ReplaceNewImage( pc_Buffer, qs_Filename);
               }
               delete pc_Loader;
            } // if starts with file:
            
            return;

         } // if text/plain type
      } // for format number
   } // block for external types

   ba_Data    = event->encodedData(DRAGANDDROP_APPLICATION);

   qs_Source =  QtUtility::GetDragSourceName(&ba_Data,&qs_Data);
   if(qs_Source.compare( DRAGANDDROP_TYPE_PIXELBUFFER) ==0)
   {
      PixelBuffer *pc_ReceivedImage;
      long         l_Void;
      int          i_ScanResult;

      i_ScanResult = sscanf((const char *)qs_Data,"%lx",&l_Void );

      if(i_ScanResult ==1)
      {
         PixelBuffer *pc_NewImage;
         QPoint       p_DropPosition;
         int          i_BufferX;
         int          i_BufferY;
         
         pc_ReceivedImage = (PixelBuffer *) l_Void;
         pc_NewImage = new PixelBuffer(pc_ReceivedImage);

         // get drop position in MainFram coordiantes
         p_DropPosition = event->pos();
         // map to global coordinates
         p_DropPosition = mapToGlobal(p_DropPosition);
         // map into drawing area coordinates
         p_DropPosition = m_pc_GlobalStorage->m_pc_DrawingArea->mapFromGlobal(p_DropPosition);
         
         // make buffer coordinates out out it
         m_pc_GlobalStorage->m_pc_Zoom->GetBufferCoordinate(p_DropPosition.x(),
                                                            p_DropPosition.y(),
                                                      i_BufferX,
                                                      i_BufferY);
         // now tell the motivbox, that there is
         // a new image
         m_pc_MotiveBox->show();

         // add motiv to the buffer coordinate
         m_pc_MotiveBox->AddImageToWorkArea(pc_NewImage,
                                                i_BufferX,
                                                i_BufferY);
         // ImageDropped(pc_ReceivedImage);

      } // if we got an address
   }
} // dropEvent()

//===========================================================================
//
//===========================================================================

void MainFrame::ReplaceNewImage(PixelBuffer *pc_NewImage, QString qs_Filename)
{
   // replace the old by the new image
   m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);

   // reset the zoom factor to 1
   m_pc_ImageFrame->SetZoomFactor(1);
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                     this,
                                                     "ZoomChanged");

   // remember the changes for undo
   DoChangedProcessing();

   // set caption the the new image name
   setCaption(qs_Filename);

   m_qs_Filename = qs_Filename;


} // ReplaceNewImage()

//===========================================================================
//
//===========================================================================



