#ifndef _MAINFRAME_H
#define _MAINFRAME_H


#include <qpopupmenu.h>
#include <qaccel.h>


#include "ImageFrame.h"

#include "ManipulationBox.h"
#include "ZoomBox.h"
#include "GraphikBox.h"
#include "PaintBox.h"
#include "GlobalStorage.h"
#include "ColorSelectBox.h"
#include "FilterBox.h"
#include "TracerBox.h"
#include "MaskBox.h"
#include "AboutBox.h"
#include "GridBox.h"
#include "TestObjectBox.h"
#include "CorrectionBox.h"
#include "RedEyeBox.h"
#include "MotiveBox.h"

class ApplicationFrame;

class MainFrame : public QMainWindow
{
  Q_OBJECT

public:
   MainFrame(QWidget *parent,
                  QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame);
   MainFrame(QWidget *parent,
                  QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame,
                  int i_Width,int i_Height);
   ~MainFrame();


   void CreateMenu();
   int  LoadImage(QString qs_Filename);
   ImageFrame      *GetImageFrame();


public slots:
   void About();

   void LoadFile();
   void SaveFile();
   void CloseFile();
   void ExitFile();

   void NewFileLoad();
   void NewFileBlank();

   void ManipulationBoxMenu();
   void GraphikBoxMenu();
   void PaintBoxMenu();
   void ZoomBoxMenu();
   void ColorSelectMenu();
   void FilterBoxMenu();
   void MaskBoxMenu();
   void GridBoxMenu();
   void CorrectionBoxMenu();
   void RedEyeBoxMenu();
   void MotiveBoxMenu();

   void UndoMenu();

   void TracerBoxMenu();
   void TestObjectBoxMenu();

   void DoChangedProcessing();

   void UndoButtonPressed();
   void ZoomInAccelPressed();
   void ZoomOutAccelPressed();

protected:
   void closeEvent(QCloseEvent *);

   void dragEnterEvent(QDragEnterEvent* event);
   void dragMoveEvent(QDragMoveEvent* event);
   void dropEvent(QDropEvent* event);

private: // functions
   void CreateSimple(QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame);

     virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   static void OnMessageReceiveCB(void *p_ThisData,
                                  QString qs_MessageType,
                                  void *p_Data);

   void SetUndoMenu();
   void ReplaceNewImage(PixelBuffer *pc_NewImage, QString qs_Filename);
   
private: // variables

   ApplicationFrame *m_pc_ApplicationFrame;

   ImageFrame      *m_pc_ImageFrame;

   ManipulationBox *m_pc_ManipulationBox;
   ZoomBox         *m_pc_ZoomBox;
   GraphikBox      *m_pc_GraphikBox;
   PaintBox        *m_pc_PaintBox;
   GlobalStorage   *m_pc_GlobalStorage;
   ColorSelectBox  *m_pc_ColorSelectBox;
   TracerBox       *m_pc_TracerBox;
   FilterBox       *m_pc_FilterBox;
   MaskBox         *m_pc_MaskBox;
   AboutBox        *m_pc_AboutBox;
   GridBox         *m_pc_GridBox;
   TestObjectBox   *m_pc_TestObjectBox;
   CorrectionBox   *m_pc_CorrectionBox;
   RedEyeBox       *m_pc_RedEyeBox;
   MotiveBox       *m_pc_MotiveBox;

   QPopupMenu      *m_UndoMenu;
   QAccel          *m_Accelerator ;

   int              m_i_UndoId;
   
   QString          m_qs_Filename;

}; // class MainFrame

#endif //  _MAINFRAME_H
