// dialog for manipulate image size, mirror and rotating

#include <stdio.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qbuttongroup.h>


#include "PixelBuffer.h"
#include "ManipulationBox.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "ApplicationStorage.h"
#include "ApplicationFrame.h"
#include "MainFrame.h"
#include "Selection.h"

#define MANIPULATIONBOXLAYER "Manipulation-Frame"

ManipulationBox::ManipulationBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global)
{
   QBoxLayout    *l_Layout;

   QVGroupBox   *ScaleBox;

   QHGroupBox   *RotateBox;
   QHGroupBox   *MirrorBox;
   QVGroupBox   *CutBox;
   int           i_CharWidth;

   setCaption("Image Size Manipulation");

   m_i_CutDestination = CutDestinationReplace;
   m_i_LowerScaleEnd = -50;
   m_i_UpperScaleEnd =  50;

   m_pc_FrameLayer= m_pc_GlobalStorage->m_pc_ObjectMain->CreateLayer(MANIPULATIONBOXLAYER);
   m_pc_FrameLayer->SetVisible(false);

   m_pc_FrameRectangle = new GraphicObjectRectangle(10,10,20,20);
   m_pc_FrameRectangle->SetColor( 0x5555ff);
   m_pc_FrameLayer->AddObject(m_pc_FrameRectangle);

   i_CharWidth = g_Application->m_pc_QtUtility->GetMaximalFontWidth();
            
   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);
   ScaleBox = new QVGroupBox("Scale",m_w_TopWidget);
   l_Layout->addWidget(ScaleBox);

   // elements for the scale box
   {
      QGridLayout  *grid;
      QWidget      *GridBox;

      GridBox = new QWidget(ScaleBox);

      grid = new QGridLayout( GridBox, 2, 4, 5 );

      // edit boxes for the sizes
      m_OriginalWidth  = new QtUtilReadOnlyLineEdit(GridBox);
      m_NewWidth       = new QLineEdit(GridBox);
      m_OriginalHeight = new QtUtilReadOnlyLineEdit(GridBox);
      m_NewHeight      = new QLineEdit(GridBox);

      m_NewWidth      ->setMaximumWidth(i_CharWidth*4);
      m_NewHeight     ->setMaximumWidth(i_CharWidth*4);
      m_OriginalWidth ->setMaximumWidth(i_CharWidth*4);
      m_OriginalHeight->setMaximumWidth(i_CharWidth*4);

      // gris with all labels and edit boxes
      grid->addWidget(new QLabel("Original Width",GridBox),0,0);
      grid->addWidget(m_OriginalWidth,0,1);
      grid->addWidget(new QLabel("New Width",GridBox),0,2);
      grid->addWidget(m_NewWidth,0,3);
      grid->addWidget(new QLabel("Original Height",GridBox),1,0);
      grid->addWidget(m_OriginalHeight,1,1);
      grid->addWidget(new QLabel("New Height",GridBox),1,2);
      grid->addWidget(m_NewHeight,1,3);

      m_NewWidth->setFocus();
   }

   // scale slider within the scale box
   {
      QHBoxLayout  *SliderLayout;
      QWidget      *SliderBox;
      SliderBox = new QWidget(ScaleBox);

      m_i_CurrentSlider =  0;

      SliderLayout  = new  QHBoxLayout( SliderBox,2);
      m_ResizeSlider = new QSlider(m_i_LowerScaleEnd,
                                   m_i_UpperScaleEnd,
                                    1,
                                    m_i_CurrentSlider,
                                    Horizontal,
                                    SliderBox);
      m_ResizeSlider->setTickmarks(QSlider::Below);
      m_ResizeSlider->setTickInterval(10);
      m_ResizeSliderLabel = new QLabel("1",SliderBox);
      m_ResizeSliderLabel->setMinimumWidth(  g_Application->m_pc_QtUtility->GetMaximalFontWidth()
                                              * 3);
      SliderLayout->addWidget(m_ResizeSlider);
      SliderLayout->addWidget(m_ResizeSliderLabel);

      connect( m_ResizeSlider,
               SIGNAL(valueChanged(int)),
               SLOT(DoSliderValueChanged(int)) );
   }

   //execute button for scaling
   {
      QHBoxLayout  *ScaleExecuteLayout;
      QWidget      *ScaleExecuteBox;
      QPushButton  *ScaleExecuteButton;
      ScaleExecuteBox = new QWidget(ScaleBox);

      ScaleExecuteLayout  = new  QHBoxLayout( ScaleExecuteBox,2);

      ScaleExecuteButton = new QPushButton("Do",ScaleExecuteBox);

      // put button into the right corner
      ScaleExecuteLayout->addWidget(new QtUtilSpacer(true,false,ScaleExecuteBox) );
      ScaleExecuteLayout->addWidget(ScaleExecuteButton);

      connect( ScaleExecuteButton,
               SIGNAL(clicked()),
               SLOT( DoScale()) );

   }

   // rotate box
   RotateBox = new QHGroupBox("Rotate",m_w_TopWidget);
   {
      // 3 pixel buttons with the angels
      m_Rotate90Button  = new QUPixelButton(RotateBox,"Rotate90","90");
      m_Rotate180Button = new QUPixelButton(RotateBox,"Rotate180","180");
      m_Rotate270Button = new QUPixelButton(RotateBox,"Rotate270","270");

      connect( m_Rotate90Button ,
               SIGNAL(clicked()),
               SLOT( Rotate90() ) );
      connect( m_Rotate180Button,
               SIGNAL(clicked()),
               SLOT( Rotate180()) );
      connect( m_Rotate270Button,
               SIGNAL(clicked()),
               SLOT( Rotate270()) );
   }
   l_Layout->addWidget(RotateBox);

   // mirror box
   MirrorBox = new QHGroupBox("Mirror",m_w_TopWidget);
   {
      // 2 buttons to mirror in 2 directions
      m_MirrorXButton  = new QUPixelButton(MirrorBox,"MirrorX","Left <-> Right");
      m_MirrorYButton  = new QUPixelButton(MirrorBox,"MirrorY","Top <-> Buttom");

      connect( m_MirrorXButton ,
               SIGNAL(clicked()),
               SLOT( MirrorX() ) );
      connect( m_MirrorYButton ,
               SIGNAL(clicked()),
               SLOT( MirrorY() ) );
   }
   l_Layout->addWidget(MirrorBox);

   // box for curring subimages
   CutBox = new QVGroupBox("Cut",m_w_TopWidget);
   {
      QWidget      *w_ButtonWidget;
      QHBoxLayout  *l_ButtonLayout;
      QButtonGroup *CutDestinationBox;

      m_b_FrameModeActive = false;
      
      w_ButtonWidget = new QWidget(CutBox);

      l_ButtonLayout = new  QHBoxLayout( w_ButtonWidget,2);

      m_CutSelectionButton  = new QPushButton("Selection Mask",
                                              w_ButtonWidget);
      l_ButtonLayout->addWidget(m_CutSelectionButton);
      m_CutRectangleButton  = new QPushButton("Rectangle",
                                               w_ButtonWidget);
      l_ButtonLayout->addWidget(m_CutRectangleButton);

      connect( m_CutSelectionButton ,
               SIGNAL(clicked()),
               SLOT( CutSelection() ) );
      connect( m_CutRectangleButton ,
               SIGNAL(clicked()),
               SLOT( CutRectangle() ) );

      // block for the frame selection mode
      {

         m_b_InUpdate = false;

         m_FrameBox = new QVGroupBox("Frame Selection",CutBox);

         m_FrameActiveCheck = new QCheckBox("Enable",m_FrameBox);
         
         m_FrameGridWidget = new QWidget(m_FrameBox);

         m_FrameGridLayout = new QGridLayout( m_FrameGridWidget, 2, 4, 5 );

         // edit boxes for the sizes
         m_FrameXEdit     = new QLineEdit(m_FrameGridWidget);
         m_FrameYEdit     = new QLineEdit(m_FrameGridWidget);
         m_FrameWidthEdit = new QLineEdit(m_FrameGridWidget);
         m_FrameHeightEdit= new QLineEdit(m_FrameGridWidget);
         
         m_FrameXEdit       ->setMaximumWidth(i_CharWidth*4);
         m_FrameYEdit       ->setMaximumWidth(i_CharWidth*4);
         m_FrameWidthEdit   ->setMaximumWidth(i_CharWidth*4);
         m_FrameHeightEdit  ->setMaximumWidth(i_CharWidth*4);

         // gris with all labels and edit boxes
         m_FrameGridLayout->addWidget(new QLabel("X",m_FrameGridWidget),0,0);
         m_FrameGridLayout->addWidget(m_FrameXEdit,0,1);
         m_FrameGridLayout->addWidget(new QLabel("Y",m_FrameGridWidget),0,2);
         m_FrameGridLayout->addWidget(m_FrameYEdit,0,3);
         m_FrameGridLayout->addWidget(new QLabel("Width",m_FrameGridWidget),1,0);
         m_FrameGridLayout->addWidget(m_FrameWidthEdit,1,1);
         m_FrameGridLayout->addWidget(new QLabel("Height",m_FrameGridWidget),1,2);
         m_FrameGridLayout->addWidget(m_FrameHeightEdit,1,3);
         
         FrameCutExecuteBox = new QWidget(m_FrameBox);

         FrameCutExecuteLayout  = new  QHBoxLayout( FrameCutExecuteBox,2);

         m_FrameResetButton      = new QPushButton("Reset",FrameCutExecuteBox);
         m_FrameCutExecuteButton = new QPushButton("Do Cut",FrameCutExecuteBox);

         // put button into the right corner
         FrameCutExecuteLayout->addWidget(new QtUtilSpacer(true,false,FrameCutExecuteBox) );
         FrameCutExecuteLayout->addWidget(m_FrameResetButton);
         FrameCutExecuteLayout->addWidget(m_FrameCutExecuteButton);

         connect( m_FrameActiveCheck,
                  SIGNAL(toggled(bool)),
                  SLOT( FrameSelectionModeToggled(bool )) );
         connect( m_FrameResetButton,
                  SIGNAL(clicked()),
                  SLOT( FrameReset()) );
         connect( m_FrameCutExecuteButton,
                  SIGNAL(clicked()),
                  SLOT( FrameDoCut()) );

         // connect the edit fiels to text changed signals
         connect(m_FrameXEdit,
                 SIGNAL( textChanged (const QString &) ),
                 SLOT( FrameXTextChanged (const QString &) ));
         connect(m_FrameYEdit,
                 SIGNAL( textChanged (const QString &) ),
                 SLOT( FrameYTextChanged (const QString &) ));
         connect(m_FrameWidthEdit,
                 SIGNAL( textChanged (const QString &) ),
                 SLOT( FrameWidthTextChanged (const QString &) ));
         connect(m_FrameHeightEdit,
                 SIGNAL( textChanged (const QString &) ),
                 SLOT( FrameHeightTextChanged (const QString &) ));
      }  // block for the frame mode
               
      // mode box for the destination of the new image
      CutDestinationBox = new QButtonGroup(3,
                                    QGroupBox::Horizontal,
                                    "Destination",
                                    CutBox);

      m_CutReplaceCurrent = new QRadioButton("Replace Current",
                                             CutDestinationBox);
      m_CutCreateNew      = new QRadioButton("Create New",
                                             CutDestinationBox);

      // enable the correct mode
      switch(m_i_CutDestination)
      {
      case CutDestinationReplace:
         m_CutReplaceCurrent->setChecked(1);
         break;
      case CutDestinationNew:
         m_CutCreateNew->setChecked(1);
         break;
      default:
         break;
      }
      connect( CutDestinationBox,
               SIGNAL(clicked(int)),
               SLOT(CutDestinationChanged(int)) );

   }
   l_Layout->addWidget(CutBox);

   // set current image dimension into the edit fields
   SetImageSize();

   // add the message receiver
   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction) OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageNewBuffer");

   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction)OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageResize");
   SetFrameToDefaultValue();
   SetFrameModeSensitivity();
   DisplayFrameRectangle();
} // ManipulationBox()

//===========================================================================
//
//===========================================================================

void ManipulationBox::CutDestinationChanged(int i_Destination)
{
   m_i_CutDestination = i_Destination;
} // CutDestinationChanged()

//===========================================================================
//
//===========================================================================

void ManipulationBox::HandleCuttedImage(PixelBuffer *pc_NewImage)
{
   if(m_i_CutDestination == CutDestinationReplace)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                this,
                                                "ImageChanged");
   }
   else if(m_i_CutDestination == CutDestinationNew)
   {
      // create a new blank image
      MainFrame * pc_Image;
      ImageFrame *pc_ImageFrame;
      pc_Image = new MainFrame(0,
                              g_Application->m_pc_QApplication,
                              g_Application->m_pc_ApplicationFrame,
                              10,10);

      g_Application->m_pc_ApplicationFrame ->AddMainFrame(pc_Image);

      // replace the old by the new image
      pc_ImageFrame = pc_Image->GetImageFrame();
      pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);

      // remember the changes for undo
      pc_Image->DoChangedProcessing();

      // set caption the the new image name
      pc_Image->setCaption("Cut");
      // show the window
      pc_Image->show();
   }

} // HandleCuttedImage()

//===========================================================================
//
//===========================================================================

void *ManipulationBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures

   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      PixelBuffer *pc_NewImage;
      RectRegion   c_Region;
      int          i_X1;
      int          i_X2;
      int          i_Y1;
      int          i_Y2;

      if(m_pc_GlobalStorage == NULL)
      {
         return NULL;
      }
      m_pc_GlobalStorage->m_pc_Selection->GetSelectionRectangle(i_X1,i_Y1,i_X2,i_Y2);
      c_Region.SetXYXY(i_X1,i_Y1,i_X2,i_Y2);

      pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->CreateNewImageFromRegion(c_Region);

      if(pc_NewImage ==NULL)
      {
         return NULL;
      }
      HandleCuttedImage(pc_NewImage);

   } // If ReasonSelection

   return NULL;
} // SelectionEventArrived()

//===========================================================================
//
//===========================================================================

void *ManipulationBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   ManipulationBox *pc_This = (ManipulationBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()


//===========================================================================
//
//===========================================================================

void ManipulationBox::CutRectangle()
{
   if(  m_pc_GlobalStorage != NULL)
   {

      m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionRectangle,
                                 "Cut an Image",
                                 Callback(SelectionEventArrivedCB,this) );

   }

} // CutRectanlge()

//===========================================================================
//
//===========================================================================

void ManipulationBox::CutSelection()
{
   PixelBuffer *pc_NewImage;

   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->CreateNewImageFromSelected();
   if(pc_NewImage ==NULL)
   {
      return;
   }

   HandleCuttedImage(pc_NewImage);

} // CutSelection()

//===========================================================================
//
//===========================================================================

void ManipulationBox::Rotate90()
{
   m_pc_GlobalStorage->m_pc_MainImage->Rotate(90);
   m_pc_GlobalStorage->m_pc_ImageFrame->ChangeImageDimensions();
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // Rotate90()

//===========================================================================
//
//===========================================================================

void ManipulationBox::Rotate180()
{
   m_pc_GlobalStorage->m_pc_MainImage->Rotate(180);
   m_pc_GlobalStorage->m_pc_ImageFrame->ChangeImageDimensions();
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // Rotate180()

//===========================================================================
//
//===========================================================================

void ManipulationBox::Rotate270()
{
   m_pc_GlobalStorage->m_pc_MainImage->Rotate(270);
   m_pc_GlobalStorage->m_pc_ImageFrame->ChangeImageDimensions();
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // Rotate270()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::MirrorX()
{
   m_pc_GlobalStorage->m_pc_MainImage->MirrorX();
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // MirrorX()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::MirrorY()
{
   m_pc_GlobalStorage->m_pc_MainImage->MirrorY();
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // MirrorY()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::DoScale()
{
   PixelBuffer *pc_NewImage;
   QString      qs_Width;
   QString      qs_Height;
   int          i_Width =0;
   int          i_Height=0;

   // get the entered values
   qs_Width  = m_NewWidth ->text();
   qs_Height = m_NewHeight->text();

   // make a number out of it
   sscanf(qs_Width,"%d",&i_Width);
   sscanf(qs_Height,"%d",&i_Height);

   // valid range
   if(   i_Width  <=1
      || i_Height <= 1
     )
   {
      // no -> do nothing
      return;
   }

   // create a new image with that dimension
   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->Scale(i_Width,i_Height);
   if(pc_NewImage ==NULL)
   {
      return;
   }

   // replace old with new buffer
   m_pc_GlobalStorage->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // DoScale()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::DoSliderValueChanged(int i_NewValue)
{
   QString qs_LabelOutput;
   QString qs_NewWidth;
   QString qs_NewHeight;
   int     i_NewWidth;
   int     i_NewHeight;

   int i_OriginalWidth;
   int i_OriginalHeight;

   m_i_CurrentSlider = i_NewValue;

   if(m_pc_GlobalStorage->m_pc_MainImage == NULL)
   {
      return;
   }

   i_OriginalWidth  = m_pc_GlobalStorage->m_pc_MainImage->GetWidth();
   i_OriginalHeight = m_pc_GlobalStorage->m_pc_MainImage->GetHeight();

   if(i_NewValue >0)
   {
      double d_Value;
      d_Value = i_NewValue * 1.0/((double)m_i_UpperScaleEnd);
      d_Value = pow(10,d_Value);
      qs_LabelOutput.sprintf("%5.2f",d_Value);
      i_NewWidth  = (int)( i_OriginalWidth * d_Value);
      i_NewHeight = (int)(i_OriginalHeight * d_Value);
                  
   }
   else if(i_NewValue ==0)
   {
      qs_LabelOutput = "1";
      i_NewWidth  = i_OriginalWidth ;
      i_NewHeight = i_OriginalHeight ;
   }
   else // < 0
   {
      double d_Value;
      d_Value = (-i_NewValue) * 1.0/((double) - m_i_LowerScaleEnd);
      d_Value = pow(10,d_Value);
      d_Value = 1 / d_Value;
      qs_LabelOutput.sprintf("%5.2f",d_Value);
      i_NewWidth  = (int)( i_OriginalWidth * d_Value);
      i_NewHeight = (int)(i_OriginalHeight * d_Value);
   }

   m_ResizeSliderLabel->setText(qs_LabelOutput);

   // put the calculated dimension into the edit field
   qs_NewWidth .sprintf("%d",i_NewWidth);
   qs_NewHeight.sprintf("%d",i_NewHeight);

   m_NewWidth ->setText( qs_NewWidth);
   m_NewHeight->setText( qs_NewHeight);

} //DoSliderValueChanged()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::SetImageSize()
{
   QString qs_Width;
   QString qs_Height;

   if(m_pc_GlobalStorage->m_pc_MainImage == NULL)
   {
      return;
   }

   // change the edit field to the new size
   qs_Width .sprintf("%d",m_pc_GlobalStorage->m_pc_MainImage->GetWidth() );
   qs_Height.sprintf("%d",m_pc_GlobalStorage->m_pc_MainImage->GetHeight() );

   m_OriginalWidth ->setText( qs_Width);
   m_OriginalHeight->setText( qs_Height);

   DoSliderValueChanged(m_i_CurrentSlider);

} // SetImageSize

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::ReceivedNewImageSize()
{
   SetImageSize();

   SetFrameIntoValidImage();
} // ReceivedNewImageSize()

//===========================================================================
//
//===========================================================================


void ManipulationBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
//fprintf(stderr,"Message Received member this = %x\n",this);
   if(qs_MessageType.compare("ImageResize") ==0)
   {
      ReceivedNewImageSize();
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
      ReceivedNewImageSize();
   }
} // OnMessageReceive()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::SetFrameModeSensitivity()
{
   if( m_b_FrameModeActive)
   {
      // enable the edit fields
      m_FrameGridWidget->setEnabled(true);
      m_FrameCutExecuteButton->setEnabled(true);
      m_FrameResetButton->setEnabled(true);
   }
   else // not active
   {
      // disable the edit fields
      m_FrameGridWidget->setEnabled(false);
      m_FrameCutExecuteButton->setEnabled(false);
      m_FrameResetButton->setEnabled(false);
   }
   
} // SetFrameModeSensitivity()
   
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::FrameSelectionModeToggled(bool b_Mode)
{
   // if mode has not changed
   if(m_b_FrameModeActive == b_Mode)
   {
      return;
   }

   // remember the new mode
   m_b_FrameModeActive = b_Mode;

   if( m_b_FrameModeActive )
   {
      // make this layer visible
      m_pc_FrameLayer->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

      // set this layer as input layer
      m_pc_GlobalStorage->m_pc_ObjectMain->SetActiveLayer(MANIPULATIONBOXLAYER);

      // enable active action
      m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput();

      // set the callback for graphic object changes
      m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(GraphicObjectCallbackCB,this));
      DisplayFrameRectangle();
   }
   else
   {
      if(   m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer()
         == m_pc_FrameLayer )
      {
         // reset the callback for graphic object changes
         m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));
         m_pc_FrameLayer->SetVisible(false);

         // disable active action
         m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
      }
   }
   
   SetFrameModeSensitivity();
} // FrameSelectionModeToggled()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::FrameReset()
{
   RectRegion c_ChangedRegion;
   RectRegion c_NewRegion;

   c_ChangedRegion = m_pc_FrameRectangle->GetSize();   
   SetFrameToDefaultValue();
   c_NewRegion = m_pc_FrameRectangle->GetSize();

   c_ChangedRegion.AddRegion(c_NewRegion);

   // redisplay the region
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);
   
} // FrameReset()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::FrameDoCut()
{
   PixelBuffer *pc_NewImage;
   RectRegion  c_FrameSize;
   RectRegion  c_BufferRegion;
   int         i_Width;
   int         i_Height;

   // get dimension of the new image
   i_Width = m_pc_GlobalStorage->m_pc_MainImage->GetWidth() ;
   i_Height= m_pc_GlobalStorage->m_pc_MainImage->GetHeight() ;

   // get frame dimension
   c_FrameSize =  m_pc_FrameRectangle->GetSize();
   c_BufferRegion.SetXYWH(0,0,i_Width,i_Height);

   // clip frame at the buffer
   c_FrameSize.ClipAt(c_BufferRegion);
   
   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->CreateNewImageFromRegion(c_FrameSize);

   if(pc_NewImage ==NULL)
   {
      return ;
   }
   HandleCuttedImage(pc_NewImage);
   
} // FrameDoCut()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::GraphicObjectCallback(GraphicObjectMainEvent *pc_CallbackData)
{
   // now redisplay the size of the graphic object

   if(pc_CallbackData->i_CallbackType ==    GraphicObjectMainEvent::SelectionDisabled)
   {
      // disable the frame mode
      if( m_b_FrameModeActive )
      {
         if(   m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer()
            == m_pc_FrameLayer )
         {
            // reset the callback for graphic object changes
            m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));
            m_pc_FrameLayer->SetVisible(false);

            // disable active action
            m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
         }
         m_FrameActiveCheck->setChecked(false);
      }
   }
   else  // regular callback event
   {
      DisplayFrameRectangle();
   }
     
} // GraphicObjectCallback()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void *ManipulationBox::GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData)
{
   ManipulationBox *pc_This;
   pc_This = (ManipulationBox *) thisData;

   pc_This-> GraphicObjectCallback( (GraphicObjectMainEvent *)GraphicObjectCallbackData );

   return NULL;
} // GraphicObjectCallbackCB()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::DisplayFrameRectangle()
{
   // now redisplay the size of the graphic object
   RectRegion c_Size;
   QString    qs_Out;

   c_Size =    m_pc_FrameRectangle->GetSize();

   // prevent execution of the text changed signals
   m_b_InUpdate = true;
   
   // put the calculated dimension into the edit field
   qs_Out.sprintf("%d",c_Size.GetStartX());
   m_FrameXEdit->setText(qs_Out);

   qs_Out.sprintf("%d",c_Size.GetStartY());
   m_FrameYEdit->setText(qs_Out);

   qs_Out.sprintf("%d",c_Size.GetWidth());
   m_FrameWidthEdit->setText(qs_Out);

   qs_Out.sprintf("%d",c_Size.GetHeight());
   m_FrameHeightEdit->setText(qs_Out);

   m_b_InUpdate = false;
   
} // DisplayFrameRectangle()

//-----------------------------------------------------------------------------
// check, whether the frame already fits into the current image.
// if not, then resize the frame
//-----------------------------------------------------------------------------

void ManipulationBox::SetFrameIntoValidImage()
{
   RectRegion c_FrameSize;
   RectRegion c_ImageSize;
   int        i_Width;
   int        i_Height;

   // get dimension of the new image
   i_Width = m_pc_GlobalStorage->m_pc_MainImage->GetWidth() ;
   i_Height= m_pc_GlobalStorage->m_pc_MainImage->GetHeight() ;

   // get frame dimension
   c_FrameSize =  m_pc_FrameRectangle->GetSize();
   c_ImageSize.SetXYWH(0,0,i_Width,i_Height);

   // fits the frame into the image?
   if(! c_ImageSize.ContainsRegion(c_FrameSize))
   {
      // no!
      // resize the frame to default values
      SetFrameToDefaultValue();
   }

} // SetFrameIntoValidImage()

//-----------------------------------------------------------------------------
//  set frame to a default value, which is
// 1/4 -- 3/4 of the current image dimension
//-----------------------------------------------------------------------------

void ManipulationBox::SetFrameToDefaultValue()
{
   RectRegion c_FrameSize;
   int        i_Width;
   int        i_Height;

   // get dimension of the new image
   i_Width = m_pc_GlobalStorage->m_pc_MainImage->GetWidth() ;
   i_Height= m_pc_GlobalStorage->m_pc_MainImage->GetHeight() ;

   c_FrameSize.SetXYXY(i_Width    /4,
                       i_Height   /4,
                       i_Width  *3/4,
                       i_Height *3/4);
   m_pc_FrameRectangle->SetSize(c_FrameSize);
   DisplayFrameRectangle();
   
} // SetFrameToDefaultValue()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::FrameXTextChanged (const QString &qs_Value)
{
   RectRegion c_FrameSize;
   RectRegion c_NewSize;
   RectRegion c_CommonSize;
   char       c_Dummy;
   int        i_Width;
   int        i_Height;

   int        i_ScanfResult;
   int        i_Value;

   if(m_b_InUpdate)
   {
      return;
   }

   // get dimension of the new image
   i_Width = m_pc_GlobalStorage->m_pc_MainImage->GetWidth() ;
   i_Height= m_pc_GlobalStorage->m_pc_MainImage->GetHeight() ;

   // get frame dimension
   c_FrameSize =  m_pc_FrameRectangle->GetSize();

   i_ScanfResult = sscanf((const char *) qs_Value,
                          "%d%c",
                          &i_Value,&c_Dummy);
   if(i_ScanfResult != 1)
   {
      i_Value =0;
   }
   else
   {
      // i_value could get read -- check for valid range
      if(i_Value <0)
      {
         i_Value =0;
      }
      else if(i_Value > (i_Width -2))
      {
         i_Value = i_Width-2;
      }
   }

   c_NewSize.SetXYWH(i_Value,
                     c_FrameSize.GetStartY(),
                     c_FrameSize.GetWidth(),
                     c_FrameSize.GetHeight());

   // common size is the region to redraw
   c_CommonSize = c_FrameSize;
   c_CommonSize.AddRegion(c_NewSize);

   // set new size to the object
   m_pc_FrameRectangle->SetSize(c_NewSize);

   // redisplay the region
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_CommonSize);

} // FrameXTextChanged()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::FrameYTextChanged (const QString &qs_Value)
{
   RectRegion c_FrameSize;
   RectRegion c_NewSize;
   RectRegion c_CommonSize;
   char       c_Dummy;
   int        i_Width;
   int        i_Height;

   int        i_ScanfResult;
   int        i_Value;

   if(m_b_InUpdate)
   {
      return;
   }

   // get dimension of the new image
   i_Width = m_pc_GlobalStorage->m_pc_MainImage->GetWidth() ;
   i_Height= m_pc_GlobalStorage->m_pc_MainImage->GetHeight() ;

   // get frame dimension
   c_FrameSize =  m_pc_FrameRectangle->GetSize();

   i_ScanfResult = sscanf((const char *) qs_Value,
                          "%d%c",
                          &i_Value,&c_Dummy);
   if(i_ScanfResult != 1)
   {
      i_Value =0;
   }
   else
   {
      // i_value could get read -- check for valid range
      if(i_Value <0)
      {
         i_Value =0;
      }
      else if(i_Value > (i_Height -2))
      {
         i_Value = i_Height-2;
      }
   }

   c_NewSize.SetXYWH(c_FrameSize.GetStartX(),
                     i_Value,
                     c_FrameSize.GetWidth(),
                     c_FrameSize.GetHeight());

   // common size is the region to redraw
   c_CommonSize = c_FrameSize;
   c_CommonSize.AddRegion(c_NewSize);

   // set new size to the object
   m_pc_FrameRectangle->SetSize(c_NewSize);

   // redisplay the region
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_CommonSize);

} // FrameYTextChanged()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::FrameWidthTextChanged (const QString &qs_Value)
{
   RectRegion c_FrameSize;
   RectRegion c_NewSize;
   RectRegion c_CommonSize;
   char       c_Dummy;
   int        i_Width;
   int        i_Height;

   int        i_ScanfResult;
   int        i_Value;

   if(m_b_InUpdate)
   {
      return;
   }

   // get dimension of the new image
   i_Width = m_pc_GlobalStorage->m_pc_MainImage->GetWidth() ;
   i_Height= m_pc_GlobalStorage->m_pc_MainImage->GetHeight() ;

   // get frame dimension
   c_FrameSize =  m_pc_FrameRectangle->GetSize();

   i_ScanfResult = sscanf((const char *) qs_Value,
                          "%d%c",
                          &i_Value,&c_Dummy);
   if(i_ScanfResult != 1)
   {
      i_Value =1;
   }
   else
   {
      // i_value could get read -- check for valid range
      if(i_Value <1)
      {
         i_Value =1 ;
      }
      else if(i_Value > (i_Width ))
      {
         i_Value = i_Width;
      }
   }

   c_NewSize.SetXYWH(c_FrameSize.GetStartX(),
                     c_FrameSize.GetStartY(),
                     i_Value,
                     c_FrameSize.GetHeight());

   // common size is the region to redraw
   c_CommonSize = c_FrameSize;
   c_CommonSize.AddRegion(c_NewSize);

   // set new size to the object
   m_pc_FrameRectangle->SetSize(c_NewSize);

   // redisplay the region
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_CommonSize);

} // FrameWidthTextChanged()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void ManipulationBox::FrameHeightTextChanged (const QString &qs_Value)
{
   RectRegion c_FrameSize;
   RectRegion c_NewSize;
   RectRegion c_CommonSize;
   char       c_Dummy;
   int        i_Width;
   int        i_Height;

   int        i_ScanfResult;
   int        i_Value;

   if(m_b_InUpdate)
   {
      return;
   }

   // get dimension of the new image
   i_Width = m_pc_GlobalStorage->m_pc_MainImage->GetWidth() ;
   i_Height= m_pc_GlobalStorage->m_pc_MainImage->GetHeight() ;

   // get frame dimension
   c_FrameSize =  m_pc_FrameRectangle->GetSize();

   i_ScanfResult = sscanf((const char *) qs_Value,
                          "%d%c",
                          &i_Value,&c_Dummy);
   if(i_ScanfResult != 1)
   {
      i_Value =1;
   }
   else
   {
      // i_value could get read -- check for valid range
      if(i_Value <1)
      {
         i_Value =1;
      }
      else if(i_Value > (i_Height ))
      {
         i_Value = i_Height;
      }
   }

   c_NewSize.SetXYWH(c_FrameSize.GetStartX(),
                     c_FrameSize.GetStartY(),
                     c_FrameSize.GetWidth(),
                     i_Value
                     );

   // common size is the region to redraw
   c_CommonSize = c_FrameSize;
   c_CommonSize.AddRegion(c_NewSize);

   // set new size to the object
   m_pc_FrameRectangle->SetSize(c_NewSize);

   // redisplay the region
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_CommonSize);

} // FrameHeightTextChanged()

//===========================================================================
//
//===========================================================================

void ManipulationBox::hideEvent(QHideEvent *)
{
   // make the layer invisible
   m_FrameActiveCheck->setChecked(false);

}  // hideEvent()

//===========================================================================
//
//===========================================================================

