
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>

#include "MaskBox.h"

#include "PixelBuffer.h"
#include "Zoom.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Utilities.h"
#include "MaskSettings.h"


MaskBox::MaskBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global),
                Tracer("MaskBox")
{
   QHBoxLayout  *MaskLayout;
   QVGroupBox   *SafetyMaskBox;
   QVGroupBox   *SelectionMaskBox;
   QBoxLayout   *l_Layout;
   QWidget      *MaskWidget;

   TRACE << "Mask Started\n" ;

   setCaption("Masks");

   m_pc_GlobalStorage = pc_Global;

   m_SafetyMaskColor = PIXEL_MAKE(0xff,0xaa,0x33);
   m_pc_GlobalStorage->m_pc_Zoom->SetSafetyMaskColor(m_SafetyMaskColor);

   m_i_PaintMode = m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode();

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);

   MaskWidget = new QWidget(m_w_TopWidget);
   l_Layout  ->addWidget(MaskWidget);
   MaskLayout  = new  QHBoxLayout( MaskWidget,2);

   // box for Safety mask
   SafetyMaskBox = new QVGroupBox("Safety Mask",MaskWidget);
   MaskLayout->addWidget(SafetyMaskBox);
   {
      QHBoxLayout  *ColorLayout;
      QLabel       *ColorLabel;
      QWidget      *ColorWidget;

      m_SafetyMaskVisible  = new QCheckBox("Visible",SafetyMaskBox);
      m_SafetyMaskActive   = new QCheckBox("Active" ,SafetyMaskBox);
      m_SafetyInvertButton = new QPushButton("Invert",SafetyMaskBox);
      m_SafetyClearButton  = new QPushButton("Clear",SafetyMaskBox);

      ColorWidget  = new  QWidget(SafetyMaskBox);
      ColorLayout  = new  QHBoxLayout( ColorWidget,2);
      ColorLabel   = new  QLabel("Mask Color",ColorWidget);
      m_SafetyMaskColorWidget = new QUColorDragWidget(ColorWidget,
                                               QUColorDragWidget::ModePipette,
                                               m_pc_GlobalStorage  );
      m_SafetyMaskColorWidget ->SetColor(m_SafetyMaskColor);

      ColorLayout->addWidget(ColorLabel);
      ColorLayout->addWidget(m_SafetyMaskColorWidget);

      connect( m_SafetyMaskVisible ,
               SIGNAL(toggled(bool)),
               SLOT( SafetyMaskVisibleClicked(bool) ) );

      connect( m_SafetyMaskActive ,
               SIGNAL(toggled(bool)),
               SLOT( SafetyMaskActiveClicked(bool) ) );

      connect( m_SafetyInvertButton ,
               SIGNAL(clicked()),
               SLOT( SafetyMaskInvert() ) );

      connect( m_SafetyClearButton ,
               SIGNAL(clicked()),
               SLOT( SafetyMaskClear() ) );

      connect( m_SafetyMaskColorWidget ,
               SIGNAL( ColorChanged(Pixel) ),
               SLOT( SafetyMaskColorChanged(Pixel ) ) );

   } // box for safety

   // box for Selection mask
   SelectionMaskBox = new QVGroupBox("Selection Mask",MaskWidget);
   MaskLayout->addWidget(SelectionMaskBox);
   {
      QHBoxLayout  *ColorLayout;
      QLabel       *ColorLabel;
      QWidget      *ColorWidget;

      m_SelectionMaskVisible  = new QCheckBox("Visible",SelectionMaskBox);
      m_SelectionMaskActive   = new QCheckBox("Active" ,SelectionMaskBox);
      m_SelectionInvertButton = new QPushButton("Invert",SelectionMaskBox);
      m_SelectionClearButton  = new QPushButton("Clear",SelectionMaskBox);

      ColorWidget  = new  QWidget(SelectionMaskBox);
      ColorLayout  = new  QHBoxLayout( ColorWidget,2);
      ColorLabel   = new  QLabel("Mask Color",ColorWidget);
      m_SelectionMaskColorWidget = new QUColorDragWidget(ColorWidget,
                                               QUColorDragWidget::ModePipette,
                                               m_pc_GlobalStorage);
      m_SelectionMaskColorWidget ->SetColor(m_SelectionMaskColor);

      ColorLayout->addWidget(ColorLabel);
      ColorLayout->addWidget(m_SelectionMaskColorWidget);

      connect( m_SelectionMaskVisible ,
               SIGNAL(toggled(bool)),
               SLOT( SelectionMaskVisibleClicked(bool) ) );

      connect( m_SelectionMaskActive ,
               SIGNAL(toggled(bool)),
               SLOT( SelectionMaskActiveClicked(bool) ) );

      connect( m_SelectionInvertButton ,
               SIGNAL(clicked()),
               SLOT( SelectionMaskInvert() ) );

      connect( m_SelectionClearButton ,
               SIGNAL(clicked()),
               SLOT( SelectionMaskClear() ) );

      connect( m_SelectionMaskColorWidget ,
               SIGNAL( ColorChanged(Pixel) ),
               SLOT( SelectionMaskColorChanged(Pixel ) ) );

   } // box for Selection

   // block for the mode
   {

      m_ModeWidget = new QButtonGroup(3,
                                    QGroupBox::Vertical,
                                    "Paint Mode Type",
                                    m_w_TopWidget);
      l_Layout->addWidget(m_ModeWidget);

      m_ModeInImage         = new QRadioButton("Image",m_ModeWidget);
      m_ModeInSafetyMask    = new QRadioButton("Safety Mask",m_ModeWidget);
      m_ModeInSelectionMask = new QRadioButton("Selection Mask",m_ModeWidget);

      switch(m_i_PaintMode)
      {
      case MaskSettings::MaskPaintModeImage:
         m_ModeInImage->setChecked(1);
         break;
      case MaskSettings::MaskPaintModeInSafety:
         m_ModeInSafetyMask->setChecked(1);
         break;
      case MaskSettings::MaskPaintModeInSelection:
         m_ModeInSelectionMask->setChecked(1);
         break;
      default:
         break;
      }
      connect( m_ModeWidget,
               SIGNAL(clicked(int)),
               SLOT(PaintModeChanged(int)) );
   } // block for the mode

   // block for the MaskDrawMmode
   {

      m_MaskDrawModeWidget = new QButtonGroup(3,
                                    QGroupBox::Vertical,
                                    "Mask Operation",
                                    m_w_TopWidget);
      l_Layout->addWidget(m_MaskDrawModeWidget);

      m_MaskDrawModeExtend   = new QRadioButton("Extend",m_MaskDrawModeWidget);
      m_MaskDrawModeReduce   = new QRadioButton("Reduce",m_MaskDrawModeWidget);

      switch(m_i_PaintMode)
      {
      case MaskSettings::MaskPaintDrawExtend:
         m_MaskDrawModeExtend->setChecked(1);
         break;
      case MaskSettings::MaskPaintDrawReduce:
         m_MaskDrawModeReduce->setChecked(1);
         break;
      default:
         break;
      }
      connect( m_MaskDrawModeWidget,
               SIGNAL(clicked(int)),
               SLOT(MaskDrawModeChanged(int)) );
   } // block for the mode

   SetSensitivity();

} // MaskBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void MaskBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()


//===========================================================================
//
//===========================================================================

void MaskBox::SafetyMaskVisibleClicked(bool b_On)
{
   m_pc_GlobalStorage->m_pc_Zoom->SetSafetyMaskVisible(b_On);
   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

} // SafetyMaskVisibleClicked()

//===========================================================================
//
//===========================================================================

void MaskBox::SelectionMaskVisibleClicked(bool b_On)
{
   m_pc_GlobalStorage->m_pc_Zoom->SetSelectionMaskVisible(b_On);
   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

} // SelectionMaskVisibleClicked()

//===========================================================================
//
//===========================================================================

void MaskBox::SafetyMaskActiveClicked(bool b_On)
{
   m_pc_GlobalStorage->m_pc_MainImage->SetSafetyMaskActive(b_On);

} // SafetyMaskActiveClicked()

//===========================================================================
//
//===========================================================================

void MaskBox::SelectionMaskActiveClicked(bool b_On)
{
   m_pc_GlobalStorage->m_pc_MainImage->SetSelectionMaskActive(b_On);

} // SelectionMaskActiveClicked()

//===========================================================================
//
//===========================================================================

void MaskBox::SafetyMaskColorChanged(Pixel p_Color)
{
   m_SafetyMaskColor = p_Color;
   m_pc_GlobalStorage->m_pc_Zoom->SetSafetyMaskColor(m_SafetyMaskColor);

   // if mask is visible then force a redraw
   if( m_pc_GlobalStorage->m_pc_Zoom->GetSafetyMaskVisible() )
   {
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // SafetyMaskColorChanged()

//===========================================================================
//
//===========================================================================

void MaskBox::SelectionMaskColorChanged(Pixel p_Color)
{
   m_SelectionMaskColor = p_Color;
   m_pc_GlobalStorage->m_pc_Zoom->SetSelectionMaskColor(m_SelectionMaskColor);

   // if mask is visible then force a redraw
   if( m_pc_GlobalStorage->m_pc_Zoom->GetSelectionMaskVisible() )
   {
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // SelectionMaskColorChanged()

//===========================================================================
//
//===========================================================================

void MaskBox::SafetyMaskInvert()
{
   PixelBuffer *pc_Mask;

   // get the physical buffer
   pc_Mask= m_pc_GlobalStorage->m_pc_MainImage->GetSafetyMask();
   if(pc_Mask == NULL)
   {
      return;
   }

   // change the mask
   pc_Mask->InvertBuffer();

   // tell the modification
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                  this,
                                                  "ImageChanged");

   // if mask is visible then force a redraw
   if( m_pc_GlobalStorage->m_pc_Zoom->GetSafetyMaskVisible() )
   {
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // SafetyMaskInvert()

//===========================================================================
//
//===========================================================================

void MaskBox::SelectionMaskInvert()
{
   PixelBuffer *pc_Mask;

   // get the physical buffer
   pc_Mask= m_pc_GlobalStorage->m_pc_MainImage->GetSelectionMask();
   if(pc_Mask == NULL)
   {
      return;
   }

   // change the mask
   pc_Mask->InvertBuffer();

   // tell the modification
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                  this,
                                                  "ImageChanged");

   // if mask is visible then force a redraw
   if( m_pc_GlobalStorage->m_pc_Zoom->GetSelectionMaskVisible() )
   {
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // SelectionMaskInvert()

//===========================================================================
//
//===========================================================================

void MaskBox::SafetyMaskClear()
{
   PixelBuffer *pc_Mask;

   // get the physical buffer
   pc_Mask= m_pc_GlobalStorage->m_pc_MainImage->GetSafetyMask();
   if(pc_Mask == NULL)
   {
      return;
   }

   // change the mask
   pc_Mask->Clean();

   // tell the modification
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                  this,
                                                  "ImageChanged");

   // if mask is visible then force a redraw
   if( m_pc_GlobalStorage->m_pc_Zoom->GetSafetyMaskVisible() )
   {
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // SafetyMaskClear()

//===========================================================================
//
//===========================================================================

void MaskBox::SelectionMaskClear()
{
   PixelBuffer *pc_Mask;

   // get the physical buffer
   pc_Mask= m_pc_GlobalStorage->m_pc_MainImage->GetSelectionMask();
   if(pc_Mask == NULL)
   {
      return;
   }

   // change the mask
   pc_Mask->Clean();

   // tell the modification
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                  this,
                                                  "ImageChanged");

   // if mask is visible then force a redraw
   if( m_pc_GlobalStorage->m_pc_Zoom->GetSelectionMaskVisible() )
   {
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // SelectionMaskClear()

//===========================================================================
//
//===========================================================================


void MaskBox::PaintModeChanged(int i_NewMode)
{
   m_pc_GlobalStorage->m_pc_MaskSettings->SetPaintMode(i_NewMode);
   m_i_PaintMode = i_NewMode;
   SetSensitivity();

} // PaintModeChanged

//===========================================================================
//
//===========================================================================


void MaskBox::MaskDrawModeChanged(int i_NewMode)
{
   m_pc_GlobalStorage->m_pc_MaskSettings->SetDraw(i_NewMode);
   m_i_MaskDrawMode = i_NewMode;

} // MaskDrawModeChanged

//===========================================================================
//
//===========================================================================

void MaskBox::SetSensitivity()
{
   // set the draw mode widget active, depending on the state of
   // the paint mode
   if( m_i_PaintMode == MaskSettings::MaskPaintModeImage)
   {
      m_MaskDrawModeWidget->setEnabled(false);
   }
   else
   {
      m_MaskDrawModeWidget->setEnabled(true);
   }
} // SetSensitivity()

//===========================================================================
//
//===========================================================================

