
#include "MessageServer.h"
#include "Tracer.h"

//===========================================================================
//
//===========================================================================

MessageReceiverElement::MessageReceiverElement(MessageCallbackFunction p_Function,
                      void * p_CallbackData,
			QString qs_MessageType)
{
   m_qs_MessageType = qs_MessageType;
   m_p_Function     = p_Function;
   m_p_CallbackData = p_CallbackData;

} // MessageReceiverElement() constructor

//===========================================================================
//
//===========================================================================

MessageServer::MessageServer() 
                    : Tracer("MessageServer")
{
   // the list owns the objects
   m_ReceiverList.setAutoDelete( true ); 
} // MessageServer()

//===========================================================================
//
//===========================================================================

MessageServer::~MessageServer()
{
   
}

//===========================================================================
//
//===========================================================================

void MessageServer::AddMessageReceiver(MessageCallbackFunction p_Function,
                      void * p_CallbackData,
			QString qs_MessageType)
{
   MessageReceiverElement *pc_NewReceiver;

   TRACE << "Add Receiver "
         << (const char *) qs_MessageType
         << "\n";

   pc_NewReceiver = new MessageReceiverElement(p_Function,
                                        p_CallbackData,
					qs_MessageType);

   if(pc_NewReceiver == NULL)
   {
      return;
   }

   m_ReceiverList.append(pc_NewReceiver);

} // AddMessageReceiver()


//===========================================================================
//
//===========================================================================

void MessageServer::RemoveMessageReceiver(MessageCallbackFunction p_Function,
                      void * p_CallbackData,
			QString qs_MessageType)
{
   int i_ActualPosition;

   TRACE << "Remove Receiver "
         << (const char *) qs_MessageType
         << "\n";

   i_ActualPosition =0;

   while(i_ActualPosition < (int) m_ReceiverList.count() )
   {
      MessageReceiverElement *pc_ActualReceiver;

      pc_ActualReceiver = m_ReceiverList.at(i_ActualPosition);
      if(    pc_ActualReceiver->m_p_Function == p_Function
          && pc_ActualReceiver->m_p_CallbackData == p_CallbackData
          && pc_ActualReceiver->m_qs_MessageType == qs_MessageType
        )
      {
         m_ReceiverList.remove(i_ActualPosition);
      }
      else
      {
         // go to next element
         i_ActualPosition += 1;
      }
      
   }

} // RemoveMessageReceiver()


//===========================================================================
//
//===========================================================================

void MessageServer::RemoveMessageReceiver(void * p_CallbackData)
{
   int i_ActualPosition;

   i_ActualPosition =0;

   while(i_ActualPosition < (int) m_ReceiverList.count() )
   {
      MessageReceiverElement *pc_ActualReceiver;

      pc_ActualReceiver = m_ReceiverList.at(i_ActualPosition);
      if( pc_ActualReceiver->m_p_CallbackData == p_CallbackData )
      {
         m_ReceiverList.remove(i_ActualPosition);
      }
      else
      {
         // go to next element
         i_ActualPosition += 1;
      }
      
   }

} // RemoveMessageReceiver()

//===========================================================================
//
//===========================================================================

void MessageServer::SendMessage(int i_MessageRange, // unused now
               void * p_Myself,
               QString qs_MessageType,
	       void *p_MessageData)
{
   int i_ActualPosition;
   int i_MessageCount;

   TRACE << "Send Message "
         << (const char *) qs_MessageType
         << "\n";

   i_MessageCount = m_ReceiverList.count();

   // look in the receiver list
   for( i_ActualPosition = 0 ;
        i_ActualPosition < i_MessageCount;
        i_ActualPosition ++)
   {
      MessageReceiverElement *pc_ActualReceiver;

      pc_ActualReceiver = m_ReceiverList.at(i_ActualPosition);

      // if it is the message type, and its not myself
      if(   pc_ActualReceiver->m_qs_MessageType.compare(qs_MessageType)==0
         && pc_ActualReceiver->m_p_CallbackData != p_Myself)
      {
         TRACE << "Send to Receiver \n";
         pc_ActualReceiver->m_p_Function(
          		pc_ActualReceiver->m_p_CallbackData,
                        qs_MessageType,
                        p_MessageData);
         TRACE << "Message Sent \n";
      }

   }

} // SendMessage()

//===========================================================================
//
//===========================================================================

void MessageReceiver::OnMessageReceiveCB(void *p_ThisData,
				 QString qs_MessageType,
                                 void *p_Data)
{
   MessageReceiver *p_This;
   p_This =(MessageReceiver *) p_ThisData;
   p_This->OnMessageReceive(qs_MessageType,p_Data);
}

void MessageReceiver::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
}

//===========================================================================
//
//===========================================================================


