#ifndef _MESSAGESERVER_H
#define _MESSAGESERVER_H

#include <qstring.h>
#include <qptrlist.h>
#include <qdialog.h>

#include "PTypes.h"
#include "Tracer.h"

typedef void *(*MessageCallbackFunction)(void *, QString, void *);

// Valid Message are
// 	ImageResize
// 	ImageNewBuffer
//	ZoomChanged
//	ImageChanged
//	OperationTypeChange


class MessageReceiverElement
{
public:
   MessageReceiverElement(MessageCallbackFunction p_Function,
                      void * p_CallbackData,
			QString qs_MessageType);

   QString m_qs_MessageType;
   MessageCallbackFunction m_p_Function;
   void *	m_p_CallbackData;
}; // MessageReceiverElement

class MessageServer : public Tracer
{

public: 
   MessageServer();
   ~MessageServer();

   void AddMessageReceiver(MessageCallbackFunction p_Function,
                      void * p_CallbackData,
			QString qs_MessageType);

   
   void RemoveMessageReceiver(MessageCallbackFunction p_Function,
                      void * p_CallbackData,
			QString qs_MessageType);

   void RemoveMessageReceiver(void * p_CallbackData);


   void SendMessage(int i_MessageRange, // unused now
               void *p_Myself,
               QString qs_MessageType,
	       void *p_MessageData=NULL);


private:
	QPtrList<MessageReceiverElement> m_ReceiverList;
   
};


class MessageReceiver 
{

public:
    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   static void OnMessageReceiveCB(void *p_ThisData,
				 QString qs_MessageType,
                                 void *p_Data);

} ;

#endif //  _MESSAGESERVER_H
