#ifndef _MOTIVEBOX_H
#define _MOTIVEBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>

#include "PTypes.h"
#include "StandardDialog.h"
#include "GraphicObjectLayer.h"
#include "GraphicObjectMain.h"
#include "QULabelSlider.h"
#include "QUClickLabel.h"
#include "QUImageDragLabel.h"

class MotiveBox :  public StandardDialog
{
Q_OBJECT

public:
   MotiveBox(QWidget *parent=0,
			GlobalStorage *pc_Global=0);

   ~MotiveBox();

   void AddImageToWorkArea(PixelBuffer *pc_Image,
               int i_PositionX,
               int i_PositionY);

public slots:
   void ActiveModeToggled(bool b_Mode);
   void ViewTransparenToggled(bool b_Mode);

   void CutSelectionMaskPressed();
   void CutRectanglePressed();
   void CutFreehandPressed();
   void DeleteButtonPressed();
   void DeleteAllButtonPressed();
   void InsertButtonPressed();
   void InsertAllButtonPressed();

   void TransparencyEnableChanged(bool b_on);
   void TransparencyFactorChanged(int i_Factor);
   
   void TransparencyBorderEnableChanged(bool b_on);
   void TransparencyBorderFactorChanged(int i_Factor);

   void SizeXTextChanged (const QString &);
   void SizeYTextChanged (const QString &);

   void SizeLabelClicked(void);
   void MotiveDropped(PixelBuffer *pc_ImageBuffer);

protected:
   virtual void showEvent(QShowEvent *);
   virtual void hideEvent(QHideEvent *);
private: // functions

   void SetObjectInfo(GraphicObject *pc_Object);
   void PutMotivesOverTheImage();
   void SetSensitivity();
   void CutRectangleImage();
   void CutFreeHandImage();

    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   void *SelectionEventArrived(void *pc_Data);
   static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);

   void GraphicObjectCallback(GraphicObjectMainEvent *pc_CallbackData);
   static void *GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData);

private: // variables
   GraphicObjectLayer     *m_pc_ObjectLayer;

   bool       m_b_ObjectModeActive;
   QCheckBox  *m_ActivateCheck;

   bool       m_b_ObjectViewTransparent;
   QCheckBox  *m_ObjectViewTransparentCheck;

   QPushButton *m_CutSelectionMask;
   QPushButton *m_CutRectangle;
   QPushButton *m_CutFreeHand;
   QPushButton *m_DeleteButton;
   QPushButton *m_DeleteAllButton;
   QPushButton *m_InsertButton;
   QPushButton *m_InsertAllButton;

   QCheckBox     *m_TransparencyEnableCheck;
   QULabelSlider *m_TransparencySlider;

   QCheckBox     *m_TransparencyBorderEnableCheck;
   QULabelSlider *m_TransparencyBorderSlider;

   QWidget      *m_SelectedInfoWidget;
   QLineEdit    *m_OriginalWidthEdit;
   QLineEdit    *m_OriginalHeightEdit;
   QLineEdit    *m_ScaleXEdit;   
   QLineEdit    *m_ScaleYEdit;

   QUClickLabel *m_ScaleXLabel;
   QUClickLabel *m_ScaleYLabel;
   
   int          m_i_PixmapDimension;
   QUImageDragLabel *m_SelectedImageLabel;
   
   enum
   {
      MotiveSelectionNone,
      MotiveSelectionRectangle,
      MotiveSelectionFreehand
   };
   
   int  m_i_SelectionMode;
   int  m_i_TransparencyFactor;
   bool m_b_TransparencyEnabled;
   int  m_i_TransparencyBorderFactor;
   bool m_b_TransparencyBorderEnabled;

   bool m_b_InTextUpdate;

   bool m_b_FactorDisplayed;

}; // class MotiveBox


#endif // _MotiveBOX_H
