
#include "MouseEvent.h"
#include "DrawingArea.h"
#include "Callback.h"
#include "GlobalStorage.h"
#include "Zoom.h"
#include "StatusBar.h"


//===========================================================================
//
//===========================================================================

MouseEvent::MouseEvent(GlobalStorage *pc_GlobalStorage)
{
   DrawingArea *pc_DrawingArea;

   m_CurrentEventCallback.Clear();

   m_pc_GlobalStorage = pc_GlobalStorage;
   m_pc_GlobalStorage->m_pc_MouseEvent = this;


   pc_DrawingArea = m_pc_GlobalStorage->m_pc_DrawingArea;

   if(pc_DrawingArea != NULL)
   {
      pc_DrawingArea->InstallMouseStartCallback(Callback(MouseStartEventCB,this));
      pc_DrawingArea->InstallMouseDoCallback(Callback(MouseDoEventCB,this));
      pc_DrawingArea->InstallMouseEndCallback(Callback(MouseEndEventCB,this));
      pc_DrawingArea->InstallMouseOverCallback(Callback(MouseOverEventCB,this));
   }

} // MouseEvent()

//===========================================================================
//
//===========================================================================

MouseEvent::~MouseEvent()
{
   // disable all callbacks
   if(m_pc_GlobalStorage->m_pc_DrawingArea)
   {
      DrawingArea *pc_DrawingArea;
      pc_DrawingArea = m_pc_GlobalStorage->m_pc_DrawingArea;

      pc_DrawingArea->InstallMouseStartCallback(Callback(NULL,NULL));
      pc_DrawingArea->InstallMouseDoCallback(Callback(NULL,NULL));
      pc_DrawingArea->InstallMouseEndCallback(Callback(NULL,NULL));
      pc_DrawingArea->InstallMouseOverCallback(Callback(NULL,NULL));
   }
} // ~MouseEvent()

//===========================================================================
//
//===========================================================================

void *MouseEvent::EventArrived(QMouseEvent *pc_MouseEvent,int e_Type)
{
   int i_X;
   int i_Y;

// fprintf(stderr,"MouseEvent %d\n",e_Type);

   i_X= pc_MouseEvent->x();
   i_Y= pc_MouseEvent->y();
   
   // get state of shift and control
   m_i_LastButtonState = pc_MouseEvent->state();

   m_i_LastWindowX = i_X;
   m_i_LastWindowY = i_Y;
   m_i_LastEventType = e_Type;

   m_pc_GlobalStorage->m_pc_Zoom->GetBufferCoordinate(i_X,
                                                      i_Y,
                                                      m_i_LastBufferX,
                                                      m_i_LastBufferY);

/*
   fprintf(stderr,"Coords Real(%d/%d) Buffer(%d/%d)\n",
                   i_X,i_Y,
                   m_i_LastBufferX,m_i_LastBufferY);
*/
   if(true   // display ever, also during operation
      || e_Type == MouseEvent::EventOver)
   {
      DisplayCoordinates(m_i_LastBufferX,m_i_LastBufferY);

      // do not pass this event into further modules like selection
   }

   m_CurrentEventCallback.Execute(NULL);

   return NULL;
} // EventArrived()


//===========================================================================
//
//===========================================================================

void *MouseEvent::MouseStartEventCB(void *Thisdata,void *ClientData)
{
   QMouseEvent * pc_Event;
   MouseEvent  * pc_This;


   pc_This  = (MouseEvent *)  Thisdata;
   pc_Event = (QMouseEvent *) ClientData;

   return pc_This->EventArrived(pc_Event, MouseEvent::EventStart);

} // MouseStartEventCB()

//===========================================================================
//
//===========================================================================

void *MouseEvent::MouseDoEventCB(void *Thisdata,void *ClientData)
{
   QMouseEvent * pc_Event;
   MouseEvent  * pc_This;


   pc_This  = (MouseEvent *)  Thisdata;
   pc_Event = (QMouseEvent *) ClientData;

   return pc_This->EventArrived(pc_Event, MouseEvent::EventDo);

} // MouseDoEventCB()

//===========================================================================
//
//===========================================================================

void *MouseEvent::MouseEndEventCB(void *Thisdata,void *ClientData)
{
   QMouseEvent * pc_Event;
   MouseEvent  * pc_This;


   pc_This  = (MouseEvent *)  Thisdata;
   pc_Event = (QMouseEvent *) ClientData;

   return pc_This->EventArrived(pc_Event, MouseEvent::EventEnd);
} // MouseEndEventCB()

//===========================================================================
//
//===========================================================================

void *MouseEvent::MouseOverEventCB(void *Thisdata,void *ClientData)
{
   QMouseEvent * pc_Event;
   MouseEvent  * pc_This;


   pc_This  = (MouseEvent *)  Thisdata;
   pc_Event = (QMouseEvent *) ClientData;

   return pc_This->EventArrived(pc_Event, MouseEvent::EventOver);
} // MouseOverEventCB()

//===========================================================================
//
//===========================================================================

void MouseEvent::GetLastWindowCoordinates(int &i_X, int &i_Y)
{
   i_X = m_i_LastWindowX;
   i_Y = m_i_LastWindowY;

} // GetLastWindowCoordinates()

//===========================================================================
//
//===========================================================================

void MouseEvent::GetLastBufferCoordinates(int &i_X, int &i_Y)
{
   i_X = m_i_LastBufferX;
   i_Y = m_i_LastBufferY;
} // GetLastBufferwCoordinates()

//===========================================================================
//
//===========================================================================

int MouseEvent::GetEventType()
{
   return m_i_LastEventType;
} // GetLastBufferwCoordinates()

//===========================================================================
//
//===========================================================================


void MouseEvent::AddEventCallback(Callback EventCallback)
{
   m_CurrentEventCallback = EventCallback; 
} // AddEventCallback()

//===========================================================================
//
//===========================================================================


void MouseEvent::RemoveEventCallback(Callback EventCallback)
{
   m_CurrentEventCallback.Clear(); 
} // RemoveEventCallback()

//===========================================================================
//
//===========================================================================

void MouseEvent::DisplayCoordinates(int i_X,int i_Y)
{
   if(    m_pc_GlobalStorage != NULL
       && m_pc_GlobalStorage->m_pc_StatusBar != NULL)
   {
      m_pc_GlobalStorage->m_pc_StatusBar->SetCoordinate(i_X,i_Y);
   }

} // DisplayCoordinates

//===========================================================================
//
//===========================================================================

int MouseEvent::GetButtonState()
{
   return m_i_LastButtonState;
} // GetButtonState()

//===========================================================================
//
//===========================================================================

