
#include <stdio.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>

#include "NewImageBox.h"
#include "QtUtilities.h"
#include "ApplicationStorage.h"

//============================================================================
//
//============================================================================

NewImageBox::NewImageBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	QDialog(parent,"NewImageBox",true)
{
   QBoxLayout   *l_DialogButtons;
   QBoxLayout   *m_l_TopLayout;

   QWidget      *w_InputBox;
   QString      qs_Text;

   setCaption("Create Blank Image");

   m_i_Width  = 100;
   m_i_Height = 100;

   m_l_TopLayout = new QVBoxLayout( this,1);


   w_InputBox = new QWidget(this);
   m_l_TopLayout->addWidget(w_InputBox);
   {
      QGridLayout  *grid;
      int           i_CharWidth;

      i_CharWidth = g_Application->m_pc_QtUtility->GetMaximalFontWidth();
      
      grid = new QGridLayout( w_InputBox , 2, 2, 5 );

      m_WidthEdit   = new QLineEdit(w_InputBox);
      m_HeightEdit  = new QLineEdit(w_InputBox);
      m_WidthEdit ->setMaximumWidth(i_CharWidth*4);
      m_HeightEdit->setMaximumWidth(i_CharWidth*4);


      grid->addWidget(new QLabel("Width",w_InputBox),0,0);
      grid->addWidget(m_WidthEdit,0,1);
      grid->addWidget(new QLabel("Height",w_InputBox),1,0);
      grid->addWidget(m_HeightEdit,1,1);

      connect( m_WidthEdit,
            SIGNAL(textChanged ( const QString & )),
            SLOT( TextChanged(const QString &) ) );
      connect( m_HeightEdit,
            SIGNAL(textChanged ( const QString & )),
            SLOT( TextChanged(const QString &) ) );

   }

   // --- the buttons button
   l_DialogButtons = new QHBoxLayout( m_l_TopLayout,5);

   m_w_OKButton    = new QPushButton( "OK", this );
   m_w_AbortButton = new QPushButton( "Abort", this );
   connect( m_w_OKButton,
            SIGNAL(clicked()),
            SLOT( OKPressed()) );

   connect( m_w_AbortButton,
            SIGNAL(clicked()),
            SLOT( AbortPressed() ) );

   l_DialogButtons->addWidget(new QtUtilSpacer(true,false,this) );
   l_DialogButtons->addWidget(m_w_OKButton);
   l_DialogButtons->addWidget(m_w_AbortButton);

   // initialize the text strings
   qs_Text = qs_Text.setNum(m_i_Width);
   m_WidthEdit->setText(qs_Text);
   qs_Text = qs_Text.setNum(m_i_Height);
   m_HeightEdit->setText(qs_Text);

} // NewImageBox()

//============================================================================
//
//============================================================================

void NewImageBox::OKPressed()
{
   accept();
} // OKPressed()

//============================================================================
//
//============================================================================

void NewImageBox::AbortPressed()
{
   reject();
} // AbortPressed()

//============================================================================
//
//============================================================================

void NewImageBox::TextChanged(const QString &qs_NewText)
{
   QString qs_Text;
   char    c_Char;
   bool    b_Valid;
   int     i_Result;

   b_Valid = true;

   // get the actual value in the width edit
   qs_Text = m_WidthEdit->text();

   // get the number
   i_Result = sscanf((const char *) qs_Text,"%d%c",&m_i_Width,&c_Char);
   if(i_Result != 1)
   {
      b_Valid = false;
   }

   // get the actual value in the height edit
   qs_Text = m_HeightEdit->text();

   // get the number
   i_Result = sscanf((const char *) qs_Text,"%d%c",&m_i_Height,&c_Char);
   if(i_Result != 1)
   {
      b_Valid = false;
   }

   // width and hight must be in some borders
   if(   m_i_Width  < 0
      || m_i_Width  > 30000
      || m_i_Height < 0
      || m_i_Height > 30000
      )
   {
      b_Valid = false;
   }

   // only if the values are valid, the ok button is set
   if(b_Valid)
   {
      m_w_OKButton->setEnabled(true);
   }
   else
   {
      m_w_OKButton->setEnabled(false);
   }

} // TextChanged()

//============================================================================
//
//============================================================================

int NewImageBox::GetWidth()
{
   return m_i_Width;
} // GetWidth()

//============================================================================
//
//============================================================================

int NewImageBox::GetHeight()
{
   return m_i_Height;
} // GetHeight()

//============================================================================
//
//============================================================================


