
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>
#include <qgrid.h>


#include "PaintBox.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Selection.h"
#include "BufferPainter.h"
#include "MouseEvent.h"
#include "Utilities.h"

#include "QULabelSlider.h"
#include "BufferFill.h"
#include "MaskSettings.h"
#include "ApplicationStorage.h"

//============================================================================
//
//============================================================================

PaintBox::PaintBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global),
                Tracer("PaintBox")
{
   QBoxLayout    *l_Layout;

   QHGroupBox   *ActivationBox;

   setCaption("Paint");
   TRACE << "Paint Started\n" ;

   m_b_TransparencyEnabled   = false;
   m_i_CurrentThickness      = 5;
   m_i_TransparencyClearMode = ClearModeAutomatic;

   m_p_FillDistanceColor        = 0x808080;
   m_b_FromStartPoint           = true;
   m_b_FromColor                = false;
   m_i_FillDistance             = 0;
   m_i_LightnessFactor          = 0;
   m_i_FilterWidth              = 1;
   m_b_FilterNearDistanceActive = false;
   m_i_FilterNearDistance       = 30;
   m_i_FilterAlgorithm          = BufferPainter::FilterAlgorithmMiddle;


   m_pc_BufferPainter = new BufferPainter();
   m_pc_BufferPainter->SetLineWidth(1);
   m_pc_BufferPainter->SetDrawInnerEnable(false);
   m_pc_BufferPainter->SetTransparencyEnable(false);
   m_pc_BufferPainter->SetTransparencyFactor(50);
   CreateBrushShape(m_i_CurrentThickness);
   m_pc_BufferPainter->SetFilterWidth(m_i_FilterWidth);
   m_pc_BufferPainter->SetFilterNearDistanceActive(m_b_FilterNearDistanceActive);
   m_pc_BufferPainter->SetFilterNearDistance(m_i_FilterNearDistance);
   m_pc_BufferPainter->SetFilterAlgorithm(m_i_FilterAlgorithm);

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);

   // --- box for activation
   ActivationBox = new QHGroupBox("Paint Operation",m_w_TopWidget);
   l_Layout->addWidget(ActivationBox);
   {
      // buttons to establish painting mode

      m_BrushButton         = new QUPixelButton(ActivationBox,
                                                "Brush",
                                                "Brush");
      m_FillButton          = new QUPixelButton(ActivationBox,
                                                "Fill",
                                                "Fill");
      m_LightnessButton     = new QUPixelButton(ActivationBox,
                                                 "Sun",
                                                 "Lightness");
      m_SaturationButton    = new QUPixelButton(ActivationBox,
                                                "Saturation",
                                                "Saturation");
      m_FilterButton        = new QUPixelButton(ActivationBox,
                                                "Filter",
                                                "Filter");

      connect( m_BrushButton ,
               SIGNAL(clicked()),
               SLOT( ActivateBrush() ) );
      connect( m_FillButton ,
               SIGNAL(clicked()),
               SLOT( ActivateFill() ) );
      connect( m_LightnessButton ,
               SIGNAL(clicked()),
               SLOT( ActivateLightness() ) );
      connect( m_SaturationButton ,
               SIGNAL(clicked()),
               SLOT( ActivateSaturation() ) );
      connect( m_FilterButton ,
               SIGNAL(clicked()),
               SLOT( ActivateFilter() ) );
   }


   // box for brush style
   m_BrushSizeModeBox = new QVGroupBox("Brush Style",m_w_TopWidget);
   l_Layout->addWidget(m_BrushSizeModeBox);

   {
      QULabelSlider *pc_NewSlider;
      pc_NewSlider = new QULabelSlider(m_BrushSizeModeBox,
                                       true,
                                       0,30,
                                       m_i_CurrentThickness,
                                       "Brush Size",
                                       6);

      m_b_Continuous = true;
      m_BrushContinuousEnable = new QCheckBox("Continous",m_BrushSizeModeBox);
      m_BrushContinuousEnable->setChecked(m_b_Continuous);

      connect( pc_NewSlider,
               SIGNAL(valueChanged(int)),
               SLOT( ThicknessChanged(int )) );

      connect( m_BrushContinuousEnable,
               SIGNAL(toggled(bool)) ,
               SLOT( ContinuousEnableChanged(bool) )
             );

   } // box for brush style


   // box for brush color
   m_BrushColorModeBox = new QVGroupBox("Color",m_w_TopWidget);
   l_Layout->addWidget(m_BrushColorModeBox);

   {
      m_BrushColorWidget = new QUColorOrPattern(m_BrushColorModeBox,
                                               m_pc_GlobalStorage);
      m_BrushColorWidget->SetColor(0x335577);

      connect( m_BrushColorWidget,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( BrushColorChanged(Pixel ) )
             );
      connect( m_BrushColorWidget,
               SIGNAL(ImageDropped(PixelBuffer *)) ,
               SLOT( BrushPatternChanged(PixelBuffer *) )
             );
      connect( m_BrushColorWidget,
               SIGNAL(ModeChange(bool)) ,
               SLOT( BrushModeChanged(bool) )
             );

   } // box for Color style

   // -- box for transparency
   m_TransparencyBox = new QVGroupBox("Transparency",m_w_TopWidget);
   l_Layout->addWidget(m_TransparencyBox);

   {
      // inner is off

      m_TransparencyEnable = new QCheckBox("Enable",m_TransparencyBox);
      m_TransparencyEnable->setChecked(m_b_TransparencyEnabled);

      m_TransparencySlider = new QULabelSlider(m_TransparencyBox,true,
                                       0,100,50,
                                       "Factor",6);

      connect( m_TransparencyEnable,
               SIGNAL(toggled(bool)) ,
               SLOT( TransparencyEnableChanged(bool) )
             );
      connect( m_TransparencySlider,
               SIGNAL(valueChanged(int)),
               SLOT( TransparencyFactorChanged(int )) );

   } // block for tranparency box

   // -- box for Mask clearing mode
   m_MaskClearModeBox = new QVGroupBox(" Mask Clear Mode",m_w_TopWidget);
   l_Layout->addWidget(m_MaskClearModeBox);

   {
      m_TransparencyClearCombo = new QComboBox(false,m_MaskClearModeBox,"TransparencyMode");
      m_TransparencyClearCombo->insertItem("manual",0);
      m_TransparencyClearCombo->insertItem("automatic",1);
      m_TransparencyClearCombo->setCurrentItem(m_i_TransparencyClearMode);
      m_ClearTransparencyMaskButton = new QPushButton("Clear",m_MaskClearModeBox);

      connect( m_TransparencyClearCombo,
               SIGNAL(activated(int)),
               SLOT( TransparencyAutomaticChanged(int )) );
      connect( m_ClearTransparencyMaskButton ,
               SIGNAL(clicked()),
               SLOT( TransparencyMaskClear() ) );

   } // block for mask clearing mode

   // -- box for fill mode
   m_FillModeBox = new QVGroupBox("Fill Mode",m_w_TopWidget);
   l_Layout->addWidget(m_FillModeBox);
   {
      QWidget *GridWidget;
      QGridLayout *grid;
      m_FillDistanceSlider = new QULabelSlider(m_FillModeBox,
                                               true,
                                               0,200,
                                               m_i_FillDistance,
                                               "Color Distance",
                                               6);

      new QLabel("Distance from",m_FillModeBox);
      GridWidget = new QWidget(m_FillModeBox);
      grid = new QGridLayout( GridWidget, 2, 3, 10 );

      m_FillModeFromOrigin = new QRadioButton("Startpoint",GridWidget);
      m_FillModeFromColor  = new QRadioButton("Color",GridWidget);

      m_FillDistanceColor = new QUColorDragWidget(GridWidget,
                                               QUColorDragWidget::ModePipette,
                                               m_pc_GlobalStorage);
      m_FillDistanceColor->SetColor(m_p_FillDistanceColor);

      grid->addWidget(m_FillModeFromOrigin,0,1);
      grid->addWidget(m_FillModeFromColor,1,1);
      grid->addWidget(m_FillDistanceColor,1,2);
      grid->addColSpacing(0,g_Application->m_pc_QtUtility->GetMaximalFontWidth());
      grid->setColStretch(1,10);
      grid->setColStretch(2,2);

      m_FillModeFromOrigin ->setChecked(true);
      m_FillModeFromColor  ->setChecked(false);

      connect( m_FillModeFromOrigin,
               SIGNAL(toggled(bool)) ,
               SLOT( FillDistanceFromStartPoint(bool) )
             );
      connect( m_FillModeFromColor,
               SIGNAL(toggled(bool)) ,
               SLOT( FillDistanceFromColor(bool) )
             );
      connect( m_FillDistanceSlider,
               SIGNAL(valueChanged(int)),
               SLOT( FillDistanceChanged(int )) );
      connect( m_FillDistanceColor,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( FillColorChanged(Pixel ) )
             );



   } // block for the fill mode


   // -- box for lightness
   m_LightnessModeBox = new QVGroupBox("Value Modification",
                                       m_w_TopWidget);
   l_Layout->addWidget(m_LightnessModeBox);
   {
       m_LightnessSlider = new QULabelSlider(m_LightnessModeBox,
                                             true,
                                             -50,
                                             50,
                                             m_i_LightnessFactor,
                                             "Change",
                                             6);
      connect( m_LightnessSlider,
               SIGNAL(valueChanged(int)),
               SLOT( LightnessFactorChanged(int )) );

   } // lighness box


   // -- box for Filter mode
   m_FilterModeBox = new QVGroupBox("Filter",m_w_TopWidget);
   l_Layout->addWidget(m_FilterModeBox);
   {
      QButtonGroup * m_AlgorithmBox;
      m_FilterWidthSlider = new QULabelSlider(m_FilterModeBox,
                                              true,
                                              1,10,
                                              m_i_FilterWidth,
                                              "Width",
                                              6);
      m_FilterDistanceActiveCheck = new QCheckBox("Filter only near colors",
                                            m_FilterModeBox);

      m_FilterDistanceSlider = new QULabelSlider(m_FilterModeBox,
                                              true,
                                              1,100,
                                              m_i_FilterNearDistance,
                                              "Distance",
                                              6);

      m_AlgorithmBox = new QButtonGroup(3,
                                    QGroupBox::Horizontal,
                                    "Filter Algorithm",
                                    m_FilterModeBox);

      m_FilterMiddleButton  = new QRadioButton("Middle",m_AlgorithmBox);
      m_FilterMedianButton  = new QRadioButton("Median",m_AlgorithmBox);

      switch(m_i_FilterAlgorithm)
      {
      case BufferPainter::FilterAlgorithmMiddle:
         m_FilterMiddleButton->setChecked(1);
         break;
      case BufferPainter::FilterAlgorithmMedian:
         m_FilterMedianButton->setChecked(1);
         break;
      default:
         break;
      }
      connect( m_AlgorithmBox,
               SIGNAL(clicked(int)),
               SLOT(FilterAlgorithmChanged(int)) );



      connect( m_FilterWidthSlider,
               SIGNAL( valueChanged(int)),
               SLOT(   FilterWidthChanged(int )) );

      connect( m_FilterDistanceActiveCheck,
               SIGNAL( toggled(bool)) ,
               SLOT(   FilterDistanceEnableChanged(bool) ) );

      connect( m_FilterDistanceSlider,
               SIGNAL( valueChanged(int)),
               SLOT(   FilterDistanceChanged(int )) );

   } // filter mode box

   // final settings
   m_pc_BufferPainter->SetBorderColor(m_BrushColorWidget->GetColor());
   m_pc_BufferPainter->SetInnerColor(m_BrushColorWidget->GetColor());

   SetSensitivity();

   // initial show elements for painting / brush
   m_TransparencyBox ->show();
   m_FillModeBox     ->hide();
   m_LightnessModeBox->hide();
   m_FilterModeBox   ->hide();
} // GraphikBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void PaintBox::ActivateBrush()
{
   // enable the valid subwidgets
   m_BrushSizeModeBox ->show();
   m_BrushColorModeBox->show();
   m_TransparencyBox  ->show();
   m_MaskClearModeBox ->show();
   m_FillModeBox      ->hide();
   m_LightnessModeBox ->hide();
   m_FilterModeBox    ->hide();

   ResizeDisplay();

   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  "Brush",
                                                  Callback(SelectionEventArrivedCB,this));

   // used one bit deep mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(1,false);
   m_i_CurrentGraphikMode = PaintBrush;

} // ActivateBrush()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::ActivateFill()
{
   // enable the valid subwidgets
   m_BrushSizeModeBox ->hide();
   m_BrushColorModeBox->show();
   m_TransparencyBox  ->show();
   m_MaskClearModeBox ->hide();
   m_FillModeBox      ->show();
   m_LightnessModeBox ->hide();
   m_FilterModeBox    ->hide();

   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionPoint,
                                                  "Fill",
                                                  Callback(SelectionEventArrivedCB,this));

   // used one bit deep mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(1,false);
   m_i_CurrentGraphikMode = PaintFill;

} // ActivateFill()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void PaintBox::ActivateLightness()
{
   // enable the valid subwidgets
   m_BrushSizeModeBox ->show();
   m_BrushColorModeBox->hide();
   m_TransparencyBox  ->hide();
   m_MaskClearModeBox ->show();
   m_FillModeBox      ->hide();
   m_LightnessModeBox ->show();
   m_FilterModeBox    ->hide();

   ResizeDisplay();

   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  "Lightness",
                                                  Callback(SelectionEventArrivedCB,this));

   // used one bit deep mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(1,false);
   m_i_CurrentGraphikMode = PaintLightness;

} // ActivateLightness()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void PaintBox::ActivateSaturation()
{
   m_BrushSizeModeBox ->show();
   m_BrushColorModeBox->hide();
   m_TransparencyBox  ->hide();
   m_MaskClearModeBox ->show();
   m_FillModeBox      ->hide();
   m_LightnessModeBox ->show();
   m_FilterModeBox    ->hide();

   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  "Saturation",
                                                  Callback(SelectionEventArrivedCB,this));

   // used one bit deep mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(1,false);
   m_i_CurrentGraphikMode = PaintSaturation;

} // ActivateSaturation()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::ActivateFilter()
{
   // enable the valid subwidgets
   m_BrushSizeModeBox ->show();
   m_BrushColorModeBox->hide();
   m_TransparencyBox  ->hide();
   m_MaskClearModeBox ->show();
   m_FillModeBox      ->hide();
   m_LightnessModeBox ->hide();
   m_FilterModeBox    ->show();

   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  "Filter",
                                                  Callback(SelectionEventArrivedCB,this));

   m_i_CurrentGraphikMode = PaintFilter;

   // create a pixelmask, to hold the complete picture and additional a mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(32,true);
   TransparencyMaskClear();

} // ActivateFilter()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()


//===========================================================================
//
//===========================================================================


void *PaintBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures
      return NULL;
   }
   if( !cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      return NULL;
   }
   // from now only real selections


   m_i_MouseEventType = m_pc_GlobalStorage->m_pc_Selection->GetMouseEventType();
   if(m_i_MouseEventType == MouseEvent::EventStart)
   {
      m_b_LastPoint = false;
   }

   // perform different actions depending on the graphic mode
   switch(m_i_CurrentGraphikMode)
   {
   case PaintBrush:
      DrawBrush();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintLightness:
      DrawLightness();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintSaturation:
      DrawSaturation();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintFilter:
      DrawFilter();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintFill:
      FillAtPoint();
      break;
   default:
      break;
   } // switch

   // at mouse end sometimes the transparency mask hast to be cleared
   if(m_i_MouseEventType == MouseEvent::EventEnd)
   {
      m_b_LastPoint = false;

      if(m_i_TransparencyClearMode == ClearModeAutomatic)
      {
         TransparencyMaskClear();
      }
   }
   else if(m_i_MouseEventType == MouseEvent::EventStart
           || m_i_MouseEventType == MouseEvent::EventDo)
   {
      m_b_LastPoint = true;
      m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(m_i_LastX,m_i_LastY);
   }

   return NULL;
} // SelectionEventArrived()


//===========================================================================
//
//===========================================================================


void *PaintBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   PaintBox *pc_This = (PaintBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================


void PaintBox::DrawBrush()
{
   PixelBuffer *pc_Destination;
   RectRegion c_ChangedRegion;
   RectRegion c_LastChangedRegion;
   int        i_X;
   int        i_Y;

   if(m_i_MouseEventType == MouseEvent::EventOver)
   {
      return;
   }

   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;

   if( m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode() == MaskSettings::MaskPaintModeInSafety)
   {
      pc_Destination = pc_Destination->GetSafetyMask();
      if(m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw() == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else if( m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode() == MaskSettings::MaskPaintModeInSelection)
   {
      pc_Destination = pc_Destination->GetSelectionMask();
      if(m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw() == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else
   {
      m_pc_BufferPainter->SetMaskMode(false,false);
   }


   if(   m_b_TransparencyEnabled)
   {

      m_pc_BufferPainter->SetOneTimeDraw(true);
   }

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   if( ! m_b_LastPoint)
   {
      // center brush to coordinate
      i_X -= m_i_CurrentThickness/2;
      i_Y -= m_i_CurrentThickness/2;

      // draw brush
      m_pc_BufferPainter->DrawBrush(pc_Destination,i_X,i_Y);

      c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
   }
   else // last point available
   {
      int i_LastX;
      int i_LastY;

      i_LastX = m_i_LastX;
      i_LastY = m_i_LastY;

      // center brush to coordinate
      i_X -= m_i_CurrentThickness/2;
      i_Y -= m_i_CurrentThickness/2;
      i_LastX -= m_i_CurrentThickness/2;
      i_LastY -= m_i_CurrentThickness/2;

      if(    i_LastX == i_X
          && i_LastY == i_Y)
      {
         // same point, only draw one
         // draw brush
         m_pc_BufferPainter->DrawBrush(pc_Destination,i_X,i_Y);

         c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
      }
      else
      {
         // Draw a line
         int i_DiffX;
         int i_DiffY;
         int i_RunX;
         int i_RunY;
         i_DiffX = i_LastX - i_X;
         i_DiffY = i_LastY - i_Y;
         if(abs (i_DiffX) > abs(i_DiffY) )
         {
            // DiffX is larger;
            if(i_DiffX <0)
            {
               // swap coordinates
               i_DiffX = - i_DiffX;
               i_DiffY = - i_DiffY;
               Utilities::swap(i_LastX,i_X);
               Utilities::swap(i_LastY,i_Y);
            }
            // draw first point
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_X,
                                             i_Y);
            c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

            // draw middle points
            for(i_RunX = 1; i_RunX < i_DiffX; i_RunX ++)
            {
               i_RunY = i_DiffY *i_RunX / i_DiffX;
               m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_RunX+i_X,
                                             i_RunY+i_Y);

            }
            // draw last point
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_LastX,
                                             i_LastY);
            c_LastChangedRegion = m_pc_BufferPainter->GetChangedRegion();

         }
         else
         {
            // DiffY is larger;
            if(i_DiffY <0)
            {
               // swap coordinates
               i_DiffX = - i_DiffX;
               i_DiffY = - i_DiffY;
               Utilities::swap(i_LastX,i_X);
               Utilities::swap(i_LastY,i_Y);

            }
            // draw first point
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_X,
                                             i_Y);
            c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

            // draw middle points
            for(i_RunY = 1; i_RunY < i_DiffY; i_RunY ++)
            {
               i_RunX = i_DiffX *i_RunY / i_DiffY;
               m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_RunX+i_X,
                                             i_RunY+i_Y);

            }
            // draw last point
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_LastX,
                                             i_LastY);
            c_LastChangedRegion = m_pc_BufferPainter->GetChangedRegion();

         }

         // union of the first and last region
         c_ChangedRegion.AddRegion(c_LastChangedRegion);

      } // multiple points



   } // else last point


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);


} // DrawBrush()


//===========================================================================
//
//===========================================================================


void PaintBox::DrawLightness()
{
   DrawSpecial(PaintLightness);
} // DrawLightness()

//===========================================================================
//
//===========================================================================


void PaintBox::DrawSaturation()
{
   DrawSpecial(PaintSaturation);
} // DrawSaturation()

//===========================================================================
//
//===========================================================================

void PaintBox::DrawFilter()
{
   DrawSpecial(PaintFilter);
} // DrawSaturation()

//===========================================================================
//
//===========================================================================

void PaintBox::DrawSpecial(int i_Mode)
{
   PixelBuffer *pc_Destination;
   RectRegion c_ChangedRegion;
   RectRegion c_LastChangedRegion;
   int        i_X;
   int        i_Y;

   if(m_i_MouseEventType == MouseEvent::EventOver)
   {
      return;
   }

   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;

   if( m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode() == MaskSettings::MaskPaintModeInSafety)
   {
      pc_Destination = pc_Destination->GetSafetyMask();
      if(m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw() == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else if( m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode() == MaskSettings::MaskPaintModeInSelection)
   {
      pc_Destination = pc_Destination->GetSelectionMask();
      if(m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw() == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else
   {
      m_pc_BufferPainter->SetMaskMode(false,false);
   }

   // use the one time draw buffer
   m_pc_BufferPainter->SetOneTimeDraw(true);
   if(i_Mode == PaintLightness)
   {
      m_pc_BufferPainter->SetLightnessMode(true);
      m_pc_BufferPainter->SetSaturationMode(false);
      m_pc_BufferPainter->SetFilterMode(false);
   }
   else if(i_Mode == PaintSaturation)
   {
      m_pc_BufferPainter->SetLightnessMode(false);
      m_pc_BufferPainter->SetSaturationMode(true);
      m_pc_BufferPainter->SetFilterMode(false);
   }
   else if(i_Mode == PaintFilter)
   {
      m_pc_BufferPainter->SetLightnessMode(false);
      m_pc_BufferPainter->SetSaturationMode(false);
      m_pc_BufferPainter->SetFilterMode(true);
   }


   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   if( ! m_b_LastPoint)
   {
      // center brush to coordinate
      i_X -= m_i_CurrentThickness/2;
      i_Y -= m_i_CurrentThickness/2;

      // draw brush
      m_pc_BufferPainter->DrawBrush(pc_Destination,i_X,i_Y);

      c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
   }
   else // last point available
   {
      int i_LastX;
      int i_LastY;

      i_LastX = m_i_LastX;
      i_LastY = m_i_LastY;

      // center brush to coordinate
      i_X -= m_i_CurrentThickness/2;
      i_Y -= m_i_CurrentThickness/2;
      i_LastX -= m_i_CurrentThickness/2;
      i_LastY -= m_i_CurrentThickness/2;

      if(    i_LastX == i_X
          && i_LastY == i_Y)
      {
         // same point, only draw one
         // draw brush
         m_pc_BufferPainter->DrawBrush(pc_Destination,i_X,i_Y);

         c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
      }
      else
      {
         // Draw a line
         int i_DiffX;
         int i_DiffY;
         int i_RunX;
         int i_RunY;
         i_DiffX = i_LastX - i_X;
         i_DiffY = i_LastY - i_Y;
         if(abs (i_DiffX) > abs(i_DiffY) )
         {
            // DiffX is larger;
            if(i_DiffX <0)
            {
               // swap coordinates
               i_DiffX = - i_DiffX;
               i_DiffY = - i_DiffY;
               Utilities::swap(i_LastX,i_X);
               Utilities::swap(i_LastY,i_Y);
            }
            // draw first point
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_X,
                                             i_Y);
            c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

            // draw middle points
            for(i_RunX = 1; i_RunX < i_DiffX; i_RunX ++)
            {
               i_RunY = i_DiffY *i_RunX / i_DiffX;
               m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_RunX+i_X,
                                             i_RunY+i_Y);

            }
            // draw last point
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_LastX,
                                             i_LastY);
            c_LastChangedRegion = m_pc_BufferPainter->GetChangedRegion();

         }
         else
         {
            // DiffY is larger;
            if(i_DiffY <0)
            {
               // swap coordinates
               i_DiffX = - i_DiffX;
               i_DiffY = - i_DiffY;
               Utilities::swap(i_LastX,i_X);
               Utilities::swap(i_LastY,i_Y);

            }
            // draw first point
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_X,
                                             i_Y);
            c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

            // draw middle points
            for(i_RunY = 1; i_RunY < i_DiffY; i_RunY ++)
            {
               i_RunX = i_DiffX *i_RunY / i_DiffY;
               m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_RunX+i_X,
                                             i_RunY+i_Y);

            }
            // draw last point
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_LastX,
                                             i_LastY);
            c_LastChangedRegion = m_pc_BufferPainter->GetChangedRegion();

         }

         // union of the first and last region
         c_ChangedRegion.AddRegion(c_LastChangedRegion);

      } // multiple points



   } // else last point


   // disable the lightness mode
   m_pc_BufferPainter->SetLightnessMode(false);
   m_pc_BufferPainter->SetSaturationMode(false);
   m_pc_BufferPainter->SetFilterMode(false);

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);


} // DrawSpecial()


//===========================================================================
//
//===========================================================================

void PaintBox::ThicknessChanged(int i_Width)
{

   m_i_CurrentThickness = i_Width;
   CreateBrushShape(m_i_CurrentThickness);

} // ThicknessChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::TransparencyEnableChanged(bool b_on)
{
   m_b_TransparencyEnabled = b_on;
   m_pc_BufferPainter->SetTransparencyEnable(b_on);

   if( m_b_TransparencyEnabled)
   {
      // allocate the one time draw buffer
      m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask();
   }

   SetSensitivity();

} // ransparencyEnableChanged

//===========================================================================
//
//===========================================================================

void PaintBox::TransparencyFactorChanged(int i_Factor)
{
   m_pc_BufferPainter->SetTransparencyFactor(i_Factor);
} // TransparencyFactorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::LightnessFactorChanged(int i_Factor)
{
   m_i_LightnessFactor = i_Factor;
   m_pc_BufferPainter->SetLightnessFactor(i_Factor);
} // LightnessFactorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::BrushColorChanged(Pixel c_Color)
{
   m_pc_BufferPainter->SetBorderColor(c_Color);
   m_pc_BufferPainter->SetInnerColor(c_Color);
} // BrushColorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::BrushPatternChanged(PixelBuffer *pc_Buffer)
{
   m_pc_BufferPainter->SetBorderPattern(pc_Buffer);
   m_pc_BufferPainter->SetInnerPattern(pc_Buffer);

}  // BrushPatternChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::BrushModeChanged(bool b_PatternMode)
{
   m_pc_BufferPainter->SetBorderDrawMode(b_PatternMode);
   m_pc_BufferPainter->SetInnerDrawMode(b_PatternMode);

} // BrushModeChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::CreateBrushShape(int i_Thickness)
{
   PixelBuffer *pc_Brush;
   float        f_Center;
   int          i_X;
   int          i_Y;
   int          i_Half;

   // create Buffer
   pc_Brush = new PixelBuffer(i_Thickness,i_Thickness,1,false,false);

   // create Circle
   f_Center = (i_Thickness-1) / 2.0;
   i_Half   = (i_Thickness+1) / 2;
   for( i_Y=0 ; i_Y < i_Half; i_Y ++)
   {
      for( i_X=0 ; i_X < i_Half; i_X ++)
      {
         float f_DifferenceX;
         float f_DifferenceY;
         float f_TotalDifference;
         f_DifferenceX = f_Center - i_X;
         f_DifferenceY = f_Center - i_Y;
         f_TotalDifference = sqrt(   f_DifferenceX*f_DifferenceX
                                   + f_DifferenceY*f_DifferenceY );
         f_TotalDifference *= 2;
         if(f_TotalDifference <= i_Thickness)
         {
            // draw in all four quadrants
            pc_Brush->Set(i_X                ,i_Y                ,1);
            pc_Brush->Set(i_X                ,(i_Thickness-1)-i_Y,1);
            pc_Brush->Set((i_Thickness-1)-i_X,i_Y                ,1);
            pc_Brush->Set((i_Thickness-1)-i_X,(i_Thickness-1)-i_Y,1);
         }

      } // for X
   } // for Y


   // Set the brush

   m_pc_BufferPainter->SetBrushShape(pc_Brush);


} // CreateBrushShape();

//===========================================================================
//
//===========================================================================

void PaintBox::ContinuousEnableChanged(bool b_on)
{
   m_b_Continuous = b_on;
   SetSensitivity();
} // ContinuousEnableChanged()

//===========================================================================
//
//===========================================================================


void PaintBox::FillAtPoint()
{
   PixelBuffer *pc_DestinationBuffer ;
   BufferFill   c_ActualFill;
   RectRegion   c_ChangedRegion;
   int          i_MaskMode;
   int          i_X;
   int          i_Y;

   // set the distance options
   c_ActualFill.SetColorDistance(m_i_FillDistance);
   c_ActualFill.SetColorDistanceColor(m_p_FillDistanceColor);
   if(m_b_FromStartPoint)
   {
      c_ActualFill.SetColorDistanceFrom(BufferFill::BufferFillDistanceOrigin);
   }
   else
   {
      c_ActualFill.SetColorDistanceFrom(BufferFill::BufferFillDistanceColor);
   }

   m_pc_BufferPainter->SetOneTimeDraw(false);
   pc_DestinationBuffer = NULL;

   i_MaskMode = m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode();

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   c_ActualFill.m_pc_SourceBuffer      = m_pc_GlobalStorage->m_pc_MainImage;
   if(   i_MaskMode == MaskSettings::MaskPaintModeInSafety
      || i_MaskMode == MaskSettings::MaskPaintModeInSelection)
   {
      pc_DestinationBuffer = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                                             1,
                                             false,false);
      c_ActualFill.m_pc_DestinationBuffer = pc_DestinationBuffer;
      c_ActualFill.m_ChangeColor = m_pc_GlobalStorage->m_pc_MainImage->Get(i_X,i_Y);
      c_ActualFill.m_NewColor    = 1;
   }
   else if(m_b_TransparencyEnabled )
   {
      pc_DestinationBuffer = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                                             1,
                                             false,false);
      c_ActualFill.m_pc_DestinationBuffer = pc_DestinationBuffer;
      c_ActualFill.m_ChangeColor = m_pc_GlobalStorage->m_pc_MainImage->Get(i_X,i_Y);
      c_ActualFill.m_NewColor    = 1;
   }
   else if(m_i_FillDistance > 0 )
   {
      pc_DestinationBuffer = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                                             1,
                                             false,false);
      c_ActualFill.m_pc_DestinationBuffer = pc_DestinationBuffer;
      c_ActualFill.m_ChangeColor = m_pc_GlobalStorage->m_pc_MainImage->Get(i_X,i_Y);
      c_ActualFill.m_NewColor    = 1;
   }
   else
   {
      pc_DestinationBuffer = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                                             1,
                                             false,false);
      c_ActualFill.m_pc_DestinationBuffer = pc_DestinationBuffer;
      c_ActualFill.m_ChangeColor = m_pc_GlobalStorage->m_pc_MainImage->Get(i_X,i_Y);
      c_ActualFill.m_NewColor    = 1;
   }


   c_ActualFill.StartFill(i_X, i_Y);

   c_ChangedRegion = c_ActualFill.GetChangedRegion();
   if(i_MaskMode == MaskSettings::MaskPaintModeInSafety)
   {
      // draw transparent from the destinationBuffer into the
      // realImage
      if(    m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
          == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
      m_pc_BufferPainter->DrawMarkedPixel(m_pc_GlobalStorage->m_pc_MainImage->GetSafetyMask(),
                        pc_DestinationBuffer,
                        &c_ChangedRegion);
   }
   else if(i_MaskMode == MaskSettings::MaskPaintModeInSelection)
   {
      // draw transparent from the destinationBuffer into the
      // realImage
      if(m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw() == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
      m_pc_BufferPainter->DrawMarkedPixel(m_pc_GlobalStorage->m_pc_MainImage->GetSelectionMask(),
                        pc_DestinationBuffer,
                        &c_ChangedRegion);
   }
   else // no mask Mode
   {
      if(   m_b_TransparencyEnabled
         || m_i_FillDistance > 0
         || pc_DestinationBuffer != NULL  
        )
      {
         // draw transparent from the destinationBuffer into the
         // realImage
         m_pc_BufferPainter->DrawMarkedPixel(m_pc_GlobalStorage->m_pc_MainImage,
                        pc_DestinationBuffer,
                        &c_ChangedRegion);
      }
   }

   c_ChangedRegion = c_ActualFill.GetChangedRegion();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   // delete the destinationbuffer
   if(pc_DestinationBuffer != NULL)
   {
      delete pc_DestinationBuffer;
   }

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                 this,
                                                 "ImageChanged");
} // FillAtPoint

//===========================================================================
//
//===========================================================================

void PaintBox::TransparencyAutomaticChanged(int i_NewValue)
{
   m_i_TransparencyClearMode = i_NewValue;
   SetSensitivity();

} // TransparencyAutomaticChanged()

//===========================================================================
//
//===========================================================================

void PaintBox:: TransparencyMaskClear()
{
   bool b_Copied;

   b_Copied = false;

   // if in filter mode, then also copy the original image
   if(m_i_CurrentGraphikMode == PaintFilter)
   {
      PixelBuffer *pc_UsedPixelMask ;

      pc_UsedPixelMask = m_pc_GlobalStorage->m_pc_MainImage->GetUsedPixelMask();
      if(pc_UsedPixelMask != NULL)
      {
         if(pc_UsedPixelMask->GetDepth() >1)
         {
            // copy the original mage
            pc_UsedPixelMask->CopyFrom(m_pc_GlobalStorage->m_pc_MainImage,false,false,false);
            // clean up the additional mask
            m_pc_GlobalStorage->m_pc_MainImage->UsedPixelMaskClean();
            // remember that its done
            b_Copied = true;
         }
      }
   }

   // if not done
   if( !b_Copied)
   {
      // the do the regular way
      m_pc_GlobalStorage->m_pc_MainImage->UsedPixelMaskClean();
   }

} // TransparencyMaskClear()


//===========================================================================
//
//===========================================================================

void PaintBox::SetSensitivity()
{
   if(m_b_TransparencyEnabled)
   {
      m_TransparencySlider->setEnabled(true);

   }
   else
   {
      // transparency is disabled
      m_TransparencySlider->setEnabled(false);

   }

   // set the availiability of the filter distance slider
   if(m_b_FilterNearDistanceActive )
   {
      m_FilterDistanceSlider ->setEnabled(true);
   }
   else
   {
      m_FilterDistanceSlider ->setEnabled(false);
   }

} // SetSensitivity()

//===========================================================================
//
//===========================================================================

void PaintBox::FillColorChanged(Pixel p_Color)
{
   m_p_FillDistanceColor = p_Color;
} // FillColorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FillDistanceChanged(int i_Factor)
{
   m_i_FillDistance = i_Factor;
} // FillDistanceChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FillDistanceFromStartPoint(bool b_On)
{
   m_b_FromStartPoint = b_On;
   m_FillModeFromColor  ->setChecked(!b_On);
} // FillDistanceFromStartPoint()

//===========================================================================
//
//===========================================================================

void PaintBox::FillDistanceFromColor(bool b_On)
{
   m_b_FromColor = b_On;
   m_FillModeFromOrigin ->setChecked(!b_On);

} // FillDistanceFromColor()

//===========================================================================
//
//===========================================================================

void PaintBox::FilterWidthChanged(int i_Factor)
{
   m_i_FilterWidth = i_Factor;
   m_pc_BufferPainter->SetFilterWidth(m_i_FilterWidth);
} // FilterWidthChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FilterDistanceEnableChanged(bool b_On)
{
   m_b_FilterNearDistanceActive = b_On;
   m_pc_BufferPainter->SetFilterNearDistanceActive(m_b_FilterNearDistanceActive);
   SetSensitivity();
} // FilterDistanceEnableChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FilterDistanceChanged(int i_Distance)
{
   m_i_FilterNearDistance = i_Distance;
   m_pc_BufferPainter->SetFilterNearDistance(m_i_FilterNearDistance);
} // FilterDistanceChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FilterAlgorithmChanged(int i_NewMode)
{
   m_i_FilterAlgorithm = i_NewMode;
   m_pc_BufferPainter->SetFilterAlgorithm(m_i_FilterAlgorithm);
} // FilterAlgorithmChanged()

//===========================================================================
//
//===========================================================================


