#ifndef _PAINTBOX_H
#define _PAINTBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qvgroupbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>

#include "StandardDialog.h"
#include "BufferPainter.h"
#include "QULabelSlider.h"
#include "QUColorDragWidget.h"
#include "QUPixelButton.h"
#include "Tracer.h"
#include "QUColorOrPattern.h"


class PaintBox :  public StandardDialog ,
                  public Tracer
{
Q_OBJECT

public:
   PaintBox(QWidget *parent=0,
            GlobalStorage *pc_Global=0);

public slots:

   void ActivateBrush();
   void ActivateFill();
   void ActivateLightness();
   void ActivateSaturation();
   void ActivateFilter();


   void ThicknessChanged(int i_Width);
   void ContinuousEnableChanged(bool b_on);


   void BrushColorChanged(Pixel c_Color);
   void BrushPatternChanged(PixelBuffer *pc_Buffer);
   void BrushModeChanged(bool b_PatternMode);

   void CreateBrushShape(int i_Thickness);

   void TransparencyEnableChanged(bool b_on);
   void TransparencyFactorChanged(int i_Factor);
   void TransparencyAutomaticChanged(int i_NewValue);
   void TransparencyMaskClear();

   void FillColorChanged(Pixel c_Color);
   void FillDistanceChanged(int i_Factor);
   void FillDistanceFromStartPoint(bool b_On);
   void FillDistanceFromColor(bool b_On);

   void LightnessFactorChanged(int i_Factor);

   void FilterWidthChanged(int i_Factor);
   void FilterDistanceEnableChanged(bool b_on);
   void FilterDistanceChanged(int i_Distance);
   void FilterAlgorithmChanged(int i_NewMode);

private:


   enum
   {
      PaintBrush,
      PaintFill,
      PaintLightness,
      PaintSaturation,
      PaintFilter
   };

    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   void *SelectionEventArrived(void *pc_Data);

   static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);

   void DrawBrush();
   void DrawLightness();
   void DrawSaturation();
   void DrawFilter();
   void FillAtPoint();

   void DrawSpecial(int i_Mode);
   void SetSensitivity();

   // variables
   // widgets
   QUColorOrPattern  *m_BrushColorWidget;
   QULabelSlider     *m_TransparencySlider;
   QUPixelButton     *m_BrushButton;
   QUPixelButton     *m_FillButton;
   QUPixelButton     *m_LightnessButton;
   QUPixelButton     *m_SaturationButton;
   QUPixelButton     *m_FilterButton;
   QPushButton       *m_ClearTransparencyMaskButton;
   QComboBox         *m_TransparencyClearCombo;
   QCheckBox         *m_TransparencyEnable;
   QCheckBox         *m_BrushContinuousEnable;
   QSlider           *m_ThicknessSlider;

   QULabelSlider     *m_FillDistanceSlider;
   QUColorDragWidget *m_FillDistanceColor;
   QButtonGroup      *m_FillModeWidget;
   QRadioButton      *m_FillModeFromOrigin;
   QRadioButton      *m_FillModeFromColor;

   QULabelSlider     *m_FilterWidthSlider;
   QCheckBox         *m_FilterDistanceActiveCheck;
   QULabelSlider     *m_FilterDistanceSlider;
   QRadioButton      *m_FilterMiddleButton;
   QRadioButton      *m_FilterMedianButton;

   QVGroupBox        *m_BrushSizeModeBox;
   QVGroupBox        *m_BrushColorModeBox;
   QVGroupBox        *m_TransparencyBox;
   QVGroupBox        *m_MaskClearModeBox;
   QVGroupBox        *m_FillModeBox;
   QVGroupBox        *m_LightnessModeBox;
   QVGroupBox        *m_FilterModeBox;

   QULabelSlider     *m_LightnessSlider;

   BufferPainter *m_pc_BufferPainter;
   bool           m_b_Continuous;
   bool           m_b_TransparencyEnabled;
   bool           m_b_LastPoint;
   int            m_i_LastX;
   int            m_i_LastY;
   int            m_i_MouseEventType;
   int            m_i_CurrentThickness;
   int            m_i_TransparencyClearMode;
   int            m_i_CurrentGraphikMode;
   int            m_i_LightnessFactor;

   // mode for transparecy single draw mask clearing
   enum
   {
      ClearModeManual=0,
      ClearModeAutomatic=1,
   };

   Pixel m_p_FillDistanceColor;
   int   m_i_FillDistance;
   bool  m_b_FromStartPoint;
   bool  m_b_FromColor;

   // variables for the filter settings
    int          m_i_FilterWidth;
    bool         m_b_FilterNearDistanceActive;
    int          m_i_FilterNearDistance;
    int          m_i_FilterAlgorithm;


};


#endif // _PAINTBOX_H
