// 2 dimensional pixel buffer

#include <stdio.h>
#include <string.h>

#include "PixelBuffer.h"
#include "Utilities.h"

//===========================================================================
// help function to just allocate the buffer
//===========================================================================

void PixelBuffer::AllocBuffer(int i_Width,
                              int i_Height,
                              int i_Depth,
                              bool b_Clear)
{
   switch(i_Depth)
   {
   case BITNUMBER_BIT:
      m_i_BufferSize = ((i_Width * i_Height) + 7) /8;
      break;
   case BITNUMBER_BYTE:
      m_i_BufferSize = (i_Width * i_Height) ;
      break;
   case BITNUMBER_LONG:
      m_i_BufferSize = ((i_Width * i_Height) * 4);
      break;
   default:
      // TODO
      fprintf(stderr,"BITDEPTH NOT IMPLEMENTED IN PIXELBUFFER");
      i_Width        =1;
      i_Height       =1;
      m_i_BufferSize = 1;
      break;

   }

   m_b_Valid = true;

   m_p_Buffer = new unsigned char [m_i_BufferSize];

   if(m_p_Buffer == NULL)
   {
      m_i_BufferWidth  = 0;
      m_i_BufferHeight = 0;
      m_b_Valid          = false;
      return;
   }
   // clean the buffer
   if( b_Clear )
   {
      memset(m_p_Buffer,0,m_i_BufferSize);
   }

   m_i_BufferWidth  = i_Width;
   m_i_BufferHeight = i_Height;
   m_i_BufferDepth  = i_Depth;


} // AllocBuffer()


//===========================================================================
//
//===========================================================================

PixelBuffer:: PixelBuffer(int i_Width,
                          int i_Height,
                          int i_Depth,
                          bool b_UseSelection,
                          bool b_UseSafety,
                          bool b_UsedPixelMask )
{

   m_pc_SelectionMask = NULL;
   m_pc_SafetyMask    = NULL;

   // allocate regular buffer
   AllocBuffer(i_Width,i_Height,i_Depth);

   // allocate selection mask
   m_b_UseSelectionMask = false;
   if(b_UseSelection)
   {
      m_pc_SelectionMask = new PixelBuffer(m_i_BufferWidth,
                                           m_i_BufferHeight,
                                           1,
                                           false,
                                           false,
                                           false);
      if( !m_pc_SelectionMask->IsValid())
      {
         delete m_pc_SelectionMask;
         m_pc_SelectionMask = NULL;
         m_b_Valid = false;

      }
   }
   else
   {
      m_pc_SelectionMask = NULL;
   }

   // allocate safety mask
   m_b_UseSafetyMask = false;
   if(b_UseSafety)
   {
      m_pc_SafetyMask = new PixelBuffer(m_i_BufferWidth,
                                        m_i_BufferHeight,
                                        1,
                                        false,
                                        false,
                                        false);
      if( !m_pc_SafetyMask->IsValid())
      {
         delete m_pc_SafetyMask;
         m_pc_SafetyMask = NULL;
         m_b_Valid = false;
      }
   }
   else
   {
      m_pc_SafetyMask = NULL;
   }

   // allocated used pixel mask
   if(b_UsedPixelMask)
   {
      m_pc_UsedPixelMask = new PixelBuffer(m_i_BufferWidth,
                                           m_i_BufferHeight,
                                           1,
                                           false,
                                           false,
                                           false);
      if( !m_pc_UsedPixelMask->IsValid())
      {
         delete m_pc_UsedPixelMask;
         m_pc_UsedPixelMask = NULL;
         m_b_Valid = false;
      }
   }
   else
   {
      m_pc_UsedPixelMask = NULL;
   }

} // PixelBuffer() constructor

//===========================================================================
// no contents is copied
//===========================================================================

PixelBuffer:: PixelBuffer(PixelBuffer *pc_Example,
                          int i_Depth,
                          bool b_UseSelection,
                          bool b_UseSafety,
                          bool b_UsedPixelMask )
{
   int i_Width;
   int i_Height;

   i_Width  = pc_Example->m_i_BufferWidth;
   i_Height = pc_Example->m_i_BufferHeight;

   m_pc_SelectionMask = NULL;
   m_pc_SafetyMask    = NULL;

   AllocBuffer(i_Width,i_Height,i_Depth);

   m_b_UseSelectionMask = false;
   if(b_UseSelection)
   {
      m_pc_SelectionMask = new PixelBuffer(m_i_BufferWidth,
                                           m_i_BufferHeight,
                                           1,
                                           false,
                                           false,
                                           false);
      if( !m_pc_SelectionMask->IsValid())
      {
         delete m_pc_SelectionMask;
         m_pc_SelectionMask = NULL;
         m_b_Valid = false;

      }
   }
   else
   {
      m_pc_SelectionMask = NULL;
   }

   m_b_UseSafetyMask = false;
   if(b_UseSafety)
   {
      m_pc_SafetyMask = new PixelBuffer(m_i_BufferWidth,
                                        m_i_BufferHeight,
                                        1,
                                        false,
                                        false,
                                        false);
      if( !m_pc_SafetyMask->IsValid())
      {
         delete m_pc_SafetyMask;
         m_pc_SafetyMask = NULL;
         m_b_Valid = false;
      }
   }
   else
   {
      m_pc_SafetyMask = NULL;
   }

   if(b_UsedPixelMask)
   {
      m_pc_UsedPixelMask = new PixelBuffer(m_i_BufferWidth,
                                           m_i_BufferHeight,
                                           1,
                                           false,
                                           false,
                                           false);
      if( !m_pc_UsedPixelMask->IsValid())
      {
         delete m_pc_UsedPixelMask;
         m_pc_UsedPixelMask = NULL;
         m_b_Valid = false;
      }
   }
   else
   {
      m_pc_UsedPixelMask = NULL;
   }

} // PixelBuffer() constructor

//===========================================================================
// make a 1-to-1 copy, event with the contents
//===========================================================================

PixelBuffer:: PixelBuffer(PixelBuffer *pc_Example )
{
   int i_Width;
   int i_Height;
   int i_Depth;


   i_Width  = pc_Example->m_i_BufferWidth;
   i_Height = pc_Example->m_i_BufferHeight;
   i_Depth  = pc_Example->m_i_BufferDepth;

   m_pc_SelectionMask = NULL;
   m_pc_SafetyMask    = NULL;

   AllocBuffer(i_Width,i_Height,i_Depth);

   if(m_p_Buffer != NULL)
   {
      memcpy(m_p_Buffer,pc_Example->m_p_Buffer,m_i_BufferSize);
   }

   // copy selectionMask
   if(pc_Example->m_pc_SelectionMask != NULL)
   {
      m_pc_SelectionMask = new PixelBuffer(pc_Example->m_pc_SelectionMask);
   }
   else
   {
      m_pc_SelectionMask = NULL;
   }

   // copy the savety mask
   if(pc_Example->m_pc_SafetyMask != NULL)
   {
      m_pc_SafetyMask = new PixelBuffer(pc_Example->m_pc_SafetyMask);
   }
   else
   {
      m_pc_SafetyMask = NULL;
   }

   // copy the used pixel mask
   if(pc_Example->m_pc_UsedPixelMask != NULL)
   {
      m_pc_UsedPixelMask = new PixelBuffer(pc_Example->m_pc_UsedPixelMask);
   }
   else
   {
      m_pc_UsedPixelMask = NULL;
   }


   m_b_UseSelectionMask = pc_Example->m_b_UseSelectionMask;
   m_b_UseSafetyMask    = pc_Example->m_b_UseSafetyMask;


} // PixelBuffer() constructor

//===========================================================================
//
//===========================================================================

PixelBuffer::~PixelBuffer()
{
   // delete the selection mask
   if(m_pc_SelectionMask != NULL)
   {
      delete m_pc_SelectionMask;
      m_pc_SelectionMask = NULL;
   }

   // delete the safety mask
   if(m_pc_SafetyMask != NULL)
   {
      delete m_pc_SafetyMask;
      m_pc_SafetyMask = NULL;
   }

   // delete the used pixel mask
   if(m_pc_UsedPixelMask != NULL)
   {
      delete m_pc_UsedPixelMask;
      m_pc_UsedPixelMask = NULL;
   }

   // delete the buffer itself
   if(m_p_Buffer != NULL)
   {
      delete[] m_p_Buffer;
      m_p_Buffer = NULL;
   }


} // ~PixelBuffer()

//===========================================================================
//
//===========================================================================

bool PixelBuffer::IsValid()
{
   return m_b_Valid;
} // IsValid()

//===========================================================================
//
//===========================================================================

PixelBuffer *PixelBuffer::GetSelectionMask()
{
   return m_pc_SelectionMask;
} // GetSelectionMask()

//===========================================================================
//
//===========================================================================

PixelBuffer *PixelBuffer::GetSafetyMask()
{
   return m_pc_SafetyMask;
} // GetSafetyMask()

//===========================================================================
//
//===========================================================================

PixelBuffer *PixelBuffer::GetUsedPixelMask()
{
   return m_pc_UsedPixelMask;
} // GetUsedPixelMask()
//===========================================================================
//
//===========================================================================

void PixelBuffer::SetRaw(int i_X, int i_Y, Pixel value)
{
   int i_PixelPosition;
   // check, whether in coordinate range
   if(    i_X < 0
       || i_Y < 0
       || i_X >= m_i_BufferWidth
       || i_Y >= m_i_BufferHeight
     )
   {
      return;
   }

   // calculate linear position
   i_PixelPosition = m_i_BufferWidth*i_Y + i_X;

   switch(m_i_BufferDepth)
   {
   case BITNUMBER_BIT:
      {
         unsigned char *p_Position;
         unsigned char uc_ShiftedValue;
         int i_ByteNumber;
         int i_BitNumber;
         i_ByteNumber = i_PixelPosition / 8;
         i_BitNumber  = i_PixelPosition % 8;
         uc_ShiftedValue = 1<< i_BitNumber;
         p_Position   = & m_p_Buffer[i_ByteNumber];

         if(value)
         {
            // set the bit
            * p_Position |= uc_ShiftedValue;
         }
         else
         {
            // clear the bit
            * p_Position &= (~ uc_ShiftedValue);
         }

      }
      break;
   case BITNUMBER_BYTE:
      {
         PBYTE  * p_Position;
         p_Position = (PBYTE *) m_p_Buffer;
         p_Position += i_PixelPosition;
         * p_Position = (PBYTE) value;
      }
      break;
   case BITNUMBER_LONG:
      {
         PLONG * p_Position;
         p_Position = (PLONG *) m_p_Buffer;
         p_Position += i_PixelPosition;
         * p_Position = (PLONG) value;
      }
      break;
   default:
      // TODO
      fprintf(stderr,"BITDEPTH NOT IMPLEMENTED IN PIXELBUFFER:SetRaw");
      break;

   }

} // SetRaw()

//===========================================================================
//
//===========================================================================

void PixelBuffer::Set(int i_X, int i_Y, Pixel value)
{
   // check, whether in coordinate range
   if(    i_X < 0
       || i_Y < 0
       || i_X >= m_i_BufferWidth
       || i_Y >= m_i_BufferHeight
     )
   {
      return;
   }

   // test, whether a safetymask is valid)
   if(    m_b_UseSafetyMask
       && m_pc_SafetyMask != NULL)
   {
      //safety mask enabled and available
      Pixel p_Safety;
      p_Safety = m_pc_SafetyMask->Get(i_X, i_Y);
      if(p_Safety != 0)
      {
         // pixel secured, don't change
         return;
      }
   }

   // if passed, then set realy
   SetRaw(i_X,i_Y,value);

} // Set()

//===========================================================================
//
//===========================================================================

Pixel PixelBuffer::Get(int i_X, int i_Y)
{
   Pixel p_Value;
   int   i_PixelPosition;

   p_Value = 0;

   // check, whether in coordinate range
   if(    i_X < 0
       || i_Y < 0
       || i_X >= m_i_BufferWidth
       || i_Y >= m_i_BufferHeight
     )
   {
      return p_Value;
   }

   i_PixelPosition = m_i_BufferWidth*i_Y + i_X;

   switch(m_i_BufferDepth)
   {
   case BITNUMBER_BIT:
      {
         unsigned char *p_Position;
         unsigned char uc_ShiftedValue;
         int i_ByteNumber;
         int i_BitNumber;
         i_ByteNumber = i_PixelPosition / 8;
         i_BitNumber  = i_PixelPosition % 8;
         uc_ShiftedValue = 1<< i_BitNumber;
         p_Position   = & m_p_Buffer[i_ByteNumber];

         p_Value =  ( * p_Position) & uc_ShiftedValue;
         if(p_Value != 0)
         {
            p_Value = 1;
         }

      }
      break;
   case BITNUMBER_BYTE:
      {
         PBYTE  * p_Position;
         p_Position = (PBYTE *) m_p_Buffer;
         p_Position += i_PixelPosition;
         p_Value = (Pixel) (* p_Position) ;
      }
      break;
   case BITNUMBER_LONG:
      {
         PLONG * p_Position;
         p_Position = (PLONG *) m_p_Buffer;
         p_Position += i_PixelPosition;
         p_Value = (Pixel) (* p_Position) ;
      }
      break;
   default:
      // TODO
      fprintf(stderr,"BITDEPTH NOT IMPLEMENTED IN PIXELBUFFER:Get");
      break;

   }

   return p_Value;
} // Get()

//===========================================================================
//
//===========================================================================

bool PixelBuffer::IsSelected(int i_X, int i_Y)
{
   if(   ! m_b_UseSelectionMask
      ||   m_pc_SelectionMask == NULL)
   {
      return false;
   }
   return m_pc_SelectionMask->Get(i_X,i_Y);
   
} // IsSelected()

//===========================================================================
//
//===========================================================================

int PixelBuffer::GetDepth()
{
   return m_i_BufferDepth;
}
//===========================================================================
//
//===========================================================================

int PixelBuffer::GetWidth()
{
   return m_i_BufferWidth;
}

//===========================================================================
//
//===========================================================================

int PixelBuffer::GetHeight()
{
   return m_i_BufferHeight;
}


//===========================================================================
//
//===========================================================================

void * PixelBuffer::GetPixelAddress(int i_X,int i_Y)
{
   int i_PixelPosition;
   // check, whether in coordinate range
   if(    i_X < 0
       || i_Y < 0
       || i_X >= m_i_BufferWidth
       || i_Y >= m_i_BufferHeight
     )
   {
      return NULL;
   }

   i_PixelPosition = m_i_BufferWidth*i_Y + i_X;

   switch(m_i_BufferDepth)
   {
   case BITNUMBER_BIT:
      // TODO
      fprintf(stderr,"BITDEPTH NOT IMPLEMENTED IN PixelBuffer::GetPixelAddress");
      // it also seems no to be usefull
      break;
   case BITNUMBER_BYTE:
      {
         PBYTE  * p_Position;
         p_Position = (PBYTE *) m_p_Buffer;
         p_Position += i_PixelPosition;
         return (void *) p_Position ;
      }
      break;
   case BITNUMBER_LONG:
      {
         PLONG * p_Position;
         p_Position = (PLONG *) m_p_Buffer;
         p_Position += i_PixelPosition;
         return (void *) p_Position ;
      }
      break;
   default:
      // TODO
      fprintf(stderr,"BITDEPTH NOT IMPLEMENTED IN PIXELBUFFER:SetRaw");
      break;

   }
   return NULL;

} // GetPixelAddress()


//===========================================================================
//
//===========================================================================

bool PixelBuffer::Rotate(int i_Degree)
{
   if(i_Degree == 180)
   {
      Pixel p1;
      Pixel p2;
      int   i_LineFrom;
      int   i_LineTo;

      i_LineFrom = 0;
      i_LineTo   = m_i_BufferHeight -1;
      while( i_LineFrom <= i_LineTo)
      {
         if(i_LineFrom == i_LineTo)
         {
            int i;
            int i_Back;
            for(i =0 , i_Back = m_i_BufferWidth-1;
                i< m_i_BufferWidth/2;
                i++,i_Back --)
            {
               p1 = Get(i,i_LineFrom);
               p2 = Get(i_Back,i_LineFrom);
               SetRaw(i,i_LineFrom,p2);
               SetRaw(i_Back,i_LineFrom,p1);

            } // for i
         }
         else
         {
            int i;
            int i_Back;
            for(i =0, i_Back = m_i_BufferWidth-1;
                i< m_i_BufferWidth;
                i++,i_Back --)
            {
               p1 = Get(i,i_LineFrom);
               p2 = Get(i_Back,i_LineTo);
               SetRaw(i,i_LineFrom,p2);
               SetRaw(i_Back,i_LineTo,p1);

            } // for i
         }

         i_LineFrom++;
         i_LineTo --;

      }  // while

      if(m_pc_SelectionMask != NULL)
      {
         m_pc_SelectionMask->Rotate(i_Degree);
      }
      if(m_pc_SafetyMask != NULL)
      {
         m_pc_SafetyMask->Rotate(i_Degree);
      }
      if(m_pc_UsedPixelMask != NULL)
      {
         m_pc_UsedPixelMask->Rotate(i_Degree);
      }
   } // 180

   else if(   i_Degree == 90
           || i_Degree == 270)
   {
      int i_RunX;
      int i_RunY;
      PixelBuffer *pc_BitBuffer;

      pc_BitBuffer = new PixelBuffer (this, BITNUMBER_BIT,false,false);

      if (pc_BitBuffer == NULL)

      {
         return false;

      }


      // rotate used pixel mask in regular way, because it might
      // have submasks
      if(m_pc_UsedPixelMask != NULL)
      {
         m_pc_UsedPixelMask->Rotate(i_Degree);
      }

      for (i_RunY = 0; i_RunY < m_i_BufferHeight; i_RunY++)
      {
         for (i_RunX = 0; i_RunX < m_i_BufferWidth; i_RunX++)
         {
            if (pc_BitBuffer->Get( i_RunX, i_RunY) == 0)
            {

               int i_PositionX;
               int i_PositionY;
               int i_BufferRotatedX;
               int i_BufferRotatedY;
               int i_DestinationAddress;

               Pixel p_Temp;
               Pixel p_TempSelection;
               Pixel p_TempSafety;
               Pixel p_TempOld;
               Pixel p_TempSelectionOld;
               Pixel p_TempSafetyOld;

               i_PositionX = i_RunX;
               i_PositionY = i_RunY;
               p_Temp = p_TempSelection = p_TempSafety = 0;

               p_TempOld = Get( i_PositionX, i_PositionY);

               if (m_pc_SelectionMask != NULL)
	       {
                  p_TempSelectionOld = m_pc_SelectionMask->Get(i_PositionX, i_PositionY);
               }
               if (m_pc_SafetyMask != NULL)
	       {
                  p_TempSafetyOld = m_pc_SafetyMask->Get(i_PositionX, i_PositionY);
               }

               do
               {

                  pc_BitBuffer->SetRaw( i_PositionX, i_PositionY, 1);

                  if (i_Degree == 270)

                  {
                     i_BufferRotatedX = i_PositionY;
                     i_BufferRotatedY = (m_i_BufferWidth - 1) - i_PositionX;
                  }
                  else if (i_Degree == 90)

                  {
                     i_BufferRotatedX = (m_i_BufferHeight - 1) - i_PositionY;
                     i_BufferRotatedY = i_PositionX;
                  }


                  i_DestinationAddress = i_BufferRotatedX + i_BufferRotatedY * m_i_BufferHeight;


                  i_PositionX = i_DestinationAddress % m_i_BufferWidth;
                  i_PositionY = i_DestinationAddress / m_i_BufferWidth;

                  p_Temp = Get ( i_PositionX, i_PositionY);
                  SetRaw ( i_PositionX, i_PositionY, p_TempOld);

                  if (m_pc_SelectionMask != NULL)
                  {
                     p_TempSelection = m_pc_SelectionMask->Get( i_PositionX, i_PositionY);
                     m_pc_SelectionMask->SetRaw( i_PositionX, i_PositionY,p_TempSelectionOld);
                  }
                  if (m_pc_SafetyMask != NULL)
                  {
                     p_TempSafety = m_pc_SafetyMask->Get( i_PositionX, i_PositionY);
                     m_pc_SafetyMask->SetRaw( i_PositionX, i_PositionY,p_TempSafetyOld);
                  }


                  p_TempOld          = p_Temp;
                  p_TempSelectionOld = p_TempSelection;
                  p_TempSafetyOld    = p_TempSafety;


               } while (   i_PositionX != i_RunX
                        || i_PositionY != i_RunY);


            } // if pixel was untouched

         } // for i_RunX

      } // for i_RunY

      delete pc_BitBuffer;

      // -------- now swap the dimensions
      Utilities::swap(m_i_BufferWidth,m_i_BufferHeight);
      if (m_pc_SelectionMask != NULL)
      {
         Utilities::swap(m_pc_SelectionMask->m_i_BufferWidth,
                         m_pc_SelectionMask->m_i_BufferHeight);
      }
      if (m_pc_SafetyMask != NULL)
      {
         Utilities::swap(m_pc_SafetyMask->m_i_BufferWidth,
                         m_pc_SafetyMask->m_i_BufferHeight);
      }


   } // 90 or 270

   else // not 90, 180 or 270
   {
      // not supported yet
      return false;
   }

   return true;

} // Rotate

//===========================================================================
//
//===========================================================================

PixelBuffer *PixelBuffer::Scale(int i_NewWidth, int i_NewHeight)
{
   PixelBuffer *pc_NewImage;
   bool         b_HasSelection;
   bool         b_HasSafety;
   bool         b_HasUsedPixel;

   b_HasSelection = ( m_pc_SelectionMask != NULL ) ;
   b_HasSafety    = ( m_pc_SafetyMask    != NULL ) ;

   // scale the used pixel buffer in regular way, because it might have
   // submasks
   if( m_pc_UsedPixelMask != NULL)
   {
      PixelBuffer *pc_NewBuffer;
      pc_NewBuffer = m_pc_UsedPixelMask->Scale(i_NewWidth,i_NewHeight);
      delete m_pc_UsedPixelMask;
      m_pc_UsedPixelMask = pc_NewBuffer;
   }

   // create a new buffer
   pc_NewImage = new PixelBuffer(i_NewWidth,
                                 i_NewHeight,
                                 GetDepth(),
                                 b_HasSelection,
                                 b_HasSafety,
                                 false);

   if(pc_NewImage ==NULL)
   {
      return NULL;
   }

   if(i_NewHeight != m_i_BufferHeight)
   {
      // more lines in new Image
      int i_SourceLine;
      int i_DestinationLine;
      int i_Relation;

      i_SourceLine =0;
      i_Relation   = i_NewHeight  ;

      if(i_NewHeight < m_i_BufferHeight)
      {
         i_Relation /=2;
      }


      for(i_DestinationLine =0 ; i_DestinationLine < i_NewHeight; i_DestinationLine ++)
      {
         ScaleLineInBuffer(i_SourceLine,
                           pc_NewImage,
                           i_DestinationLine,
                           b_HasSelection,
                           b_HasSafety,
                           false);

         i_Relation -= m_i_BufferHeight;
         while(i_Relation <=0)
         {
            i_Relation += i_NewHeight;
            i_SourceLine ++;
         }
      } // for all lines
   }
   else  // if(i_NewHeight == m_i_BufferHeight)
   {
      // same lines
      int i_SourceLine;
      for(i_SourceLine =0 ; i_SourceLine < i_NewHeight; i_SourceLine ++)
      {
         ScaleLineInBuffer(i_SourceLine,
                           pc_NewImage,
                           i_SourceLine,
                           b_HasSelection,
                           b_HasSafety,
                           b_HasUsedPixel);
      } // for all lines
   }

   return pc_NewImage;

} // scale

//===========================================================================
//
//===========================================================================

void PixelBuffer::ScaleLineInBuffer(int i_SourceLine,
			PixelBuffer *pc_DestinationBuffer,
                        int i_DestinationLine,
                        bool b_HasSelection,
                        bool b_HasSafety,
                        bool b_HasUsedPixel)
{
   Pixel p;

   if(m_i_BufferWidth == pc_DestinationBuffer->m_i_BufferWidth)
   {
      // same width

      int i_X;

      for(i_X =0; i_X < m_i_BufferWidth; i_X ++)
      {
         p= Get(i_X,i_SourceLine);
         pc_DestinationBuffer->SetRaw(i_X,i_DestinationLine,p);

         if(b_HasSelection)
         {
            p= m_pc_SelectionMask->Get(i_X,i_SourceLine);
            pc_DestinationBuffer->m_pc_SelectionMask->SetRaw(i_X,i_DestinationLine,p);
         }

         if(b_HasSafety)
         {
            p= m_pc_SafetyMask->Get(i_X,i_SourceLine);
            pc_DestinationBuffer->m_pc_SafetyMask->SetRaw(i_X,i_DestinationLine,p);
         }
         if(b_HasUsedPixel)
         {
            p= m_pc_UsedPixelMask->Get(i_X,i_SourceLine);
            pc_DestinationBuffer->m_pc_UsedPixelMask->SetRaw(i_X,i_DestinationLine,p);
         }
      }
   }
   else
   {
      // different width
      int i_SourceX;
      int i_DestinationX;
      int i_Relation;

      i_SourceX =0;
      i_Relation   = pc_DestinationBuffer->m_i_BufferWidth ;

      if(m_i_BufferWidth > pc_DestinationBuffer->m_i_BufferWidth)
      {
         i_Relation /=2;
      }

      for(i_DestinationX =0 ;
          i_DestinationX < pc_DestinationBuffer->m_i_BufferWidth;
          i_DestinationX ++)
      {

         p= Get(i_SourceX,i_SourceLine);
         pc_DestinationBuffer->SetRaw(i_DestinationX,i_DestinationLine,p);

         if(b_HasSelection)
         {
            p= m_pc_SelectionMask->Get(i_SourceX,i_SourceLine);
            pc_DestinationBuffer->m_pc_SelectionMask->SetRaw(i_DestinationX,i_DestinationLine,p);
         }

         if(b_HasSafety)
         {
            p= m_pc_SafetyMask->Get(i_SourceX,i_SourceLine);
            pc_DestinationBuffer->m_pc_SafetyMask->SetRaw(i_DestinationX,i_DestinationLine,p);
         }

         if(b_HasUsedPixel)
         {
            p= m_pc_UsedPixelMask->Get(i_SourceX,i_SourceLine);
            pc_DestinationBuffer->m_pc_UsedPixelMask->SetRaw(i_DestinationX,i_DestinationLine,p);
         }

         i_Relation -= m_i_BufferWidth;
         while(i_Relation <=0)
         {
            i_Relation += pc_DestinationBuffer->m_i_BufferWidth;
            i_SourceX ++;
         }
      } // for all lines
   }
} // ScaleLineInBuffer()

//===========================================================================
//
//===========================================================================

void PixelBuffer::Clean(Pixel p_Value)
{
   if(p_Value == 0)
   {
      // clean the buffer
      memset(m_p_Buffer,0,m_i_BufferSize);
   }
   else // fill the buffer with the Value
   {
      int i_RunX;
      int i_RunY;
      for(i_RunY=0;i_RunY<m_i_BufferHeight;i_RunY++)
      {
         for(i_RunX=0; i_RunX<m_i_BufferWidth; i_RunX++)
         {
            SetRaw(i_RunX,i_RunY,p_Value);
         }
      }
   }  // else clean with a value
} // Clean()

//===========================================================================
//
//===========================================================================


void PixelBuffer::MirrorX()
{
   MirrorBuffer(true);
   if(m_pc_SelectionMask != NULL)
   {
      m_pc_SelectionMask->MirrorBuffer(true);
   }
   if(m_pc_SafetyMask != NULL)
   {
      m_pc_SafetyMask->MirrorBuffer(true);
   }
   if(m_pc_UsedPixelMask != NULL)
   {
      m_pc_UsedPixelMask->MirrorBuffer(true);
   }
} // MirrorX

//===========================================================================
//
//===========================================================================


void PixelBuffer::MirrorY()
{
   MirrorBuffer(false);
   if(m_pc_SelectionMask != NULL)
   {
      m_pc_SelectionMask->MirrorBuffer(false);
   }
   if(m_pc_SafetyMask != NULL)
   {
      m_pc_SafetyMask->MirrorBuffer(false);
   }
   if(m_pc_UsedPixelMask != NULL)
   {
      m_pc_UsedPixelMask->MirrorBuffer(false);
   }

} // MirrorY

//===========================================================================
//
//===========================================================================

void PixelBuffer::MirrorBuffer(bool XDirection)
{
   Pixel p_Pixel1;
   Pixel p_Pixel2;
   int   i_RunX;
   int   i_RunY;
   int   i_Half;
   int   i_Opposite;

   if(XDirection)
   {
      // mirror left to right
      i_Half = m_i_BufferWidth/2;
      for(i_RunY=0;
          i_RunY < m_i_BufferHeight;
          i_RunY ++)
      {
         for(i_RunX =0 , i_Opposite = m_i_BufferWidth-1;
             i_RunX < i_Half;
             i_RunX ++, i_Opposite --)
         {
            p_Pixel1 = Get(i_RunX    ,i_RunY);
            p_Pixel2 = Get(i_Opposite,i_RunY);

            SetRaw(i_RunX    ,i_RunY,p_Pixel2);
            SetRaw(i_Opposite,i_RunY,p_Pixel1);

         }
      }
   }
   else // YDirection
   {
      // mirror top to bottom
      i_Half = m_i_BufferHeight/2;
      for(i_RunX=0;
          i_RunX < m_i_BufferWidth;
          i_RunX ++)
      {
         for(i_RunY =0 , i_Opposite = m_i_BufferHeight-1;
             i_RunY < i_Half;
             i_RunY ++, i_Opposite --)
         {
            p_Pixel1 = Get(i_RunX,i_RunY    );
            p_Pixel2 = Get(i_RunX,i_Opposite);

            SetRaw(i_RunX ,i_RunY    ,p_Pixel2);
            SetRaw(i_RunX ,i_Opposite,p_Pixel1);

         }
      }
   } // else YDirection

} // MirrorBuffer()

//===========================================================================
//
//===========================================================================

bool PixelBuffer::HasEqualSize(PixelBuffer *pc_OtherBuffer)
{
   if( pc_OtherBuffer == NULL)
   {
      return false;
   }

   if(    m_i_BufferWidth  == pc_OtherBuffer -> m_i_BufferWidth
       && m_i_BufferHeight == pc_OtherBuffer -> m_i_BufferHeight
     )
   {
      // buffer have same dimensions
      return true;
   }

   return false;

} // HasEqualSize()

//===========================================================================
//
//===========================================================================

void PixelBuffer::CopyFrom(PixelBuffer *pc_Example,
           bool b_UseSelection,
           bool b_UseSafety,
           bool b_UsedPixel)
{
   if(pc_Example == NULL)
   {
      return;
   }

   if(    m_i_BufferWidth  == pc_Example -> m_i_BufferWidth
       && m_i_BufferHeight == pc_Example -> m_i_BufferHeight
       && m_i_BufferDepth  == pc_Example -> m_i_BufferDepth)
   {
      // copy the memory
      memcpy(m_p_Buffer,pc_Example->m_p_Buffer, m_i_BufferSize);
   }
   else
   {
      // should not happen
   }

   if(b_UseSelection &&
      m_pc_SelectionMask != NULL
     )
   {
      m_pc_SelectionMask->CopyFrom(pc_Example->m_pc_SelectionMask,false,false,false);
   }

   if(b_UseSafety &&
      m_pc_SafetyMask != NULL
     )
   {
      m_pc_SafetyMask->CopyFrom(pc_Example->m_pc_SafetyMask,false,false,false);
   }

   if(b_UsedPixel )
   {
      if( m_pc_UsedPixelMask != NULL)
      {
         // used pixel mask might have sub-masks
         m_pc_UsedPixelMask->CopyFrom(pc_Example->m_pc_UsedPixelMask);
      }
      else if( pc_Example->m_pc_UsedPixelMask != NULL)
      {
         // create mask on demand
         m_pc_UsedPixelMask = new PixelBuffer(pc_Example->m_pc_UsedPixelMask);
      }
      else
      {
          // should not happen
      }
   }
} // CopyFrom()

//===========================================================================
// resize to another buffer size.
// teh contents gets destroyed
//===========================================================================

void PixelBuffer::ResizeInternalTo(int i_NewWidth, int i_NewHeight)
{
   if(    m_i_BufferWidth  == i_NewWidth
      &&  m_i_BufferHeight == i_NewHeight
     )
   {
      // has already that size;
      return;
   }

   // delete the old Buffer
   delete m_p_Buffer;

   // allocate a new one
   // buffer gets not cleared
   AllocBuffer(i_NewWidth,i_NewHeight,m_i_BufferDepth,false);

   // resize the selection mask
   if(m_pc_SelectionMask != NULL)
   {
      m_pc_SelectionMask->ResizeInternalTo(i_NewWidth,i_NewHeight);
   }

   // resize the safety mask
   if(m_pc_SafetyMask != NULL)
   {
      m_pc_SafetyMask->ResizeInternalTo(i_NewWidth,i_NewHeight);
   }

   // resize the safety mask
   if(m_pc_UsedPixelMask != NULL)
   {
      m_pc_UsedPixelMask->ResizeInternalTo(i_NewWidth,i_NewHeight);
   }

} // ResizeInternalTo()

//===========================================================================
//
//===========================================================================

void PixelBuffer::CreateUsedPixelMask(int i_DefaultDepth,bool b_UseSubmask)
{
   if(   m_pc_UsedPixelMask != NULL)
   {
      if( m_pc_UsedPixelMask->m_i_BufferDepth == i_DefaultDepth)
      {
          // mask exists, and has the needed depth
         return;
      }

      // wrong depth, so delete the mask
      delete m_pc_UsedPixelMask;
      m_pc_UsedPixelMask = NULL;
   }

   if(m_pc_UsedPixelMask == NULL)
   {
      m_pc_UsedPixelMask = new PixelBuffer(m_i_BufferWidth,
                                           m_i_BufferHeight,
                                           i_DefaultDepth,
                                           false,
                                           false,
                                           b_UseSubmask);
   }
} // CreateUsedPixelMask()

//===========================================================================
//
//===========================================================================

void PixelBuffer::UsedPixelMaskSet(int i_X, int i_Y, Pixel Value)
{
   if(m_pc_UsedPixelMask != NULL)
   {
      if(m_pc_UsedPixelMask->m_i_BufferDepth==1)
      {
         m_pc_UsedPixelMask->Set(i_X,i_Y,Value);
      }
      else
      {
         if(m_pc_UsedPixelMask->m_pc_UsedPixelMask != NULL)
         {
            m_pc_UsedPixelMask->m_pc_UsedPixelMask->Set(i_X,i_Y,Value);
         }
         else
         {
            m_pc_UsedPixelMask->Set(i_X,i_Y,Value);
         }
      }
   }
} // UsedPixelMaskSet()

//===========================================================================
//
//===========================================================================

Pixel PixelBuffer::UsedPixelMaskGet(int i_X, int i_Y)
{
   if(m_pc_UsedPixelMask == NULL)
   {
      return 0;
   }
   if(m_pc_UsedPixelMask->m_i_BufferDepth==1)
   {
      return m_pc_UsedPixelMask->Get(i_X,i_Y);
   }
   if(m_pc_UsedPixelMask->m_pc_UsedPixelMask != NULL)
   {
      return m_pc_UsedPixelMask->m_pc_UsedPixelMask->Get(i_X,i_Y);
   }
   return m_pc_UsedPixelMask->Get(i_X,i_Y);

} // UsedPixelMaskGet()

//===========================================================================
//
//===========================================================================

Pixel PixelBuffer::UsedPixelMaskGetColorPixel(int i_X, int i_Y)
{
   if(m_pc_UsedPixelMask == NULL)
   {
      return 0;
   }
   if(m_pc_UsedPixelMask->m_i_BufferDepth==1)
   {
      return 0;
   }
   return m_pc_UsedPixelMask->Get(i_X,i_Y);

} // UsedPixelMaskGetColorPixel()
//===========================================================================
//
//===========================================================================

void PixelBuffer::UsedPixelMaskClean()
{
   if(m_pc_UsedPixelMask != NULL)
   {
      if(m_pc_UsedPixelMask->m_i_BufferDepth ==1)
      {
         m_pc_UsedPixelMask->Clean();
      }
      else
      {
         if(m_pc_UsedPixelMask->m_pc_UsedPixelMask != NULL)
         {
            m_pc_UsedPixelMask->m_pc_UsedPixelMask->Clean();
         }
         else
         {
            m_pc_UsedPixelMask->Clean();
         }
      }
   }

} // UsedPixelMaskClean()

//===========================================================================
//
//===========================================================================

bool PixelBuffer::UsedPixelMaskAvailable()
{
   if(m_pc_UsedPixelMask != NULL)
   {
      return true;
   }
   return false;

} // UsedPixelMaskAvailable()


//===========================================================================
//
//===========================================================================

bool PixelBuffer::CompareBuffer(PixelBuffer *pc_OtherBuffer,bool recursiv)
{
   bool b_Result;
   int  i_Result;

   // other buffer available?
   if(pc_OtherBuffer == NULL)
   {
      return false;
   }


   if(    m_i_BufferWidth  != pc_OtherBuffer -> m_i_BufferWidth
       || m_i_BufferHeight != pc_OtherBuffer -> m_i_BufferHeight
       || m_i_BufferDepth  != pc_OtherBuffer -> m_i_BufferDepth)
   {
      // different dimensions
      return false;
   }

   i_Result = memcmp(m_p_Buffer,
                     pc_OtherBuffer->m_p_Buffer,
                     m_i_BufferSize);
   if(i_Result != 0)
   {
      return false;
   }

   if(!recursiv)
   {
      return true;
   }

   // if recursiv now check the sub-buffers
   if(m_pc_SelectionMask != NULL)
   {
      b_Result = m_pc_SelectionMask->CompareBuffer(pc_OtherBuffer->m_pc_SelectionMask,
                                                    recursiv);
      if(! b_Result)
      {
         return false;
      }
   }
   if(m_pc_SafetyMask != NULL)
   {
      b_Result = m_pc_SafetyMask->CompareBuffer(pc_OtherBuffer->m_pc_SafetyMask,
                                                    recursiv);
      if(! b_Result)
      {
         return false;
      }
   }
   if(m_pc_UsedPixelMask != NULL)
   {
      b_Result = m_pc_UsedPixelMask->CompareBuffer(pc_OtherBuffer->m_pc_UsedPixelMask,
                                                    recursiv);
      if(! b_Result)
      {
         return false;
      }
   }
   return true;

} // CompareBuffer()

//===========================================================================
//
//===========================================================================

void PixelBuffer::SetSafetyMaskActive(bool b_Active)
{
   m_b_UseSafetyMask = b_Active;
} // SetSafetyMaskActive()

//===========================================================================
//
//===========================================================================

bool PixelBuffer::GetSafetyMaskActive()
{
   return m_b_UseSafetyMask;
} // GetSafetyMaskActive()

//===========================================================================
//
//===========================================================================

void PixelBuffer::SetSelectionMaskActive(bool b_Active)
{
   m_b_UseSelectionMask = b_Active;
} // SetSelectionMaskActive()

//===========================================================================
//
//===========================================================================

bool PixelBuffer::GetSelectionMaskActive()
{
   return m_b_UseSelectionMask;
} // GetSelectionMaskActive()

//===========================================================================
//
//===========================================================================

void PixelBuffer::InvertBuffer()
{
   unsigned char * p_InBuffer;
   int             i;

   p_InBuffer = m_p_Buffer;

   // invert the memory
   for(i=0; i< m_i_BufferSize; i++)
   {
      * p_InBuffer++ ^= 0xff;
   }

} // InvertBuffer;

//===========================================================================
//
//===========================================================================

RectRegion PixelBuffer::GetUsedRegion(Pixel p_Color)
{
   RectRegion c_Result;
   int i_RunX;
   int i_RunY;
   int i_MinX;
   int i_MinY;
   int i_MaxX;
   int i_MaxY;

   i_MinX = m_i_BufferWidth  +1;
   i_MinY = m_i_BufferHeight +1;
   i_MaxX = -1;
   i_MaxY = -1;

   for(i_RunY=0; i_RunY < m_i_BufferHeight; i_RunY ++)
   {
      for(i_RunX=0; i_RunX < m_i_BufferWidth; i_RunX ++)
      {
         if(Get(i_RunX,i_RunY) == p_Color)
         {
            if(i_MinX>i_RunX)
            {
               i_MinX = i_RunX;
            }
            if(i_MaxX<i_RunX)
            {
               i_MaxX = i_RunX;
            }
            if(i_MinY>i_RunY)
            {
               i_MinY = i_RunY;
            }
            if(i_MaxY<i_RunY)
            {
               i_MaxY = i_RunY;
            }
         }
      } // for RunX
   } // for RunY

   if(i_MinX > i_MaxX)
   {
      // nothing found
      c_Result.SetEmpty();
   }
   else
   {
      // found some pixel
      c_Result.SetXYXY(i_MinX,i_MinY,i_MaxX,i_MaxY);
   }

   return c_Result;
} // GetUsedRegion()

//===========================================================================
//
//===========================================================================


RectRegion PixelBuffer::GetRegion()
{
   RectRegion c_Result;
   
   c_Result.SetXYWH(0,0,m_i_BufferWidth,m_i_BufferHeight);

   return c_Result;

} // GetRegion()

//===========================================================================
//
//===========================================================================

PixelBuffer *PixelBuffer::CreateNewImageFromSelected(RectRegion *pc_ResultRegion,
                              PixelBuffer *pc_SelectionBuffer)
{
   PixelBuffer *pc_NewBuffer;
   PixelBuffer *pc_NewSelectionBuffer;
   PixelBuffer *pc_NewSafetyBuffer;
   PixelBuffer *pc_UseSelectionBuffer;
   RectRegion  c_SelectedRegion;
   int         i_NewWidth;
   int         i_NewHeight;
   int         i_X;
   int         i_Y;
   int         i_OffsetX;
   int         i_OffsetY;

   // use the given mask
   pc_UseSelectionBuffer = pc_SelectionBuffer;

   // if empty, use mask at the buffer
   if(pc_UseSelectionBuffer == NULL)
   {
      pc_UseSelectionBuffer = m_pc_SelectionMask;
   }

   // if still empty, then do nothing
   if( pc_UseSelectionBuffer == NULL)
   {
      return NULL;
   }

   c_SelectedRegion = pc_UseSelectionBuffer->GetUsedRegion(1);

   // nothing selected, then finish
   if(c_SelectedRegion.IsEmpty())
   {
      return NULL;
   }

   i_NewWidth  = c_SelectedRegion.GetWidth();
   i_NewHeight = c_SelectedRegion.GetHeight();

   // allocated a new buffer;
   pc_NewBuffer = new PixelBuffer(i_NewWidth,
                                  i_NewHeight,
                                  m_i_BufferDepth,
                                  true,
                                  true,
                                  false
                                  );
   // buffer must not be empty
   if(pc_NewBuffer == NULL)
   {
      return NULL;
   }

   pc_NewSelectionBuffer = pc_NewBuffer->GetSelectionMask();
   pc_NewSafetyBuffer    = pc_NewBuffer->GetSafetyMask();

   i_OffsetX = c_SelectedRegion.GetStartX();
   i_OffsetY = c_SelectedRegion.GetStartY();

   // copy contents in new buffer
   for(i_Y =0 ; i_Y < i_NewHeight; i_Y ++)
   {
      for(i_X =0 ; i_X < i_NewWidth; i_X ++)
      {
         Pixel p_Mask;

         // check, whether mask is set
         p_Mask = pc_UseSelectionBuffer->Get(i_X+i_OffsetX,i_Y+i_OffsetY);

         if(p_Mask)
         {
            Pixel p_Image;
            p_Image = Get(i_X+i_OffsetX,i_Y+i_OffsetY);
            pc_NewBuffer         ->SetRaw(i_X,i_Y,p_Image);
            pc_NewSelectionBuffer->SetRaw(i_X,i_Y,1);
            if(m_pc_SafetyMask != NULL)
            {
               Pixel p_Safety;
               p_Safety = m_pc_SafetyMask->Get(i_X+i_OffsetX,i_Y+i_OffsetY);
               pc_NewSafetyBuffer->SetRaw(i_X,i_Y,p_Safety);

            }
         }
         else // mask not set
         {
            // set white default pixel
            pc_NewBuffer         ->SetRaw(i_X,i_Y,0xffffff);
            pc_NewSelectionBuffer->SetRaw(i_X,i_Y,0);
            pc_NewSafetyBuffer   ->SetRaw(i_X,i_Y,0);
         }

      } // for i_X
   } // for i_Y


   if(pc_ResultRegion != NULL)
   {
      *pc_ResultRegion = c_SelectedRegion;
   }

   return pc_NewBuffer;
} // CreateNewImageFromSelected()

//===========================================================================
//
//===========================================================================

PixelBuffer *PixelBuffer::CreateNewImageFromRegion(RectRegion c_Region)
{
   PixelBuffer *pc_NewBuffer;
   int         i_NewWidth;
   int         i_NewHeight;
   int         i_X;
   int         i_Y;
   int         i_OffsetX;
   int         i_OffsetY;

   i_NewWidth  = c_Region.GetWidth();
   i_NewHeight = c_Region.GetHeight();

   // allocated a new buffer;
   pc_NewBuffer = new PixelBuffer(i_NewWidth,
                                  i_NewHeight,
                                  m_i_BufferDepth,
                                  true,
                                  true,
                                  false
                                  );
   // buffer must not be empty
   if(pc_NewBuffer == NULL)
   {
      return NULL;
   }

   i_OffsetX = c_Region.GetStartX();
   i_OffsetY = c_Region.GetStartY();

   // copy contents in new buffer
   for(i_Y =0 ; i_Y < i_NewHeight; i_Y ++)
   {
      for(i_X =0 ; i_X < i_NewWidth; i_X ++)
      {
         Pixel p_Image;
         p_Image = Get(i_X+i_OffsetX,i_Y+i_OffsetY);
         pc_NewBuffer         ->SetRaw(i_X,i_Y,p_Image);

      } // for i_X
   } // for i_Y

   return pc_NewBuffer;
} // CreateNewImageFromRegion

//===========================================================================
//
//===========================================================================

void PixelBuffer::ConvertColors(unsigned char *puc_Red,
                           unsigned char *puc_Green,
                           unsigned char *puc_Blue)
{
   unsigned char i_Red;
   unsigned char i_Green;
   unsigned char i_Blue;
   Pixel         p_Pixel;
   int           i_X;
   int           i_Y;

   // for all pixel
   for(i_Y=0; i_Y < m_i_BufferHeight;i_Y++)
   {
      for(i_X=0; i_X < m_i_BufferWidth; i_X++)
      {
         if(m_b_UseSelectionMask)
         {
            // selection active
            if(!m_pc_SelectionMask->Get(i_X,i_Y) )
            {
               // bit not selected. so continue
               continue;
            }
         }

         if(m_b_UseSafetyMask)
         {
            //safety active
            if(m_pc_SafetyMask->Get(i_X,i_Y) )
            {
               // and this point is saved
               continue;
            }
         }
         p_Pixel = Get(i_X,i_Y);

         i_Red   = PIXEL_RED  (p_Pixel);
         i_Green = PIXEL_GREEN(p_Pixel);
         i_Blue  = PIXEL_BLUE (p_Pixel);

         i_Red   = puc_Red  [i_Red  ];
         i_Green = puc_Green[i_Green];
         i_Blue  = puc_Blue [i_Blue ];

         SetRaw(i_X,i_Y, PIXEL_MAKE(i_Red,i_Green,i_Blue));
      }
   }
} // ConvertColors()

//===========================================================================
//
//===========================================================================

void PixelBuffer::AddSelectionMask()
{
   if(m_pc_SelectionMask ==NULL)
   {
      m_pc_SelectionMask = new PixelBuffer(m_i_BufferWidth,
                                           m_i_BufferHeight,
                                           1,
                                           false,
                                           false,
                                           false);
   }
} // AddSelectionMask

//===========================================================================
//
//===========================================================================

PixelBuffer * PixelBuffer::CreateBorderDistanceBuffer()
{
   PixelBuffer *pc_DistanceBuffer;
   int          i_X;
   int          i_Y;

   // no mask - then no selection available
   if(m_pc_SelectionMask == NULL)
   {
      return NULL;
   }

   // allocate a new buffer
   pc_DistanceBuffer = new PixelBuffer(this,8,false,false,false);
   if(pc_DistanceBuffer == NULL)
   {
      return NULL;
   }

   // now fill all masked points with 255, else 0
   for(i_Y=0; i_Y < m_i_BufferHeight; i_Y ++)
   {
      for(i_X=0; i_X < m_i_BufferWidth; i_X ++)
      {
         if(m_pc_SelectionMask->Get(i_X,i_Y))
         {
            pc_DistanceBuffer->SetRaw(i_X,i_Y,1);
         }
         else
         {
            pc_DistanceBuffer->SetRaw(i_X,i_Y,0);
         }  
      }
   } // for i_Y

   for(i_Y=1; i_Y < m_i_BufferHeight-1; i_Y ++)
   {
      for(i_X=1; i_X < m_i_BufferWidth-1; i_X ++)
      {
         int i_Current;
         int i_Upper;
         int i_Left;
         int i_New;
         i_Current = pc_DistanceBuffer->Get( i_X,i_Y);
         if(i_Current>0)
         {
            i_Upper = pc_DistanceBuffer->Get( i_X,i_Y-1);
            i_Left  = pc_DistanceBuffer->Get( i_X-1,i_Y);
            if(i_Left < i_Upper)
            {
               i_Upper = i_Left;
            }
            i_New = 1 + i_Upper;
            if(i_New >255)
            {
               i_New = 255;
            }
            pc_DistanceBuffer->Set( i_X,i_Y,i_New);
         }
         
      } // for X
   } // for Y

   
   // now reverse
   for(i_Y=m_i_BufferHeight-2; i_Y >=1;i_Y--)
   {
      for(i_X=m_i_BufferWidth-2; i_X >=1; i_X --)
      {
         int i_Current;
         int i_Lower;
         int i_Right;
         int i_New;
         i_Current = pc_DistanceBuffer->Get( i_X,i_Y);
         if(i_Current>0)
         {
            i_Lower = pc_DistanceBuffer->Get( i_X,i_Y+1);
            i_Right  = pc_DistanceBuffer->Get( i_X+1,i_Y);
            if(i_Right < i_Lower)
            {
               i_Lower = i_Right;
            }
            i_New = 1 + i_Lower;
            if(i_New > i_Current)
            {
               i_New = i_Current;
            }
            if(i_New >255)
            {
               i_New = 255;
            }
             pc_DistanceBuffer->Set( i_X,i_Y,i_New);
         }

      } //for x reverse
   }  // for Y reverse
   
   

   // return the resultbuffer
   return pc_DistanceBuffer;
   
} // CreateBorderDistanceBuffer()

//===========================================================================
//
//===========================================================================


