#ifndef _PIXELBUFFER_LOADED_H
#define _PIXELBUFFER_LOADED_H

#include "PTypes.h"
#include "RectRegion.h"
#include <stdio.h>

class PixelCoordinate
{
public:
  int m_i_X;
  int m_i_Y;
};

class PixelBuffer
{
private:
   bool            m_b_UseSelectionMask;
   PixelBuffer   * m_pc_SelectionMask;

   bool            m_b_UseSafetyMask;
   PixelBuffer   * m_pc_SafetyMask;
   
   // whether internal structure is valid as desired
   bool            m_b_Valid;

   unsigned char * m_p_Buffer;
   int             m_i_BufferSize;

   int             m_i_BufferDepth;
   int             m_i_BufferWidth;
   int             m_i_BufferHeight;


   PixelBuffer   * m_pc_UsedPixelMask;

public:
   PixelBuffer(int i_Width,
               int i_Height,
               int i_Depth,
               bool b_UseSelection = true,
               bool b_UseSafety    = true,
               bool b_UsedPixelMap = true
              );

   PixelBuffer(PixelBuffer *pc_Example,
               int i_Depth,
               bool b_UseSelection = true,
               bool b_UseSafety    = true,
               bool b_UsedPixelMap = true
              );

   PixelBuffer(PixelBuffer *pc_Example);

   ~PixelBuffer();

   bool IsValid();

   void SetRaw( int i_X, int i_Y, Pixel value);
   void Set( int i_X, int i_Y, Pixel value);
   Pixel Get(int i_X, int i_Y );

   int GetDepth();
   int GetWidth();
   int GetHeight();

   bool Rotate(int i_Degree);
   void MirrorX();
   void MirrorY();

   PixelBuffer *GetSelectionMask();
   PixelBuffer *GetSafetyMask();
   PixelBuffer *GetUsedPixelMask();
   void SetSafetyMaskActive(bool b_Active);
   bool GetSafetyMaskActive();
   void SetSelectionMaskActive(bool b_Active);
   bool GetSelectionMaskActive();

   PixelBuffer *Scale(int i_NewWidth, int i_NewHeight);


   void ResizeInternalTo(int i_NewWidth, int i_NewHeight);

   void * GetPixelAddress(int i_X, int i_Y);
   void Clean(Pixel p_Value=0);

   bool HasEqualSize(PixelBuffer *pc_OtherBuffer);
   void CopyFrom(PixelBuffer *pc_Example,
               bool b_UseSelection = true,
               bool b_UseSafety    = true,
               bool b_UsedPixelMap = true
              );

   void CreateUsedPixelMask(int i_DefaultDepth=1,
                            bool b_UseSubmask=false);
   void UsedPixelMaskSet(int i_X, int i_Y, Pixel Value);
   Pixel UsedPixelMaskGet(int i_X, int i_Y);
   Pixel UsedPixelMaskGetColorPixel(int i_X, int i_Y);
   void UsedPixelMaskClean();
   bool UsedPixelMaskAvailable();

   bool CompareBuffer(PixelBuffer *pc_OtherBuffer,bool recursiv);

   void InvertBuffer();

   RectRegion GetUsedRegion(Pixel p_Color);
   RectRegion GetRegion();

   PixelBuffer *CreateNewImageFromSelected(
               RectRegion *pc_ResultRegion=NULL,
               PixelBuffer *pc_SelectionBuffer = NULL);
   PixelBuffer *CreateNewImageFromRegion(RectRegion c_Region);
   
   void ConvertColors(unsigned char *puc_Red,
                           unsigned char *puc_Green,
                           unsigned char *puc_Blue);
   bool IsSelected(int i_X, int i_Y);

   void AddSelectionMask();

   PixelBuffer * CreateBorderDistanceBuffer();

private:
   void ScaleLineInBuffer(int i_SourceLine,
                        PixelBuffer *pc_DestinationBuffer,
                        int i_DestinationLine,
                        bool b_HasSelection,
                        bool b_HasSafety,
                        bool b_HasUsed);


   void MirrorBuffer(bool XDirection);

   void AllocBuffer(int i_Width, int i_Height, int i_Depth,bool b_Clear=true);


} ; // PixelBuffer



#endif //  _PIXELBUFFER_LOADED_H
