
#include "PixelBufferObject.h"
#include "ColorCalculation.h"


//============================================================================
//
//============================================================================

PixelBufferObject::PixelBufferObject()
{
   m_pc_Destination          = NULL;
   m_pc_Object               = NULL;
   m_c_DestinationRegion.SetEmpty();
   m_b_UseTransparency       = false;
   m_i_TransparencyFactor    = 0;
   m_b_MergeWithSmoothBorder = false;
   m_i_MergeBorderWidth      = 0;

} // PixelBufferObject()

//============================================================================
//
//============================================================================

PixelBufferObject::~PixelBufferObject()
{

} // ~PixelBufferObject()

//============================================================================
//
//============================================================================

void PixelBufferObject::SetDestinationBuffer(PixelBuffer *pc_Destination)
{
   m_pc_Destination = pc_Destination;
} // SetDestinationBuffer()

//============================================================================
//
//============================================================================

void PixelBufferObject::SetObject(GraphicObject *pc_Object)
{
   m_pc_Object = pc_Object;
} // SetObject()

//============================================================================
//
//============================================================================

void PixelBufferObject::SetRegion(RectRegion c_DestinationRegion)
{
   m_c_DestinationRegion = c_DestinationRegion;
} // SetRegion()

//============================================================================
//
//============================================================================

RectRegion  PixelBufferObject::InsertObject()
{
   GraphicObjectAttribute * pc_Attribute;
   PixelBuffer             *pc_Image;
   PixelBuffer             *pc_BorderBuffer;
   RectRegion              c_ChangedRegion;
   Pixel                   p_Color;
   float                   f_BufferStartX;
   float                   f_BufferStartY;
   float                   f_BufferEndX;
   float                   f_BufferEndY;
   float                   f_DesiredWidth;
   float                   f_DesiredHeight;
   float                   f_ZoomImageX;
   float                   f_ZoomImageY;
   int                     i_DestinationWidth;
   int                     i_DestinationHeight;
   int                     i_OriginalWidth;
   int                     i_OriginalHeight;
   int                     i_OriginalX;
   int                     i_OriginalY;


   c_ChangedRegion.SetEmpty();

   // check for an image object
   pc_Attribute = m_pc_Object->GetAttribute(GraphicObjectAttribute::Image);
   if(pc_Attribute ==NULL)
   {
      return c_ChangedRegion;
   }

   // check for attached image date
   pc_Image = pc_Attribute->m_pc_ImageData;
   if(pc_Image ==NULL)
   {
      return c_ChangedRegion;
   }

   if(m_b_MergeWithSmoothBorder)
   {
      pc_BorderBuffer = pc_Image->CreateBorderDistanceBuffer();
   }
   else
   {
      pc_BorderBuffer = NULL;
   }
   
   i_OriginalWidth  = pc_Image->GetWidth();
   i_OriginalHeight = pc_Image->GetHeight();


   i_DestinationWidth  = m_pc_Destination->GetWidth();
   i_DestinationHeight = m_pc_Destination->GetHeight();


   f_DesiredWidth  = m_c_DestinationRegion.GetWidth();
   f_DesiredHeight = m_c_DestinationRegion.GetHeight();

   f_ZoomImageX =  i_OriginalWidth  / f_DesiredWidth  ;
   f_ZoomImageY =  i_OriginalHeight / f_DesiredHeight ;

   // get position into plain variabls
   f_BufferStartX = m_c_DestinationRegion.GetStartX();
   f_BufferStartY = m_c_DestinationRegion.GetStartY();
   f_BufferEndX   = m_c_DestinationRegion.GetEndX();
   f_BufferEndY   = m_c_DestinationRegion.GetEndY();
   

   { // drawing block
      float f_PictureX;
      float f_PictureY;
      int   i_RunY;
      int   i_RunX;
      int   i_RealX;
      int   i_RealY;

      i_RealY=0;
      for(i_RunY= (int)f_BufferStartY;
          i_RunY<= (int)f_BufferEndY;
          i_RunY++)
      {
         i_RealX =0;
         for(i_RunX= (int)f_BufferStartX;
             i_RunX<= (int)f_BufferEndX;
             i_RunX++)
         {
            if(    i_RunX>=0 
                && i_RunY>=0
                && i_RunX < i_DestinationWidth
                && i_RunY < i_DestinationHeight)
            {
            
               // f_Picture has now the pixel coordinates of
               // the bounding box

               // multiply the object to real image faktor
               f_PictureX = i_RealX * f_ZoomImageX;
               f_PictureY = i_RealY * f_ZoomImageY;


               i_OriginalX = (int) f_PictureX;
               if(i_OriginalX <0)
               {
                  i_OriginalX =0;
               }
               else if( i_OriginalX >= i_OriginalWidth)
               {
                  i_OriginalX = i_OriginalWidth-1;
               }

               i_OriginalY = (int) f_PictureY;
               if(i_OriginalY <0)
               {
                  i_OriginalY =0;
               }
               else if( i_OriginalY >= i_OriginalHeight)
               {
                  i_OriginalY = i_OriginalHeight-1;
               }

               // only draw the pixel, if it selected in the
               // selectionmask
               if(pc_Image->IsSelected(i_OriginalX,i_OriginalY))
               {
                  p_Color = pc_Image->Get(i_OriginalX,i_OriginalY);

                  if(pc_BorderBuffer != NULL)
                  {
                     // insert with smooth border
                     int i_BorderValue;

                     i_BorderValue = pc_BorderBuffer->Get(i_OriginalX,i_OriginalY);
                     if(i_BorderValue > m_i_MergeBorderWidth)
                     {
                     
                        if(   m_b_UseTransparency
                           && m_i_TransparencyFactor>0)
                        {
                           Pixel p_Old;
                           Pixel p_New;
                           p_Old = m_pc_Destination->Get(i_RunX,i_RunY);
                           p_New = ColorCalculation::TransparentColor(p_Old,
                                       p_Color,
                                       m_i_TransparencyFactor);
                           m_pc_Destination->Set(i_RunX,i_RunY,p_New);

                        }
                        else // not transparent
                        {
                           m_pc_Destination->Set(i_RunX,i_RunY,p_Color);
                        } // else not transparent
                     }
                     else // use the border value
                     {
                        int   i_BorderTransparenceFactor;
                        int   i_InversBorderFactor;
                        i_InversBorderFactor = (i_BorderValue *100)
                                                / m_i_MergeBorderWidth;

                        i_BorderTransparenceFactor = 100 - i_InversBorderFactor;

                        if(   m_b_UseTransparency
                           && m_i_TransparencyFactor>0)
                        {
                           // border and transparent
                           Pixel p_Old;
                           Pixel p_New;
                           int i_TotalFactor;
                           
                           p_Old = m_pc_Destination->Get(i_RunX,i_RunY);

                           i_TotalFactor =m_i_TransparencyFactor
                                          + (( (100 - m_i_TransparencyFactor)
                                              * i_BorderTransparenceFactor)/100);

                           p_New = ColorCalculation::TransparentColor(p_Old,
                                       p_Color,
                                       i_TotalFactor);
                           m_pc_Destination->Set(i_RunX,i_RunY,p_New);

                        }
                        else // border but not transparent
                        {
                           Pixel p_Old;
                           Pixel p_New;

                           p_Old = m_pc_Destination->Get(i_RunX,i_RunY);

                           p_New = ColorCalculation::TransparentColor(p_Old,
                                       p_Color,
                                       i_BorderTransparenceFactor);
                           m_pc_Destination->Set(i_RunX,i_RunY,p_New);
                        } // else not transparent
                     } // else use the border value
                  }

                  else // not smooth insert
                  {
                     if(   m_b_UseTransparency
                        && m_i_TransparencyFactor>0)
                     {
                        Pixel p_Old;
                        Pixel p_New;
                        p_Old = m_pc_Destination->Get(i_RunX,i_RunY);
                        p_New = ColorCalculation::TransparentColor(p_Old,
                                       p_Color,
                                       m_i_TransparencyFactor);
                        m_pc_Destination->Set(i_RunX,i_RunY,p_New);
                     
                     }
                     else // not transparent
                     {
                        m_pc_Destination->Set(i_RunX,i_RunY,p_Color);
                     } // else not transparent
                  } // else not smooth insert
               } // if selected in mask
            } // if destination is in image

            i_RealX++;
         } // for X

         i_RealY ++;
      } // for Y
   } // drawing block

   // delete borderbuffer if allocated
   if(pc_BorderBuffer != NULL)
   {
      delete pc_BorderBuffer;
   }

   // return the changed region
   c_ChangedRegion = m_c_DestinationRegion;
   return c_ChangedRegion;
} // InsertObject()

//============================================================================
//
//============================================================================

void PixelBufferObject::SetTransparency(bool b_Enabled, int i_Factor)
{
   m_b_UseTransparency    = b_Enabled;
   m_i_TransparencyFactor = i_Factor;
} // SetTransparency()

//============================================================================
//
//============================================================================

void  PixelBufferObject::SetMergeBorder(bool b_Enabled, int i_BorderWidth)
{
   m_b_MergeWithSmoothBorder = b_Enabled;
   m_i_MergeBorderWidth      = i_BorderWidth;

} // SetMergeBorder()

//============================================================================
//
//============================================================================




