
#include <qdragobject.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qiconset.h>
#include <qlayout.h>


#include "QUColorDragWidget.h"
#include "Selection.h"
#include "ImageFrame.h"
#include "Tracer.h"
#include "QtUtilities.h"
#include "DragAndDropConstants.h"


//===========================================================================
//
//===========================================================================


QUColorDragWidget::QUColorDragWidget(QWidget *p_Parent,
                     unsigned int ui_Mode,
                     GlobalStorage *pc_GlobalStorage,
                     int i_Width,
                     int i_Height)
                : QWidget(p_Parent) , Tracer("QUColorDragWidget")
{

   if(ui_Mode & ModeSetSize)
   {
      // set the given size
      setFixedSize(i_Width,i_Height);
   }
   else
   {
      // set a fixed size for the widget
      setFixedSize(48,32);
   }

   // enable the dropping of objects
   setAcceptDrops(TRUE);

   // save mode and global storage
   m_ui_Mode          = ui_Mode;
   m_pc_GlobalStorage = pc_GlobalStorage;

   // initital history is empty
   m_i_ColorsInHistory = 0;

} // ColorDragWidget()


//===========================================================================
//
//===========================================================================

void QUColorDragWidget::SetColor(Pixel c_Color)
{
   QPalette pal;

   // store the color
   m_Color = c_Color;

   // change the widget color
   pal.setColor( QColorGroup::Background, c_Color  );

   setPalette( pal );

   // sve the color in the history table
   SaveColorInHistory(c_Color);

} // SetColor()

//===========================================================================
//
//===========================================================================

Pixel QUColorDragWidget::GetColor()
{
   return m_Color;
} // GetColor()

//===========================================================================
//
//===========================================================================

void QUColorDragWidget::dragEnterEvent(QDragEnterEvent* event)
{
    TRACE << "QDrag Enter Event\n";

   QByteArray ba_Data;
   bool       b_DoAccept;
   QString    qs_Source;
   int        i_Size;

   TRACE << "QDrag Enter Event\n";

   b_DoAccept = false;
   ba_Data    = event->encodedData(DRAGANDDROP_APPLICATION);

   i_Size = ba_Data.size();

   TRACE << "Data size is "
         << i_Size
         << "\n";

   qs_Source =  QtUtility::GetDragSourceName(&ba_Data);
   if(qs_Source.compare( DRAGANDDROP_TYPE_COLOR) ==0)
   {
      TRACE << "Accept is true\n";
      b_DoAccept = true;
   }

   event->accept( b_DoAccept );

} // dragEnterEvent()

//===========================================================================
//
//===========================================================================

void QUColorDragWidget::dropEvent(QDropEvent* event)
{
   QByteArray ba_Data;
   QString    qs_Source;
   QString    qs_Data;
   int        i_Size;

   TRACE << "QDrop Enter Event\n";

   ba_Data = event->encodedData(DRAGANDDROP_APPLICATION);

   i_Size = ba_Data.size();

   TRACE << "Data size is "
         << i_Size
         << "\n";

   qs_Source =  QtUtility::GetDragSourceName(&ba_Data,&qs_Data);
   if(qs_Source.compare( DRAGANDDROP_TYPE_COLOR) ==0)
   {
      Pixel c_NewColor;
      int   i_ScanResult;

      i_ScanResult = sscanf((const char *)qs_Data,"%d",&c_NewColor );

      if(i_ScanResult ==1)
      {
         TRACE << "Color Received "
               <<  (int) c_NewColor
               << "\n";
         SetColor(c_NewColor);
         ColorChanged(c_NewColor);
      }
   }

} // dropEvent()

//===========================================================================
//
//===========================================================================

void QUColorDragWidget::StartDrag()
{
   QStoredDrag * pc_Drag;
   QByteArray  * data;
   QWidget     * target;
   char          string[40];

   TRACE << "Start Drag EnterEvent\n";

   // allocate array to transfer the data
   data = new QByteArray(100);

   // write in the color
   sprintf(string,"%s=%d",DRAGANDDROP_TYPE_COLOR,m_Color);
   data->setRawData((const char *) string,strlen(string)+1);

   pc_Drag = new QStoredDrag( DRAGANDDROP_APPLICATION, this );
   pc_Drag->setEncodedData(*data);
   
   // create a pixmap in the selected color
   {
      QPixmap  pc_ColorPixmap;
      QImage  *pc_Image;
      int     i_RunX;
      int     i_RunY;
      int     i_Size;

      i_Size =16;

      // create an image in that size
      pc_Image = new QImage(i_Size,i_Size,32);
      for(i_RunY=0; i_RunY < i_Size; i_RunY++)
      {
         for(i_RunX=0; i_RunX < i_Size; i_RunX++)
         {
           pc_Image->setPixel(i_RunX,i_RunY,
                            m_Color);
         }
      }

      // make pixmap from image
      pc_ColorPixmap.convertFromImage(*pc_Image,QPixmap::Color);
      
      // attach pixmap at the drag object
      pc_Drag->setPixmap(pc_ColorPixmap,QPoint(0,0));
   }

   // start the drag
   pc_Drag->dragMove();

   // get the target
   target = pc_Drag->target();
   if(target != NULL)
   {
      // if dragged, then notify
      DragStarted(m_Color);
   }
   //delete pc_Drag;

} // StartDrag()

//===========================================================================
//
//===========================================================================

void QUColorDragWidget::mousePressEvent(QMouseEvent *theEvent)
{
   if(theEvent->button() == 1)
   {
      StartDrag();
   }
   else if(theEvent->button() == 2)
   {
      StartPipette();
   }
   if(theEvent->button() == 4)
   {
      // create a popup menu
      QPopupMenu  *m_ColorMenu;
      QImage       *m_Image;
      int          i;
      int          i_Result;

      m_ColorMenu = new QPopupMenu(this);

      m_ColorMenu->insertItem("From Image",0,0 );

      m_Image = new QImage(16,16,32);
      for(i=0;i< m_i_ColorsInHistory; i++)
      {
         QPixmap  m_Pixmap;
         QString  qs_String;
         Pixel    p_Current;
         int      i_RunX;
         int      i_RunY;

         p_Current = m_ColorHistory[i];

         for(i_RunY=0;i_RunY<16;i_RunY++)
         {
            for(i_RunX=0;i_RunX<16;i_RunX++)
            {
               m_Image->setPixel(i_RunX,i_RunY,p_Current);
            }
         }
         m_Pixmap.convertFromImage(*m_Image,QPixmap::Color);
         
         // create an iconset out of the pixmap
         QIconSet m_IconSet(m_Pixmap);
         qs_String.sprintf("Color %d",i+1);
         
         // insert the color menu
         m_ColorMenu->insertItem(m_IconSet,qs_String ,(int)i+1,(int)i+1);

      }
      i_Result = m_ColorMenu->exec(QCursor::pos());

      if(i_Result ==0)
      {
         // start the pipette
         StartPipette();
      }
      else if(i_Result >0)
      {
         // Select one of the old colors
         Pixel c_NewColor;
         
         c_NewColor = m_ColorHistory[i_Result -1];

         SetColor(c_NewColor);
         ColorChanged(c_NewColor);
      }

      delete m_ColorMenu;
      delete m_Image;
   }

} // mousePressEvent()

//===========================================================================
//
//===========================================================================

void *QUColorDragWidget::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures

      // clean up the cursor
      m_pc_GlobalStorage->m_pc_ImageFrame->SetCursor(ImageFrame::CursorNormal);

   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      Pixel p_Color;
      int   i_X;
      int   i_Y;

      if(m_pc_GlobalStorage == NULL)
      {
         return NULL;
      }

      m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

      p_Color = m_pc_GlobalStorage->m_pc_MainImage->Get(i_X,i_Y);

      TRACE << "Selection of new Color "
            << (int) p_Color
            << "\n";

      SetColor(p_Color);
      ColorChanged(p_Color);
   }
   return NULL;
} // SelectionEventArrived()

void *QUColorDragWidget::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   void *p_Result;
   QUColorDragWidget *pc_This = (QUColorDragWidget *) Thisdata;

   p_Result = pc_This->SelectionEventArrived(pc_Data);

   return p_Result;
} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================

void QUColorDragWidget::SaveColorInHistory(Pixel p_Color)
{
   int i;
   //look - whether the color is already in
   for(i=0;i<m_i_ColorsInHistory;i++)
   {
      if(m_ColorHistory[i] == p_Color)
      {
         // found the color
         // put it into the first position
         int j;
         for(j= i; j>0 ; j--)
         {
            m_ColorHistory[j] = m_ColorHistory[j-1];
         }
         m_ColorHistory[0] = p_Color;
         return;
      }
   }

   // color not in, put it in the first place
   for(i= QUCOLORDRAGWIDGET_MAX_HISTORY_COLORS -1;
       i>0;
       i--)
   {
      m_ColorHistory[i] = m_ColorHistory[i-1];
   }
   m_ColorHistory[0] = p_Color;
   if( m_i_ColorsInHistory < QUCOLORDRAGWIDGET_MAX_HISTORY_COLORS)
   {
      m_i_ColorsInHistory ++;
   }


} // SaveColorInHistory()

//===========================================================================
//
//===========================================================================

void QUColorDragWidget::StartPipette()
{
   if(     m_ui_Mode ==  ModePipette
       &&  m_pc_GlobalStorage != NULL)
   {
      // active pipette selection
      m_pc_GlobalStorage->m_pc_ImageFrame->SetCursor(ImageFrame::CursorPipette);

      m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionPoint,
                                 "Select a color",
                                 Callback(SelectionEventArrivedCB,this) );


   } // if global storage available

} // StartPipette

//===========================================================================
//
//===========================================================================


QUColorDragLabeledWidget::QUColorDragLabeledWidget(QWidget *p_Parent,
                     QString qs_Label,
                     unsigned int ui_Mode,
                     GlobalStorage *pc_GlobalStorage)
                : QWidget(p_Parent)
{
   QBoxLayout   *l_Layout;
   // create a horizontal box
   l_Layout = new  QHBoxLayout( this,1);

   // create the label
   m_w_Label = new QLabel(qs_Label,this);
   l_Layout->addWidget(m_w_Label);

   // create the color drag widget
   m_w_ColorWidget = new QUColorDragWidget(this,ui_Mode,pc_GlobalStorage);
   l_Layout->addWidget(m_w_ColorWidget);

   // connect inner slots to out signals
   connect( m_w_ColorWidget ,
               SIGNAL( ColorChanged(Pixel) ),
               SLOT( PColorChanged(Pixel) ) );
   connect( m_w_ColorWidget ,
               SIGNAL( DragStarted(Pixel) ),
               SLOT( PDragStarted(Pixel) ) );

   // add a spacer at the right end
   l_Layout->addWidget(new QtUtilSpacer(true,false,this));

} // QUColorDragLabeledWidget()

//===========================================================================
//
//===========================================================================

void QUColorDragLabeledWidget::SetColor(Pixel c_Color)
{
    m_w_ColorWidget->SetColor(c_Color);
}  // SetColor()

//===========================================================================
//
//===========================================================================

Pixel QUColorDragLabeledWidget::GetColor()
{
   return m_w_ColorWidget->GetColor();
} // GetColor()

//===========================================================================
//
//===========================================================================

void QUColorDragLabeledWidget::PColorChanged(Pixel p_Pixel)
{
   ColorChanged(p_Pixel);
} // PColorChanged()

//===========================================================================
//
//===========================================================================

void QUColorDragLabeledWidget::PDragStarted(Pixel p_Pixel)
{
   DragStarted(p_Pixel);
} // PColorDraged()

//===========================================================================
//
//===========================================================================

