
#include <qhgroupbox.h>
#include <qlayout.h>

#include "QUColorOrPattern.h"

#include "QtUtilities.h"
#include "ApplicationStorage.h"


// width and height of the color and pattern area
#define SELCTION_WIDGET_DIMENSION 64

//===========================================================================
//
//===========================================================================

QUColorOrPattern:: QUColorOrPattern(QWidget *parent,
                    GlobalStorage *pc_GlobalStorage)
                    : QWidget(parent)
{
   QHBoxLayout *l_BoxLayout;
   
   m_pc_GlobalStorage  = pc_GlobalStorage;
   m_pc_CurrentPattern = NULL;
   m_p_CurrentPixel    = 0;

   m_b_InUpdate        = false;

   // create a horizonal layout
   l_BoxLayout = new QHBoxLayout(this,2);

   // add checkbox
   m_ColorSelected = new QCheckBox("Color",this);
   l_BoxLayout->addWidget(m_ColorSelected);

   // add color widget
   m_ColorWidget = new QUColorDragWidget(this,
                                            QUColorDragWidget::ModePipette
                                          | QUColorDragWidget::ModeSetSize,
                                          m_pc_GlobalStorage,
                                          SELCTION_WIDGET_DIMENSION,
                                          SELCTION_WIDGET_DIMENSION);
   l_BoxLayout->addWidget(m_ColorWidget);
   m_ColorWidget->SetColor(m_p_CurrentPixel);
   
   // add patern checkbox
   m_PatternSelected = new QCheckBox("Pattern",this);
   l_BoxLayout->addWidget(m_PatternSelected);

   // add pattern selection label
   m_PatternWidget = new QUImageDragLabel(this,
                                            QUImageDragLabel::ModeUseWidthHeight
                                          | QUImageDragLabel::ModeEnablePopup,
                                          m_pc_GlobalStorage,
                                          SELCTION_WIDGET_DIMENSION,
                                          SELCTION_WIDGET_DIMENSION
                                          );
   l_BoxLayout->addWidget(m_PatternWidget);

   // connect all elements to slots
   connect( m_ColorSelected,
            SIGNAL(toggled(bool)),
            SLOT( ColorToggled(bool )));

   connect( m_ColorWidget,
            SIGNAL(ColorChanged(Pixel)),
            SLOT(InternalColorChanged(Pixel)) );
            
   connect( m_PatternWidget,
            SIGNAL(ImageDropped(PixelBuffer *)),
            SLOT(InternalImageDropped(PixelBuffer *)) );

   connect( m_PatternSelected,
            SIGNAL(toggled(bool)),
            SLOT( PatternToggled(bool )));

   // initital switch to the color mode
   m_b_ColorIsSelected = true;
   m_ColorSelected->setChecked(true);

} // QUColorOrPattern() - constructor

//===========================================================================
//
//===========================================================================

QUColorOrPattern::~QUColorOrPattern()
{
   // delete the pattern, if it exists
   if(m_pc_CurrentPattern != NULL)
   {
      delete m_pc_CurrentPattern;
      m_pc_CurrentPattern = NULL;
   }
} // ~QUColorOrPattern()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::PatternToggled(bool b_Mode)
{
   // is it called from myself
   if(m_b_InUpdate)
   {
      // do nothing
      return;
   }

   
   if(b_Mode)
   {
      m_b_ColorIsSelected = false;
   }
   else
   {
      m_b_ColorIsSelected = true;
   }
   m_b_InUpdate=true;
   m_ColorSelected->setChecked(!b_Mode);
   m_b_InUpdate=false;
   
   // signal the pattern mode
   ModeChange(b_Mode) ;
} // PatternToggled()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::ColorToggled(bool b_Mode)
{
   // is it called from myself
   if(m_b_InUpdate)
   {
      // do nothing
      return;
   }

   if(b_Mode)
   {
      m_b_ColorIsSelected = true;
   }
   else
   {
      m_b_ColorIsSelected = false;
   }
   m_b_InUpdate=true;
   m_PatternSelected->setChecked(!b_Mode);
   m_b_InUpdate=false;

   // signal the pattern mode
   ModeChange(!b_Mode) ;
}  // ColorToggled()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::InternalColorChanged(Pixel p_ThePixel)
{
   m_p_CurrentPixel = p_ThePixel;
   // in the moment siple lead it through
   ColorChanged(p_ThePixel);
}  // InternalColorChanged()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::InternalImageDropped(PixelBuffer *pc_Image)
{
   if(pc_Image == NULL)
   {
      return;
   }

   // delete current pattern
   if(m_pc_CurrentPattern != NULL)
   {
      delete m_pc_CurrentPattern;
      m_pc_CurrentPattern = NULL;
   }

   // and create a real copy
   m_pc_CurrentPattern = new PixelBuffer(pc_Image);
   m_PatternWidget->SetPixelBuffer(m_pc_CurrentPattern);

   // in the moment siple lead it through
   ImageDropped(pc_Image);
} // InternalImageDropped()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::SetColor(Pixel p)
{
   m_p_CurrentPixel = p;
   m_ColorWidget->SetColor(m_p_CurrentPixel);
   
} // SetColor()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::SetPattern(PixelBuffer *pc_Pattern)
{
   // store are real copy the this widget
   if(m_pc_CurrentPattern != NULL)
   {
      delete m_pc_CurrentPattern;
      m_pc_CurrentPattern = NULL;
   }
   m_pc_CurrentPattern = new PixelBuffer(pc_Pattern);

   m_PatternWidget->SetPixelBuffer(m_pc_CurrentPattern);

} //  SetPattern()

//===========================================================================
//
//===========================================================================

Pixel QUColorOrPattern::GetColor()
{
   return m_p_CurrentPixel;

} // GetColor()

//===========================================================================
//
//===========================================================================

PixelBuffer * QUColorOrPattern::GetPattern()
{
   return m_pc_CurrentPattern;

} // GetPattern()

//===========================================================================
//
//===========================================================================

bool QUColorOrPattern::IsColorActive()
{
   return m_b_ColorIsSelected;

} // IsColorActive()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::SetColorActive(bool b_Active)
{
   if(m_b_ColorIsSelected != b_Active)
   {
      m_b_ColorIsSelected = b_Active;
      m_ColorSelected->setChecked(b_Active);
   }    

} // SetColorActive()

//===========================================================================
//
//===========================================================================


