#ifndef _QUCORRECTIONCURVE_H
#define _QUCORRECTIONCURVE_H


#include <qwidget.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlineedit.h>


#include "ImageFrame.h"
#include "PixelBuffer.h"
#include "GlobalStorage.h"

#include "GraphicObject.h"
#include "GraphicObjectPoint.h"
#include "GraphicObjectLayer.h"
#include "GraphicObjectMain.h"
#include "Tracer.h"

class QUCorrectionCurve: public QWidget , public Tracer
{
Q_OBJECT


public:
   QUCorrectionCurve(QWidget *parent,
                     GlobalStorage *pc_ParentGlobalStorage,
                     int i_Options =0,
                     Pixel p_Mask =0xffffff);
   ~QUCorrectionCurve();

   enum
   {
      OptionSmall=1,
   };

public slots:

   void AddPressed();
   void DeletePressed();
   void ResetPressed();
   void LineModeChanged(int i_NewValue);
   void TextChanged(const QString &qs_Text);

signals:
   void	CurveChanged(unsigned char *uc_CorrectionCurve);

private:

   void CalculateCorrectedBuffer();
   void SetSensitivity();
   void RecalculateLines();
   void RecalculateCorrectionCurve();
   void RecalculateCorrectionCurveGamma();

   void CleanMainBuffer();
   void RecalculateCorrectionCurveLine(float f_LastX,float f_LastY,
                                       float f_ActualX,float f_ActualY);

   void GraphicObjektCallback(GraphicObjectMainEvent *pc_CallbackData);
   static void *GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData);

   void ObjectMovedLinear(GraphicObjectMainEvent *pc_ObjectCallbackData);
   void ObjectMovedGamma(GraphicObjectMainEvent *pc_ObjectCallbackData);

   void SetGammaEditField(float f_Faktor);

private:

   enum
   {
      CurveModeLinear=0,
      CurveModeGamma=1
   };

   ImageFrame      *m_pc_CurveFrame;
   ImageFrame      *m_pc_OriginalFrame;
   ImageFrame      *m_pc_CorrectedFrame;

   PixelBuffer *m_pc_CurveBuffer;
   PixelBuffer *m_pc_OriginalBuffer;
   PixelBuffer *m_pc_CorrectedBuffer;

   GlobalStorage *m_pc_ParentGlobalStorage;

   GlobalStorage *m_pc_CurveGlobalStorage;

   QPushButton *m_AddButton;
   QPushButton *m_DeleteButton;
   QPushButton *m_ResetButton;
   QComboBox   *m_ModeCombo;
   QLineEdit   *m_GammaFactorEdit;


   int m_i_ViewSize;
   int m_i_CurveMode;

   unsigned char m_uca_CorrectionCurve[256];

   GraphicObjectLayer     *pc_ObjectLayer;

#define CURVE_MAXIMUM_POINTS 10
   int m_i_CurrentCurvePoints;

   GraphicObjectPoint *m_pc_CurvePoints[CURVE_MAXIMUM_POINTS];
   GraphicObjectLine  *m_pc_CurveLines[CURVE_MAXIMUM_POINTS+1];

   GraphicObjectPoint *m_pc_GammaCurvePoint;
   float               m_f_GammaValue;
   bool                m_b_SelfUpdate;
   bool                m_b_Small;
   
   // size of the drawing area 128 or 256
   int                 m_i_AreaSize;
   Pixel               p_ColorMask;

};

#endif // _QUCORRECTIONCURVE_H
