

#include "QULabelSlider.h"

#include <qhgroupbox.h>
#include <qlayout.h>

#include "QtUtilities.h"
#include "ApplicationStorage.h"


QULabelSlider::QULabelSlider(QWidget *parent,
                 bool b_AutomaticLabel,
                 int i_From,
                 int i_To,
                 int i_Value,
                 QString qs_Description,
                 int i_DescriptionChars)
		: QWidget(parent)
{
   // create a hbox with a slider and a text on the right
   QHBoxLayout  *SliderLayout;

   m_i_CurrentValue   = i_Value;
   m_b_AutomaticLabel = b_AutomaticLabel;

   // to prevent updates whil in set functions
   m_b_DisableUpdate = false;

   m_i_DescriptionChars = i_DescriptionChars;


   // if autolabel enabled
   if(m_b_AutomaticLabel )
   {
      // set the label
      m_qs_CurrentLabel.sprintf("%5d",m_i_CurrentValue);
   }
   else
   {
      // leave it alone
      m_qs_CurrentLabel ="";
   }

   // create horizontal box
   SliderLayout  = new  QHBoxLayout( this,2);

   if( !qs_Description.isEmpty())
   {
      m_SliderDescription = new QLabel(qs_Description,this);
      if(m_i_DescriptionChars >0)
      {
         m_SliderDescription->setMinimumWidth(  g_Application->m_pc_QtUtility->GetMaximalFontWidth()
                                              * m_i_DescriptionChars);
      }
      SliderLayout->addWidget(m_SliderDescription);
   }
   else
   {
      m_SliderDescription = NULL;
   }

   // add slider
   m_Slider = new QSlider(i_From,
                          i_To,
                          1,
                          m_i_CurrentValue,
                          Horizontal,
                          this);
   // set a minimum width to the slider
   m_Slider->setMinimumWidth(  g_Application->m_pc_QtUtility->GetMaximalFontWidth()
                                              * 8);
   if( (i_To - i_From) < 20)
   {
      m_Slider->setTickmarks(QSlider::Below);
   }

   // add label
   m_SliderLabel = new QLabel(m_qs_CurrentLabel,this);
   m_SliderLabel->setMinimumWidth(g_Application->m_pc_QtUtility->GetMaximalFontWidth() *3 );
   m_SliderLabel->setAlignment( Qt::AlignRight);

   // put both into the layout
   SliderLayout->addWidget(m_Slider);
   SliderLayout->addWidget(m_SliderLabel);


   // and connect the value changed slider
   connect( m_Slider,  
               SIGNAL(valueChanged(int)), 
               SLOT(DoSliderValueChanged(int)) );


} // QULabelSlider() - constructor

//===========================================================================
//
//===========================================================================


QULabelSlider::~QULabelSlider()
{
} // ~QULabelSlider()

//===========================================================================
//
//===========================================================================


void QULabelSlider::SetValue(int i_Value,QString qs_Label)
{
   m_b_DisableUpdate= true;

   m_i_CurrentValue = i_Value;

   // set the value
   m_Slider->setValue(m_i_CurrentValue);

   // if there is a valid string, then set also the string
   if(! qs_Label.isEmpty())
   {
      SetLabel( qs_Label);
   }
   else
   {
      // no valid string
      // but automation is enabled
      if(m_b_AutomaticLabel)
      {
         m_qs_CurrentLabel.sprintf("%5d",m_i_CurrentValue);
         m_SliderLabel->setText(m_qs_CurrentLabel);
      }
   }


   m_b_DisableUpdate= false;

} // SetValue()

//===========================================================================
//
//===========================================================================

void QULabelSlider::SetLabel(QString qs_Label)
{
   m_qs_CurrentLabel = qs_Label;
   m_SliderLabel->setText(m_qs_CurrentLabel);

} // SetLabel()

//===========================================================================
//
//===========================================================================

int QULabelSlider::GetValue()
{
   return m_i_CurrentValue;
} // GetValue()

//===========================================================================
//
//===========================================================================


void QULabelSlider::DoSliderValueChanged(int i_NewValue)
{
   if(m_b_DisableUpdate)
   {
      //fprintf(stderr,"-- ignored\n");
      return;
   }
 
   m_i_CurrentValue = i_NewValue;
   if(m_b_AutomaticLabel)
   {
      m_qs_CurrentLabel.sprintf("%5d",i_NewValue);
      m_SliderLabel->setText(m_qs_CurrentLabel);
   }

   valueChanged( i_NewValue);

} // DoSliderValueChanged(int i_NewValue)

//===========================================================================
//
//===========================================================================

void QULabelSlider::SetLabelStringSize(int i_Letters)
{
   m_SliderDescription->setMinimumWidth(  g_Application->m_pc_QtUtility->GetMaximalFontWidth()
                                              * m_i_DescriptionChars);
  
}

//===========================================================================
//
//===========================================================================

   

