#include "QtUtilities.h"
#include <qtooltip.h>
#include <qapplication.h>
#include <qcursor.h>


//===========================================================================
//
//===========================================================================

QtUtilSpacer::QtUtilSpacer(bool ExpandHorizontal,
                           bool ExpandVertical,
                           QWidget *Parent)
	: QWidget(Parent)
{
   if(ExpandHorizontal || ! ExpandVertical )
   {

      setSizePolicy(QSizePolicy( QSizePolicy::Expanding,
                                 QSizePolicy::Minimum,
                                 false));
   }
   if(!ExpandHorizontal ||  ExpandVertical )
   {

      setSizePolicy(QSizePolicy( QSizePolicy::Minimum,
                                 QSizePolicy::Expanding,
                                 false));
   }
   if(ExpandHorizontal || ! ExpandVertical )
   {

      setSizePolicy(QSizePolicy( QSizePolicy::Expanding,
                                 QSizePolicy::Expanding,
                                 false));
   }

} // QtUtilSpacer()

//===========================================================================
//
//===========================================================================

QtUtilReadOnlyLineEdit::QtUtilReadOnlyLineEdit(QWidget *Parent)
     :  QLineEdit(Parent)
{
   QPalette pal;
   setReadOnly(true);

   pal.setColor( QColorGroup::Base, QColor(180,180,190) );
   pal.setColor( QColorGroup::HighlightedText, QColor(255,255,255) );

   setPalette( pal );

} // QtUtilReadOnlyLineEdit

//===========================================================================
//
//===========================================================================

QtUtility::QtUtility()
{
   QPalette pal;
   QFont  font;
   QFontMetrics fm(font);
   m_i_MaxFontWidth = fm.maxWidth();


   // black text
   pal.setColor( QColorGroup::Text, QColor(0,0,0) );
   // light red background
   pal.setColor( QColorGroup::Background, QColor(255,211,211) );

   // use this palette for tooltips
   QToolTip::setPalette( pal );

} // QtUtility()

//===========================================================================
//
//===========================================================================

int QtUtility::GetMaximalFontWidth()
{
   return m_i_MaxFontWidth;
} // GetMaximalFontWidth()

//===========================================================================
//
//===========================================================================

void QtUtility::StartWaitingCursor()
{
   QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
} // StartWaitingCursor()

//===========================================================================
//
//===========================================================================

void QtUtility::EndWaitingCursor()
{
   QApplication::restoreOverrideCursor();

} // EndWaitingCursor()

//===========================================================================
//
//===========================================================================

QString QtUtility::GetDragSourceName(QByteArray *qba_Data,
                                    QString *pqs_Value)
{
   QString qs_Return;

   char    a_Character;
   int     i;
   int     i_Size;

   qs_Return = "";

   i_Size = qba_Data->size();

   if(    i_Size > 1
       && i_Size < 100 )
   {
      for(i=0; i< i_Size ; i++)
      {
         a_Character = qba_Data->at(i);
         if(a_Character== '=')
         {
            if(pqs_Value == NULL)
            {
               return qs_Return;
            }
            // skip the equal sign
            i++ ; 
            *pqs_Value ="";
            while(i< i_Size)
            {
               a_Character = qba_Data->at(i);
               *pqs_Value += a_Character;
               i++;
            }
            return qs_Return;
         }
         qs_Return += a_Character;
      }

   } // id size long enough

   return qs_Return;
} // GetDrawSourceName()

//===========================================================================
//
//===========================================================================

QString QtUtility::GetDragSourceText(QByteArray *qba_Data)
{
   QString qs_Return;
   char    a_Character;
   int     i;
   int     i_Size;

   qs_Return = "";

   i_Size = qba_Data->size();

   if( i_Size <= 0 )
   {
      return qs_Return;
   }

   for(i=0; i< i_Size ; i++)
   {
      a_Character = qba_Data->at(i);
      qs_Return  += a_Character;
   } // for i

   return qs_Return;
} // GetDrawSourceText()

//===========================================================================
//
//===========================================================================
