
#include "stdio.h"
#include "RectRegion.h"
#include "Geometry.h"
#include "Utilities.h"
#include "PixelBuffer.h"

//===========================================================================
//
//===========================================================================

RectRegion::RectRegion()
{
   SetEmpty();
   
   // int mode is default
   m_b_FloatMode =false;

} // RectRegion()

//===========================================================================
//
//===========================================================================

RectRegion::RectRegion(int i_StartX, int i_StartY, int i_Width,int i_Height)
{
   m_i_StartX = i_StartX;
   m_i_StartY = i_StartY;
   m_i_Width  = i_Width;
   m_i_Height = i_Height;

   m_b_FloatMode = false;

   AdjustEnd();

} // RectRegion()

//===========================================================================
//
//===========================================================================

RectRegion::RectRegion(float f_StartX, float f_StartY, float f_Width,float f_Height)
{
   m_f_StartX = f_StartX;
   m_f_StartY = f_StartY;
   m_f_Width  = f_Width;
   m_f_Height = f_Height;

   m_b_FloatMode = true;

   AdjustEnd();

} // RectRegion()

//===========================================================================
//
//===========================================================================

RectRegion::RectRegion(PixelBuffer *pc_Image)
{
   if(pc_Image ==NULL)
   {
      SetEmpty();
      return;
   }
   
   m_i_StartX = 0;
   m_i_StartY = 0;
   m_i_Width  = pc_Image->GetWidth();
   m_i_Height = pc_Image->GetHeight();

   m_b_FloatMode = false;

   AdjustEnd();

} // RectRegion()

//===========================================================================
//
//===========================================================================

void RectRegion::AdjustEnd()
{
   if(m_b_FloatMode)
   {
      m_f_EndX = m_f_StartX + m_f_Width - 1.0;
      m_f_EndY = m_f_StartY + m_f_Height- 1.0;

      m_b_Empty = false;
      if(   m_f_Width  <= 0
         || m_f_Height <= 0)
      {
         m_b_Empty = true;
      }
   }
   else // integer mode
   {
      m_i_EndX = m_i_StartX + m_i_Width - 1;
      m_i_EndY = m_i_StartY + m_i_Height- 1;

      m_b_Empty = false;
      if(   m_i_Width  <= 0
         || m_i_Height <= 0)
      {
         m_b_Empty = true;
      }
   }

} // AdjustEnd()


//===========================================================================
//
//===========================================================================

void RectRegion::AdjustWidthHeight()
{
   if(m_b_FloatMode)
   {
      m_f_Width  = m_f_EndX - m_f_StartX + 1.0;
      m_f_Height = m_f_EndY - m_f_StartY + 1.0;

      m_b_Empty = false;
      if(   m_f_Width  <= 0
         || m_f_Height <= 0)
      {
         m_b_Empty = true;
      }
   }
   else // integer mode
   {
      m_i_Width  = m_i_EndX - m_i_StartX + 1;
      m_i_Height = m_i_EndY - m_i_StartY + 1;

      m_b_Empty = false;
      if(   m_i_Width  <= 0
         || m_i_Height <= 0)
      {
         m_b_Empty = true;
      }
   }

} // AdjustEnd()

//===========================================================================
//
//===========================================================================

void RectRegion::SetEmpty()
{
   if(m_b_FloatMode)
   {
      m_f_StartX =0;
      m_f_StartY =0;
      m_f_EndX   =0;
      m_f_EndY   =0;
      m_f_Width  =0;
      m_f_Height =0;
   }
   else
   {
      m_i_StartX =0;
      m_i_StartY =0;
      m_i_EndX   =0;
      m_i_EndY   =0;
      m_i_Width  =0;
      m_i_Height =0;
   }

   m_b_Empty = true;
}

//===========================================================================
//
//===========================================================================

int RectRegion::GetStartX()
{
   if(m_b_Empty)
   {
      return 0;
   }
   if(m_b_FloatMode)
   {
      return (int) m_f_StartX;
   }
   return m_i_StartX;
}
//===========================================================================
//
//===========================================================================

int RectRegion::GetStartY()
{
   if(m_b_Empty)
   {
      return 0;
   }
   if(m_b_FloatMode)
   {
      return (int) m_f_StartY;
   }
   return m_i_StartY;

}
//===========================================================================
//
//===========================================================================

int RectRegion::GetEndX()
{
   if(m_b_Empty)
   {
      return 0;
   }
   if(m_b_FloatMode)
   {
      return (int) m_f_EndX;
   }
   return m_i_EndX;
}
//===========================================================================
//
//===========================================================================

int RectRegion::GetEndY()
{
   if(m_b_Empty)
   {
      return 0;
   }
   if(m_b_FloatMode)
   {
      return (int) m_f_EndY;
   }
   return m_i_EndY;
}
//===========================================================================
//
//===========================================================================

int RectRegion::GetWidth()
{
   if(m_b_Empty)
   {
      return 0;
   }
   if(m_b_FloatMode)
   {
      return (int) m_f_Width;
   }
   return m_i_Width;
}

//===========================================================================
//
//===========================================================================

int RectRegion::GetHeight()
{
   if(m_b_Empty)
   {
      return 0;
   }
   if(m_b_FloatMode)
   {
      return (int) m_f_Height;
   }
   return m_i_Height;
}

//===========================================================================
//
//===========================================================================

float RectRegion::GetStartXFloat()
{
   if(m_b_Empty)
   {
      return 0.0;
   }
   if(m_b_FloatMode)
   {
      return  m_f_StartX;
   }
   return (float) m_i_StartX;
}
//===========================================================================
//
//===========================================================================

float RectRegion::GetStartYFloat()
{
   if(m_b_Empty)
   {
      return 0.0;
   }
   if(m_b_FloatMode)
   {
      return  m_f_StartY;
   }
   return (float) m_i_StartY;

}
//===========================================================================
//
//===========================================================================

float RectRegion::GetEndXFloat()
{
   if(m_b_Empty)
   {
      return 0.0;
   }
   if(m_b_FloatMode)
   {
      return  m_f_EndX;
   }
   return (float) m_i_EndX;
}
//===========================================================================
//
//===========================================================================

float RectRegion::GetEndYFloat()
{
   if(m_b_Empty)
   {
      return 0.0;
   }
   if(m_b_FloatMode)
   {
      return  m_f_EndY;
   }
   return (float) m_i_EndY;
}
//===========================================================================
//
//===========================================================================

float RectRegion::GetWidthFloat()
{
   if(m_b_Empty)
   {
      return 0.0;
   }
   if(m_b_FloatMode)
   {
      return  m_f_Width;
   }
   return (float) m_i_Width;
}

//===========================================================================
//
//===========================================================================

float RectRegion::GetHeightFloat()
{
   if(m_b_Empty)
   {
      return 0.0;
   }
   if(m_b_FloatMode)
   {
      return  m_f_Height;
   }
   return (float) m_i_Height;
}

//===========================================================================
//
//===========================================================================

bool RectRegion::IsEmpty()
{
   return m_b_Empty;
} // IsEmpty()

//===========================================================================
//
//===========================================================================

void RectRegion::MoveOffset(int i_XOffset, int i_YOffset)
{
   if(m_b_Empty)
   {
      return ;
   }
   if(m_b_FloatMode)
   {
      m_f_StartX += i_XOffset;
      m_f_EndX   += i_XOffset;
      m_f_StartY += i_YOffset;
      m_f_EndY   += i_YOffset;
   }
   else
   {
      m_i_StartX += i_XOffset;
      m_i_EndX   += i_XOffset;
      m_i_StartY += i_YOffset;
      m_i_EndY   += i_YOffset;
   }

} // MoveOffset()

//===========================================================================
//
//===========================================================================

void RectRegion::ClipAt(RectRegion &ClipRegion)
{
   if(m_b_Empty)
   {
      return ;
   }

   // clipregion empty
   if(ClipRegion.m_b_Empty)
   {
      // then new is also empty
      SetEmpty();
      return;
   }

   if(   ! m_b_FloatMode
      && ClipRegion.m_b_FloatMode)
   {
      // different modes.
      // change this object to float mode
      ConvertToFloat();
   }

   if(m_b_FloatMode)
   {
      if(ClipRegion.m_b_FloatMode)
      {
         // calculate new X range
         if(m_f_EndX < ClipRegion.m_f_StartX)
         {
            SetEmpty();
            return;
         }
         if(m_f_StartX > ClipRegion.m_f_EndX)
         {
            SetEmpty();
            return;
         }
         // X range is overlapping
         if(m_f_StartX < ClipRegion.m_f_StartX)
         {
            m_f_StartX = ClipRegion.m_f_StartX;
         }
         if(m_f_EndX > ClipRegion.m_f_EndX)
         {
            m_f_EndX = ClipRegion.m_f_EndX;
         }

         // and now calculate new Y range
         if(m_f_EndY < ClipRegion.m_f_StartY)
         {
            SetEmpty();
            return;
         }
         if(m_f_StartY > ClipRegion.m_f_EndY)
         {
            SetEmpty();
            return;
         }
         // X range is overlapping
         if(m_f_StartY < ClipRegion.m_f_StartY)
         {
            m_f_StartY = ClipRegion.m_f_StartY;
         }
         if(m_f_EndY > ClipRegion.m_f_EndY)
         {
            m_f_EndY = ClipRegion.m_f_EndY;
         }
         AdjustWidthHeight();


      }
      else // clipregion is integer
      {
      
         // calculate new X range
         if(m_f_EndX < (float) ClipRegion.m_i_StartX)
         {
            SetEmpty();
            return;
         }
         if(m_f_StartX > (float) ClipRegion.m_i_EndX)
         {
            SetEmpty();
            return;
         }
         // X range is overlapping
         if(m_f_StartX < (float) ClipRegion.m_i_StartX)
         {
            m_f_StartX = (float) ClipRegion.m_i_StartX;
         }
         if(m_f_EndX > (float) ClipRegion.m_i_EndX)
         {
            m_f_EndX = (float) ClipRegion.m_i_EndX;
         }

         // and now calculate new Y range
         if(m_f_EndY < (float) ClipRegion.m_i_StartY)
         {
            SetEmpty();
            return;
         }
         if(m_f_StartY > (float) ClipRegion.m_i_EndY)
         {
            SetEmpty();
            return;
         }
         // X range is overlapping
         if(m_f_StartY < (float) ClipRegion.m_i_StartY)
         {
            m_f_StartY = (float) ClipRegion.m_i_StartY;
         }
         if(m_f_EndY > (float) ClipRegion.m_i_EndY)
         {
            m_f_EndY = (float) ClipRegion.m_i_EndY;
         }
         AdjustWidthHeight();



      }
   }
   else // both are integer mode
   {
      // calculate new X range
      if(m_i_EndX < ClipRegion.m_i_StartX)
      {
         SetEmpty();
         return;
      }
      if(m_i_StartX > ClipRegion.m_i_EndX)
      {
         SetEmpty();
         return;
      }
      // X range is overlapping
      if(m_i_StartX < ClipRegion.m_i_StartX)
      {
         m_i_StartX = ClipRegion.m_i_StartX;
      }
      if(m_i_EndX > ClipRegion.m_i_EndX)
      {
         m_i_EndX = ClipRegion.m_i_EndX;
      }

      // and now calculate new Y range
      if(m_i_EndY < ClipRegion.m_i_StartY)
      {
         SetEmpty();
         return;
      }
      if(m_i_StartY > ClipRegion.m_i_EndY)
      {
         SetEmpty();
         return;
      }
      // X range is overlapping
      if(m_i_StartY < ClipRegion.m_i_StartY)
      {
         m_i_StartY = ClipRegion.m_i_StartY;
      }
      if(m_i_EndY > ClipRegion.m_i_EndY)
      {
         m_i_EndY = ClipRegion.m_i_EndY;
      }
      AdjustWidthHeight();
   } // else both integer mode

} // ClipAt();

//===========================================================================
//
//===========================================================================

void RectRegion::SetStartX(int i_StartX)
{
   if(m_b_FloatMode)
   {
      m_f_StartX = (float) i_StartX;

      if(m_f_EndX < m_f_StartX)
      {
         Utilities::swap(m_f_EndX,m_f_StartX);
      }
   }
   else
   {
   
      m_i_StartX = i_StartX;

      if(m_i_EndX < m_i_StartX)
      {
         Utilities::swap(m_i_EndX,m_i_StartX);
      }
   }
   AdjustWidthHeight();
}  // SetStartX()

//===========================================================================
//
//===========================================================================

void RectRegion::SetStartX(float f_StartX)
{
   if(m_b_FloatMode)
   {
      m_f_StartX =  f_StartX;

      if(m_f_EndX < m_f_StartX)
      {
         Utilities::swap(m_f_EndX,m_f_StartX);
      }
   }
   else
   {

      m_i_StartX = (int) f_StartX;

      if(m_i_EndX < m_i_StartX)
      {
         Utilities::swap(m_i_EndX,m_i_StartX);
      }
   }
   AdjustWidthHeight();
}  // SetStartX()

//===========================================================================
//
//===========================================================================

void RectRegion::SetStartY(int i_StartY)
{
   if(m_b_FloatMode)
   {
      m_f_StartY = (float) i_StartY;

      if(m_f_EndY < m_f_StartY)
      {
         Utilities::swap(m_f_EndY,m_f_StartY);
      }
   }
   else
   {
      m_i_StartY = i_StartY;

      if(m_i_EndY < m_i_StartY)
      {
         Utilities::swap(m_i_EndY,m_i_StartY);
      }
   }
   AdjustWidthHeight();
}  // SetStartY()

//===========================================================================
//
//===========================================================================

void RectRegion::SetStartY(float f_StartY)
{
   if(m_b_FloatMode)
   {
      m_f_StartY =  f_StartY;

      if(m_f_EndY < m_f_StartY)
      {
         Utilities::swap(m_f_EndY,m_f_StartY);
      }
   }
   else
   {
      m_i_StartY = (int) f_StartY;

      if(m_i_EndY < m_i_StartY)
      {
         Utilities::swap(m_i_EndY,m_i_StartY);
      }
   }
   AdjustWidthHeight();
}  // SetStartY()

//===========================================================================
//
//===========================================================================


void RectRegion::SetEndX(int i_EndX)
{
   if(m_b_FloatMode)
   {
      m_f_EndX = (float) i_EndX;

      if(m_f_EndX < m_f_StartX)
      {
         Utilities::swap(m_f_EndX,m_f_StartX);
      }
   }
   else
   {
   
      m_i_EndX = i_EndX;

      if(m_i_EndX < m_i_StartX)
      {
         Utilities::swap(m_i_EndX,m_i_StartX);
      }
   }

   AdjustWidthHeight();
} // SetEndX()

//===========================================================================
//
//===========================================================================


void RectRegion::SetEndX(float f_EndX)
{
   if(m_b_FloatMode)
   {
      m_f_EndX =  f_EndX;

      if(m_f_EndX < m_f_StartX)
      {
         Utilities::swap(m_f_EndX,m_f_StartX);
      }
   }
   else
   {

      m_i_EndX = (int) f_EndX;

      if(m_i_EndX < m_i_StartX)
      {
         Utilities::swap(m_i_EndX,m_i_StartX);
      }
   }

   AdjustWidthHeight();
} // SetEndX()

//===========================================================================
//
//===========================================================================


void RectRegion::SetEndY(int i_EndY)
{
   if(m_b_FloatMode)
   {
      m_f_EndY = (float) i_EndY;

      if(m_f_EndY < m_f_StartY)
      {
         Utilities::swap(m_f_EndY,m_f_StartY);
      }
   }
   else
   {
      m_i_EndY = i_EndY;

      if(m_i_EndY < m_i_StartY)
      {
         Utilities::swap(m_i_EndY,m_i_StartY);
      }
   }

   AdjustWidthHeight();
} // SetEndY()

//===========================================================================
//
//===========================================================================


void RectRegion::SetEndY(float f_EndY)
{
   if(m_b_FloatMode)
   {
      m_f_EndY = f_EndY;

      if(m_f_EndY < m_f_StartY)
      {
         Utilities::swap(m_f_EndY,m_f_StartY);
      }
   }
   else
   {
      m_i_EndY = (int) f_EndY;

      if(m_i_EndY < m_i_StartY)
      {
         Utilities::swap(m_i_EndY,m_i_StartY);
      }
   }

   AdjustWidthHeight();
} // SetEndY()


//===========================================================================
//
//===========================================================================

void RectRegion::SetWidth(int i_Width)
{
   if(m_b_FloatMode)
   {
      m_f_Width = (float) i_Width;
   }
   else
   {
      m_i_Width = i_Width;
   }

   AdjustEnd();
} // SetWidth()

//===========================================================================
//
//===========================================================================

void RectRegion::SetWidth(float f_Width)
{
   if(m_b_FloatMode)
   {
      m_f_Width = f_Width;
   }
   else
   {
      m_i_Width = (int) f_Width;
   }

   AdjustEnd();

} // SetWidth()

//===========================================================================
//
//===========================================================================

void RectRegion::SetHeight(int i_Height)
{
   if(m_b_FloatMode)
   {
      m_f_Height = (float) i_Height;
   }
   else
   {
      m_i_Height = i_Height;
   }

   AdjustEnd();

} // SetHeight()

//===========================================================================
//
//===========================================================================

void RectRegion::SetHeight(float f_Height)
{
   if(m_b_FloatMode)
   {
      m_f_Height = f_Height;
   }
   else
   {
      m_i_Height = (int) f_Height;
   }

   AdjustEnd();

} // SetHeight()

//===========================================================================
//
//===========================================================================



void RectRegion::SetXYXY(int i_X1, int i_Y1,int i_X2,int i_Y2)
{
   if(i_X1 < i_X2)
   {
      m_i_StartX = i_X1;
      m_i_EndX   = i_X2;
   }
   else
   {
      m_i_StartX = i_X2;
      m_i_EndX   = i_X1;
   }

   if(i_Y1 < i_Y2)
   {
      m_i_StartY = i_Y1;
      m_i_EndY   = i_Y2;
   }
   else
   {
      m_i_StartY = i_Y2;
      m_i_EndY   = i_Y1;
   }
   m_b_FloatMode = false;

   AdjustWidthHeight();

} // SetXYXY()

//===========================================================================
//
//===========================================================================

void RectRegion::SetXYXY(float f_X1, float f_Y1,float f_X2,float f_Y2)
{
   if(f_X1 < f_X2)
   {
      m_f_StartX = f_X1;
      m_f_EndX   = f_X2;
   }
   else
   {
      m_f_StartX = f_X2;
      m_f_EndX   = f_X1;
   }

   if(f_Y1 < f_Y2)
   {
      m_f_StartY = f_Y1;
      m_f_EndY   = f_Y2;
   }
   else
   {
      m_f_StartY = f_Y2;
      m_f_EndY   = f_Y1;
   }
   m_b_FloatMode = true;

   AdjustWidthHeight();

} // SetXYXY()

//===========================================================================
//
//===========================================================================

void RectRegion::SetXYWH(int i_X, int i_Y,int i_Width,int i_Height)
{
   m_i_StartX = i_X;
   m_i_StartY = i_Y;
   m_i_Width  = i_Width;
   m_i_Height = i_Height;

   m_b_FloatMode = false;
   AdjustEnd();

} // SetXYWH

//===========================================================================
//
//===========================================================================

void RectRegion::SetXYWH(float f_X, float f_Y,float f_Width,float f_Height)
{
   m_f_StartX = f_X;
   m_f_StartY = f_Y;
   m_f_Width  = f_Width;
   m_f_Height = f_Height;

   m_b_FloatMode = true;
   AdjustEnd();

} // SetXYWH
//===========================================================================
//
//===========================================================================



void RectRegion::printf()
{
   if(m_b_FloatMode)
   {
      fprintf(stderr," F: (%f/%f) - (%f/%f) w:%f h:%f",
		m_f_StartX,m_f_StartY,
		m_f_EndX  ,m_f_EndY,
		m_f_Width ,m_f_Height
   		);
   }
   else // integer mode
   {
      fprintf(stderr," I: (%d/%d) - (%d/%d) w:%d h:%d",
		m_i_StartX,m_i_StartY,
		m_i_EndX,m_i_EndY,
		m_i_Width,m_i_Height
   		);
   }

} // printf()

//===========================================================================
//
//===========================================================================

void RectRegion::ConvertToFloat()
{
   m_f_StartX   = (float) m_i_StartX ;
   m_f_StartY   = (float) m_i_StartY ;
   m_f_Width    = (float) m_i_Width ;
   m_f_Height   = (float) m_i_Height ;
   m_f_EndX     = (float) m_i_EndX ;
   m_f_EndY     = (float) m_i_EndY ;

   m_b_FloatMode = true;
} // ConvertToFloat()

//===========================================================================
//
//===========================================================================


bool RectRegion::PointInRegion(GeoCoordinate c_Point)
{
   if(m_b_FloatMode)
   {
      if(   c_Point.m_f_X < m_f_StartX
         || c_Point.m_f_X > m_f_EndX
         || c_Point.m_f_Y < m_f_StartY
         || c_Point.m_f_Y > m_f_EndY
        )
      {
         return false;
      }
   }
   else // integer mode
   {
      if(   c_Point.m_f_X < (float) m_i_StartX
         || c_Point.m_f_X > (float) m_i_EndX
         || c_Point.m_f_Y < (float) m_i_StartY
         || c_Point.m_f_Y > (float) m_i_EndY
        )
      {
         return false;
      }
   }

   // point is in region
   return true;
}  // PointInRegion()

//===========================================================================
//
//===========================================================================

bool RectRegion::PointInRegion(int i_X, int i_Y)
{
   if(m_b_FloatMode)
   {
      if(   i_X < (int) m_f_StartX
         || i_X > (int) m_f_EndX
         || i_Y < (int) m_f_StartY
         || i_Y > (int) m_f_EndY
        )
      {
         return false;
      }
   }
   else // integer mode
   {
      if(   i_X < m_i_StartX
         || i_X > m_i_EndX
         || i_Y < m_i_StartY
         || i_Y > m_i_EndY
        )
      {
         return false;
      }
   }

   // point is in region
   return true;
}  // PointInRegion()

//===========================================================================
//
//===========================================================================

bool RectRegion::PointInRegion(float f_X, float f_Y)
{
   if(m_b_FloatMode)
   {
      if(   f_X <  m_f_StartX
         || f_X >  m_f_EndX
         || f_Y <  m_f_StartY
         || f_Y >  m_f_EndY
        )
      {
         return false;
      }
   }
   else // integer mode
   {
      if(   (int) f_X < m_i_StartX
         || (int) f_X > m_i_EndX
         || (int) f_Y < m_i_StartY
         || (int) f_Y > m_i_EndY
        )
      {
         return false;
      }
   }

   // point is in region
   return true;
}  // PointInRegion()

//===========================================================================
//
//===========================================================================

void RectRegion::Extend(int i_Left, int i_Top, int i_Right, int i_Bottom)
{
   if(m_b_FloatMode)
   {
      m_f_StartX -= (float) i_Left;
      m_f_StartY -= (float) i_Top;
      m_f_EndX   += (float) i_Right;
      m_f_EndY   += (float) i_Bottom;
      AdjustWidthHeight();
   }
   else
   {
      m_i_StartX -= i_Left;
      m_i_StartY -= i_Top;
      m_i_EndX   += i_Right;
      m_i_EndY   += i_Bottom;
      AdjustWidthHeight();
   }

} // Extend()


//===========================================================================
//
//===========================================================================

void RectRegion::AddRegion(RectRegion &OtherRegion)
{
   // this empty, then copy entire
   if(m_b_Empty)
   {
      *this = OtherRegion;
      return;
   }

   if(m_b_FloatMode)
   {
      if(OtherRegion.GetStartXFloat() < m_f_StartX)
      {
         m_f_StartX = OtherRegion.GetStartXFloat();
      }
      if(OtherRegion.GetStartYFloat() < m_f_StartY)
      {
         m_f_StartY = OtherRegion.GetStartYFloat();
      }

      if(OtherRegion.GetEndXFloat() > m_f_EndX)
      {
         m_f_EndX = OtherRegion.GetEndXFloat();
      }
      if(OtherRegion.GetEndYFloat() > m_f_EndY)
      {
         m_f_EndY = OtherRegion.GetEndYFloat();
      }

      AdjustWidthHeight();
   }
   else // int mode
   {
      if(OtherRegion.GetStartX() < m_i_StartX)
      {
         m_i_StartX = OtherRegion.GetStartX();
      }
      if(OtherRegion.GetStartY() < m_i_StartY)
      {
         m_i_StartY = OtherRegion.GetStartY();
      }

      if(OtherRegion.GetEndX() > m_i_EndX)
      {
         m_i_EndX = OtherRegion.GetEndX();
      }
      if(OtherRegion.GetEndY() > m_i_EndY)
      {
         m_i_EndY = OtherRegion.GetEndY();
      }

      AdjustWidthHeight();
   }

} // AddRegion()

//===========================================================================
//
//===========================================================================

void RectRegion::AddPoint(int i_X, int i_Y)
{
   bool b_Changed;
   
   if(m_b_Empty)
   {
      SetXYWH(i_X,i_Y,1,1);
      m_b_Empty = false;
      return;
   }
   
   b_Changed = false;
   
   if(m_b_FloatMode)
   {
      if(m_f_StartX > (float) i_X)
      {
         m_f_StartX = (float) i_X;
         b_Changed = true;
      }
      else if(m_f_EndX < (float) i_X)
      {
         m_f_EndX = (float) i_X;
         b_Changed = true;
      }

      if(m_f_StartY > (float) i_Y)
      {
         m_f_StartY = (float) i_Y;
         b_Changed = true;
      }
      else if(m_f_EndY < (float) i_Y)
      {
         m_f_EndY = (float) i_Y;
         b_Changed = true;
      }

   }
   else // int mode
   {
      if(m_i_StartX >  i_X)
      {
         m_i_StartX =  i_X;
         b_Changed = true;
      }
      else if(m_i_EndX <  i_X)
      {
         m_i_EndX =  i_X;
         b_Changed = true;
      }

      if(m_i_StartY >  i_Y)
      {
         m_i_StartY =  i_Y;
         b_Changed = true;
      }
      else if(m_i_EndY <  i_Y)
      {
         m_i_EndY =  i_Y;
         b_Changed = true;
      }

   } // else in mode

   if(b_Changed)
   {
      AdjustWidthHeight();
   }
} // AddPoint()

//===========================================================================
//
//===========================================================================

bool RectRegion::ContainsRegion(RectRegion &InnerRegion)
{
   if(m_b_FloatMode)
   {
      if(InnerRegion.GetStartXFloat() < m_f_StartX)
      {
         return false;
      }
      if(InnerRegion.GetEndXFloat() > m_f_EndX)
      {
         return false;
      }
      if(InnerRegion.GetStartYFloat() < m_f_StartY)
      {
         return false;
      }
      if(InnerRegion.GetEndYFloat() > m_f_EndY)
      {
         return false;
      }
      
   }
   else // int mode
   {
      if(InnerRegion.GetStartX() < m_i_StartX)
      {
         return false;
      }
      if(InnerRegion.GetEndX() > m_i_EndX)
      {
         return false;
      }
      if(InnerRegion.GetStartY() < m_i_StartY)
      {
         return false;
      }
      if(InnerRegion.GetEndY() > m_i_EndY)
      {
         return false;
      }
   }

   return true;   
} //  ContainsRegion()

//===========================================================================
// check whether this region has a least 0 point common with
// the other region.
//===========================================================================

bool RectRegion::OverlapsRegion(RectRegion &OtherRegion)
{
   if(m_b_FloatMode)
   {
      float f_OtherStartX;
      float f_OtherStartY;
      float f_OtherEndX;
      float f_OtherEndY;

      f_OtherStartX = OtherRegion.GetStartXFloat();
      f_OtherStartY = OtherRegion.GetStartYFloat();
      f_OtherEndX   = OtherRegion.GetEndXFloat();
      f_OtherEndY   = OtherRegion.GetEndYFloat();

      // is point 1 in other
      if(    m_f_StartX >= f_OtherStartX
          && m_f_StartX <= f_OtherEndX
          && m_f_StartY >= f_OtherStartY
          && m_f_StartY <= f_OtherEndY  )
      {
         return true;
      }
      // is point 2 in other
      if(    m_f_EndX   >= f_OtherStartX
          && m_f_EndX   <= f_OtherEndX
          && m_f_StartY >= f_OtherStartY
          && m_f_StartY <= f_OtherEndY  )
      {
         return true;
      }
      // is point 3 in other
      if(    m_f_EndX   >= f_OtherStartX
          && m_f_EndX   <= f_OtherEndX
          && m_f_EndY   >= f_OtherStartY
          && m_f_EndY   <= f_OtherEndY  )
      {
         return true;
      }
      // is point 4 in other
      if(    m_f_StartX >= f_OtherStartX
          && m_f_StartX <= f_OtherEndX
          && m_f_EndY   >= f_OtherStartY
          && m_f_EndY   <= f_OtherEndY  )
      {
         return true;
      }

      // no point of this is on other,
      // remains only the possibility, the other is completely in this
      // this can be done by only checking one point

      // is point 1 of other, whether it is in this
      if(    f_OtherStartX >= m_f_StartX
          && f_OtherStartX <= m_f_EndX
          && f_OtherStartY >= m_f_StartY
          && f_OtherStartY <= m_f_EndY  )
      {
         return true;
      }
      
   }
   else // int mode
   {
      int i_OtherStartX;
      int i_OtherStartY;
      int i_OtherEndX;
      int i_OtherEndY;

      i_OtherStartX = OtherRegion.GetStartX();
      i_OtherStartY = OtherRegion.GetStartY();
      i_OtherEndX   = OtherRegion.GetEndX();
      i_OtherEndY   = OtherRegion.GetEndY();

      // is point 1 in other
      if(    m_i_StartX >= i_OtherStartX
          && m_i_StartX <= i_OtherEndX
          && m_i_StartY >= i_OtherStartY
          && m_i_StartY <= i_OtherEndY  )
      {
         return true;
      }
      // is point 2 in other
      if(    m_i_EndX   >= i_OtherStartX
          && m_i_EndX   <= i_OtherEndX
          && m_i_StartY >= i_OtherStartY
          && m_i_StartY <= i_OtherEndY  )
      {
         return true;
      }
      // is point 3 in other
      if(    m_i_EndX   >= i_OtherStartX
          && m_i_EndX   <= i_OtherEndX
          && m_i_EndY   >= i_OtherStartY
          && m_i_EndY   <= i_OtherEndY  )
      {
         return true;
      }
      // is point 4 in other
      if(    m_i_StartX >= i_OtherStartX
          && m_i_StartX <= i_OtherEndX
          && m_i_EndY   >= i_OtherStartY
          && m_i_EndY   <= i_OtherEndY  )
      {
         return true;
      }

      // no point of this is on other,
      // remains only the possibility, the other is completely in this
      // this can be done by only checking one point

      // is point 1 of other, whether it is in this
      if(    i_OtherStartX >= m_i_StartX
          && i_OtherStartX <= m_i_EndX
          && i_OtherStartY >= m_i_StartY
          && i_OtherStartY <= m_i_EndY  )
      {
         return true;
      }

   }  // int mode

   // no overlapping
   
   return false;
} // OverlapsRegion

//===========================================================================
//
//===========================================================================

void RectRegion::Set(PixelBuffer *pc_Image)
{
   if(pc_Image ==NULL)
   {
      SetEmpty();
      return;
   }

   m_b_FloatMode = false;
   SetXYWH(0,0,pc_Image->GetWidth(),pc_Image->GetHeight());

} // set

//===========================================================================
//
//===========================================================================

