#ifndef _RECTREGION_LOADED_H
#define _RECTREGION_LOADED_H

class GeoCoordinate;
class PixelBuffer;

class RectRegion
{
   friend class GeoCoordinate;
   private:
   int m_i_StartX;
   int m_i_StartY;
   int m_i_Width;
   int m_i_Height;
   int m_i_EndX;
   int m_i_EndY;

   // the same again for the float mode
   float m_f_StartX;
   float m_f_StartY;
   float m_f_Width;
   float m_f_Height;
   float m_f_EndX;
   float m_f_EndY;

   bool m_b_Empty;
   bool m_b_FloatMode;

   void AdjustEnd();
   void AdjustWidthHeight();
   
   void ConvertToFloat();

   public:
   // constructors
   RectRegion(int i_StartX,int i_StartX,int i_Width,int i_Height);
   RectRegion(float i_StartX,float i_StartX,float i_Width,float i_Height);
   RectRegion();
   RectRegion(PixelBuffer *pc_Image);

   void SetEmpty();
   void SetXYXY(float i_X1, float i_Y1,float i_X2,float i_Y2);
   void SetXYXY(int i_X1, int i_Y1,int i_X2,int i_Y2);
   void SetXYWH(float i_X , float i_Y ,float i_Width,float i_Height);
   void SetXYWH(int i_X , int i_Y ,int i_Width,int i_Height);
   void SetStartX(int i_StartX);
   void SetStartY(int i_StartY);
   void SetEndX(int i_EndX);
   void SetEndY(int i_EndY);
   void SetStartX(float f_StartX);
   void SetStartY(float f_StartY);
   void SetEndX(float f_EndX);
   void SetEndY(float f_EndY);
   void SetWidth(int i_Width);
   void SetWidth(float f_Width);
   void SetHeight(int i_Height);
   void SetHeight(float i_Height);
   
   void Set(PixelBuffer *pc_Image);

   int GetStartX();
   int GetStartY();
   int GetEndX();
   int GetEndY();
   int GetWidth();
   int GetHeight();
   float GetStartXFloat();
   float GetStartYFloat();
   float GetEndXFloat();
   float GetEndYFloat();
   float GetWidthFloat();
   float GetHeightFloat();
   bool IsEmpty();

   void MoveOffset(int i_XOffset, int i_YOffset);
   void MoveOffset(float i_XOffset, float i_YOffset);
   void ClipAt(RectRegion & ClipRegion);
   void Extend(int i_Left, int i_Top, int i_Right, int i_Bottom);

   bool PointInRegion(GeoCoordinate c_Point);
   bool PointInRegion(int i_X, int i_Y);
   bool PointInRegion(float f_X, float f_Y);
   bool ContainsRegion(RectRegion &InnerRegion);
   bool OverlapsRegion(RectRegion &OtherRegion);

   //Intersect();
   void AddRegion(RectRegion &OtherRegion);
   void AddPoint(int i_X, int i_Y);

   void printf();

} ; // RectRegion

#endif
