
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>

#include "RedEyeBox.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Utilities.h"

#include "QULabelSlider.h"
#include "Selection.h"
#include "Callback.h"
#include "RedEyeReduction.h"

#define REDEYEBOXLAYER "RedEyelayer"


RedEyeBox::RedEyeBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global),
                Tracer("RedEyeBox")
{
   QBoxLayout    *l_Layout;

   setCaption("Red Eye Reduction");
   TRACE << "Red Eye Reduction Started\n" ;

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);


   m_i_DarkeningValue   = 1;
   m_b_ObjectModeActive = false;
   m_b_InRedraw         = false;

   // create a new layer
   m_pc_ObjectLayer= m_pc_GlobalStorage->m_pc_ObjectMain->CreateLayer(REDEYEBOXLAYER);
   m_pc_ObjectLayer->SetVisible(false);


   // create tha active checkbox
   m_ActivateCheck = new QCheckBox("Active",m_w_TopWidget);
   l_Layout->addWidget(m_ActivateCheck);
   

   // create the region box with 3 buttons
   m_RegionBox = new QHGroupBox("Elliptic Region ",m_w_TopWidget);

   l_Layout->addWidget(m_RegionBox);

   m_AddButton       = new QPushButton("Add",m_RegionBox);
   m_DeleteButton    = new QPushButton("Delete",m_RegionBox);
   m_DeleteAllButton = new QPushButton("Delete All",m_RegionBox);

   m_DarkeningSlider = new QULabelSlider(m_w_TopWidget,
                           true,
                           0,5,
                           m_i_DarkeningValue,
                           "Darkening",6);
   l_Layout->addWidget(m_DarkeningSlider);
   
   
   // create the reduce button
   m_ReduceButton    = new QPushButton("Reduce",m_w_TopWidget);
   l_Layout->addWidget(m_ReduceButton);

   // connect all elements with slots
   connect( m_ActivateCheck,
            SIGNAL(toggled(bool)),
            SLOT( ActiveModeToggled(bool )));

   connect( m_AddButton,
            SIGNAL( clicked()),
            SLOT( AddButtonPressed( )));

   connect( m_DeleteButton,
            SIGNAL( clicked()),
            SLOT( DeleteButtonPressed( )));

   connect( m_DeleteAllButton,
            SIGNAL( clicked()),
            SLOT( DeleteAllButtonPressed( )));

   connect( m_DarkeningSlider,
            SIGNAL(valueChanged(int)),
            SLOT(DarkeningSliderMoved(int))
         );
         
   connect( m_ReduceButton,
            SIGNAL( clicked()),
            SLOT( ReduceButtonPressed( )));

   SetSensitivity();
   
} // RedEyeBox()

//===========================================================================
//
//===========================================================================


void RedEyeBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================

void RedEyeBox::GraphicObjectCallback(GraphicObjectMainEvent *pc_CallbackData)
{
   // now redisplay the size of the graphic object

   if(pc_CallbackData->i_CallbackType == GraphicObjectMainEvent::SelectionDisabled)
   {
      // disable the frame mode
      if( m_b_ObjectModeActive )
      {
         if(   m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer()
            == m_pc_ObjectLayer )
         {
            // reset the callback for graphic object changes
            m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));
            // disnable active action
            m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
         }
         m_pc_ObjectLayer->SetVisible(false);

         m_b_InRedraw = true;
         m_ActivateCheck->setChecked(false);
         m_b_InRedraw = false;
      }
   }
   else  // regular callback event
   {
   }
} // GraphicObjectCallback()

//===========================================================================
//
//===========================================================================

void *RedEyeBox::GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData)
{
   RedEyeBox *pc_This;
   pc_This = (RedEyeBox *) thisData;

   pc_This-> GraphicObjectCallback( (GraphicObjectMainEvent *)GraphicObjectCallbackData );

   return NULL;
} // GraphicObjectCallbackCB()

//===========================================================================
//
//===========================================================================

void RedEyeBox::DoActivate()
{
    if(m_b_ObjectModeActive )
    {
       return;
    }

    m_ActivateCheck->setChecked(true);
    
} // DoActivate()

//===========================================================================
//
//===========================================================================

void RedEyeBox::ActiveModeToggled(bool b_Mode)
{
   if(m_b_InRedraw)
   {
   }

   if(b_Mode == m_b_ObjectModeActive)
   {
      // no change
      return;
   }

   m_b_ObjectModeActive = b_Mode;

   if( m_b_ObjectModeActive)
   {
      // make the layer visible
      m_pc_ObjectLayer->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

      // set this layer as input layer
      m_pc_GlobalStorage->m_pc_ObjectMain->SetActiveLayer(REDEYEBOXLAYER);

      // enable active action
      m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput();

      // set the callback for graphic object changes
      m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(GraphicObjectCallbackCB,this));
   }
   else // not active
   {
      // make invisible
      m_pc_ObjectLayer->SetVisible(false);

      if(   m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer()
         == m_pc_ObjectLayer )
      {
         // remove the event callback
         m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));

         // disable input
         m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);

         // redraw the image
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
      }
   } // else not active
   
   SetSensitivity();
   
} // ActiveModeToggled()

//===========================================================================
//
//===========================================================================

void *RedEyeBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures

   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      GraphicObjectEllipse *pc_NewEllipse;
      int i_CenterX;
      int i_CenterY;
      int i_RadiusX;
      int i_RadiusY;
      
      if(m_pc_GlobalStorage == NULL)
      {
         return NULL;
      }

      // get coordianted from the selection
      m_pc_GlobalStorage->m_pc_Selection->GetSelectionEllipse(i_CenterX,
                                                              i_CenterY,
                                                              i_RadiusX,
                                                              i_RadiusY);

      // create a new object                                                              
      pc_NewEllipse = new GraphicObjectEllipse(i_CenterX-i_RadiusX,
                                               i_CenterY-i_RadiusY,
                                               i_CenterX+i_RadiusX,
                                               i_CenterY+i_RadiusY
                                               );

      // add the ellipse into the layer
      m_pc_ObjectLayer->AddObject(pc_NewEllipse);
      
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

      // enable the reduce button
      SetSensitivity();
   }
   return NULL;

}  // SelectionEventArrived()


void *RedEyeBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   RedEyeBox *pc_This = (RedEyeBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================

void RedEyeBox::AddButtonPressed()
{
   if(  m_pc_GlobalStorage != NULL)
   {
      m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionEllipse,
                                 "Create Red Eye Ellipse",
                                 Callback(SelectionEventArrivedCB,this)
                                 );

   }


} // AddButtonPressed()

//===========================================================================
//
//===========================================================================

void RedEyeBox::DeleteButtonPressed()
{
   GraphicObject *pc_Object;
   int i_Number;

   // get number of selected objects
   // and the first selected one
   i_Number = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);
   
   while(i_Number >0)
   {
      m_pc_ObjectLayer->DeleteObject(pc_Object);
      
      // get new number;
      i_Number = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);
   }
   
   
   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   SetSensitivity();

} // DeleteButtonPressed()

//===========================================================================
//
//===========================================================================

void RedEyeBox::DeleteAllButtonPressed()
{
   // delete all objects from the layer
   m_pc_ObjectLayer->DeleteAllObjects();

   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   SetSensitivity();

} // DeleteAllButtonPressed()

//===========================================================================
//
//===========================================================================

void RedEyeBox::SetSensitivity()
{
   if(m_b_ObjectModeActive)
   {
      m_RegionBox->setEnabled(true);

      if(m_pc_ObjectLayer->GetObjectCount()>0)
      {
         m_ReduceButton->setEnabled(true);
      }
      else
      {
         m_ReduceButton->setEnabled(false);
      }
   }
   else
   {
      m_RegionBox->setEnabled(false);
      m_ReduceButton->setEnabled(false);
   }
} // SetSensitivity()

//===========================================================================
//
//===========================================================================

void RedEyeBox::ReduceButtonPressed()
{
   RedEyeReduction c_Reduction;
   RectRegion      c_ChangedRegion;

   c_Reduction.SetLayer(m_pc_ObjectLayer);
   c_Reduction.SetImage(m_pc_GlobalStorage->m_pc_MainImage);
   c_Reduction.SetDarkeningFactor(m_i_DarkeningValue);

   c_Reduction.Reduce(c_ChangedRegion);

   if( ! c_ChangedRegion.IsEmpty() )
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   }
} // ReduceButtonPressed()

//===========================================================================
//
//===========================================================================

void RedEyeBox::DarkeningSliderMoved(int i_NewValue)
{
   m_i_DarkeningValue = i_NewValue;
} // DarkeningSliderMoved()

//===========================================================================
//
//===========================================================================

