#include <stdio.h>
#include <stdlib.h>

#include <qnamespace.h>

#include "Selection.h"
#include "GlobalStorage.h"
#include "MouseEvent.h"
#include "DrawingArea.h"
#include "Utilities.h"
#include "StatusBar.h"
#include "Zoom.h"
#include "ImageFrame.h"

//===========================================================================
//
//===========================================================================

SelectionCallbackData::SelectionCallbackData()
{
   m_cs_OperationString = "";
   i_CallbackReason     = ReasonNone;
} // SelectionCallbackData()

//===========================================================================
//
//===========================================================================


Selection::Selection(GlobalStorage *pc_Global)
		: Tracer("Selection")
{
   m_pc_Global = pc_Global;

   m_i_CurrentSelectionMode = SelectionNothing;
   m_b_InFreeDrawSelection  = false;

   // clean up the callback
   m_SelectionCallback.Clear();
   
   // initital no short selection
   m_b_ShortTimeSelectionActive = false;
   
   m_b_IgnoreUntilMouseRelease  = false;
   
   m_i_ButtonState =0;
} // Selection - constructor

//===========================================================================
//
//===========================================================================


Selection::~Selection()
{
} // ~Selection


//===========================================================================
//
//===========================================================================


void *Selection::MouseEventArrived(void *pc_Data)
{
   int i_MouseEventType ;

   int i_ActiveSelectionMode;

   TRACE << "Selection::MouseEventArrived\n";

   if(m_b_ShortTimeSelectionActive)
   {
      i_ActiveSelectionMode = m_i_CurrentShortTimeSelectionMode;
   }
   else
   {
      i_ActiveSelectionMode = m_i_CurrentSelectionMode;
   }

   // get event type
   i_MouseEventType = m_pc_Global->m_pc_MouseEvent->GetEventType( );
   
   // get state of the shift and control button
   m_i_ButtonState  = m_pc_Global->m_pc_MouseEvent->GetButtonState();

   if(m_b_IgnoreUntilMouseRelease)
   {
      if(i_MouseEventType == MouseEvent::EventEnd)
      {
         m_b_IgnoreUntilMouseRelease = false;
      }
      return NULL;
   }

   switch(i_ActiveSelectionMode)
   {
   case SelectionPoint:
      if(i_MouseEventType == MouseEvent::EventStart)
      {
         // this is the signal we waited for

         m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X1,
                                                              m_i_Y1);
         ExecuteRegularCallback();
      }
      break;
   case SelectionLine:
      DrawLine(i_MouseEventType);
      break;
   case SelectionRectangle:
      DrawRectangle(i_MouseEventType);
      break;
   case SelectionQuadrat:
      DrawQuadrat(i_MouseEventType);
      break;
   case SelectionStream:
      // stream everythig through
      m_i_CurrentMouseEventType = i_MouseEventType;
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X1,
                                                              m_i_Y1);
      ExecuteRegularCallback();

      break;
   case SelectionEllipse:
      DrawEllipse(i_MouseEventType);
      break;
   case SelectionCircle:
      DrawCircle(i_MouseEventType);
      break;
   case SelectionFreeDraw:
      DrawFreeDraw(i_MouseEventType);
      break;
   case SelectionNothing:
      break;
   default:
      break;
   } // switch mode


   return NULL;
} // MouseEventArrived


//===========================================================================
//
//===========================================================================


void *Selection::MouseEventArrivedCB(void *Thisdata,void *pc_Data)
{
   Selection *pc_This = (Selection *) Thisdata;
   return pc_This->MouseEventArrived(pc_Data);
}

//===========================================================================
//
//===========================================================================

void Selection::ActivateSelection(int i_SelectionType,
                                  QString qs_Operation,
                                  Callback SelectionCallback)
{
   TRACE << "Active Selection "
         << i_SelectionType
         << "\n";

   CleanUpShortTimeSelection();

   if( ! m_SelectionCallback.IsEmpty())
   {
      // tell the old callback that it is disabled
      SelectionCallbackData cb;
      cb.m_cs_OperationString = m_qs_Operation;
      cb.i_CallbackReason     = SelectionCallbackData::ReasonOperationDisabled;
      cb.i_SelectionType      = m_i_CurrentSelectionMode;
      m_SelectionCallback.Execute (&cb);
   }

   // disable a possible short time selection
   m_b_ShortTimeSelectionActive = false;

   m_SelectionCallback      = SelectionCallback;
   m_i_CurrentSelectionMode = i_SelectionType;
   m_qs_Operation           = qs_Operation;

   m_pc_Global->m_pc_MouseEvent->AddEventCallback(
                            Callback(MouseEventArrivedCB,this));

   if(    m_pc_Global != NULL
       && m_pc_Global->m_pc_StatusBar != NULL)
   {
      m_pc_Global->m_pc_StatusBar->SetOperationMode(m_qs_Operation);
   }

} // ActivateSelection()

//===========================================================================
// deactivate a selection, but only if type and callback are
// just installed
//===========================================================================

void Selection::DeactivateSelection(int i_SelectionType,
                                  QString qs_Operation,
                                  Callback SelectionCallback)
{
   if(m_i_CurrentSelectionMode == i_SelectionType
      &&  SelectionCallback ==   m_SelectionCallback)
   {
      m_SelectionCallback.Clear();
      m_i_CurrentSelectionMode = SelectionNothing;
      m_qs_Operation           = "";


      if(    m_pc_Global != NULL
       && m_pc_Global->m_pc_StatusBar != NULL)
      {
         m_pc_Global->m_pc_StatusBar->SetOperationMode(m_qs_Operation);
      }
      
   } // if it is the same
   
} // DeactivateSelection()

//===========================================================================
// deactivate every selection
//===========================================================================

void Selection::DeactivateSelection()
{
   m_SelectionCallback.Clear();
   m_i_CurrentSelectionMode = SelectionNothing;
   m_qs_Operation           = "";


   if(    m_pc_Global != NULL
       && m_pc_Global->m_pc_StatusBar != NULL)
   {
      m_pc_Global->m_pc_StatusBar->SetOperationMode(m_qs_Operation);
   }

} // DeactivateSelection()

//===========================================================================
//
//===========================================================================

void Selection::ActivateShortTimeSelection(int i_SelectionType,
                                  QString qs_Operation,
                                  Callback SelectionCallback)
{
   TRACE << "Active Short Time Selection\n";

   if( ! m_ShortTimeSelectionCallback.IsEmpty())
   {
      // tell the old callback that it is disabled
      SelectionCallbackData cb;
      cb.m_cs_OperationString = m_qs_Operation;
      cb.i_CallbackReason     = SelectionCallbackData::ReasonOperationDisabled;
      cb.i_SelectionType      = m_i_CurrentShortTimeSelectionMode;
      m_ShortTimeSelectionCallback.Execute (&cb);
   }


   m_ShortTimeSelectionCallback      = SelectionCallback;
   m_i_CurrentShortTimeSelectionMode = i_SelectionType;
   m_qs_ShortTimeOperation           = qs_Operation;

   m_pc_Global->m_pc_MouseEvent->AddEventCallback(
                            Callback(MouseEventArrivedCB,this));

   if(    m_pc_Global != NULL
       && m_pc_Global->m_pc_StatusBar != NULL)
   {
      m_pc_Global->m_pc_StatusBar->SetOperationMode(m_qs_ShortTimeOperation);
   }

   // enable short time selection
   m_b_ShortTimeSelectionActive = true;

} // ActivateShortTimeSelection()

//===========================================================================
//
//===========================================================================

void Selection::GetSelectionPoint( int &i_X, int &i_Y)
{
   i_X = m_i_X1;
   i_Y = m_i_Y1;

} // GetSelectionPoint

//===========================================================================
//
//===========================================================================

void Selection::GetSelectionLine(int &i_X1,int &i_Y1,int &i_X2,int &i_Y2)
{
   i_X1 = m_i_X1;
   i_Y1 = m_i_Y1;
   i_X2 = m_i_X2;
   i_Y2 = m_i_Y2;
   
} // GetSelectionLine


//===========================================================================
//
//===========================================================================

void Selection::GetSelectionRectangle(int &i_X1,int &i_Y1,int &i_X2,int &i_Y2)
{
   i_X1 = m_i_X1;
   i_Y1 = m_i_Y1;
   i_X2 = m_i_X2;
   i_Y2 = m_i_Y2;

} // GetSelectionRectangle

//===========================================================================
//
//===========================================================================

void Selection::GetSelectionQuadrat(int &i_X1,int &i_Y1,int &i_X2,int &i_Y2)
{
   int i_EndX;
   int i_EndY;
   CalculateQuadratCoordinates(m_i_X1,m_i_Y1,
                               m_i_X2,m_i_Y2,
                               i_EndX,i_EndY);
   i_X1 = m_i_X1;
   i_Y1 = m_i_Y1;
   i_X2 = i_EndX;
   i_Y2 = i_EndY;

} // GetSelectionQuadrat

//===========================================================================
//
//===========================================================================

void Selection::GetSelectionEllipse(int &i_CenterX,int &i_CenterY,
                                    int &i_RadiusX,int &i_RadiusY)
{
   i_CenterX = m_i_X1;
   i_CenterY = m_i_Y1;
   i_RadiusX = abs(m_i_X1 - m_i_X2);
   i_RadiusY = abs(m_i_Y1 - m_i_Y2);

} // GetSelectionEllipse


//===========================================================================
//
//===========================================================================

void Selection::GetSelectionCircle(int &i_CenterX,int &i_CenterY,
                                    int &i_Radius)
{
   i_CenterX = m_i_X1;
   i_CenterY = m_i_Y1;
   i_Radius  = (int) Utilities:: Hypothenusis(m_i_X1 - m_i_X2,m_i_Y1 - m_i_Y2);

} // GetSelectionCircle

//===========================================================================
//
//===========================================================================

int Selection::GetMouseEventType()
{
   return m_i_CurrentMouseEventType;
} // GetMouseEventType()

//===========================================================================
// subroutine, to draw the selection elipse on the screen
//===========================================================================

void Selection::DrawEllipse(int i_MouseEventType)
{
   QRect q_Ellipse;
   // do different things on the event types
   if(i_MouseEventType == MouseEvent::EventStart)
   {
      // this is the signal we waited for
      // put the elipse Center into the first point

      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X1,
                                                           m_i_Y1);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX1,
                                                              m_i_WindowY1);
      m_b_LastLinePointUsed = false;
   }
   else if(i_MouseEventType == MouseEvent::EventDo)
   {

      QPainter painter(m_pc_Global->m_pc_DrawingArea);
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);

      // in *1 is the center, in *2 is the current point

      if(m_b_LastLinePointUsed)
      {
         // delete the last line

         q_Ellipse = CalculateEllipseCoordinates(m_i_WindowX1,m_i_WindowY1,
                                                m_i_WindowX2,m_i_WindowY2);

         painter.drawEllipse(q_Ellipse);

      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                           m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      // draw a new line
      q_Ellipse = CalculateEllipseCoordinates(m_i_WindowX1,m_i_WindowY1,
                                                m_i_WindowX2,m_i_WindowY2);

      painter.drawEllipse(q_Ellipse);
      m_b_LastLinePointUsed = true;
   }
   else if(i_MouseEventType == MouseEvent::EventEnd)
   {
      QPainter painter(m_pc_Global->m_pc_DrawingArea);
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);

      if(m_b_LastLinePointUsed)
      {
         // delete the last ellipse
         q_Ellipse = CalculateEllipseCoordinates(m_i_WindowX1,m_i_WindowY1,
                                                m_i_WindowX2,m_i_WindowY2);

         painter.drawEllipse(q_Ellipse);

      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                             m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      ExecuteRegularCallback();
   }
} // DrawElipse()

//===========================================================================
//
//===========================================================================

QRect Selection::CalculateEllipseCoordinates(int i_CenterX, int i_CenterY,
                                            int i_X2, int i_Y2)
{
   QRect q_ReturnRect;
   int i_DiffX;
   int i_DiffY;

   i_DiffX = i_CenterX - i_X2;
   i_DiffY = i_CenterY - i_Y2;

   // make differnce positive
   if(i_DiffX <0)
   {
      i_DiffX = - i_DiffX;
   }

   if(i_DiffY <0)
   {
      i_DiffY = - i_DiffY;
   }

   q_ReturnRect.setCoords(i_CenterX-i_DiffX,
                          i_CenterY-i_DiffY,
                          i_CenterX+i_DiffX,
                          i_CenterY+i_DiffY
                          );

   return q_ReturnRect;
} // CalculateElipseCoordinates()


//===========================================================================
// subroutine, to draw the selection circle on the screen
//===========================================================================

void Selection::DrawCircle(int i_MouseEventType)
{
   QRect q_Ellipse;
   // do different things on the event types
   if(i_MouseEventType == MouseEvent::EventStart)
   {
      // this is the signal we waited for
      // put the elipse Center into the first point

      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X1,
                                                           m_i_Y1);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX1,
                                                              m_i_WindowY1);
      m_b_LastLinePointUsed = false;
   }
   else if(i_MouseEventType == MouseEvent::EventDo)
   {

      QPainter painter(m_pc_Global->m_pc_DrawingArea);
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);

      // in *1 is the center, in *2 is the current point

      if(m_b_LastLinePointUsed)
      {
         // delete the last line

         q_Ellipse = CalculateCircleCoordinates(m_i_WindowX1,m_i_WindowY1,
                                                m_i_WindowX2,m_i_WindowY2);

         painter.drawEllipse(q_Ellipse);

      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                           m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      // draw a new line
      q_Ellipse = CalculateCircleCoordinates(m_i_WindowX1,m_i_WindowY1,
                                                m_i_WindowX2,m_i_WindowY2);

      painter.drawEllipse(q_Ellipse);
      m_b_LastLinePointUsed = true;
   }
   else if(i_MouseEventType == MouseEvent::EventEnd)
   {
      QPainter painter(m_pc_Global->m_pc_DrawingArea);
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);

      if(m_b_LastLinePointUsed)
      {
         // delete the last ellipse
         q_Ellipse = CalculateCircleCoordinates(m_i_WindowX1,m_i_WindowY1,
                                                m_i_WindowX2,m_i_WindowY2);

         painter.drawEllipse(q_Ellipse);

      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                             m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      ExecuteRegularCallback();
   }
} // DrawCircle()

//===========================================================================
//
//===========================================================================

QRect Selection::CalculateCircleCoordinates(int i_CenterX, int i_CenterY,
                                            int i_X2, int i_Y2)
{
   QRect q_ReturnRect;
   int i_DiffX;
   int i_DiffY;

   int i_Radius ;


   i_DiffX = i_CenterX - i_X2;
   i_DiffY = i_CenterY - i_Y2;

   i_Radius = (int) Utilities:: Hypothenusis(i_DiffX,i_DiffY);



   q_ReturnRect.setCoords(i_CenterX-i_Radius,
                          i_CenterY-i_Radius,
                          i_CenterX+i_Radius,
                          i_CenterY+i_Radius
                          );

   return q_ReturnRect;
} // CalculateCircleCoordinates()

//===========================================================================
// subroutine, to draw the selection circle on the screen
//===========================================================================

void Selection::DrawRectangle(int i_MouseEventType)
{
   if(i_MouseEventType == MouseEvent::EventStart)
   {
      // this is the signal we waited for

      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X1,
                                                              m_i_Y1);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX1,
                                                              m_i_WindowY1);
      m_b_LastLinePointUsed = false;
   }
   else if(i_MouseEventType == MouseEvent::EventDo)
   {
      QPainter painter(m_pc_Global->m_pc_DrawingArea);
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);
      if(m_b_LastLinePointUsed)
      {
         // delete the last line
         painter.moveTo(m_i_WindowX1,m_i_WindowY1);
         painter.lineTo(m_i_WindowX1,m_i_WindowY2);
         painter.lineTo(m_i_WindowX2,m_i_WindowY2);
         painter.lineTo(m_i_WindowX2,m_i_WindowY1);
         painter.lineTo(m_i_WindowX1,m_i_WindowY1);
      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                              m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      // draw a new line
      painter.moveTo(m_i_WindowX1,m_i_WindowY1);
      painter.lineTo(m_i_WindowX1,m_i_WindowY2);
      painter.lineTo(m_i_WindowX2,m_i_WindowY2);
      painter.lineTo(m_i_WindowX2,m_i_WindowY1);
      painter.lineTo(m_i_WindowX1,m_i_WindowY1);
      m_b_LastLinePointUsed = true;
   }
   else if(i_MouseEventType == MouseEvent::EventEnd)
   {
      QPainter painter(m_pc_Global->m_pc_DrawingArea);
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);

      if(m_b_LastLinePointUsed)
      {
         // delete the last line
         painter.moveTo(m_i_WindowX1,m_i_WindowY1);
         painter.lineTo(m_i_WindowX1,m_i_WindowY2);
         painter.lineTo(m_i_WindowX2,m_i_WindowY2);
         painter.lineTo(m_i_WindowX2,m_i_WindowY1);
         painter.lineTo(m_i_WindowX1,m_i_WindowY1);

      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                              m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      ExecuteRegularCallback();
   }

} // DrawRectangle()

//===========================================================================
//
//===========================================================================

void Selection::DrawQuadrat(int i_MouseEventType)
{
   int i_EndX;
   int i_EndY;
   if(i_MouseEventType == MouseEvent::EventStart)
   {
      // this is the signal we waited for

      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X1,
                                                              m_i_Y1);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX1,
                                                              m_i_WindowY1);
      m_b_LastLinePointUsed = false;
   }
   else if(i_MouseEventType == MouseEvent::EventDo)
   {
      QPainter painter(m_pc_Global->m_pc_DrawingArea);
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);
      if(m_b_LastLinePointUsed)
      {
         // delete the last line
         CalculateQuadratCoordinates(m_i_WindowX1,m_i_WindowY1,
                                     m_i_WindowX2,m_i_WindowY2,
                                     i_EndX,i_EndY);
         painter.moveTo(m_i_WindowX1,m_i_WindowY1);
         painter.lineTo(m_i_WindowX1,i_EndY);
         painter.lineTo(i_EndX,i_EndY);
         painter.lineTo(i_EndX,m_i_WindowY1);
         painter.lineTo(m_i_WindowX1,m_i_WindowY1);
      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                              m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      // draw a new line
         CalculateQuadratCoordinates(m_i_WindowX1,m_i_WindowY1,
                                     m_i_WindowX2,m_i_WindowY2,
                                     i_EndX,i_EndY);
      painter.moveTo(m_i_WindowX1,m_i_WindowY1);
      painter.lineTo(m_i_WindowX1,i_EndY);
      painter.lineTo(i_EndX,i_EndY);
      painter.lineTo(i_EndX,m_i_WindowY1);
      painter.lineTo(m_i_WindowX1,m_i_WindowY1);
      m_b_LastLinePointUsed = true;
   }
   else if(i_MouseEventType == MouseEvent::EventEnd)
   {
      QPainter painter(m_pc_Global->m_pc_DrawingArea);
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);

      if(m_b_LastLinePointUsed)
      {
         // delete the last line
         CalculateQuadratCoordinates(m_i_WindowX1,m_i_WindowY1,
                                     m_i_WindowX2,m_i_WindowY2,
                                     i_EndX,i_EndY);
         painter.moveTo(m_i_WindowX1,m_i_WindowY1);
         painter.lineTo(m_i_WindowX1,i_EndY);
         painter.lineTo(i_EndX,i_EndY);
         painter.lineTo(i_EndX,m_i_WindowY1);
         painter.lineTo(m_i_WindowX1,m_i_WindowY1);

      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                              m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      ExecuteRegularCallback();
   }

} // DrawQuadrat()

//===========================================================================
//
//===========================================================================

void Selection::CalculateQuadratCoordinates(int i_StartX, int i_StartY,
                                    int i_X2, int i_Y2,
                                    int &i_EndX,int &i_EndY)
{
   bool b_NegativX;
   bool b_NegativY;
   int  i_DiffX;
   int  i_DiffY;

   i_DiffX    = i_X2 - i_StartX;
   i_DiffY    = i_Y2 - i_StartY;

   b_NegativX = false;
   b_NegativY = false;

   if(i_DiffX <0)
   {
      b_NegativX = true;
      i_DiffX    = - i_DiffX;
   }
   if(i_DiffY <0)
   {
      b_NegativY = true;
      i_DiffY    = - i_DiffY;
   }

   // put the larger value into diffx
   if(i_DiffX < i_DiffY)
   {
      i_DiffX = i_DiffY;
   }

   // calculate the resulting values
   if(b_NegativX)
   {
      i_EndX = i_StartX - i_DiffX;
   }
   else
   {
      i_EndX = i_StartX + i_DiffX;
   }
   if(b_NegativY)
   {
      i_EndY = i_StartY - i_DiffX;
   }
   else
   {
      i_EndY = i_StartY + i_DiffX;
   }


} // CalculateQuadratCoordinates()

//===========================================================================
//
//===========================================================================

void Selection::DrawFreeDraw(int i_MouseEventType)
{
   PixelCoordinate c_ActualPosition;
   PixelCoordinate c_LastPosition;
   int             i_Size;

   // only do something a the press down point
   if(i_MouseEventType == MouseEvent::EventStart)
   {
      // get coordinate of button down
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(c_ActualPosition.m_i_X,
                                                             c_ActualPosition.m_i_Y);

      TRACE << "Draw free" << c_ActualPosition.m_i_X << c_ActualPosition.m_i_Y ;
      // first point, initialize the list
      m_CoordinateList.clear();
      m_b_LastLinePointUsed =false;

      // add point to the list
      m_CoordinateList.push_back(c_ActualPosition);

      m_b_InFreeDrawSelection = true;

   }
   else if(i_MouseEventType == MouseEvent::EventDo)
   {

      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(c_ActualPosition.m_i_X,
                                                             c_ActualPosition.m_i_Y);

      // get the last coordiante
      i_Size = m_CoordinateList.size();
      if(i_Size >= 1)
      {
         c_LastPosition = m_CoordinateList.at(i_Size-1);

         TRACE <<  c_LastPosition.m_i_X 
               <<  c_LastPosition.m_i_Y
               <<  " -- > "
               <<  c_ActualPosition.m_i_X
               <<  c_ActualPosition.m_i_Y
               << "\n";
         
         if(   c_LastPosition.m_i_X != c_ActualPosition.m_i_X
            || c_LastPosition.m_i_Y != c_ActualPosition.m_i_Y
            )
         {

            QPainter painter(m_pc_Global->m_pc_DrawingArea);
            painter.setPen(QColor(0xFFFFFF));
            painter.setRasterOp(Qt::XorROP);

            // draw endpoint double, to have no empty spaces
            painter.drawPoint(c_LastPosition.m_i_X  ,c_LastPosition.m_i_Y);

            painter.moveTo(c_ActualPosition.m_i_X,c_ActualPosition.m_i_Y);
            painter.lineTo(c_LastPosition.m_i_X  ,c_LastPosition.m_i_Y);

            // add the point to the list
            m_CoordinateList.push_back(c_ActualPosition);
         }

      } // if previous points available

   }
   else if(i_MouseEventType == MouseEvent::EventEnd)
   {
      // also add the last coordinate
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(c_ActualPosition.m_i_X,
                                                             c_ActualPosition.m_i_Y);

      // get the last coordiante
      i_Size = m_CoordinateList.size();
      if(i_Size >= 1)
      {
         c_LastPosition = m_CoordinateList.at(i_Size-1);
         if(   c_LastPosition.m_i_X != c_ActualPosition.m_i_X
            || c_LastPosition.m_i_Y != c_ActualPosition.m_i_Y
            )
         {

            QPainter painter(m_pc_Global->m_pc_DrawingArea);
            painter.setPen(QColor(0xFFFFFF));
            painter.setRasterOp(Qt::XorROP);

            painter.moveTo(c_ActualPosition.m_i_X,c_ActualPosition.m_i_Y);
            painter.lineTo(c_LastPosition.m_i_X  ,c_LastPosition.m_i_Y);

            // add the point to the list
            m_CoordinateList.push_back(c_ActualPosition);
         }

         
      }
      // now clean up the selection area
      i_Size = m_CoordinateList.size();
      if(i_Size >= 1)
      {
         PixelCoordinate c_CurrentPosition;
         int i_Number;

         QPainter painter(m_pc_Global->m_pc_DrawingArea);
         painter.setPen(QColor(0xFFFFFF));
         painter.setRasterOp(Qt::XorROP);

         // draw again all lines
         c_LastPosition = m_CoordinateList.at(0);
         for(i_Number =0; i_Number < (i_Size-1) ; i_Number++)
         {
            c_CurrentPosition = m_CoordinateList.at(i_Number +1);

            // draw endpoint double, to have no empty spaces
            painter.drawPoint(c_LastPosition.m_i_X  ,c_LastPosition.m_i_Y);

            painter.moveTo(c_CurrentPosition.m_i_X,
                           c_CurrentPosition.m_i_Y);
            painter.lineTo(c_LastPosition.m_i_X,
                           c_LastPosition.m_i_Y);
            c_LastPosition = c_CurrentPosition;
            
            
         } // for i_Number
      } // if size >=1

      // now convert the list from window Coordinates into buffer coordinates
      ConvertListIntoBufferCoordinates();
      // additionaly call the selection ready callback
      ExecuteRegularCallback();
   }
} // DrawPolygon

//===========================================================================
//
//===========================================================================

void Selection::ConvertListIntoBufferCoordinates()
{
   Zoom * pc_Zoom;
   int i_Size;
   int i;
   int i_DiffX;
   int i_DiffY;

   PixelCoordinate c_ActualWindowPosition;
   PixelCoordinate c_ActualBufferPosition;
   PixelCoordinate c_LastBufferPosition;


   pc_Zoom = m_pc_Global->m_pc_Zoom;
   // clean up the buffer list
   m_BufferCoordinateList.clear();

   // get the size of the window list
   i_Size = m_CoordinateList.size();

   // empty list, then stop
   if(   i_Size ==0
      || pc_Zoom == NULL)
   {
      return;
   }

   // initialize the differences with 0
   i_DiffX=0;
   i_DiffY=0;

   // do the first point ever
   c_ActualWindowPosition = m_CoordinateList.at(0);

   pc_Zoom->GetBufferCoordinate(c_ActualWindowPosition.m_i_X,
                                  c_ActualWindowPosition.m_i_Y,
                                  c_ActualBufferPosition.m_i_X,
                                  c_ActualBufferPosition.m_i_Y);
   //

   m_BufferCoordinateList.push_back(c_ActualBufferPosition);
   c_LastBufferPosition = c_ActualBufferPosition;
   for(i=1; i< i_Size; i++)
   {
      c_ActualWindowPosition = m_CoordinateList.at(i);
      pc_Zoom->GetBufferCoordinate(c_ActualWindowPosition.m_i_X,
                                  c_ActualWindowPosition.m_i_Y,
                                  c_ActualBufferPosition.m_i_X,
                                  c_ActualBufferPosition.m_i_Y);
      // same point ?
      if(   c_ActualBufferPosition.m_i_X != c_LastBufferPosition.m_i_X
         || c_ActualBufferPosition.m_i_Y != c_LastBufferPosition.m_i_Y
         )
      {
         // not the same, store it
         int i_NewDiffX;
         int i_NewDiffY;
         // calculate new line increment
         i_NewDiffX = c_ActualBufferPosition.m_i_X - c_LastBufferPosition.m_i_X;
         i_NewDiffY = c_ActualBufferPosition.m_i_Y - c_LastBufferPosition.m_i_Y;
         if(    i_NewDiffX == i_DiffX
            &&  i_NewDiffY == i_DiffY
            )
         {
            // same line increment like the last time
            // replace the last store point by this
            m_BufferCoordinateList.pop_back();
            m_BufferCoordinateList.push_back(c_ActualBufferPosition);

         }
         else // different line increment
         {
            m_BufferCoordinateList.push_back(c_ActualBufferPosition);
         }

         c_LastBufferPosition = c_ActualBufferPosition;
         i_DiffX = i_NewDiffX;
         i_DiffY = i_NewDiffY;
      } // if not same point

   } // for i


} // ConvertListIntoBufferCoordinates()

//===========================================================================
//
//===========================================================================

QValueVector<PixelCoordinate> *Selection::GetSelectionPointList()
{
   return &m_BufferCoordinateList;
} // GetSelectionPointList()

//===========================================================================
//
//===========================================================================

void Selection::ExecuteRegularCallback()
{
   if( m_b_ShortTimeSelectionActive)
   {

      TRACE << "Selection goes to short time selection\n";

      // Set back zo the original operation mode
      if(    m_pc_Global != NULL
          && m_pc_Global->m_pc_StatusBar != NULL)
      {
         if( ! m_SelectionCallback.IsEmpty())
         {
            // re-display original string
            m_pc_Global->m_pc_StatusBar->SetOperationMode(m_qs_Operation);
         }
         else
         {
            // clean up status string
            m_pc_Global->m_pc_StatusBar->SetOperationMode("");
         }

      }
      if( ! m_ShortTimeSelectionCallback.IsEmpty())
      {
         // tell the old callback that it is disabled
         SelectionCallbackData cb;
         cb.m_cs_OperationString = m_qs_ShortTimeOperation;
         cb.i_CallbackReason     = SelectionCallbackData::ReasonSelection;
         cb.i_SelectionType      = m_i_CurrentShortTimeSelectionMode;
         m_ShortTimeSelectionCallback.Execute (&cb);
      }

      CleanUpShortTimeSelection();

      return;
   }

   if( ! m_SelectionCallback.IsEmpty())
   {
      // tell the old callback that it is disabled
      SelectionCallbackData cb;
      cb.m_cs_OperationString = m_qs_Operation;
      cb.i_CallbackReason     = SelectionCallbackData::ReasonSelection;
      cb.i_SelectionType      = m_i_CurrentSelectionMode;
      m_SelectionCallback.Execute (&cb);
   }
} // ExecuteRegularCallback()

//===========================================================================
//
//===========================================================================

void Selection::CleanUpShortTimeSelection()
{
   if( ! m_ShortTimeSelectionCallback.IsEmpty())
   {
      int i_MouseEventType;
      // tell the old callback that it is disabled
      SelectionCallbackData cb;

      TRACE << "Clean up short time selection\n";

      cb.m_cs_OperationString = m_qs_ShortTimeOperation;
      cb.i_CallbackReason     = SelectionCallbackData::ReasonOperationDisabled;
      cb.i_SelectionType      = m_i_CurrentShortTimeSelectionMode;
      m_ShortTimeSelectionCallback.Execute (&cb);
      m_ShortTimeSelectionCallback.Clear();

      i_MouseEventType = m_pc_Global->m_pc_MouseEvent->GetEventType( );
      if(i_MouseEventType != MouseEvent::EventEnd)
      {
         //disable events until event end has arrived
         m_b_IgnoreUntilMouseRelease = true;
      }
   }

   m_b_ShortTimeSelectionActive = false;
} // CleanUpShortTimeSelection()

//===========================================================================
//
//===========================================================================

int Selection::GetButtonState()
{
   return m_i_ButtonState;
} // GetButtnoState()

//===========================================================================
//
//===========================================================================

void Selection::DrawLine(int i_MouseEventType)
{
   if(i_MouseEventType == MouseEvent::EventStart)
   {
      // this is the signal we waited for

      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X1,
                                                              m_i_Y1);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX1,
                                                              m_i_WindowY1);
      m_b_LastLinePointUsed = false;
   }
   else if(i_MouseEventType == MouseEvent::EventDo)
   {
      QPainter painter(m_pc_Global->m_pc_DrawingArea);

      // draw with full xor mode
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);

      if(m_b_LastLinePointUsed)
      {
         // delete the last line
         painter.moveTo(m_i_WindowX1,m_i_WindowY1);
         painter.lineTo(m_i_WindowX2,m_i_WindowY2);

      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                              m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      // draw a new line
      painter.moveTo(m_i_WindowX1,m_i_WindowY1);
      painter.lineTo(m_i_WindowX2,m_i_WindowY2);
      m_b_LastLinePointUsed = true;
   }
   else if(i_MouseEventType == MouseEvent::EventEnd)
   {
      QPainter painter(m_pc_Global->m_pc_DrawingArea);
      painter.setPen(QColor(0xFFFFFF));
      painter.setRasterOp(Qt::XorROP);

      if(m_b_LastLinePointUsed)
      {
         // delete the last line
         painter.moveTo(m_i_WindowX1,m_i_WindowY1);
         painter.lineTo(m_i_WindowX2,m_i_WindowY2);
      }
      m_pc_Global->m_pc_MouseEvent->GetLastBufferCoordinates(m_i_X2,
                                                              m_i_Y2);
      m_pc_Global->m_pc_MouseEvent->GetLastWindowCoordinates(m_i_WindowX2,
                                                              m_i_WindowY2);

      ExecuteRegularCallback();
   } // end

} // DrawLine()

//===========================================================================
//
//===========================================================================


