#ifndef _SELECTION_H
#define _SELECTION_H

#include <qvaluevector.h>
#include <qrect.h>

#include "PixelBuffer.h"
#include "GlobalStorage.h"
#include "Callback.h"
#include "Tracer.h"


class SelectionCallbackData
{
public:
   SelectionCallbackData();
   QString m_cs_OperationString;
   enum
   {
      ReasonNone,
      ReasonSelection,
      ReasonOperationDisabled

   };
   int i_CallbackReason;
   int i_SelectionType;

};


class Selection : public Tracer
{

public:
   Selection(GlobalStorage *pc_Global);
   ~Selection();

   enum {
         SelectionNothing,
         SelectionPoint,
         SelectionRectangle,
         SelectionLine,
         SelectionStream,
         SelectionEllipse,
         SelectionCircle,
         SelectionQuadrat,
         SelectionFreeDraw
   };


   void ActivateSelection(int i_SelectionType,
                          QString qs_Operation,
                          Callback SelectionCallback);
   void DeactivateSelection(int i_SelectionType,
                          QString qs_Operation,
                          Callback SelectionCallback);
   void DeactivateSelection();


   void ActivateShortTimeSelection(int i_SelectionType,
                          QString qs_Operation,
                          Callback SelectionCallback);

   void GetSelectionPoint( int &i_X, int &i_Y);
   void GetSelectionLine(int &i_X1,int &i_Y1,int &i_X2,int &i_Y2);
   void GetSelectionRectangle(int &i_X1,int &i_Y1,int &i_X2,int &i_Y2);
   void GetSelectionQuadrat(int &i_X1,int &i_Y1,int &i_X2,int &i_Y2);
   void GetSelectionEllipse(int &i_CenterX,int &i_CenterY,
                                    int &i_RadiusX,int &i_RadiusY);

   void GetSelectionCircle(int &i_CenterX,int &i_CenterY,
                                    int &i_Radius);
   QValueVector<PixelCoordinate> *GetSelectionPointList();
   int GetMouseEventType();
   int GetButtonState();

private: // functions

   void *MouseEventArrived(void *pc_Data);

   static void *MouseEventArrivedCB(void *Thisdata,void *pc_Data);

   void DrawLine(int i_MouseEventType);
   void DrawEllipse(int i_MouseEventType);
   void DrawCircle(int i_MouseEventType);
   void DrawRectangle(int i_MouseEventType);
   void DrawQuadrat(int i_MouseEventType);
   void DrawFreeDraw(int i_MouseEventType);

   QRect CalculateEllipseCoordinates(int i_CenterX, int i_CenterY,
                                     int i_X2, int i_Y2);
   QRect CalculateCircleCoordinates(int i_CenterX, int i_CenterY,
                                    int i_X2, int i_Y2);
   void CalculateQuadratCoordinates(int i_StartX, int i_StartY,
                                    int i_X2, int i_Y2,
                                    int &i_EndX,int &i_EndY);
   void ConvertListIntoBufferCoordinates();
   void ExecuteRegularCallback();

   void CleanUpShortTimeSelection();
private: // variables

   bool m_b_ShortTimeSelectionActive;

   Callback m_SelectionCallback;
   QString  m_qs_Operation;
   int m_i_CurrentSelectionMode;

   Callback m_ShortTimeSelectionCallback;
   QString  m_qs_ShortTimeOperation;
   int m_i_CurrentShortTimeSelectionMode;

   GlobalStorage *m_pc_Global;


   int m_i_X1;
   int m_i_Y1;
   int m_i_X2;
   int m_i_Y2;

   int m_b_LastLinePointUsed;
   int m_i_WindowX1;
   int m_i_WindowY1;
   int m_i_WindowX2;
   int m_i_WindowY2;
   int m_i_CurrentMouseEventType;
   int m_i_ButtonState;

   QValueVector<PixelCoordinate> m_CoordinateList;
   QValueVector<PixelCoordinate> m_BufferCoordinateList;
   bool m_b_InFreeDrawSelection;
   bool m_b_IgnoreUntilMouseRelease;


} ; // class selection


#endif // _SELECTION_H
