
#include <stdio.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>

#include "TestObjectBox.h"
#include "ImageFrame.h"
#include "QUCorrectionCurve.h"
#include "ApplicationFrame.h"
#include "IconLibrary.h"
#include "QUImageDragLabel.h"
#include "QUColorOrPattern.h"

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

TestObjectBox::TestObjectBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               StandardDialog(parent,pc_Global)
{
   QBoxLayout    *l_Layout;

   setCaption("TestObjectBox");


   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);


   m_ActivateButton     = new QPushButton("Enable All",m_w_TopWidget);
   l_Layout->addWidget(m_ActivateButton);
   
   {
      QUCorrectionCurve *pc_Correction;
      pc_Correction = new QUCorrectionCurve(m_w_TopWidget,
                                            pc_Global,
                                            QUCorrectionCurve::OptionSmall);
      l_Layout->addWidget(pc_Correction);
   }

   {
      QUColorOrPattern *pc_Pinsel;
      pc_Pinsel = new QUColorOrPattern(m_w_TopWidget,
                                            pc_Global);
      l_Layout->addWidget(pc_Pinsel );
                                            
   }
   {

      PixelBuffer *pc_ImageBuffer;
      m_DragLabel = new QUImageDragLabel(m_w_TopWidget,
                                      QUImageDragLabel::ModeUseWidthHeight,
                                      pc_Global,
                                      50,
                                      50);
                                      
      l_Layout->addWidget(m_DragLabel);
      pc_ImageBuffer = g_Application->m_pc_IconLibrary->GetIcon("Perico");
      if(pc_ImageBuffer != NULL)
      {
         m_DragLabel->SetPixelBuffer(pc_ImageBuffer);
      }
   }
   
   connect( m_ActivateButton,
            SIGNAL(clicked())  ,
            SLOT(Activate()) );

   CreateTestObjects();

} // GraphikBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void TestObjectBox::Activate()
{

   m_pc_ActiveLayer->SetVisible(true);
   m_pc_ActiveLayer = m_pc_GlobalStorage->m_pc_ObjectMain->SetActiveLayer("Test");
   m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput();

   m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(true);
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
} // Activate()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void TestObjectBox::CreateTestObjects()
{
  GraphicObjectPoint      *pc_Point;
  GraphicObjectLine       *pc_Line;
  GraphicObjectRectangle  *pc_Rectangle;
  GraphicObjectEllipse    *pc_Ellipse;
  GraphicObjectImage      *pc_ImageObject;
  PixelBuffer             *pc_Image;

   // set the object Layer visible
   m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(false);

   m_pc_ActiveLayer = m_pc_GlobalStorage->m_pc_ObjectMain->CreateLayer("Test");

   if(m_pc_ActiveLayer != NULL)
   {
      pc_Point = new GraphicObjectPoint (100,100);
      pc_Point->SetColor(0x000000);
      m_pc_ActiveLayer->AddObject(pc_Point);

      pc_Line = new GraphicObjectLine(10,33,60,79);
      pc_Line->SetColor(0x000000);
      m_pc_ActiveLayer->AddObject(pc_Line);

      pc_Line = new GraphicObjectLine(110,33,68,79);
      pc_Line->SetColor(0x0000f0);
      m_pc_ActiveLayer->AddObject(pc_Line);
      pc_Line = new GraphicObjectLine(55,33,12,71);
      pc_Line->SetColor(0x00f000);
      m_pc_ActiveLayer->AddObject(pc_Line);
      pc_Line = new GraphicObjectLine(87,44,55,53);
      pc_Line->SetColor(0xf00000);
      m_pc_ActiveLayer->AddObject(pc_Line);

      pc_Rectangle = new GraphicObjectRectangle(101,102,140,155);
      pc_Rectangle->SetColor(0xf0f000);
      m_pc_ActiveLayer->AddObject(pc_Rectangle);

      pc_Ellipse = new GraphicObjectEllipse(140,160,190,202);
      pc_Ellipse->SetColor(0xf0f060);
      m_pc_ActiveLayer->AddObject(pc_Ellipse);

      pc_Image = g_Application->m_pc_IconLibrary->GetIcon("Perico");
      {
         PixelBuffer *pc_SelectionMask;
         pc_Image->AddSelectionMask();
         pc_SelectionMask = pc_Image->GetSelectionMask();
         pc_SelectionMask->Clean(1);
         pc_Image->SetSelectionMaskActive(true);
      }
      pc_ImageObject = new GraphicObjectImage(12,130,pc_Image);
      m_pc_ActiveLayer->AddObject(pc_ImageObject);
            
   }
   m_pc_ActiveLayer->SetVisible(false);

} // CreateTestObjects()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
