#include <stdio.h>
#include <qvaluevector.h>

#include "Tracer.h"

QValueVector<TracerEntry> g_TraceBase;

//============================================================================
//
//============================================================================

TracerEntry::TracerEntry()
{
   m_qs_Name        = "";
   m_i_Instances    = 1;
   m_b_TraceEnabled = false;
}

//============================================================================
//
//============================================================================

TracerEntry::~TracerEntry()
{
}

//============================================================================
//
//============================================================================

void TracerEntry::TraceEnable(QString qs_Name,bool b_State)
{
   int i;
   int i_Size;

   // look for the element
   i_Size = g_TraceBase.size();
   for(i=0;i<i_Size;i++)
   {
      TracerEntry c_Entry;

      c_Entry = g_TraceBase.at(i);
      if(qs_Name.compare(c_Entry.m_qs_Name) == 0)
      {
         c_Entry.m_b_TraceEnabled = b_State;
         return;
      }
   }

} // TraceEnable()

//============================================================================
//
//============================================================================

Tracer::Tracer(QString qs_Name)
{
   int i;
   int i_Size;
   int i_InsertIndex;


   m_i_TracerIndex = -1;
   m_qs_Name       = qs_Name;

   // look, whether in the tracerbase this name exists
   i_Size = g_TraceBase.size();
   i_InsertIndex = -1;
   for(i=0;i<i_Size;i++)
   {
      TracerEntry c_Entry;
      int         i_CompareResult;

      c_Entry = g_TraceBase.at(i);
      i_CompareResult = m_qs_Name.compare(c_Entry.m_qs_Name);
      if(i_CompareResult == 0)
      {
         // already in
         c_Entry.m_i_Instances ++;
         m_i_TracerIndex = i;
         return;
      }
      else if(   i_CompareResult < 0
              && i_InsertIndex <0 )
      {
         i_InsertIndex = i;
      }

   }
   // not found -- create new element
   {
      TracerEntry c_NewEntry;
      c_NewEntry.m_qs_Name        = m_qs_Name;
      c_NewEntry.m_i_Instances    = 1;
      c_NewEntry.m_b_TraceEnabled = false;
      if(i_InsertIndex>=0)
      {
         QValueVector<TracerEntry>::iterator it;
         int i_Index;
         for( it = g_TraceBase.begin(),i_Index=0;
              (it != g_TraceBase.end()) && (i_Index < i_InsertIndex) ;
              ++it,i_Index++ )
         {
         }
         g_TraceBase.insert( it,c_NewEntry);
      }
      else
      {
         g_TraceBase.push_back(c_NewEntry);
      }
      m_i_TracerIndex = g_TraceBase.size()-1;
   }

} // Tracer()

//============================================================================
//
//============================================================================


Tracer::~Tracer()
{
  int i;
  int i_Size;
  // Search for the element
   i_Size = g_TraceBase.size();

   for(i=0;i<i_Size;i++)
   {
      TracerEntry c_Entry;

      c_Entry = g_TraceBase.at(i);
      if(m_qs_Name.compare(c_Entry.m_qs_Name) == 0)
      {
         // decrease instances
         c_Entry.m_i_Instances --;
         break;
      }
   }
} // ~Tracer()

//============================================================================
//
//============================================================================

 Tracer  Tracer::operator<<( char * cp_String)
{
   if( TraceActive())
   {
      fprintf(stderr,"%s",cp_String);
   }
   return *this;
}

//============================================================================
//
//============================================================================

 Tracer  Tracer::operator<<( int i_Number)
{
   if( TraceActive())
   {
      fprintf(stderr,"%d ",i_Number);
   }
   return *this;
}

//============================================================================
//
//============================================================================

 Tracer  Tracer::operator<<( double d_Number)
{
   if( TraceActive())
   {
      fprintf(stderr,"%lf ",d_Number);
   }
   return *this;
}

//============================================================================
//
//============================================================================

 Tracer Tracer::operator<<( bool b_Value)
{
   if( TraceActive())
   {
      fprintf(stderr,"%s ",b_Value ? "true" : "false");
   }
   return *this;
}

//============================================================================
//
//============================================================================

int Tracer::GetNumberOfModules()
{
  int i_Size;
  // look, whether in the tracerbase this name exists
  i_Size = g_TraceBase.size();

  return i_Size;
} // GetNumberOfModules()

//============================================================================
//
//============================================================================

TracerEntry Tracer::GetEntry(int i_Index)
{
   TracerEntry c_Entry;
   int         i_Size;

   // look for the element
   i_Size = g_TraceBase.size();

   if(    i_Index <0
       || i_Index >= i_Size)
   {
      return c_Entry;
   }

   c_Entry = g_TraceBase.at(i_Index);

   return c_Entry;
} // GetEntry()

//============================================================================
//
//============================================================================

void Tracer::SetDataPointer(int i_Index,void *p_Data)
{
   TracerEntry c_Entry;
   int         i_Size;

   // look for the element
   i_Size = g_TraceBase.size();

   if(    i_Index <0
       || i_Index >= i_Size)
   {
      return ;
   }

   c_Entry = g_TraceBase.at(i_Index);

   c_Entry.m_p_ListItem = p_Data;

   g_TraceBase[i_Index] = c_Entry;

} // SetPointer()

//============================================================================
//
//============================================================================

void Tracer::SetActive(int i_Index, bool b_Active)
{
   TracerEntry c_Entry;
   int         i_Size;

   // look for the element
   i_Size = g_TraceBase.size();

   if(    i_Index <0
       || i_Index >= i_Size)
   {
      return ;
   }

   c_Entry = g_TraceBase.at(i_Index);

   c_Entry.m_b_TraceEnabled = b_Active;
                       
   g_TraceBase[i_Index] = c_Entry;

} // SetActive()

//============================================================================
//
//============================================================================

bool Tracer::TraceActive()
{

  int i;
  int i_Size;
  // Search for the element
   i_Size = g_TraceBase.size();

   for(i=0;i<i_Size;i++)
   {
      TracerEntry c_Entry;

      c_Entry = g_TraceBase.at(i);
      if(m_qs_Name.compare(c_Entry.m_qs_Name) == 0)
      {
         // decrease instances
         return c_Entry.m_b_TraceEnabled ;
      }
   } // for i

   return false;

} // TraceActive()

//============================================================================
//
//============================================================================


