
#include <stdio.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>

#include "Tracer.h"
#include "TracerBox.h"


TracerBox::TracerBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global)
{
   QBoxLayout    *l_Layout;
   QHGroupBox   *ButtonBox;


   setCaption("TracerBox");

   // nothing selected
   m_i_CurrentSelected = -1 ;

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);

   {
      m_ListView = new QListView(m_w_TopWidget);
      l_Layout->addWidget(m_ListView);

      m_ListView->addColumn("Module",200);
      
      m_ListView->setColumnWidthMode(0,QListView::Maximum);      
   }
   
   // -- box with the all/nothing buttons
   ButtonBox = new QHGroupBox("All Groups",m_w_TopWidget);
   l_Layout->addWidget(ButtonBox);
   {
      m_AllButton     = new QPushButton("Enable All",ButtonBox);
      m_NothingButton = new QPushButton("No Output",ButtonBox);
   
      connect( m_AllButton ,
               SIGNAL(clicked()),
               SLOT( ActivateAll() ) );
      connect( m_NothingButton ,
               SIGNAL(clicked()),
               SLOT( ActivateNothing() ) );
    }

   connect( m_ListView,
            SIGNAL(clicked( QListViewItem *)),
            SLOT(ListViewClicked( QListViewItem *) )
            );

} // TracerBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void TracerBox::Reload()
{
   int i;

   // nothing selected
   m_i_CurrentSelected = -1 ;

   m_i_NumberOfTraces = Tracer::GetNumberOfModules();

   // load all elements from tracer into the listbox
   {

      // delete the old contents
      m_ListView->clear();

      for(i=0; i < m_i_NumberOfTraces; i++)
      {
         QCheckListItem *p_Check;
         TracerEntry     c_CurrentEntry;
         QString         qs_OutputString;

         c_CurrentEntry = Tracer::GetEntry(i);

         qs_OutputString = c_CurrentEntry.m_qs_Name;

         p_Check = new QCheckListItem(m_ListView,
                                      qs_OutputString,
                                      QCheckListItem::CheckBox);

         if( c_CurrentEntry.m_b_TraceEnabled)
         {
            p_Check->setOn(true);
         }
         Tracer::SetDataPointer(i, (void *)p_Check);          
         
      } // for i
   }
} // Reload

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void TracerBox::ActivateAll()
{
   int i;
   for(i=0; i< m_i_NumberOfTraces; i++)
   {

       Tracer::SetActive(i,true);
   }
   Reload();
} // ActivateAll()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void TracerBox::ActivateNothing()
{
   int i;
   for(i=0; i< m_i_NumberOfTraces; i++)
   {

       Tracer::SetActive(i,false);
   }
   Reload();
} // ActivateNothing()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void TracerBox::ListViewClicked ( QListViewItem * item )
{
   bool b_On;
   int i;
   int pos;

   pos = item->itemPos();
   
   if(((QCheckListItem*)item)->isOn())
   {
      b_On = true;
   }
   else
   {
      b_On=false;
   }

   m_i_NumberOfTraces = Tracer::GetNumberOfModules();

   for(i=0; i < m_i_NumberOfTraces; i++)
   {
      TracerEntry c_CurrentEntry;

      c_CurrentEntry = Tracer::GetEntry(i);
      if( c_CurrentEntry.m_p_ListItem == (void *) item)
      {
         Tracer::SetActive(i,b_On);
         break;
      }
   } // for i
} // ListViewClicked()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

