
#include <stdio.h>

#include "Undo.h"

//============================================================================
//
//============================================================================

Undo::Undo()
{
   m_b_UndoPossible = false;
   m_b_RedoPossible = false;

   m_pc_ActualPicture  = NULL;
   m_pc_OldPicture  = NULL;


} // Undo() - constructor

//============================================================================
//
//============================================================================

Undo::~Undo()
{
   m_b_UndoPossible = false;
   m_b_RedoPossible = false;

   if(m_pc_ActualPicture != NULL)
   {
      delete m_pc_ActualPicture;
      m_pc_ActualPicture = NULL;
   }

   if(m_pc_OldPicture != NULL)
   {
      delete m_pc_OldPicture;
      m_pc_OldPicture = NULL;
   }


} // Undo() - destructor

//============================================================================
//
//============================================================================

void Undo::Reset()
{
   m_b_UndoPossible = false;
   m_b_RedoPossible = false;

   if(m_pc_ActualPicture != NULL)
   {
      delete m_pc_ActualPicture;
      m_pc_ActualPicture = NULL;
   }

   if(m_pc_OldPicture != NULL)
   {
      delete m_pc_OldPicture;
      m_pc_OldPicture = NULL;
   }

} // Reset();

//============================================================================
//
//============================================================================

void Undo::SaveBufferToUndo(PixelBuffer *pc_Original)
{
   // put actual into old
   if(m_pc_OldPicture == NULL)
   {
      // copy the pointer
      m_pc_OldPicture = m_pc_ActualPicture;
      // invalidate the actual
      m_pc_ActualPicture = NULL;
   }
   else // Old was not empty
   {
      if( m_pc_OldPicture->HasEqualSize(m_pc_ActualPicture))
      {
         // swap the buffers
         // because the actual buffer is overwritten in the next step
         PixelBuffer *pc_Temp;

         pc_Temp            = m_pc_OldPicture;
         m_pc_OldPicture    = m_pc_ActualPicture;
         m_pc_ActualPicture = pc_Temp;
      }
      else // different Size
      {
         delete m_pc_OldPicture;
         m_pc_OldPicture      = m_pc_ActualPicture;
         m_pc_ActualPicture   = NULL;
      }
   }

   // now copy the Original into the actual
   if(m_pc_ActualPicture == NULL)
   {
      m_pc_ActualPicture = new PixelBuffer(pc_Original);
   }
   else
   {
      if(m_pc_ActualPicture->HasEqualSize(pc_Original))
      {
         m_pc_ActualPicture->CopyFrom(pc_Original);
      }
      else // different size
      {
         // delete the actual Buffer
         delete m_pc_ActualPicture;
         m_pc_ActualPicture = new PixelBuffer(pc_Original);
      }

   }


   // set the flags
   m_b_RedoPossible = false;
   m_b_UndoPossible = true;
} // SaveBufferToUndo

//============================================================================
//
//============================================================================

void Undo::DoUndo(PixelBuffer * &pc_BufferToGetRestored,
                  bool &b_SizeChanged)
{
   // copy from the OldBuffer into the Get restored Buffer.
   // swap the actual and the old picture

   PixelBuffer *pc_Temp;
   
   b_SizeChanged = false;

   // is an undo possible ?
   if(! m_b_UndoPossible
      || m_pc_ActualPicture == NULL
      || m_pc_OldPicture    == NULL
     )
   {
      return ;
   }

   // copy the old image into the buffer

   if(m_pc_OldPicture->HasEqualSize(pc_BufferToGetRestored) )
   {
      // just copy the contents
      pc_BufferToGetRestored->CopyFrom(m_pc_OldPicture);
   }
   else
   {
      // resize the original picture before
      pc_BufferToGetRestored->ResizeInternalTo(m_pc_OldPicture->GetWidth(),
                       m_pc_OldPicture->GetHeight());

      // now copy into the resized buffer
      pc_BufferToGetRestored->CopyFrom(m_pc_OldPicture);
      b_SizeChanged = true;
   }


   // swap the Buffers
   pc_Temp            = m_pc_OldPicture;
   m_pc_OldPicture    = m_pc_ActualPicture;
   m_pc_ActualPicture = pc_Temp;


   m_b_UndoPossible = false;
   m_b_RedoPossible = true;

} // DoUndo

//============================================================================
//
//============================================================================

void Undo::DoRedo(PixelBuffer * &pc_BufferToGetRestored,
			bool &b_SizeChanged)
{
   // copy from the OldBuffer into the Get restored Buffer.
   // swap the actual and the old picture

   PixelBuffer *pc_Temp;

   b_SizeChanged = false;

   // is an undo possible ?
   if(! m_b_RedoPossible
      || m_pc_ActualPicture == NULL
      || m_pc_OldPicture    == NULL
     )
   {
      return ;
   }
   // copy the old image into the buffer

   if(m_pc_OldPicture->HasEqualSize(pc_BufferToGetRestored) )
   {
      // just copy the contents
      pc_BufferToGetRestored->CopyFrom(m_pc_OldPicture);
   }
   else
   {
      // resize the original picture before
      pc_BufferToGetRestored->ResizeInternalTo(m_pc_OldPicture->GetWidth(),
                       m_pc_OldPicture->GetHeight());

      // now copy into the resized buffer
      pc_BufferToGetRestored->CopyFrom(m_pc_OldPicture);
   
      b_SizeChanged = true;

   }


   // swap the Buffers
   pc_Temp            = m_pc_OldPicture;
   m_pc_OldPicture    = m_pc_ActualPicture;
   m_pc_ActualPicture = pc_Temp;


   m_b_UndoPossible = true;
   m_b_RedoPossible = false;


} // DoRedo()

//============================================================================
//
//============================================================================

bool Undo::IsRedoPossible()
{
   return m_b_RedoPossible;
} // IsRedoPossible()

//============================================================================
//
//============================================================================

bool Undo::IsUndoPossible()
{
   return m_b_UndoPossible;
} // IsUndoPossible()

//============================================================================
//
//============================================================================

PixelBuffer *Undo::GetActualPicture()
{
   return m_pc_ActualPicture;
} // GetActualPicture()

//============================================================================
//
//============================================================================

