// zoom functions

#include "Utilities.h"
#include "PTypes.h"
#include "Zoom.h"
#include "DrawingArea.h"
#include "GraphicObjectLayer.h"
#include "GraphicObjectMain.h"

//===========================================================================
//
//===========================================================================

Zoom::Zoom(GlobalStorage *pc_GlobalStorage)
      : Tracer("Zoom")
{
   int i;
   m_pc_GlobalStorage =  pc_GlobalStorage;
   m_pc_GlobalStorage->m_pc_Zoom = this;

   m_i_ZoomFactor     =1;
   m_i_LeftStartPixel =0;
   m_i_TopStartPixel  =0;


   m_pc_DestinationImage = NULL ;
   m_c_DestinationArea.SetEmpty();
   m_i_DestinationMaxWidth  = 1;
   m_i_DestinationMaxHeight = 1;

   // disable visiblilty of both masks
   m_b_SelectionMaskVisible = false;
   m_b_SafetyMaskVisible    = false;

   // disable the grid mode
   m_p_GridColor   =0xff33cc;
   m_b_GridEnabled =false;
   m_i_GridXOffset =1;
   m_i_GridYOffset =1;
   m_i_GridWidth   =2;
   m_i_GridHeight  =2;

   // reset the correction curves
   for(i=0;i<256;i++)
   {
      m_uca_CorrectionRed[i]   = (unsigned char) i;
      m_uca_CorrectionGreen[i] = (unsigned char) i;
      m_uca_CorrectionBlue[i]  = (unsigned char) i;
   }
   m_b_CorrectionEnabled = false;

   m_b_DrawObjectImagesTransparent = false;
} // Zoom constructor

//===========================================================================
//
//===========================================================================


Zoom::~Zoom()
{
   // nothing special until now
}


//===========================================================================
//
//===========================================================================

void Zoom::SetDestinationImage(QImage *pc_OutputImage)
{
   m_pc_DestinationImage = pc_OutputImage;
}

//===========================================================================
//
//===========================================================================

void Zoom::SetSourceBuffer(PixelBuffer *pc_Source)
{
   m_pc_SourceBuffer = pc_Source;
}
//===========================================================================
//
//===========================================================================


void Zoom::SetZoomFactor(int i_Factor)
{
   m_i_ZoomFactor = i_Factor;
}

//===========================================================================
//
//===========================================================================

void Zoom::SetSourceLeftTop(int i_LeftStart,int i_TopStart)
{
   m_i_LeftStartPixel = i_LeftStart;
   m_i_TopStartPixel  = i_TopStart;
}
//===========================================================================
//
//===========================================================================


void Zoom::SetDestinationArea(RectRegion c_DestinationArea)
{
   m_c_DestinationArea = c_DestinationArea;
} // SetDestinationArea()

//===========================================================================
//
//===========================================================================

void Zoom::SetMaximumPossibleDestinationArea(RectRegion c_DestinationArea)
{
   m_MaximalPossibleDestinationArea = c_DestinationArea;
} // SetMaximumPossibleDestinationArea

//===========================================================================
//
//===========================================================================


void Zoom::DoZoom()
{
   if(m_i_ZoomFactor ==1 )
   {
      if(   m_b_SelectionMaskVisible
         || m_b_SafetyMaskVisible
         || m_b_CorrectionEnabled
         )
      {
         DoZoom1To1WithMasks();
      }
      else // no mask activ
      {
         DoZoom1To1();
      }
   }
   else if(m_i_ZoomFactor > 1)
   {
      if(   m_b_SelectionMaskVisible
         || m_b_SafetyMaskVisible
         || m_b_CorrectionEnabled
         )
      {
         DoZoom1ToLargerWithMasks();
      }
      else
      {
         DoZoom1ToLarger();
      }
   }
   else //  m_i_ZoomFactor < 1
   {
      if(   m_b_SelectionMaskVisible
         || m_b_SafetyMaskVisible
         || m_b_CorrectionEnabled
         )
      {
         DoZoom1ToSmallerWithMasks();
      }
      else
      {
         DoZoom1ToSmaller();
      }
   }

   if(m_pc_GlobalStorage->m_pc_ObjectMain != NULL)
   {
      if(m_pc_GlobalStorage->m_pc_ObjectMain->IsVisible())
      {
         // objects are visible, add them
	 ZoomGraphicObjects();
      }
   }

   if(m_b_GridEnabled)
   {
      AddGridLines();
   }

} // DoZoom()

//===========================================================================
//
//===========================================================================


void Zoom::DoZoom1To1()
{
   PLONG *pc_InDestination;
   int    i_X;
   int    i_Y;
   int    i_StartX;
   int    i_EndX;
   int    i_StartY;
   int    i_EndY;

   int    i_StartSourceX;
   int    i_StartSourceY;
   int    i_SourceY;

   int    i_MaxSourceX;
   int    i_MaxSourceY;

   i_MaxSourceX = m_pc_SourceBuffer->GetWidth();
   i_MaxSourceY = m_pc_SourceBuffer->GetHeight();

   // subtract the current picture offset
   i_MaxSourceX -= m_i_LeftStartPixel;
   i_MaxSourceY -= m_i_TopStartPixel;


   i_StartX = m_c_DestinationArea.GetStartX();
   i_StartY = m_c_DestinationArea.GetStartY();
   i_EndX   = m_c_DestinationArea.GetEndX();
   i_EndY   = m_c_DestinationArea.GetEndY();


   // clip at the maximum possible pixel position
   if(i_EndX >= i_MaxSourceX)
   {
      i_EndX = i_MaxSourceX-1;
   }
   if(i_EndY >= i_MaxSourceY)
   {
      i_EndY = i_MaxSourceY-1;
   }

   TRACE << "Zoom sx:" << i_StartX
         << "sy:"      << i_StartY
         << "ex:"      << i_EndX
         << "ey"       << i_EndY
         << "\n";

   i_StartSourceX = i_StartX +  m_i_LeftStartPixel;
   i_StartSourceY = i_StartY +  m_i_TopStartPixel;

   if(   i_StartX<= i_EndX
     && i_StartY<= i_EndY)
   {

      if(m_pc_SourceBuffer->GetDepth() == 8)
      {
         PBYTE *pc_Source;
         i_SourceY = i_StartSourceY;
         for(i_Y=i_StartY; i_Y<=i_EndY;i_Y ++ )
         {
            pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y) + i_StartX;
            pc_Source = (PBYTE *) m_pc_SourceBuffer->GetPixelAddress(i_StartSourceX,i_SourceY);
            for(i_X=i_StartX; i_X<=i_EndX;i_X ++)
            {
               *pc_InDestination ++ = (PLONG) (* pc_Source ++);

            } // for i_X
            i_SourceY++;
         } // for i_Y
      } // if Depth = 8
      else if(m_pc_SourceBuffer->GetDepth() == 32)
      {
         PLONG *pc_Source;
         i_SourceY = i_StartSourceY;
         for(i_Y=i_StartY; i_Y<=i_EndY;i_Y ++)
         {
            pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y) + i_StartX;
            pc_Source = (PLONG *) m_pc_SourceBuffer->GetPixelAddress(i_StartSourceX,i_SourceY);
            for(i_X=i_StartX; i_X<=i_EndX; i_X ++)
            {
               *pc_InDestination ++ = * pc_Source ++;

            } // for i_X
            i_SourceY++;
         } // for i_Y
      } // if Depth = 32

      // set the output area
      m_c_ZoomOutputArea.SetXYXY(i_StartX,i_StartY,i_EndX,i_EndY);
   }
   else // start > end
   {
      // nothing to zoom
      m_c_ZoomOutputArea.SetEmpty();

   }

} // DoZoom1To1()

//===========================================================================
//
//===========================================================================


void Zoom::DoZoom1To1WithMasks()
{
   PixelBuffer *pc_SafetyBuffer;
   PixelBuffer *pc_SelectionBuffer;
   Pixel        p_Pixel;
   PLONG       *pc_InDestination;
   int          i_X;
   int          i_Y;
   int          i_StartX;
   int          i_EndX;
   int          i_StartY;
   int          i_EndY;

   int          i_StartSourceX;
   int          i_StartSourceY;
   int          i_SourceX;
   int          i_SourceY;

   int          i_MaxSourceX;
   int          i_MaxSourceY;
   int          i_ApplySafety;
   int          i_ApplySelection;


   if(m_b_SafetyMaskVisible)
   {
      pc_SafetyBuffer    = m_pc_SourceBuffer->GetSafetyMask();
   }
   else
   {
      pc_SafetyBuffer    = NULL;
   }

   if(m_b_SelectionMaskVisible)
   {
      pc_SelectionBuffer = m_pc_SourceBuffer->GetSelectionMask();
   }
   else
   {
      pc_SelectionBuffer = NULL;
   }

   i_MaxSourceX = m_pc_SourceBuffer->GetWidth();
   i_MaxSourceY = m_pc_SourceBuffer->GetHeight();

   // subtract the current picture offset
   i_MaxSourceX -= m_i_LeftStartPixel;
   i_MaxSourceY -= m_i_TopStartPixel;


   i_StartX = m_c_DestinationArea.GetStartX();
   i_StartY = m_c_DestinationArea.GetStartY();
   i_EndX   = m_c_DestinationArea.GetEndX();
   i_EndY   = m_c_DestinationArea.GetEndY();


   // clip at the maximum possible pixel position
   if(i_EndX >= i_MaxSourceX)
   {
      i_EndX = i_MaxSourceX-1;
   }
   if(i_EndY >= i_MaxSourceY)
   {
      i_EndY = i_MaxSourceY-1;
   }

   TRACE << "ZoomWithMask sx:" << i_StartX
         << "sy:"      << i_StartY
         << "ex:"      << i_EndX
         << "ey"       << i_EndY
         << "\n";

   i_StartSourceX = i_StartX +  m_i_LeftStartPixel;
   i_StartSourceY = i_StartY +  m_i_TopStartPixel;

   if(   i_StartX<= i_EndX
      && i_StartY<= i_EndY)
   {

      if(m_pc_SourceBuffer->GetDepth() == 32)
      {
         PLONG *pc_Source;
         i_SourceY = i_StartSourceY;

         for(i_Y=i_StartY; i_Y<=i_EndY;i_Y ++)
         {
            pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y) + i_StartX;
            pc_Source = (PLONG *) m_pc_SourceBuffer->GetPixelAddress(i_StartSourceX,i_SourceY);
            i_SourceX= i_StartSourceX;
            for(i_X=i_StartX; i_X<=i_EndX; i_X ++)
            {
               p_Pixel = * pc_Source ++;
               
               // apply color correction
               if(m_b_CorrectionEnabled)
               {
                  p_Pixel = ApplyCorrection(p_Pixel);
               }

               if(pc_SafetyBuffer == NULL)
               {
                  i_ApplySafety = 0;
               }
               else
               {
                  i_ApplySafety = pc_SafetyBuffer->Get(i_SourceX,i_SourceY);
               }
               if(pc_SelectionBuffer == NULL)
               {
                  i_ApplySelection = 0;
               }
               else
               {
                  i_ApplySelection = pc_SelectionBuffer->Get(i_SourceX,i_SourceY);
               }

               // merge Pixel with Selection Color
               if(i_ApplySelection)
               {
                  p_Pixel = MergeColors(p_Pixel,m_p_SelectionColor);
               }

               // merge Pixel with Safety Color
               if(i_ApplySafety)
               {
                  p_Pixel = MergeColors(p_Pixel,m_p_SafetyColor);
               }

               *pc_InDestination ++ =  p_Pixel;
               i_SourceX ++;

            } // for i_X
            i_SourceY++;
         } // for i_Y
      } // if Depth = 32

      // set the output area
      m_c_ZoomOutputArea.SetXYXY(i_StartX,i_StartY,i_EndX,i_EndY);
   }
   else // start > end
   {
      // nothing to zoom
      m_c_ZoomOutputArea.SetEmpty();

   }

} // DoZoom1To1WithMasks()

//===========================================================================
//
//===========================================================================


void Zoom::DoZoom1ToLargerWithMasks()
{
   PixelBuffer *pc_SafetyBuffer;
   PixelBuffer *pc_SelectionBuffer;
   PLONG       *pc_InDestination;
   Pixel        p_Pixel;
   int          i_X;
   int          i_Y;
   int          i_StartX;
   int          i_EndX;
   int          i_StartY;
   int          i_EndY;

   int          i_StartSourceX;
   int          i_StartSourceY;
   int          i_SourceY;

   int          i_MaxSourceX;
   int          i_MaxSourceY;

   int          i_ApplySafety;
   int          i_ApplySelection;



   if(m_b_SafetyMaskVisible)
   {
      pc_SafetyBuffer    = m_pc_SourceBuffer->GetSafetyMask();
   }
   else
   {
      pc_SafetyBuffer    = NULL;
   }

   if(m_b_SelectionMaskVisible)
   {
      pc_SelectionBuffer = m_pc_SourceBuffer->GetSelectionMask();
   }
   else
   {
      pc_SelectionBuffer = NULL;
   }


   i_MaxSourceX = m_pc_SourceBuffer->GetWidth();
   i_MaxSourceY = m_pc_SourceBuffer->GetHeight();


   i_StartX = m_c_DestinationArea.GetStartX();
   i_StartY = m_c_DestinationArea.GetStartY();
   i_EndX   = m_c_DestinationArea.GetEndX();
   i_EndY   = m_c_DestinationArea.GetEndY();


   TRACE << "Zoom 1 to larger sx:" << i_StartX
         << "sy:"      << i_StartY
         << "ex:"      << i_EndX
         << "ey"       << i_EndY
         << "\n";

   i_StartSourceX = (i_StartX / m_i_ZoomFactor)+  m_i_LeftStartPixel;
   i_StartSourceY = (i_StartY / m_i_ZoomFactor)+  m_i_TopStartPixel;

   if(   i_StartX<= i_EndX
      && i_StartY<= i_EndY
      && i_StartSourceX < i_MaxSourceX
      && i_StartSourceY < i_MaxSourceY)
   {

      if(m_pc_SourceBuffer->GetDepth() == 32)
      {
         PLONG *pc_Source;
         i_SourceY = i_StartSourceY;
         for(i_Y=i_StartY; i_Y<=i_EndY;i_Y ++)
         {
            int i_SourceX;
            pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y) + i_StartX;
            pc_Source = (PLONG *) m_pc_SourceBuffer->GetPixelAddress(i_StartSourceX,i_SourceY);
            if(pc_Source == NULL)
            {
               // should not occur, but test for stability
               continue;
            }
            i_SourceX = i_StartSourceX;

            p_Pixel = * pc_Source ;
            
            // apply color correction
            if(m_b_CorrectionEnabled)
            {
               p_Pixel = ApplyCorrection(p_Pixel);
            }

            if(pc_SafetyBuffer == NULL)
            {
               i_ApplySafety = 0;
            }
            else
            {
               i_ApplySafety = pc_SafetyBuffer->Get(i_SourceX,i_SourceY);
            }
            if(pc_SelectionBuffer == NULL)
            {
               i_ApplySelection = 0;
            }
            else
            {
               i_ApplySelection = pc_SelectionBuffer->Get(i_SourceX,i_SourceY);
            }

            // merge Pixel with Selection Color
            if(i_ApplySelection)
            {
               p_Pixel = MergeColors(p_Pixel,m_p_SelectionColor);
            }

               // merge Pixel with Safety Color
            if(i_ApplySafety)
            {
               p_Pixel = MergeColors(p_Pixel,m_p_SafetyColor);
            }

            for(i_X=i_StartX; i_X<=i_EndX; i_X ++)
            {
               *pc_InDestination ++ = p_Pixel ;
               if( (i_X % m_i_ZoomFactor) == (m_i_ZoomFactor-1))
               {
                  pc_Source ++;
                  i_SourceX++;
                  p_Pixel = * pc_Source ;
                  if(pc_SafetyBuffer == NULL)
                  {
                     i_ApplySafety = 0;
                  }
                  else
                  {
                     i_ApplySafety = pc_SafetyBuffer->Get(i_SourceX,i_SourceY);
                  }
                  if(pc_SelectionBuffer == NULL)
                  {
                     i_ApplySelection = 0;
                  }
                  else
                  {
                     i_ApplySelection = pc_SelectionBuffer->Get(i_SourceX,i_SourceY);
                  }

                  // merge Pixel with Selection Color
                  if(i_ApplySelection)
                  {
                     p_Pixel = MergeColors(p_Pixel,m_p_SelectionColor);
                  }

                     // merge Pixel with Safety Color
                  if(i_ApplySafety)
                  {
                     p_Pixel = MergeColors(p_Pixel,m_p_SafetyColor);
                  }

                  if(i_SourceX >= i_MaxSourceX)
                  {
                     // reduce the X-end for the next time
                     // also used for region calculation
                     i_EndX = i_X;
                     break;
                  }
               }

            } // for i_X



            if( (i_Y % m_i_ZoomFactor) == (m_i_ZoomFactor-1))
            {

               i_SourceY++;
               if(i_SourceY>= i_MaxSourceY)
               {
                  // reduce the y-end. Used for region calculation
                  i_EndY = i_Y;
                  break;
               }
            }

         } // for i_Y
      } // if Depth = 32

      m_c_ZoomOutputArea.SetXYXY(i_StartX,i_StartY,i_EndX,i_EndY);
   }
   else // start > end
   {
      // nothing to zoom
      m_c_ZoomOutputArea.SetEmpty();

   }


} // DoZoom1ToLargerWithMasks()

//===========================================================================
//
//===========================================================================


void Zoom::DoZoom1ToLarger()
{
   PLONG *pc_InDestination;
   int    i_X;
   int    i_Y;
   int    i_StartX;
   int    i_EndX;
   int    i_StartY;
   int    i_EndY;

   int    i_StartSourceX;
   int    i_StartSourceY;
   int    i_SourceY;

   int    i_MaxSourceX;
   int    i_MaxSourceY;


   i_MaxSourceX = m_pc_SourceBuffer->GetWidth();
   i_MaxSourceY = m_pc_SourceBuffer->GetHeight();


   i_StartX = m_c_DestinationArea.GetStartX();
   i_StartY = m_c_DestinationArea.GetStartY();
   i_EndX   = m_c_DestinationArea.GetEndX();
   i_EndY   = m_c_DestinationArea.GetEndY();


   TRACE << "Zoom 1 to larger sx:" << i_StartX
         << "sy:"      << i_StartY
         << "ex:"      << i_EndX
         << "ey"       << i_EndY
         << "\n";

   i_StartSourceX = (i_StartX / m_i_ZoomFactor)+  m_i_LeftStartPixel;
   i_StartSourceY = (i_StartY / m_i_ZoomFactor)+  m_i_TopStartPixel;

   if(   i_StartX<= i_EndX
      && i_StartY<= i_EndY
      && i_StartSourceX < i_MaxSourceX
      && i_StartSourceY < i_MaxSourceY)
   {

      if(m_pc_SourceBuffer->GetDepth() == 32)
      {
         PLONG *pc_Source;
         i_SourceY = i_StartSourceY;
         for(i_Y=i_StartY; i_Y<=i_EndY;i_Y ++)
         {
            int i_SourceX;
            pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y) + i_StartX;
            pc_Source = (PLONG *) m_pc_SourceBuffer->GetPixelAddress(i_StartSourceX,i_SourceY);
            if(pc_Source == NULL)
            {
               // should not occur, but test for stability
               continue;
            }
            i_SourceX = i_StartSourceX;

            for(i_X=i_StartX; i_X<=i_EndX; i_X ++)
            {
               *pc_InDestination ++ = * pc_Source ;
               if( (i_X % m_i_ZoomFactor) == (m_i_ZoomFactor-1))
               {
                  pc_Source ++;
                  i_SourceX++;
                  if(i_SourceX >= i_MaxSourceX)
                  {
                     // reduce the X-end for the next time
                     // also used for region calculation
                     i_EndX = i_X;
                     break;
                  }
               }

            } // for i_X



            if( (i_Y % m_i_ZoomFactor) == (m_i_ZoomFactor-1))
            {

               i_SourceY++;
               if(i_SourceY>= i_MaxSourceY)
               {
                  // reduce the y-end. Used for region calculation
                  i_EndY = i_Y;
                  break;
               }
            }

         } // for i_Y
      } // if Depth = 32

      m_c_ZoomOutputArea.SetXYXY(i_StartX,i_StartY,i_EndX,i_EndY);
   }
   else // start > end
   {
      // nothing to zoom
      m_c_ZoomOutputArea.SetEmpty();

   }

} // DoZoom1ToLarger()

//===========================================================================
// get buffer out of a window coordinate
// perform also a clipping
//===========================================================================

void Zoom::GetBufferCoordinate(int i_WindowX,
                               int i_WindowY,
                               int &i_BufferX,
                               int &i_BufferY)
{
   // calculate position

   if(m_i_ZoomFactor>=1)
   {
      i_BufferX = ( i_WindowX /m_i_ZoomFactor)  + m_i_LeftStartPixel;
      i_BufferY = ( i_WindowY /m_i_ZoomFactor)  + m_i_TopStartPixel;
   }
   else
   {
      int i_Smaller;
      i_Smaller = - m_i_ZoomFactor;

      i_BufferX = ( i_WindowX  * i_Smaller)  + m_i_LeftStartPixel;
      i_BufferY = ( i_WindowY  * i_Smaller)  + m_i_TopStartPixel;

   }

   // clip into the image
   if(i_BufferX <0)
   {
      i_BufferX =0;
   }
   if(i_BufferX >= m_pc_SourceBuffer->GetWidth())
   {
      i_BufferX = m_pc_SourceBuffer->GetWidth() -1;
   }

   if(i_BufferY <0)
   {
      i_BufferY =0;
   }
   if(i_BufferY >= m_pc_SourceBuffer->GetHeight())
   {
      i_BufferY = m_pc_SourceBuffer->GetHeight() -1;
   }

} // Get BufferCoordiante


//===========================================================================
// Get a window coordinate for a given buffer coordinate
//===========================================================================

bool Zoom::GetWindowCoordinate(int i_BufferX,
                               int i_BufferY,
                               int &i_WindowX,
                               int &i_WindowY,
                               bool b_DoClip)
{
   // calculate position
   bool b_IsClipped;

   b_IsClipped = false;
   if(m_i_ZoomFactor>=1)
   {

      i_WindowX = (i_BufferX - m_i_LeftStartPixel) *m_i_ZoomFactor;
      i_WindowY = (i_BufferY - m_i_TopStartPixel ) *m_i_ZoomFactor;
   }
   else
   {
      int i_Smaller;

      i_Smaller = - m_i_ZoomFactor;

      i_WindowX = (i_BufferX - m_i_LeftStartPixel) /i_Smaller ;
      i_WindowY = (i_BufferY - m_i_TopStartPixel ) /i_Smaller ;

   }

   if(b_DoClip)
   {
      // clip into the image

      int i_DrawingAreaWidth;
      int i_DrawingAreaHeight;
  
      i_DrawingAreaWidth  = m_pc_GlobalStorage->m_pc_DrawingArea->GetWidth();  
      i_DrawingAreaHeight = m_pc_GlobalStorage->m_pc_DrawingArea->GetHeight();


      if(i_WindowX <0)
      {
         i_WindowX =0;
         b_IsClipped = true;
      }

      if(i_WindowX >= i_DrawingAreaWidth)
      {
         i_WindowX = i_DrawingAreaWidth -1;
         b_IsClipped = true;
      }

      if(i_WindowY <0)
      {
         i_WindowY =0;
         b_IsClipped = true;
      }

      if(i_WindowY >= i_DrawingAreaHeight)
      {
         i_WindowY = i_DrawingAreaHeight -1;
         b_IsClipped = true;
      }

   } // if DoClip

   return b_IsClipped;

} // GetWindowCoordinate()



//===========================================================================
//
//===========================================================================

RectRegion Zoom::GetZoomedOutputArea()
{
   return m_c_ZoomOutputArea;

} // GetZoomedOutputArea()

//===========================================================================
//
//===========================================================================

void Zoom::DoZoom1ToSmaller()
{
   PLONG *pc_InDestination;
   int    i_Smaller;
   int    i_X;
   int    i_Y;
   int    i_StartX;
   int    i_EndX;
   int    i_StartY;
   int    i_EndY;

   int    i_StartSourceX;
   int    i_StartSourceY;
   int    i_SourceY;

   int    i_MaxSourceX;
   int    i_MaxSourceY;

   // for easier calculations
   i_Smaller = - m_i_ZoomFactor;

   i_MaxSourceX = m_pc_SourceBuffer->GetWidth();
   i_MaxSourceY = m_pc_SourceBuffer->GetHeight();


   i_StartX = m_c_DestinationArea.GetStartX();
   i_StartY = m_c_DestinationArea.GetStartY();
   i_EndX   = m_c_DestinationArea.GetEndX();
   i_EndY   = m_c_DestinationArea.GetEndY();


   TRACE << "Zoom to smaller sx:" << i_StartX
         << "sy:"      << i_StartY
         << "ex:"      << i_EndX
         << "ey"       << i_EndY
         << "\n";


   i_StartSourceX = (i_StartX * i_Smaller)+  m_i_LeftStartPixel;
   i_StartSourceY = (i_StartY * i_Smaller)+  m_i_TopStartPixel;

   if(i_StartSourceX <0)
   {
      i_StartSourceX=0;
   }

   if(i_StartSourceY <0)
   {
      i_StartSourceY=0;
   }

   // default nothing to zoom
   m_c_ZoomOutputArea.SetEmpty();
   if(    i_StartSourceX >= i_MaxSourceX
       || i_StartSourceY >= i_MaxSourceY)
   {
      TRACE << "Start larger than buffer\n";

   }

   else if(   i_StartX<= i_EndX
           && i_StartY<= i_EndY)
   {

      if(m_pc_SourceBuffer->GetDepth() == 32)
      {
         PLONG *pc_Source;
         i_SourceY = i_StartSourceY;

         for(i_Y=i_StartY; i_Y<=i_EndY;i_Y ++)
         {
            int i_SourceX;
            pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y) + i_StartX;
            pc_Source = (PLONG *) m_pc_SourceBuffer->GetPixelAddress(i_StartSourceX,i_SourceY);
            i_SourceX = i_StartSourceX;

            for(i_X=i_StartX; i_X<=i_EndX; i_X ++)
            {
               *pc_InDestination ++ = * pc_Source ;
               pc_Source += i_Smaller;
               i_SourceX += i_Smaller;
               if(i_SourceX >= i_MaxSourceX)
               {
                  // reduce the X-end for the next time
                  // also used for region calculation
                  i_EndX = i_X;
                  break;
               }

            } // for i_X




            i_SourceY += i_Smaller;
            if(i_SourceY>= i_MaxSourceY)
            {
               // reduce the y-end. Used for region calculation
               i_EndY = i_Y;
               break;
            }

         } // for i_Y
      } // if Depth = 32

      // set the output area
      m_c_ZoomOutputArea.SetXYXY(i_StartX,i_StartY,i_EndX,i_EndY);
   }
   else // start > end
   {
      // nothing to zoom
      m_c_ZoomOutputArea.SetEmpty();

   }

} // DoZoom1ToSmaller()

//===========================================================================
//
//===========================================================================

void Zoom::DoZoom1ToSmallerWithMasks()
{
   PixelBuffer *pc_SafetyBuffer;
   PixelBuffer *pc_SelectionBuffer;
   PLONG       *pc_InDestination;
   Pixel       p_Pixel;
   int         i_Smaller;
   int         i_X;
   int         i_Y;
   int         i_StartX;
   int         i_EndX;
   int         i_StartY;
   int         i_EndY;

   int         i_StartSourceX;
   int         i_StartSourceY;
   int         i_SourceY;

   int         i_MaxSourceX;
   int         i_MaxSourceY;

   int         i_ApplySafety;
   int         i_ApplySelection;



   if(m_b_SafetyMaskVisible)
   {
      pc_SafetyBuffer    = m_pc_SourceBuffer->GetSafetyMask();
   }
   else
   {
      pc_SafetyBuffer    = NULL;
   }

   if(m_b_SelectionMaskVisible)
   {
      pc_SelectionBuffer = m_pc_SourceBuffer->GetSelectionMask();
   }
   else
   {
      pc_SelectionBuffer = NULL;
   }

   // for easier calculations
   i_Smaller = - m_i_ZoomFactor;

   i_MaxSourceX = m_pc_SourceBuffer->GetWidth();
   i_MaxSourceY = m_pc_SourceBuffer->GetHeight();


   i_StartX = m_c_DestinationArea.GetStartX();
   i_StartY = m_c_DestinationArea.GetStartY();
   i_EndX   = m_c_DestinationArea.GetEndX();
   i_EndY   = m_c_DestinationArea.GetEndY();


   TRACE << "Zoom to smaller sx:" << i_StartX
         << "sy:"      << i_StartY
         << "ex:"      << i_EndX
         << "ey"       << i_EndY
         << "\n";


   i_StartSourceX = (i_StartX * i_Smaller)+  m_i_LeftStartPixel;
   i_StartSourceY = (i_StartY * i_Smaller)+  m_i_TopStartPixel;

   if(i_StartSourceX <0)
   {
      i_StartSourceX=0;
   }

   if(i_StartSourceY <0)
   {
      i_StartSourceY=0;
   }

   // default nothing to zoom
   m_c_ZoomOutputArea.SetEmpty();
   if(    i_StartSourceX >= i_MaxSourceX
       || i_StartSourceY >= i_MaxSourceY)
   {
      TRACE << "Start larger than buffer\n";

   }

   else if(   i_StartX<= i_EndX
           && i_StartY<= i_EndY)
   {

      if(m_pc_SourceBuffer->GetDepth() == 32)
      {
         PLONG *pc_Source;
         i_SourceY = i_StartSourceY;

         for(i_Y=i_StartY; i_Y<=i_EndY;i_Y ++)
         {
            int i_SourceX;
            pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y) + i_StartX;
            pc_Source = (PLONG *) m_pc_SourceBuffer->GetPixelAddress(i_StartSourceX,i_SourceY);
            i_SourceX = i_StartSourceX;

            for(i_X=i_StartX; i_X<=i_EndX; i_X ++)
            {
               p_Pixel = * pc_Source ;
               
               // apply color correction
               if(m_b_CorrectionEnabled)
               {
                  p_Pixel = ApplyCorrection(p_Pixel);
               }

               if(pc_SafetyBuffer == NULL)
               {
                  i_ApplySafety = 0;
               }
               else
               {
                  i_ApplySafety = pc_SafetyBuffer->Get(i_SourceX,i_SourceY);
               }
               if(pc_SelectionBuffer == NULL)
               {
                  i_ApplySelection = 0;
               }
               else
               {
                  i_ApplySelection = pc_SelectionBuffer->Get(i_SourceX,i_SourceY);
               }

               // merge Pixel with Selection Color
               if(i_ApplySelection)
               {
                  p_Pixel = MergeColors(p_Pixel,m_p_SelectionColor);
               }

                  // merge Pixel with Safety Color
               if(i_ApplySafety)
               {
                  p_Pixel = MergeColors(p_Pixel,m_p_SafetyColor);
               }

               *pc_InDestination ++ = p_Pixel ;
               pc_Source += i_Smaller;
               i_SourceX += i_Smaller;
               if(i_SourceX >= i_MaxSourceX)
               {
                  // reduce the X-end for the next time
                  // also used for region calculation
                  i_EndX = i_X;
                  break;
               }

            } // for i_X




            i_SourceY += i_Smaller;
            if(i_SourceY>= i_MaxSourceY)
            {
               // reduce the y-end. Used for region calculation
               i_EndY = i_Y;
               break;
            }

         } // for i_Y
      } // if Depth = 32

      // set the output area
      m_c_ZoomOutputArea.SetXYXY(i_StartX,i_StartY,i_EndX,i_EndY);

   }
   else // start > end
   {
      // nothing to zoom
      m_c_ZoomOutputArea.SetEmpty();

   }


} // DoZoom1ToSmallerWithMasks()

//===========================================================================
// for graphic objects and zoomed in display, move object on the
// middle of the pixel.
//===========================================================================

void Zoom::CenterCoordinate(int &i_BufferX, int &i_BufferY)
{
   if(m_i_ZoomFactor>2)
   {

      i_BufferX += m_i_ZoomFactor / 2;
      i_BufferY += m_i_ZoomFactor / 2;
   }
} // CenterCoordinate()

//===========================================================================
// for graphic objects and zoomed in display, move object on the
// middle of the pixel.
//===========================================================================

void Zoom::CenterCoordinate(float &f_BufferX, float &f_BufferY)
{
   if(m_i_ZoomFactor>2)
   {

      f_BufferX += m_i_ZoomFactor / 2;
      f_BufferY += m_i_ZoomFactor / 2;
   }
} // CenterCoordinate()


//===========================================================================
//
//===========================================================================

void Zoom::SetSafetyMaskVisible(bool b_Visible)
{
   m_b_SafetyMaskVisible = b_Visible;
} //SetSafetyMaskVisible()

//===========================================================================
//
//===========================================================================

bool Zoom::GetSafetyMaskVisible()
{
   return m_b_SafetyMaskVisible;
} // GetSafetyMaskVisible()

//===========================================================================
//
//===========================================================================


void Zoom::SetSelectionMaskVisible(bool b_Visible)
{
   m_b_SelectionMaskVisible = b_Visible;
} // SetSelectionMaskVisible()

//===========================================================================
//
//===========================================================================

bool Zoom::GetSelectionMaskVisible()
{
   return m_b_SelectionMaskVisible;
} // GetSelectionMaskVisible()

//===========================================================================
//
//===========================================================================

void Zoom::SetSafetyMaskColor(Pixel p_Color)
{
   m_p_SafetyColor = p_Color;
} // SetSafetyMaskColor()

//===========================================================================
//
//===========================================================================

void Zoom::SetSelectionMaskColor(Pixel p_Color)
{
   m_p_SelectionColor = p_Color;
} // SetSelectionMaskColor()

//===========================================================================
//
//===========================================================================

Pixel Zoom::MergeColors(Pixel p1,Pixel p2)
{
   int r;
   int g;
   int b;
   
   r = PIXEL_RED  (p1) + PIXEL_RED  (p2);
   g = PIXEL_GREEN(p1) + PIXEL_GREEN(p2);
   b = PIXEL_BLUE (p1) + PIXEL_BLUE (p2);

   return PIXEL_MAKE(r>>1,g>>1,b>>1);

} // MergeColors()

//===========================================================================
//
//===========================================================================

void Zoom::AddGridLines()
{
   // display grid only when regular or zoomed in
   if(m_i_ZoomFactor<1)
   {
      return;
   }
   if( m_i_ZoomFactor == 1)
   {
      PLONG *pc_InDestination;
      int    i_X;
      int    i_Y;
      int    i_StartX;
      int    i_EndX;
      int    i_StartY;
      int    i_EndY;

      int    i_StartSourceX;
      int    i_StartSourceY;
      int    i_SourceY;

      int    i_MaxSourceX;
      int    i_MaxSourceY;

      i_MaxSourceX = m_pc_SourceBuffer->GetWidth();
      i_MaxSourceY = m_pc_SourceBuffer->GetHeight();

      // subtract the current picture offset
      i_MaxSourceX -= m_i_LeftStartPixel;
      i_MaxSourceY -= m_i_TopStartPixel;


      i_StartX = m_c_DestinationArea.GetStartX();
      i_StartY = m_c_DestinationArea.GetStartY();
      i_EndX   = m_c_DestinationArea.GetEndX();
      i_EndY   = m_c_DestinationArea.GetEndY();


      // clip at the maximum possible pixel position
      if(i_EndX >= i_MaxSourceX)
      {
         i_EndX = i_MaxSourceX-1;
      }
      if(i_EndY >= i_MaxSourceY)
      {
         i_EndY = i_MaxSourceY-1;
      }

      TRACE << "Zoom Grid sx:" << i_StartX
            << "sy:"      << i_StartY
            << "ex:"      << i_EndX
            << "ey"       << i_EndY
           << "\n";

      i_StartSourceX = i_StartX +  m_i_LeftStartPixel;
      i_StartSourceY = i_StartY +  m_i_TopStartPixel;

      TRACE << "Zoom Grid StartSourceX:" << i_StartSourceX
            << "StartSourceY:"           << i_StartSourceY
           << "\n";

      if(   i_StartX<= i_EndX
         && i_StartY<= i_EndY)
      {

         PLONG *pc_Source;
         int   i_StartXGridOffset;

         i_StartXGridOffset = i_StartSourceX - m_i_GridXOffset;
         if(i_StartXGridOffset <0)
         {
            i_StartXGridOffset += m_i_GridWidth;
         }
         i_StartXGridOffset %= m_i_GridWidth;
         if( i_StartXGridOffset >0)
         {
            i_StartXGridOffset = m_i_GridWidth - i_StartXGridOffset;
         }

         i_SourceY = i_StartSourceY;
         for(i_Y=i_StartY; i_Y<=i_EndY;i_Y ++)
         {
            pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y) + i_StartX;
            pc_Source = (PLONG *) m_pc_SourceBuffer->GetPixelAddress(i_StartSourceX,i_SourceY);
            if((i_SourceY % m_i_GridHeight) == m_i_GridYOffset)
            {
               // complete line with grid
               for(i_X=i_StartX; i_X<=i_EndX; i_X ++)
               {
                  *pc_InDestination ++ = m_p_GridColor;
               } // for i_X

            }
            else // lines only with vertical grids
            {
               i_X = i_StartX + i_StartXGridOffset;
               pc_InDestination += i_StartXGridOffset;

               while( i_X<=i_EndX)
               {
                  *pc_InDestination  = m_p_GridColor;
                  i_X               += m_i_GridWidth;
                  pc_InDestination  += m_i_GridWidth;
               } // for i_X
            }
            i_SourceY++;
         } // for i_Y

      }
   } // if zoom = 1to1
   else
   {
      PLONG *pc_InDestination;
      int    i_StartXGridOffset;
      int    i_X;
      int    i_Y;
      int    i_StartX;
      int    i_EndX;
      int    i_StartY;
      int    i_EndY;

      int    i_StartSourceX;
      int    i_StartSourceY;
      int    i_SourceY;

      int    i_MaxSourceX;
      int    i_MaxSourceY;

      i_MaxSourceX = m_pc_SourceBuffer->GetWidth();
      i_MaxSourceY = m_pc_SourceBuffer->GetHeight();


      i_StartX = m_c_DestinationArea.GetStartX();
      i_StartY = m_c_DestinationArea.GetStartY();
      i_EndX   = m_c_DestinationArea.GetEndX();
      i_EndY   = m_c_DestinationArea.GetEndY();


      TRACE << "Zoom Grid 1 to larger sx:" << i_StartX
            << "sy:"      << i_StartY
            << "ex:"      << i_EndX
            << "ey"       << i_EndY
            << "\n";

      i_StartSourceX = (i_StartX / m_i_ZoomFactor)+  m_i_LeftStartPixel;
      i_StartSourceY = (i_StartY / m_i_ZoomFactor)+  m_i_TopStartPixel;

      if(   i_StartX<= i_EndX
         && i_StartY<= i_EndY
         && i_StartSourceX < i_MaxSourceX
         && i_StartSourceY < i_MaxSourceY)
      {

         PLONG *pc_Source;
         int   m_i_ZoomedGridWidth;

         m_i_ZoomedGridWidth = m_i_GridWidth*m_i_ZoomFactor;

         i_StartXGridOffset = i_StartSourceX - m_i_GridXOffset;
         if(i_StartXGridOffset <0)
         {
            i_StartXGridOffset += m_i_GridWidth;
         }
         i_StartXGridOffset %= m_i_GridWidth;
         if( i_StartXGridOffset >0)
         {
            i_StartXGridOffset = m_i_GridWidth - i_StartXGridOffset;
         }


         i_SourceY = i_StartSourceY;

         for(i_Y=i_StartY; i_Y<=i_EndY;i_Y ++)
         {
            pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y) + i_StartX;
            pc_Source = (PLONG *) m_pc_SourceBuffer->GetPixelAddress(i_StartSourceX,i_SourceY);
            if(pc_Source == NULL)
            {
               // should not occur, but test for stability
               continue;
            }

            if(   (i_SourceY % m_i_GridHeight) == m_i_GridYOffset
               && (i_Y % m_i_ZoomFactor) ==0 )
            {
               // full line with grid
               for(i_X=i_StartX; i_X<=i_EndX; i_X ++)
               {
                  *pc_InDestination ++ = m_p_GridColor;
               }
            }
            else // regular line
            {
               i_X = (i_StartX / m_i_ZoomFactor);
               i_X +=  i_StartXGridOffset;
               i_X *= m_i_ZoomFactor;

               pc_InDestination -= (i_StartX % m_i_ZoomFactor);
               pc_InDestination += i_StartXGridOffset*m_i_ZoomFactor;

               while( i_X<=i_EndX)
               {
                  *pc_InDestination  = m_p_GridColor;
                  i_X               += m_i_ZoomedGridWidth;
                  pc_InDestination  += m_i_ZoomedGridWidth;
               } // for i_X


            } // else regular line

            if( (i_Y % m_i_ZoomFactor) == (m_i_ZoomFactor-1))
            {

               i_SourceY++;
               if(i_SourceY>= i_MaxSourceY)
               {
                  // reduce the y-end. Used for region calculation
                  i_EndY = i_Y;
                  break;
               }
            }

         } // for i_Y
      }
   } // else zoom 1 to larger

} // AddGridLines()

//===========================================================================
//
//===========================================================================

void Zoom::SetGridColor(Pixel p_Color)
{
   m_p_GridColor = p_Color;
} // SetGridColor()

//===========================================================================
//
//===========================================================================

void Zoom::SetGridVisible(bool b_Visible)
{
   m_b_GridEnabled = b_Visible;
} // SetGridVisible()

//===========================================================================
//
//===========================================================================

void Zoom::SetGridParameters(int i_StartX,
                             int i_StartY, 
                             int i_Width, 
                             int i_Height)
{
   m_i_GridXOffset = i_StartX % i_Width;
   m_i_GridYOffset = i_StartY % i_Height;
   m_i_GridWidth   = i_Width;
   m_i_GridHeight  = i_Height;

} // SetGridParameters()

//===========================================================================
//
//===========================================================================

void Zoom::SetCorrectionCurve(unsigned char *puc_Red,
                           unsigned char *puc_Green,
                           unsigned char *puc_Blue)
{
   int i;
   
   if(puc_Red != NULL)
   {
      for(i=0;i<256;i++)
      {
         m_uca_CorrectionRed[i] = puc_Red[i];
      }
   }

   if(puc_Green != NULL)
   {
      for(i=0;i<256;i++)
      {
         m_uca_CorrectionGreen[i] = puc_Green[i];
      }
   }

   if(puc_Blue != NULL)
   {
      for(i=0;i<256;i++)
      {
         m_uca_CorrectionBlue[i] = puc_Blue[i];
      }
   }


} // SetCorrectionCurve()

//===========================================================================
//
//===========================================================================

void Zoom::SetCorrectionEnabled(bool b_Enabled)
{
   m_b_CorrectionEnabled = b_Enabled;

} // SetCorrectionEnabled()

//===========================================================================
//
//===========================================================================

Pixel Zoom::ApplyCorrection(Pixel p_Input)
{
   unsigned char i_Red;
   unsigned char i_Green;
   unsigned char i_Blue;

   i_Red   = PIXEL_RED  (p_Input);
   i_Green = PIXEL_GREEN(p_Input);
   i_Blue  = PIXEL_BLUE (p_Input);

   i_Red   = m_uca_CorrectionRed[i_Red  ];
   i_Green = m_uca_CorrectionGreen[i_Green];
   i_Blue  = m_uca_CorrectionBlue[i_Blue ];

   return (Pixel) PIXEL_MAKE(i_Red,i_Green,i_Blue);
} // ApplyCorrection()

//===========================================================================
//
//===========================================================================

void Zoom::DrawObjectImagesTransparent(bool b_On)
{
   m_b_DrawObjectImagesTransparent = b_On;
} // DrawObjectImagesTransparent()


//===========================================================================
//
//===========================================================================


