#ifndef _ZOOM_LOADED_H
#define _ZOOM_LOADED_H

#include <qimage.h>

#include "PixelBuffer.h"
#include "RectRegion.h"
#include "GraphicObject.h"

#include "GlobalStorage.h"
#include "Tracer.h"

class Zoom : public Tracer
{
   // allow access to private Pixelbuffer variables
   friend class PixelBuffer;

public:
   Zoom(GlobalStorage *pc_GlobalStorage);
   ~Zoom();


   void SetDestinationImage(QImage *pc_OutputImage);
   void SetSourceBuffer(PixelBuffer *pc_Source);

   void SetZoomFactor(int i_Factor);
   void SetSourceLeftTop(int i_LeftStart,int i_TopStart);

   void SetDestinationArea(RectRegion c_DestinationArea);
   void SetMaximumPossibleDestinationArea(RectRegion c_DestinationArea);


   void DoZoom();


   void GetBufferCoordinate(int i_WindowX,
                            int i_WindowY,
                            int &i_BufferX,
                            int &i_BufferY);

   bool GetWindowCoordinate(int i_BufferX,
                            int i_BufferY,
                            int &i_WindowX,
                            int &i_WindowY,
                            bool b_DoClip);

   RectRegion GetZoomedOutputArea();

   void SetSafetyMaskVisible(bool b_Visible);
   bool GetSafetyMaskVisible();
   void SetSafetyMaskColor(Pixel p_Color);

   void SetSelectionMaskVisible(bool b_Visible);
   bool GetSelectionMaskVisible();
   void SetSelectionMaskColor(Pixel p_Color);
   void SetGridColor(Pixel p_Color);
   void SetGridVisible(bool b_visible);
   void SetGridParameters(int i_StartX,
                          int i_StartY, 
                          int i_Width, 
                          int i_Height);
                          
   void SetCorrectionCurve(unsigned char *puc_Red,
                           unsigned char *puc_Green,
                           unsigned char *puc_Blue);
   void SetCorrectionEnabled(bool b_Enabled);
   void DrawObjectImagesTransparent(bool b_On);

private: // functions

   void DoZoom1To1();
   void DoZoom1To1WithMasks();
   void DoZoom1ToLarger();
   void DoZoom1ToLargerWithMasks();
   void DoZoom1ToSmaller();
   void DoZoom1ToSmallerWithMasks();

   void ZoomGraphicObjects();
   void ZoomDrawSingleObject(GraphicObject *pc_Object);
   void ZoomDrawObjectPoint(GraphicObject *pc_Object,float f_OffsetX, float f_OffsetY);
   void ZoomDrawObjectLine(GraphicObject *pc_Object,float f_OffsetX, float f_OffsetY);
   void ZoomDrawObjectRectangle(GraphicObject *pc_Object,float f_OffsetX, float f_OffsetY);
   void ZoomDrawObjectEllipse(GraphicObject *pc_Object,float f_OffsetX, float f_OffsetY);
   void ZoomDrawObjectImage(GraphicObject *pc_Object,float f_OffsetX, float f_OffsetY);
   void ZoomDrawObjectLineSub( GeoLine              c_Line,
                               Pixel p_Color);
   void ZoomDrawObjectLineSubZoomed( GeoLine              c_Line,
                                     Pixel p_Color);
   void ZoomDrawObjectBorder(RectRegion c_Region,Pixel p_BorderColor);

   void CenterCoordinate(int &i_BufferX, int &i_BufferY);
   void CenterCoordinate(float &f_BufferX, float &f_BufferY);

   Pixel MergeColors(Pixel p1,Pixel p2);
   void AddGridLines();
   
   Pixel ApplyCorrection(Pixel p_Input);
   

private:	// variables
   //-----------------------

   GlobalStorage *m_pc_GlobalStorage ;
   int m_i_ZoomFactor;

   // in ImagePixel
   int m_i_LeftStartPixel;
   int m_i_TopStartPixel;


   QImage * m_pc_DestinationImage;
   // screen pixel top/Left = (0/0)
   RectRegion m_c_DestinationArea;

   // if entire region must get rredrawm
   RectRegion m_MaximalPossibleDestinationArea;


   RectRegion m_c_ZoomOutputArea;

   // in screen pixel
   int m_i_DestinationMaxWidth;
   int m_i_DestinationMaxHeight;


   PixelBuffer * m_pc_SourceBuffer;

   bool m_b_SafetyMaskVisible;
   bool m_b_SelectionMaskVisible;

   Pixel m_p_SafetyColor;
   Pixel m_p_SelectionColor;

   // settings for the grid mode
   Pixel m_p_GridColor;
   bool  m_b_GridEnabled;
   int   m_i_GridXOffset;
   int   m_i_GridYOffset;
   int   m_i_GridWidth;
   int   m_i_GridHeight;

   // settings for the correctioncurve
   unsigned char m_uca_CorrectionRed[256];
   unsigned char m_uca_CorrectionGreen[256];
   unsigned char m_uca_CorrectionBlue[256];
   bool          m_b_CorrectionEnabled;

   bool          m_b_DrawObjectImagesTransparent;


} ; // class zoom



#endif // _ZOOM_LOADED_H
