// Box to show the current zoomfactor.

#include <stdio.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>

#include "Selection.h"
#include "PixelBuffer.h"
#include "ZoomBox.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "Zoom.h"
#include "ApplicationStorage.h"

//===========================================================================
//
//===========================================================================

ZoomBox::ZoomBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global)
{

   QBoxLayout   *l_Layout;
   QVGroupBox   *ZoomGroupBox;

   int          i_CharWidth;

   // zoom-range
   m_i_MaxPlusZoom   = 10;
   m_i_MaxMinusZoom  = -7;

   m_b_PreventUpdate = false;

   i_CharWidth = g_Application->m_pc_QtUtility->GetMaximalFontWidth();

   setCaption("Image Zooming");

   l_Layout     = new  QVBoxLayout( m_w_TopWidget,1);
   ZoomGroupBox = new QVGroupBox("Zoom",m_w_TopWidget);
   l_Layout->addWidget(ZoomGroupBox);

   {
      QHBoxLayout  *SliderLayout;
      QWidget      *SliderBox;
      SliderBox = new QWidget(ZoomGroupBox);

      m_i_CurrentSlider     =  1;
      m_i_CurrentZoomFactor =  1;

      SliderLayout  = new  QHBoxLayout( SliderBox,2);
      m_ZoomSlider = new QSlider(m_i_MaxMinusZoom+2, // 0,-1 are ignored
                                    m_i_MaxPlusZoom,
                                    1,
                                    m_i_CurrentSlider,
                                    Horizontal,
                                    SliderBox);
      m_ZoomSlider->setTickmarks(QSlider::Below);
      m_ZoomSliderLabel = new QLabel("  1",SliderBox);
      m_ZoomSliderLabel->setMinimumWidth(i_CharWidth * 3);

      SliderLayout->addWidget(m_ZoomSlider);
      SliderLayout->addWidget(m_ZoomSliderLabel);

      connect( m_ZoomSlider,
               SIGNAL(valueChanged(int)),
               SLOT(DoSliderValueChanged(int)) );

      // create the entire button
      m_EntireImageButton = new QPushButton("Entire Image",ZoomGroupBox);
      connect( m_EntireImageButton,
               SIGNAL(clicked()),
               SLOT(EntireImagePressed()) );

      // create the entire button
      m_SelectAreaButton = new QPushButton("SelectArea",ZoomGroupBox);
      connect( m_SelectAreaButton,
               SIGNAL(clicked()),
               SLOT(SelectAreaPressed()) );

   }

   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction)OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ZoomChanged");

} // ManipulationBox()

//===========================================================================
//
//===========================================================================

void ZoomBox::DoSliderValueChanged(int i_NewValue)
{
   QString qs_LabelOutput;


   m_i_CurrentSlider = i_NewValue;
   m_i_CurrentZoomFactor = m_i_CurrentSlider;
   if(m_i_CurrentZoomFactor <=0)
   {
      m_i_CurrentZoomFactor -= 2;
   }
   
   // set a value into the label
   qs_LabelOutput.sprintf("%3d",m_i_CurrentZoomFactor);
   m_ZoomSliderLabel->setText(qs_LabelOutput);

   // if not update is prevented
   if(! m_b_PreventUpdate )
   {
      // apply the factor to the image
      m_pc_GlobalStorage->m_pc_ImageFrame->SetZoomFactor(m_i_CurrentZoomFactor);
   }

} //DoSliderValueChanged()

//===========================================================================
//
//===========================================================================

void ZoomBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   else if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
   else if(qs_MessageType.compare("ZoomChanged") ==0)
   {
      int i_SliderValue;
      m_i_CurrentZoomFactor = m_pc_GlobalStorage->m_pc_ImageFrame->GetZoomFactor();
      // now set the slider regarding the zoom factor
      i_SliderValue = m_i_CurrentZoomFactor;
      if(i_SliderValue <0)
      {
         i_SliderValue +=2;
      }
      m_b_PreventUpdate = true;
      m_ZoomSlider->setValue(i_SliderValue);
      m_b_PreventUpdate = false;

   }
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================

void ZoomBox::EntireImagePressed()
{
   int i_ImageX;
   int i_ImageY;
   int i_MaxDrawingAreaX;
   int i_MaxDrawingAreaY;
   int i_FactorX;
   int i_FactorY;
   int i_NewFactor;

   // get image dimension
   i_ImageX = m_pc_GlobalStorage->m_pc_MainImage->GetWidth();
   i_ImageY = m_pc_GlobalStorage->m_pc_MainImage->GetHeight();

   // get Drawing Area dimension
   i_MaxDrawingAreaX = m_pc_GlobalStorage->m_pc_DrawingArea->GetWidth();
   i_MaxDrawingAreaY = m_pc_GlobalStorage->m_pc_DrawingArea->GetHeight();

   // calculate the zoomfactor for each dimension
   i_FactorX = CalculateMaximumZoomFactor( i_ImageX,i_MaxDrawingAreaX);
   i_FactorY = CalculateMaximumZoomFactor( i_ImageY,i_MaxDrawingAreaY);

   // select the smallest of both
   i_NewFactor= i_FactorX;
   if(i_FactorY < i_NewFactor)
   {
      i_NewFactor = i_FactorY;
   }

   // clip zoom range
   if( i_NewFactor>  m_i_MaxPlusZoom)
   {
      i_NewFactor = m_i_MaxPlusZoom;
   }
   if( i_NewFactor<  m_i_MaxMinusZoom)
   {
      i_NewFactor = m_i_MaxMinusZoom;
   }

   // now change to that zoomfactor
   if(i_NewFactor <0)
   {
      i_NewFactor +=2;
   }
   // update is not prevented. so the set value moves the slider and sets
   // also the image to a new size.
   m_ZoomSlider->setValue(i_NewFactor);

} // EntireImagePressed()

//===========================================================================
//
//===========================================================================

int ZoomBox::CalculateMaximumZoomFactor(int i_Image,int i_DrawingArea)
{
   int i_Factor;
   if(i_Image < i_DrawingArea)
   {
      // Factor >= 1
      i_Factor = i_DrawingArea / i_Image;
   }
   else
   {
      float f_Factor;
      // negative factor
      f_Factor = (float)i_Image / (float)i_DrawingArea ;

      if( (i_Image % i_DrawingArea) == 0)
      {
         // it is a multiple
         i_Factor = - (i_Image / i_DrawingArea);
      }

      else // remainder at division
      {
         // select an number smaller
         i_Factor = - (int)(f_Factor+1);
      }

   } // else negative

   return i_Factor;
} // CalculateMaximumZoomFactor()

//===========================================================================
//
//===========================================================================

void ZoomBox::SelectAreaPressed()
{
   if(  m_pc_GlobalStorage != NULL)
   {

      m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionRectangle,
                                 "Select Zoom Area",
                                 Callback(SelectionEventArrivedCB,this) );

   }

} // SelectAreaPressed()

//===========================================================================
//
//===========================================================================


void *ZoomBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   ZoomBox *pc_This = (ZoomBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================

void *ZoomBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures

   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      RectRegion   c_Region;
      RectRegion   c_ClipRegion;
      int          i_X1;
      int          i_X2;
      int          i_Y1;
      int          i_Y2;
      int          i_MaxDrawingAreaX;
      int          i_MaxDrawingAreaY;
      float        f_NewZoomX;
      float        f_NewZoomY;
      int          i_CommonNewZoom;

      if(m_pc_GlobalStorage == NULL)
      {
         return NULL;
      }
      m_pc_GlobalStorage->m_pc_Selection->GetSelectionRectangle(i_X1,i_Y1,i_X2,i_Y2);
      c_Region.SetXYXY(i_X1,i_Y1,i_X2,i_Y2);
      c_ClipRegion = m_pc_GlobalStorage->m_pc_MainImage->GetRegion();
      c_Region.ClipAt(c_ClipRegion);

      // get Drawing Area dimension
      i_MaxDrawingAreaX = m_pc_GlobalStorage->m_pc_DrawingArea->GetWidth();
      i_MaxDrawingAreaY = m_pc_GlobalStorage->m_pc_DrawingArea->GetHeight();

      f_NewZoomX = (float)i_MaxDrawingAreaX / c_Region.GetWidth()  ;
      f_NewZoomY = (float)i_MaxDrawingAreaY / c_Region.GetHeight();

      // is zoom smaller one, make it neagtive
      if(  f_NewZoomX <1)
      {
         f_NewZoomX = - (1.0 / f_NewZoomX);
        
      } 
      if(  f_NewZoomY <1)
      {
         f_NewZoomY = - (1.0 / f_NewZoomY);

      }

      // select the smaller one of X and Y
      if(  f_NewZoomX < f_NewZoomY)
      {
         i_CommonNewZoom = (int) f_NewZoomX;
      }
      else
      {
         i_CommonNewZoom = (int) f_NewZoomY;
      }
      if(i_CommonNewZoom <0)
      {
         i_CommonNewZoom -=1 ;
      }

      // clip in valid zoom ranges
      if(i_CommonNewZoom <  m_i_MaxMinusZoom)
      {
         i_CommonNewZoom =  m_i_MaxMinusZoom;         
      }
      if(i_CommonNewZoom >  m_i_MaxPlusZoom)
      {
         i_CommonNewZoom =  m_i_MaxPlusZoom;
      }
      
      // remains the same? do nothing
      if(i_CommonNewZoom == m_i_CurrentZoomFactor)
      {
         return NULL;
      }
      {
         int i_SliderValue;
         // now set the slider regarding the zoom factor
         i_SliderValue = i_CommonNewZoom ;
         if(i_SliderValue <0)
         {
            i_SliderValue +=2;
         }
         m_b_PreventUpdate = true;
         m_ZoomSlider->setValue(i_SliderValue);
         m_b_PreventUpdate = false;

      }

      m_pc_GlobalStorage->m_pc_ImageFrame->SetZoomAndRegion(i_CommonNewZoom,c_Region);
         
      // zoomfaktor changed, now calculate the stating offset

   } // If ReasonSelection

   return NULL;
} // SelectionEventArrived()

//===========================================================================
//
//===========================================================================

void ZoomBox::ZoomInOneStep()
{
   if(m_i_CurrentSlider < m_i_MaxPlusZoom)
   {
      m_ZoomSlider->setValue(m_i_CurrentSlider + 1);
   }

} // ZoomInOneStep()

//===========================================================================
//
//===========================================================================

void ZoomBox::ZoomOutOneStep()
{
   if(m_i_CurrentSlider > m_i_MaxMinusZoom)
   {
      m_ZoomSlider->setValue(m_i_CurrentSlider - 1);
   }
   
}  // ZoomOutOneStep

//===========================================================================
//
//===========================================================================

