// Zoom functions.
// special section for display objects
#include <math.h>

#include "Utilities.h"
#include "PTypes.h"
#include "Zoom.h"
#include "DrawingArea.h"
#include "GraphicObjectLayer.h"
#include "GraphicObjectMain.h"

//===========================================================================
//
//===========================================================================


void Zoom::ZoomGraphicObjects()
{
   int i_LayerCount;
   int i;

   TRACE<< "ZoomGraphicObjects()\n";
   // get the number of layers
   i_LayerCount = m_pc_GlobalStorage->m_pc_ObjectMain->GetLayerCount();

   // for all layers
   for(i=0; i< i_LayerCount; i++)
   {
      GraphicObjectLayer * pc_ActualLayer;
      pc_ActualLayer = m_pc_GlobalStorage->m_pc_ObjectMain->GetLayer(i);

      TRACE << "ZoomGraphicObjects() - layer " << i << "\n" ;
      if(pc_ActualLayer->IsVisible())
      {
         // get all obejcts of the layer and draw them
         int i_NumberObjects;
         int i_ActualObjectIndex;

         i_NumberObjects = pc_ActualLayer-> GetObjectCount();
         TRACE << "ZoomGraphicObjects() - is visible, objects ="
               << i_NumberObjects
               << "\n" ;
         for(i_ActualObjectIndex = 0;
             i_ActualObjectIndex < i_NumberObjects;
             i_ActualObjectIndex++)
         {
            GraphicObject *pc_ActualObject;

            pc_ActualObject = pc_ActualLayer->GetObject(i_ActualObjectIndex);

            // now draw the single object
            ZoomDrawSingleObject(pc_ActualObject);

         } // for all objects

      } // if layer visible

   } // for i

} // ZoomGrapicObjects()

//===========================================================================
//
//===========================================================================

void Zoom::ZoomDrawSingleObject(GraphicObject *pc_Object)
{
   int i_Type;
   
   TRACE << "ZoomDrawSingleObject()\n" ;
   if(pc_Object == NULL)
   {
      return;
   }
   // don't draw invisible objects
   if(!pc_Object->IsVisible())
   {
      return;
   }

   // switch on the object type
   i_Type = pc_Object ->GetObjectType();
   switch (i_Type)
   {
   case GraphicObject::TypePoint:
      ZoomDrawObjectPoint(pc_Object,0.0,0.0);
      break;
   case GraphicObject::TypeLine:
      ZoomDrawObjectLine(pc_Object,0.0,0.0);
      break;
   case GraphicObject::TypeRectangle:
      ZoomDrawObjectRectangle(pc_Object,0.0,0.0);
      break;
   case GraphicObject::TypeEllipse:
      ZoomDrawObjectEllipse(pc_Object,0.0,0.0);
      break;
   case GraphicObject::TypeImage:
      ZoomDrawObjectImage(pc_Object,0.0,0.0);
      break;
   default:
      break;
   } // switch type

   if(pc_Object->IsSelected())
   {
      RectRegion c_ObjectSize;
      Pixel      p_Color;
      
      // get working color
      p_Color = m_pc_GlobalStorage->m_pc_ObjectMain->GetWorkColor();
      // get Size of Object
      c_ObjectSize = pc_Object->GetSize();
      // Draw a rectangle border around it
      ZoomDrawObjectBorder(c_ObjectSize,p_Color);

   } // if it is selected

} // ZoomDrawSingleObject()

//===========================================================================
//
//===========================================================================

void Zoom::ZoomDrawObjectPoint(GraphicObject *pc_Object,
                               float f_OffsetX,
                               float f_OffsetY)
{
   GraphicObjectAttribute * pc_Attribute;
   GeoCoordinate           c_Position;
   Pixel                   p_Color;

   TRACE << "ZoomDrawObjectPoint()\n" ;


   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::StartPoint);
   if(pc_Attribute ==NULL)
   {
      return;
   }

   c_Position= pc_Attribute->m_c_Coordinate;

   c_Position.Transform(f_OffsetX,f_OffsetY);

   // check, whether point is in current possible drawing region
   if(m_MaximalPossibleDestinationArea.PointInRegion(c_Position) )
   {
      // calclute the buffer coordinates
      int i_BufferX;
      int i_BufferY;

      p_Color = pc_Object->GetColor();

      // first converto to integer
      i_BufferX = (int) c_Position.GetX();
      i_BufferY = (int) c_Position.GetY();

      i_BufferX -= m_i_LeftStartPixel;
      i_BufferY -= m_i_TopStartPixel;
      if(   i_BufferX >=0
         && i_BufferY >=0)
      {
         int i;
         if(m_i_ZoomFactor>=1)
         {

            i_BufferX *= m_i_ZoomFactor;
            i_BufferY *= m_i_ZoomFactor;
         }
         else
         {

            i_BufferX /= (-m_i_ZoomFactor) ;
            i_BufferY /= (-m_i_ZoomFactor) ;

         }

         CenterCoordinate(i_BufferX, i_BufferY);

         // now the real drawing part
         for(i= -5; i<= 5 ; i++)
         {
            int p_ActualX;
            int p_ActualY1;
            int p_ActualY2;

            // draw a cross
            p_ActualX = i_BufferX + i;
            p_ActualY1 = i_BufferY + i;
            p_ActualY2 = i_BufferY - i;

            if(m_c_DestinationArea.PointInRegion(p_ActualX,p_ActualY1))
            {
               m_pc_DestinationImage->setPixel(p_ActualX,p_ActualY1,p_Color);
            }
            if(m_c_DestinationArea.PointInRegion(p_ActualX,p_ActualY2))
            {
               m_pc_DestinationImage->setPixel(p_ActualX,p_ActualY2,p_Color);
            }

         } // for i

      } // if coordinate > 0

   } // if in region


} // ZoomDrawObjectPoint()

//===========================================================================
//
//===========================================================================

void Zoom::ZoomDrawObjectRectangle(GraphicObject *pc_Object,
                               float f_OffsetX,
                               float f_OffsetY)
{
   // get the coordinates and draw 4 lines
   GraphicObjectAttribute * pc_Attribute;
   GeoCoordinate           c_StartPosition;
   GeoCoordinate           c_EndPosition;
   GeoLine                 c_Line[4];
   Pixel                   p_Color;
   int                     i;

   TRACE << "ZoomDrawObjectRectangle()\n" ;

   // get the color of the rectangle
   p_Color = pc_Object->GetColor();

   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::StartPoint);
   if(pc_Attribute ==NULL)
   {
      return;
   }

   c_StartPosition= pc_Attribute->m_c_Coordinate;

   c_StartPosition.Transform(f_OffsetX,f_OffsetY);

   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::EndPoint);
   if(pc_Attribute ==NULL)
   {
      return;
   }

   c_EndPosition= pc_Attribute->m_c_Coordinate;

   c_EndPosition.Transform(f_OffsetX,f_OffsetY);

   // create 4 seperate lines
   c_Line[0].Set(c_StartPosition,
                 GeoCoordinate(c_StartPosition.GetX(),c_EndPosition.GetY())
                 );
   c_Line[1].Set(c_StartPosition,
                 GeoCoordinate(c_EndPosition.GetX(),c_StartPosition.GetY())
                 );
   c_Line[2].Set(GeoCoordinate(c_StartPosition.GetX(),c_EndPosition.GetY()),
                 c_EndPosition
                 );
   c_Line[3].Set(GeoCoordinate(c_EndPosition.GetX(),c_StartPosition.GetY()),
                 c_EndPosition
                 );

   // now draw all 4 lines
   for(i=0;i<4;i++)
   {
      ZoomDrawObjectLineSub(c_Line[i],p_Color);
   }


} // ZoomDrawObjectRectangle()

//===========================================================================
//
//===========================================================================

void Zoom::ZoomDrawObjectEllipse(GraphicObject *pc_Object,
                               float f_OffsetX,
                               float f_OffsetY)
{
   // get the coordinates and draw 4 lines
   GraphicObjectAttribute * pc_Attribute;
   GeoCoordinate           c_StartPosition;
   GeoCoordinate           c_EndPosition;
   GeoLine                 c_Line;
   Pixel                   p_Color;
   float                   f_CenterX;
   float                   f_CenterY;
   float                   f_RadiusX;
   float                   f_RadiusY;
   float                   f_Factor;
   float                   f_WidthX;
   float                   f_LastX;
   float                   f_BufferStartX;
   float                   f_BufferEndX;
   float                   f_BufferStartY;
   float                   f_BufferEndY;
   int                     i_RunY;
   int                     i_Width;
   int                     i_RunYMax;

   TRACE << "ZoomDrawObjectEllipse()\n" ;

   // first version - assume ellipse is symetrical to the axis
   
   // get the color of the ellipse
   p_Color = pc_Object->GetColor();

   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::StartPoint);
   if(pc_Attribute ==NULL)
   {
      return;
   }

   c_StartPosition= pc_Attribute->m_c_Coordinate;

   c_StartPosition.Transform(f_OffsetX,f_OffsetY);

   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::EndPoint);
   if(pc_Attribute ==NULL)
   {
      return;
   }

   c_EndPosition= pc_Attribute->m_c_Coordinate;

   c_EndPosition.Transform(f_OffsetX,f_OffsetY);

   // get position into plain variabls
   f_BufferStartX = c_StartPosition.GetX();
   f_BufferStartY = c_StartPosition.GetY();
   f_BufferEndX   = c_EndPosition.GetX();
   f_BufferEndY   = c_EndPosition.GetY();

   // remove the zoom offset
   f_BufferStartX -= m_i_LeftStartPixel;
   f_BufferStartY -= m_i_TopStartPixel;
   f_BufferEndX   -= m_i_LeftStartPixel;
   f_BufferEndY   -= m_i_TopStartPixel;

   // multiply the zoom factor
   if(m_i_ZoomFactor>=1)
   {
      f_BufferStartX *= m_i_ZoomFactor;
      f_BufferStartY *= m_i_ZoomFactor;
      f_BufferEndX   *= m_i_ZoomFactor;
      f_BufferEndY   *= m_i_ZoomFactor;
   }
   else
   {
      f_BufferStartX /= (-m_i_ZoomFactor) ;
      f_BufferStartY /= (-m_i_ZoomFactor) ;
      f_BufferEndX   /= (-m_i_ZoomFactor) ;
      f_BufferEndY   /= (-m_i_ZoomFactor) ;
   }


   // calculate center and radius of the ellipse
   f_CenterX = (f_BufferStartX + f_BufferEndX) / 2.0;
   f_RadiusX = fabs(f_CenterX - f_BufferStartX ) ;

   f_CenterY = (f_BufferStartY + f_BufferEndY) / 2.0;
   f_RadiusY = fabs(f_CenterY - f_BufferStartY);

   // now draw the ellipse
   f_Factor =   (float)  (f_RadiusX * f_RadiusX)
              / ((float) (f_RadiusY * f_RadiusY));

   // first the middle line
   f_WidthX = sqrt (( float) (f_Factor * (  f_RadiusY * f_RadiusY)  ));

   i_Width      = (int) (f_WidthX +0.5);

   f_LastX =0.0;

   if(i_Width > 0)
   {
      // left part
      i_Width = (int) (f_WidthX );
      c_Line.Set(f_CenterX-i_Width,f_CenterY,
                 f_CenterX-i_Width,f_CenterY
                 );
                 
      ZoomDrawObjectLineSubZoomed(c_Line,p_Color);

      // right part
      c_Line.Set(f_CenterX+i_Width,f_CenterY,
                 f_CenterX+i_Width,f_CenterY
                 );
      ZoomDrawObjectLineSubZoomed(c_Line,p_Color);

      f_LastX = f_WidthX;
   }

   i_RunYMax = (int) (f_RadiusY);
   
   for(i_RunY= 1;i_RunY<= i_RunYMax; i_RunY ++)
   {

      f_WidthX = sqrt (( float) (f_Factor * (  f_RadiusY * f_RadiusY
                                             - i_RunY * i_RunY)));
      // force a closing at the end
      if(i_RunY == i_RunYMax)
      {
         f_WidthX = 0.0;
      }
      
      if(   f_WidthX >= 0.0
         && f_LastX  >= 0.0 )
      {
         i_Width = (int) f_WidthX;
         // draw upper lines
         c_Line.Set(f_CenterX-i_Width,       f_CenterY+i_RunY,
                       f_CenterX-f_LastX,f_CenterY+i_RunY
                    );
         ZoomDrawObjectLineSubZoomed(c_Line,p_Color);

         c_Line.Set(f_CenterX+i_Width       ,f_CenterY+i_RunY,
                       f_CenterX+f_LastX       ,f_CenterY+i_RunY
                    );
         ZoomDrawObjectLineSubZoomed(c_Line,p_Color);


         // draw lower line
         c_Line.Set(f_CenterX-i_Width,       f_CenterY-i_RunY,
                       f_CenterX-f_LastX,       f_CenterY-i_RunY
                    );
         ZoomDrawObjectLineSubZoomed(c_Line,p_Color);

         c_Line.Set(f_CenterX+f_LastX,f_CenterY-i_RunY,
                       f_CenterX+i_Width       ,f_CenterY-i_RunY
                    );
         ZoomDrawObjectLineSubZoomed(c_Line,p_Color);

         f_LastX = f_WidthX;

      } // if valid points

   } // for all lines


} // ZoomDrawObjectEllipse()

//===========================================================================
//
//===========================================================================

void Zoom::ZoomDrawObjectLine(GraphicObject *pc_Object,
                               float f_OffsetX,
                               float f_OffsetY)
{
   GraphicObjectAttribute * pc_Attribute;
   GeoCoordinate           c_StartPosition;
   GeoCoordinate           c_EndPosition;
   GeoLine                 c_Line;
   GeoLine                 c_ClipLine;
   Pixel                   p_Color;

   TRACE << "ZoomDrawObjectLine()\n" ;

   // get the color of the line
   p_Color = pc_Object->GetColor();


   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::StartPoint);
   if(pc_Attribute ==NULL)
   {
      return;
   }

   c_StartPosition= pc_Attribute->m_c_Coordinate;

   c_StartPosition.Transform(f_OffsetX,f_OffsetY);

   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::EndPoint);
   if(pc_Attribute ==NULL)
   {
      return;
   }

   c_EndPosition= pc_Attribute->m_c_Coordinate;

   c_EndPosition.Transform(f_OffsetX,f_OffsetY);

   c_Line.Set(c_StartPosition,c_EndPosition);

   // now test, whether the line is in the visible area

   ZoomDrawObjectLineSub(c_Line,p_Color);

} // ZoomDrawObjectLine()

//===========================================================================
//
//===========================================================================

void Zoom::ZoomDrawObjectLineSub(GeoLine        c_Line,
                                 Pixel          p_Color)
{
   GeoLine                 c_ClipLine;
   bool                    b_IsInRect;
   float f_BufferStartX;
   float f_BufferStartY;
   float f_BufferEndX;
   float f_BufferEndY;
   int   i_X1;
   int   i_Y1;
   int   i_X2;
   int   i_Y2;
   int   i_DiffX;
   int   i_DiffY;
   // now test, whether the line is in the visible area

   TRACE << "ZoomDrawObjectLineSub()\n" ;
   b_IsInRect = c_Line.ClipInRectRegion(m_MaximalPossibleDestinationArea,c_ClipLine);

   // if it is visible
   if(! b_IsInRect)
   {
      TRACE << "Line is not in Clipping Region\n" ;
      return;
   }

   // get coordinates in 4 variables
   c_Line.Get(f_BufferStartX,f_BufferStartY,
                    f_BufferEndX,f_BufferEndY);

   f_BufferStartX -= m_i_LeftStartPixel;
   f_BufferStartY -= m_i_TopStartPixel;
   f_BufferEndX   -= m_i_LeftStartPixel;
   f_BufferEndY   -= m_i_TopStartPixel;

   if(m_i_ZoomFactor>=1)
   {
      f_BufferStartX *= m_i_ZoomFactor;
      f_BufferStartY *= m_i_ZoomFactor;
      f_BufferEndX   *= m_i_ZoomFactor;
      f_BufferEndY   *= m_i_ZoomFactor;
   }
   else
   {
      f_BufferStartX /= (-m_i_ZoomFactor) ;
      f_BufferStartY /= (-m_i_ZoomFactor) ;
      f_BufferEndX   /= (-m_i_ZoomFactor) ;
      f_BufferEndY   /= (-m_i_ZoomFactor) ;
   }

   CenterCoordinate(f_BufferStartX, f_BufferStartY);
   CenterCoordinate(f_BufferEndX  , f_BufferEndY);

   // now draw a line between the buffer coordiantes
   i_X1 = (int) f_BufferStartX;
   i_Y1 = (int) f_BufferStartY;
   i_X2 = (int) f_BufferEndX;
   i_Y2 = (int) f_BufferEndY;


   // calculate differences
   i_DiffX = i_X2 - i_X1;
   if(i_DiffX<0)
   {
      i_DiffX = - i_DiffX;
   }

   i_DiffY = i_Y2 - i_Y1;
   if(i_DiffY<0)
   {
      i_DiffY = - i_DiffY;
   }

   if(i_DiffY == 0)
   {
      // horizontal line
      PLONG *pc_InDestination;
      int   i_RunX;

      // make loop for X
      if(i_X1> i_X2)
      {
         Utilities::swap(i_X1,i_X2);
      }

      pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y1) + i_X1;
      for(i_RunX= i_X1; i_RunX <= i_X2;i_RunX++)
      {
         if(m_c_DestinationArea.PointInRegion(i_RunX,i_Y1))
         {
            *pc_InDestination = p_Color;
         }
         pc_InDestination++;
      }  // for i_RunX
   } // horizontal
   else if(i_DiffX == 0)
   {
      // vertical line
      int i_RunY;

      // make loop for X
      if(i_Y1> i_Y2)
      {
         Utilities::swap(i_Y1,i_Y2);
      }
      for(i_RunY= i_Y1; i_RunY <= i_Y2;i_RunY++)
      {
         if(m_c_DestinationArea.PointInRegion(i_X1,i_RunY))
         {
            m_pc_DestinationImage->setPixel(i_X1,i_RunY,p_Color);
         }
      }  // for i_RunY
   } // vertical
   else if(i_DiffX >= i_DiffY)
   {
      int i_RunX;
      int i_RunY;
      int i_YOffset;
      int i_Divider;

      // make loop for X
      if(i_X1> i_X2)
      {
         Utilities::swap(i_X1,i_X2);
         Utilities::swap(i_Y1,i_Y2);
      }
      if(i_Y1>i_Y2)
      {
         i_YOffset = -1;
      }
      else if(i_Y1<i_Y2)
      {
         i_YOffset = 1;
      }
      else
      {
         i_YOffset = 0;
      }

      i_Divider = i_DiffX /2;
      i_RunY = i_Y1;
      for(i_RunX= i_X1; i_RunX <= i_X2;i_RunX++)
      {
         if(m_c_DestinationArea.PointInRegion(i_RunX,i_RunY))
         {
            m_pc_DestinationImage->setPixel(i_RunX,i_RunY,p_Color);
          }
         i_Divider -= i_DiffY;
         if(i_Divider <0)
         {
            i_Divider += i_DiffX;
            i_RunY    += i_YOffset;
         }
      }
   }
   else  // DiffY ist larger
   {
      // make loop for Y
      int i_RunX;
      int i_RunY;
      int i_XOffset;
      int i_Divider;

      if(i_Y1> i_Y2)
      {
         Utilities::swap(i_X1,i_X2);
         Utilities::swap(i_Y1,i_Y2);
      }
      if(i_X1>i_X2)
      {
         i_XOffset = -1;
      }
      else if(i_X1<i_X2)
      {
         i_XOffset = 1;
      }
      else
      {
         i_XOffset = 0;
      }

      i_Divider = i_DiffY /2;
      i_RunX = i_X1;
      for(i_RunY= i_Y1; i_RunY <= i_Y2;i_RunY++)
      {
         if(m_c_DestinationArea.PointInRegion(i_RunX,i_RunY))
         {
            m_pc_DestinationImage->setPixel(i_RunX,i_RunY,p_Color);
         }
         i_Divider -= i_DiffX;
         if(i_Divider <0)
         {
            i_Divider += i_DiffY;
            i_RunX    += i_XOffset;
         }
      }
   } // diffy larger

} // ZoomDrawObjectLineSub()

//===========================================================================
// assume - coordinated are in zoomed coordinates
//===========================================================================

void Zoom::ZoomDrawObjectLineSubZoomed(GeoLine        c_Line,
                                 Pixel          p_Color)
{
   GeoLine  c_ClipLine;
   bool     b_IsInRect;
   float    f_BufferStartX;
   float    f_BufferStartY;
   float    f_BufferEndX;
   float    f_BufferEndY;
   int      i_X1;
   int      i_Y1;
   int      i_X2;
   int      i_Y2;
   int      i_DiffX;
   int      i_DiffY;
   // now test, whether the line is in the visible area

   TRACE << "ZoomDrawObjectLineSub()\n" ;

   b_IsInRect = c_Line.ClipInRectRegion(m_c_DestinationArea,c_ClipLine);

   // if it is visible
   if(! b_IsInRect)
   {
      TRACE << "Line is not in Clipping Region\n" ;
      return;
   }

   // get coordinates in 4 variables
   c_Line.Get(f_BufferStartX,f_BufferStartY,
                    f_BufferEndX,f_BufferEndY);


   // now draw a line between the buffer coordiantes
   i_X1 = (int) f_BufferStartX;
   i_Y1 = (int) f_BufferStartY;
   i_X2 = (int) f_BufferEndX;
   i_Y2 = (int) f_BufferEndY;


   // calculate differences
   i_DiffX = i_X2 - i_X1;
   if(i_DiffX<0)
   {
      i_DiffX = - i_DiffX;
   }

   i_DiffY = i_Y2 - i_Y1;
   if(i_DiffY<0)
   {
      i_DiffY = - i_DiffY;
   }

   if(i_DiffY == 0)
   {
      // horizontal line
      PLONG *pc_InDestination;
      int    i_RunX;

      // make loop for X
      if(i_X1> i_X2)
      {
         Utilities::swap(i_X1,i_X2);
      }
      pc_InDestination = (PLONG *) m_pc_DestinationImage->scanLine(i_Y1) + i_X1;
      for(i_RunX= i_X1; i_RunX <= i_X2;i_RunX++)
      {
         if(m_c_DestinationArea.PointInRegion(i_RunX,i_Y1))
         {
            *pc_InDestination = p_Color;
         }
         // increase pointer
         pc_InDestination++;
      }  // for i_RunX
   } // horizontal
   else if(i_DiffX == 0)
   {
      // vertical line
      int i_RunY;

      // make loop for X
      if(i_Y1> i_Y2)
      {
         Utilities::swap(i_Y1,i_Y2);
      }
      for(i_RunY= i_Y1; i_RunY <= i_Y2;i_RunY++)
      {
         if(m_c_DestinationArea.PointInRegion(i_X1,i_RunY))
         {
            m_pc_DestinationImage->setPixel(i_X1,i_RunY,p_Color);
         }
      }  // for i_RunY
   } // vertical
   else if(i_DiffX >= i_DiffY)
   {
      int i_RunX;
      int i_RunY;
      int i_YOffset;
      int i_Divider;

      // make loop for X
      if(i_X1> i_X2)
      {
         Utilities::swap(i_X1,i_X2);
         Utilities::swap(i_Y1,i_Y2);
      }
      if(i_Y1>i_Y2)
      {
         i_YOffset = -1;
      }
      else if(i_Y1<i_Y2)
      {
         i_YOffset = 1;
      }
      else
      {
         i_YOffset = 0;
      }

      i_Divider = i_DiffX /2;
      i_RunY = i_Y1;
      for(i_RunX= i_X1; i_RunX <= i_X2;i_RunX++)
      {
         if(m_c_DestinationArea.PointInRegion(i_RunX,i_RunY))
         {
            m_pc_DestinationImage->setPixel(i_RunX,i_RunY,p_Color);
          }
         i_Divider -= i_DiffY;
         if(i_Divider <0)
         {
            i_Divider += i_DiffX;
            i_RunY    += i_YOffset;
         }
      }
   }
   else  // DiffY ist larger
   {
      // make loop for Y
      int i_RunX;
      int i_RunY;
      int i_XOffset;
      int i_Divider;

      if(i_Y1> i_Y2)
      {
         Utilities::swap(i_X1,i_X2);
         Utilities::swap(i_Y1,i_Y2);
      }
      if(i_X1>i_X2)
      {
         i_XOffset = -1;
      }
      else if(i_X1<i_X2)
      {
         i_XOffset = 1;
      }
      else
      {
         i_XOffset = 0;
      }

      i_Divider = i_DiffY /2;
      i_RunX = i_X1;
      for(i_RunY= i_Y1; i_RunY <= i_Y2;i_RunY++)
      {
         if(m_c_DestinationArea.PointInRegion(i_RunX,i_RunY))
         {
            m_pc_DestinationImage->setPixel(i_RunX,i_RunY,p_Color);
         }
         i_Divider -= i_DiffX;
         if(i_Divider <0)
         {
            i_Divider += i_DiffY;
            i_RunX    += i_XOffset;
         }
      }
   } // diffy larger

} // ZoomDrawObjectLineSubZoomed()

//===========================================================================
//
//===========================================================================

void Zoom::ZoomDrawObjectBorder(RectRegion c_Region,Pixel p_BorderColor)
{
   // get the coordinates and draw 4 lines
   GeoLine  c_Line[4];
   int      i;


   TRACE << "ZoomDrawObjectBorder\n";

   // create 4 seperate lines
   c_Line[0].Set( c_Region.GetStartXFloat(),
                  c_Region.GetStartYFloat(),
                  c_Region.GetStartXFloat(),
                  c_Region.GetEndYFloat()
                );
   c_Line[1].Set( c_Region.GetStartXFloat(),
                  c_Region.GetStartYFloat(),
                  c_Region.GetEndXFloat(),
                  c_Region.GetStartYFloat()
                );
   c_Line[2].Set( c_Region.GetEndXFloat(),
                  c_Region.GetStartYFloat(),
                  c_Region.GetEndXFloat(),
                  c_Region.GetEndYFloat()
                );
   c_Line[3].Set( c_Region.GetStartXFloat(),
                  c_Region.GetEndYFloat(),
                  c_Region.GetEndXFloat(),
                  c_Region.GetEndYFloat()
                );

   // now draw all 4 lines
   for(i=0;i<4;i++)
   {
      ZoomDrawObjectLineSub(c_Line[i],p_BorderColor);
   }

   // in future draw edges for moving, sizing and rotating

} // ZoomDrawObjectBorder()

//===========================================================================
//
//===========================================================================

void Zoom::ZoomDrawObjectImage(GraphicObject *pc_Object,
                               float f_OffsetX,
                     	       float f_OffsetY)
{
   // get the coordinates and draw 4 lines
   GraphicObjectAttribute * pc_Attribute;
   GeoCoordinate            c_StartPosition;
   GeoCoordinate            c_EndPosition;
   GeoLine                  c_Line[4];
   Pixel                    p_Color;
   PixelBuffer             *pc_Image;
   float                    f_BufferStartX;
   float                    f_BufferStartY;
   float                    f_BufferEndX;
   float                    f_BufferEndY;
   float                    f_DesiredWidth;
   float                    f_DesiredHeight;
   float                    f_ZoomImageX;
   float                    f_ZoomImageY;
   int                      i_OriginalWidth;
   int                      i_OriginalHeight;
   int                      i;
   int                      i_OriginalX;
   int                      i_OriginalY;

   TRACE << "ZoomDrawObjectImage()\n" ;

   // get the color of the rectangle
   p_Color = pc_Object->GetColor();

   // -- block drawing as outline
   
   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::StartPoint);
   if(pc_Attribute ==NULL)
   {
      return;
   }
   c_StartPosition= pc_Attribute->m_c_Coordinate;


   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::EndPoint);
   if(pc_Attribute ==NULL)
   {
      return;
   }
   c_EndPosition= pc_Attribute->m_c_Coordinate;

      
   c_EndPosition.Transform(f_OffsetX,f_OffsetY);
   c_StartPosition.Transform(f_OffsetX,f_OffsetY);

   // create 4 seperate lines
   c_Line[0].Set(c_StartPosition,
                 GeoCoordinate(c_StartPosition.GetX(),c_EndPosition.GetY())
                 );
   c_Line[1].Set(c_StartPosition,
                 GeoCoordinate(c_EndPosition.GetX(),c_StartPosition.GetY())
                 );
   c_Line[2].Set(GeoCoordinate(c_StartPosition.GetX(),c_EndPosition.GetY()),
                 c_EndPosition
                 );
   c_Line[3].Set(GeoCoordinate(c_EndPosition.GetX(),c_StartPosition.GetY()),
                 c_EndPosition
                 );

   // now draw all 4 lines
   for(i=0;i<4;i++)
   {
      ZoomDrawObjectLineSub(c_Line[i],p_Color);
   }

   // -- real image drawing block

   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::Image);
   if(pc_Attribute ==NULL)
   {
      return;
   }
   pc_Image = pc_Attribute->m_pc_ImageData;
   if(pc_Image ==NULL)
   {
      return;
   }
   i_OriginalWidth  = pc_Image->GetWidth();
   i_OriginalHeight = pc_Image->GetHeight();
   

   f_DesiredWidth  = c_EndPosition.GetX() - c_StartPosition.GetX() +1;
   f_DesiredHeight = c_EndPosition.GetY() - c_StartPosition.GetY() +1;

   f_ZoomImageX =  i_OriginalWidth  / f_DesiredWidth  ;
   f_ZoomImageY =  i_OriginalHeight / f_DesiredHeight ;

   
   c_StartPosition.Transform(f_OffsetX,f_OffsetY);
   c_EndPosition.Transform(f_OffsetX,f_OffsetY);

   // get position into plain variabls
   f_BufferStartX = c_StartPosition.GetX();
   f_BufferStartY = c_StartPosition.GetY();
   f_BufferEndX   = c_EndPosition.GetX();
   f_BufferEndY   = c_EndPosition.GetY();

   // remove the zoom offset
   f_BufferStartX -= m_i_LeftStartPixel;
   f_BufferStartY -= m_i_TopStartPixel;
   f_BufferEndX   -= m_i_LeftStartPixel;
   f_BufferEndY   -= m_i_TopStartPixel;

   // multiply the zoom factor
   if(m_i_ZoomFactor>=1)
   {
      f_BufferStartX *= m_i_ZoomFactor;
      f_BufferStartY *= m_i_ZoomFactor;
      f_BufferEndX   *= m_i_ZoomFactor;
      f_BufferEndY   *= m_i_ZoomFactor;
   }
   else
   {
      f_BufferStartX /= (-m_i_ZoomFactor) ;
      f_BufferStartY /= (-m_i_ZoomFactor) ;
      f_BufferEndX   /= (-m_i_ZoomFactor) ;
      f_BufferEndY   /= (-m_i_ZoomFactor) ;
   }

   { // drawing block
      int i_RunY;
      int i_RunX;
      int i_RealX;
      int i_RealY;
      float f_PictureX;
      float f_PictureY;

      i_RealY=0;
      for(i_RunY= (int)f_BufferStartY;
          i_RunY<= (int)f_BufferEndY;
          i_RunY++)
      {

         // calculate y point transformation before out of
         // the X-loop.
         if(m_i_ZoomFactor>=1)
         {
            f_PictureY = i_RealY / (float) m_i_ZoomFactor;
         }
         else
         {
            f_PictureY = i_RealY * ((float) -m_i_ZoomFactor);
         }
         // multiply the object to real image faktor
         f_PictureY *= f_ZoomImageY;

         i_OriginalY = (int) f_PictureY;
         if(i_OriginalY <0)
         {
            i_OriginalY =0;
         }
         else if( i_OriginalY >= i_OriginalHeight)
         {
            i_OriginalY = i_OriginalHeight-1;
         }

         i_RealX =0;
         for(i_RunX= (int)f_BufferStartX;
             i_RunX<= (int)f_BufferEndX;
             i_RunX++)
         {
            if(m_c_DestinationArea.PointInRegion(i_RunX,i_RunY))
            {
               if(m_i_ZoomFactor>=1)
               {
                  f_PictureX = i_RealX / (float) m_i_ZoomFactor;
               }
               else
               {
                  f_PictureX = i_RealX * ((float) -m_i_ZoomFactor);
               }

               // f_Picture has now the pixel coordinates of
               // the bounding box

               // multiply the object to real image faktor
               f_PictureX *= f_ZoomImageX;

               i_OriginalX = (int) f_PictureX;
               if(i_OriginalX <0)
               {
                  i_OriginalX =0;
               }
               else if( i_OriginalX >= i_OriginalWidth)
               {
                  i_OriginalX = i_OriginalWidth-1;
               }
               

               // only draw the pixel, if it selected in the
               // selectionmask
               if(pc_Image->IsSelected(i_OriginalX,i_OriginalY))
               {
                  p_Color = pc_Image->Get(i_OriginalX,i_OriginalY);

                  if(m_b_DrawObjectImagesTransparent)
                  {
                     Pixel p_Old;
                     p_Old = m_pc_DestinationImage->pixel(i_RunX,i_RunY);
                     p_Old = MergeColors(p_Old,p_Color);
                     m_pc_DestinationImage->setPixel(i_RunX,i_RunY,p_Old);
                     
                  }
                  else // not transparent
                  {
                     m_pc_DestinationImage->setPixel(i_RunX,i_RunY,p_Color);
                  }  // else not transparent
               }
            }

            i_RealX++;
         } // for X
         
         i_RealY ++;
      } // for Y
   } // drawing block

   
} // ZoomDrawObjectImage()


//===========================================================================
//
//===========================================================================

